// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  AvailabilityState.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b11147       6.1.0.3   04/14/2008   vandersn  New part.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid;

import java.io.Serializable;

import com.ibm.ws.objectgrid.Constants;

/**
 * Each shard in a distributed ObjectGrid has an availability state associated with it.
 * This state refers to the shard's ability to process incoming requests.
 *
 * @since WAS XD 6.1.0.3, XC10
 */
public final class AvailabilityState implements Serializable {

    private static final long serialVersionUID = -4050671632967940100L;

    /**
     * An AvailabilityState.OFFLINE indicates that a shard is offline and unable to process
     * requests.
     */
    public final static AvailabilityState OFFLINE = new AvailabilityState((byte) 0);

    /**
     * An AvailabilityState.PRELOAD indicates that a shard is in the preload state.  When
     * in the preload state, a shard will reject all requests that are not initiated from
     * a client that is preloading data into the ObjectGrid.
     */
    public final static AvailabilityState PRELOAD = new AvailabilityState((byte) 1);

    /**
     * An AvailabilityState.ONLINE indicates that a shard is online.  A shard that is online
     * is available for processing requests.
     */
    public final static AvailabilityState ONLINE = new AvailabilityState((byte) 2);

    /**
     * An AvailabilityState.QUIESCE indicates that a shard is in quiesce.  Quiesce is a
     * transitional state.  Shards that are in the quiesce state are on their way to being
     * offline.
     *
     * A shard in the quiesce state will allow all pending transactions to complete
     * before moving to the AvailabilityState.OFFLINE, assuming that all pending transactions complete
     * within 30 seconds after entering the quiesce state.
     */
    public final static AvailabilityState QUIESCE = new AvailabilityState((byte) 3);

    /**
     * An AvailabilityState.UNKNOWN indicates that the availability state of the shard could
     * not be determined.
     */
    public final static AvailabilityState UNKNOWN = new AvailabilityState((byte) 4);


    final private byte ivUniqueId;

    /**
     * Returns the internal identifier for this state.
     * @return the internal id.
     * @since 7.1.1
     */
    public int getId()
    {
    	return (int)ivUniqueId;
    }

    private AvailabilityState() {
        this((byte) 0);
    }

    private AvailabilityState(byte uniqueId) {
        ivUniqueId = uniqueId;
    }

    private Object readResolve () throws java.io.ObjectStreamException {
        switch(ivUniqueId){
            case 0: return OFFLINE;
            case 1: return AvailabilityState.PRELOAD;
            case 2: return ONLINE;
            case 3: return AvailabilityState.QUIESCE;
            default: return UNKNOWN;
        }
    }

    public String toString() {
        switch (ivUniqueId) {
        case 0:
            return Constants.AVAILABILITY_STATE_OFFLINE;
        case 1:
            return Constants.AVAILABILITY_STATE_PRELOAD;
        case 2:
            return Constants.AVAILABILITY_STATE_ONLINE;
        case 3:
            return Constants.AVAILABILITY_STATE_QUIESCE;
        default:
            return Constants.AVAILABILITY_STATE_UNKNOWN;
        }
    }

}
