// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.6 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/LockTimeoutException.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:21:28
//
// Module  :  LockTimeoutException.java
//
// Source File Description:
//
// See javadoc for this class.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   02/23/2005   neyema    new code drop
// 283173       WASX.XD   06/12/2005   kjlaw     get/setLockRequestQueueDetails methods.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid;

import java.security.AccessController;
import java.security.PrivilegedAction;

/**
 * This exception is used by the lock manager to indicate that the maximum wait time
 * for a lock has been exceeded.  The timeout may or may not be the
 * result of a deadlock.  If it is a deadlock, the timeout is used
 * to break the deadlock.
 *
 * @since WAS XD 6.0, XC10
 */
public class LockTimeoutException extends LockException
{
    /**
     * The <code>serialVersionUID</code> for this class.
     */
    private static final long serialVersionUID = 3806095146526796058L;

	private static final boolean produceJavaCoreOnLockTimeout = Boolean
			.parseBoolean(getSystemProperty(
					"com.ibm.websphere.xs.ProduceJavaCoreOnLockTimeout", "false"));
	
	private static final boolean isIBMJDK;

	static {
        String vmVendor = getSystemProperty("java.vm.vendor", "none");
        isIBMJDK = vmVendor.toLowerCase().matches(".*ibm corporation.*");
	}

    private static final String getSystemProperty(final String propName, final String defaultValue) {
        return AccessController.doPrivileged(new PrivilegedAction<String>() {
            public String run() {
                return System.getProperty(propName, defaultValue);
            }
        });
    }

    /**
     * Detail of lock requests on the lock request queue at the
     * time the lock timeout occurred.
     */
    private String ivDetails;

    /**
     * Constructs a new LockTimeoutException with <code>null</code> as its detail
     * message. The cause is not initialized, and may subsequently be initialized
     * by a call to the <code>initCause</code> method.
     *
     * @see #initCause(Throwable)
     */
    public LockTimeoutException() {
        super();
        if (produceJavaCoreOnLockTimeout) {
            forceJavaCore();
        }
    }

    /**
     * Constructs a new LockTimeoutException with the specified detail message. The
     * cause is not initialized, and may subsequently be initialized by a call to
     * the <code>initCause</code> method.
     *
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     *
     * @see #initCause(Throwable)
     * @see #getMessage()
     */
    public LockTimeoutException(String message) {
        super(message);
        if (produceJavaCoreOnLockTimeout) {
            forceJavaCore();
        }
    }

    /**
     * Constructs a new LockTimeoutException with the specified detail message and
     * cause.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this LockTimeoutException's detail
     * message.
     *
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). (A<code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown).
     *
     * @see #getCause()
     * @see #getMessage()
     */
    public LockTimeoutException(String message, Throwable cause) {
        super(message, cause);
        if (produceJavaCoreOnLockTimeout) {
            forceJavaCore();
        }
    }

    /**
     * Constructs a new LockTimeoutException with a specified cause.
     * The cause and a detail message of <code>(cause==null ? null : cause.toString())</code>
     * is used (which typically contains the class and detail message of <code>cause</code>).
     * This constructor is useful for LockTimeoutExceptions that are little more
     * than wrappers for other throwables.
     *
     * @param cause is the exception that caused this exception to be thrown,
     *        which is saved for later retrieval by the <code>getCause()</code> method.
     *        A <code>null</code> value is permitted and indicates that the cause
     *        is nonexistent or is unknown.
     *
     * @see #getCause()
     */
    public LockTimeoutException(Throwable cause) {
        super(cause);
        if (produceJavaCoreOnLockTimeout) {
            forceJavaCore();
        }
    }

    /**
     * Provides detailed information about the state of the lock queue when
     * the lock timeout occurred.
     *
     * @return   the argument that was passed to the <code>setLockRequestQueueDetails(String)</code>
     *           method of this class or <code>null</code> if the
     *           <code>setLockRequestQueueDetails</code> method was not previously called for this object.
     */
    public String getLockRequestQueueDetails()
    {
        return ivDetails;
    }

    public void forceJavaCore(){
    	if (isIBMJDK) {
    		try {
    			com.ibm.jvm.Dump.JavaDump();
    		} catch (Throwable e) {
    			// ignore the exception
    		}
    	}
    }

    /**
     * Sets the details of the lock requests on
     * the lock request queue at the time the lock timeout occurred.
     *
     * @param string the details of lock requests on the lock request queue at
     *               the time the lock timeout occurred.
     */
    public void setLockRequestQueueDetails(String string)
    {
        ivDetails = string;
    }

    /**
     * Returns the detail message string of this exception. The returned
     * String includes the request queue details as well as the message
     * provided to the constructor.
     *
     * @return  the detail message string of this object instance
     */
    public String getMessage()
    {
        return super.getMessage() + ivDetails;
    }
}