// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005-2010
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.10 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/ObjectGridManagerFactory.java, WAS.objectgrid, WASX.XD, b0614.107 2/17/06 20:14:10
//
// Module  :  ObjectGridManagerFactory.java
//
// Source File Description:
//
//    The ObjectGridFactory class provides ways to get user APIs.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 286802       WASX.XD   06/27/2005   cheng1    Factor out non-public factory methods from ObjectGridManager
// XD601_M1_B.9 WASX.XD   08/31/2005   jhanders  Add call to check product type.
// XD601_M3_B.1 WASX.XD   0/05/2005   kjlaw     remote check product type call.
// d347453      WXD601.XD 02/14/2006   jitang    More doPriv needed when running in WAS
// b19719       XS7.1     05/19/2010   jhanders   Add ObjectGridRASUtil.clientBootstrap to allow for ObjectGridManagerFactory to be called before ServerFactory
// b21012       XS7.1     08/27/2010   jhanders  Fix trace initialization to not cause security to fail in WAS.
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.security.AccessController;
import java.security.PrivilegedAction;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.runtime.RuntimeInfo;
import com.ibm.ws.objectgrid.util.ObjectGridRASUtil;

/**
 * This factory class is a high level helper class to get <code>ObjectGridManager</code> instances.
 *
 * @since WAS XD 6.0, XC10
 */

public final class ObjectGridManagerFactory {

    static final String CLASS_NAME = ObjectGridManagerFactory.class.getName();

    private final static TraceComponent tc;

    private static final ObjectGridManager objectGridManager;

    static {
        Class<?> ogmClass = AccessController.doPrivileged(new PrivilegedAction<Class<?>>() {


            public Class<?> run() {
            	RuntimeInfo ri = RuntimeInfo.instance();
                if (!ri.isLoggingProvidedByTheFramework()) {
                	ObjectGridRASUtil.clientBootstrap();
                    ObjectGridRASUtil.clientTraceInitialization();
                }
                

                // Trace the version information here so that we
                // have some diagnostic information about installation
                // in case we get exception in this class initializer
                // as a result for wrong jar being installed.  Note,
                // there is a risk that this class could be loaded before
                // RAS initialization has occurred.  We do not think is
                // very likely to every happen, but if it does, the worst
                // that should happen is we lose some useful trace information.
                // Given that, we decided it was worth the risk to avoid other
                // problems that could occur if we move where this traceVersionInfo
                // call occurs.
                ri.traceVersionInfo();

                String className = "com.ibm.ws.objectgrid.ObjectGridManagerImpl";
                try {
                    return Class.forName(className, true, ObjectGridManagerFactory.class.getClassLoader());
                } catch (ClassNotFoundException cne) {
                    FFDCFilter.processException(cne, CLASS_NAME + ".<clinit>", "67");
                    throw new ObjectGridRuntimeException("ClassNotFoundException", cne);
                }
            }
        });


        tc = Tr.register(CLASS_NAME,
                Constants.TR_GROUP_NAME, Constants.TR_RESOURCE_BUNDLE_NAME);

        try {
            // Now let's invoke ObjectGridManagerImpl.instance method, which
            // should work if correct jars are installed.
            Method instanceMethod = ogmClass.getDeclaredMethod("instance", new Class[0]);
            objectGridManager = (ObjectGridManager) instanceMethod.invoke(ogmClass, new Object[0]);
        } catch (NoSuchMethodException nsme) {
            FFDCFilter.processException(nsme, CLASS_NAME + ".<clinit>", "55");
            if (tc.isDebugEnabled()) {
                Tr.debug(tc, "NoSuchMethodException " + nsme);
            }
            throw new ObjectGridRuntimeException("InstantiationException", nsme);
        } catch (InvocationTargetException ite) {
            FFDCFilter.processException(ite, CLASS_NAME + ".<clinit>", "61");
            if (tc.isDebugEnabled()) {
                Tr.debug(tc, "InvocationTargetException " + ite);
            }
            throw new ObjectGridRuntimeException("InvocationTargetException", ite);
        } catch (IllegalAccessException iae) {
            FFDCFilter.processException(iae, CLASS_NAME + ".<clinit>", "64");
            if (tc.isDebugEnabled()) {
                Tr.debug(tc, "IllegalAccessException " + iae);
            }
            throw new ObjectGridRuntimeException("IllegalAccessException", iae);
        } catch (Throwable e) {
            FFDCFilter.processException(e, CLASS_NAME + ".<clinit>", "74");
            if (tc.isDebugEnabled()) {
                Tr.debug(tc, "Exception " + e);
            }
            throw new ObjectGridRuntimeException("Exception", e);
        }
    }

    /**
     * Returns the <code>ObjectGridManager</code> singleton.
     *
     * @return The ObjectGridManager singleton
     *
     * @see ObjectGridManager
     */
    public static final ObjectGridManager getObjectGridManager(){
        return objectGridManager;
    }
}
