// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  StateManager.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b11147       6.1.0.3   04/14/2008   vandersn  New part.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid;

/**
 * The StateManager can be used to retrieve the availability state of an ObjectGrid.  Use the
 * {@link StateManagerFactory#getStateManager()} method to retrieve a StateManager instance.
 *
 * @since WAS XD 6.1.0.3, XC10
 */
public interface StateManager {

    /**
     * Get the AvailabilityState of an ObjectGrid.  A random shard within the ObjectGrid is chosen
     * for reporting availability state.
     *
     * @param objectGrid
     *          the availability state of the specified remote ObjectGrid will be retrieved
     * @return the AvailabilityState of the remote ObjectGrid
     *
     * @throws IllegalArgumentException. If parameter objectGrid, is either null or it is of type 'LOCAL'. See {@link ObjectGrid#getObjectGridType()}.
     * @throws TargetNotAvailableException if there are no active shards for the ObjectGrid specified.
     */
    public AvailabilityState getObjectGridState(ObjectGrid objectGrid);


    /**
     * Set the AvailabilityState for an ObjectGrid. Each shard in the ObjectGrid will be transitioned to the state specified.
     *
     * This method does not return until each shard in the ObjectGrid has transitioned to the AvailabilityState specified or if it
     * times-out.
     *
     * @param state
     *            the AvailabilityState to transition to.
     * @param objectGrid
     *            the ObjectGrid to transaction to the specified AvailabilityState.
     * @throws IllegalArgumentException.
     *         <ol>
     *          <li>If parameter ObjectGrid. is either null or is of type 'LOCAL'. See {@link ObjectGrid#getObjectGridType()}.</li>
     *          <li>If parameter AvailabilityState is null.</li>
     *         </ol>
     */
    public void setObjectGridState(AvailabilityState state, ObjectGrid objectGrid);
}
