/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 * b9388                  10/01/2007   vandersn  additional zone support
 */

package com.ibm.websphere.objectgrid;

import com.ibm.websphere.objectgrid.server.ServerProperties;

/**
 * 
 * This exception is thrown when a container is started in an incompatible zone or when a problem 
 * has been detected with the zone configuration within the deployment policy.
 * 
 * A container can be placed into a custom zone by specifying the zone name when the container is started.
 * When no zone name is specified, the container is placed into the DefaultZone.  When containers have been started in 
 * one or more custom zones for a particular ObjectGrid, it is an error to start a subsequent container
 * in the DefaultZone for that ObjectGrid.  A ZoneConfigurationException will be thrown when this is detected.
 * 
 * Similarly, when containers are started in the DefaultZone for a particular ObjectGrid, it is an error to start 
 * a subsequent container in a custom zone for that ObjectGrid.  A ZoneConfigurationException will
 * be thrown in this case.
 * 
 * A container's deployment policy is validated when the container starts.  Validation is done on zone configuration within
 * the deployment policy if zones are configured.  If a problem is found with the zone configuration this exception is thrown.
 * 
 * @since WAS XD 6.1 FIX3
 */

public class ZoneConfigurationException extends ObjectGridException {
    
    /**
     * A generic error code.  See the exception message for more detail.
     */
    public static final int GENERIC = 0;

    /**
     * One of the error codes for this exception.
     * 
     * When containers have already been started in one or more custom zones for a particular ObjectGrid, 
     * it is an error to start a subsequent container within the DefaultZone for the same ObjectGrid.
     * 
     * @see ServerProperties#DEFAULT_ZONE
     */
    public static final int CONTAINER_NOT_ALLOWED_IN_DEFAULT_ZONE = 1;

    /**
     * One of the error codes for this exception.
     * 
     * When containers have already been started in the DefaultZone for a particular ObjectGrid,
     * it is an error to start a subsequent container within a custom zone for the same ObjectGrid.
     * 
     * @see ServerProperties#DEFAULT_ZONE
     */
    public static final int CONTAINER_NOT_ALLOWED_IN_CUSTOM_ZONE = 2; 
    
    // Do not use error code 3 (ObjectGridContainerImpl.FROM_SERVER = 3). Skip to 4 for next error code.

    private static final long serialVersionUID = 7444553284746711627L;

    private int errorCode = GENERIC;

    /**
     * Constructs a new ZoneConfigurationException with <code>null</code> as its detail
     * message. The cause is not initialized, and may subsequently be initialized
     * by a call to the <code>initCause</code> method.
     * 
     * @see #initCause(Throwable)
     */
    public ZoneConfigurationException() {
        super();
    }
    
    /**
     * Constructs a new ZoneConfigurationException with the specified detail message. The
     * cause is not initialized, and may subsequently be initialized by a call to
     * the <code>initCause</code> method.
     * 
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     * 
     * @see #initCause(Throwable)
     * @see #getMessage() 
     */

    public ZoneConfigurationException(String message) {
        super(message);
        this.errorCode = GENERIC;
    }

    /**
     * Constructs a new ZoneConfigurationException with the specified detail message and error code. The
     * cause is not initialized, and may subsequently be initialized by a call to
     * the <code>initCause</code> method.
     * 
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     * @param errorCode
     *           the errorCode corresponds to one of the constants of this class
     * 
     * @see #initCause(Throwable)
     * @see #getMessage() 
     */
    public ZoneConfigurationException(String message, int errorCode) {
        super(message);
        this.errorCode = errorCode;
    }

    /**
     * Constructs a new ZoneConfigurationException with a specified cause.
     * The cause and a detail message of <code>(cause==null ? null : cause.toString())</code> 
     * is used (which typically contains the class and detail message of <code>cause</code>).
     * This constructor is useful for ZoneConfigurationException that are little more
     * than wrappers for other throwables.
     * 
     * @param cause is the exception that caused this exception to be thrown,
     *        which is saved for later retrieval by the <code>getCause()</code> method.
     *        A <code>null</code> value is permitted and indicates that the cause 
     *        is nonexistent or is unknown.
     * 
     * @see #getCause()
     */
    public ZoneConfigurationException(Throwable cause) {
        super(cause);
    }

    /**
     * Constructs a new ZoneConfigurationException with the specified detail message and
     * cause.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this ZoneConfigurationException detail
     * message.
     * 
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). (A<code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown).
     * 
     * @see #getCause()
     * @see #getMessage() 
     */
    public ZoneConfigurationException(String message, Throwable cause) {
        super(message, cause);
    }

    /**
     * Constructs a new ZoneConfigurationException with the specified detail message,
     * cause, and error code.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this ZoneConfigurationException detail
     * message.
     * 
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). (A<code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown).
     * @param errorCode
     *       the errorCode corresponds to one of the constants of this class
     * 
     * @see #getCause()
     * @see #getMessage() 
     */
    public ZoneConfigurationException(String message, Throwable cause, int errorCode) {
        super(message);
        this.errorCode = errorCode;
    }
    
    /**
     * Returns the error code that was set by one of the constructors that 
     * accepts an error code, or <code>GENERIC</code> if one of the other
     * constructors was called.
     * 
     * @return the error code.  One of the constants of this exception class.
     */
    public int getErrorCode() {
        return errorCode;        
    }

}
