// IBM Confidential OCO Source Material
//
// 5724-J34 COPYRIGHT International Business Machines Corp. 2005, 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ConfigProperty.java
//
// Source File Description: 
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.x  WASX.XD   02/23/2005   rajeshk    new code drop
// b896                   04/30/2006   vandersn   open config APIs
//
//

package com.ibm.websphere.objectgrid.config;

import java.io.Serializable;

/**
 * <p>ConfigProperty can be used to attach properties to a <code>Plugin</code>. 
 * A ConfigProperty has the following attributes:</p>
 * <ul>
 * <li>name: the property name </li> 
 * <li>value: the property value</li> 
 * <li>configPropertyType: the configuration property type </li>
 * </ul>
 * </p>
 * 
 * <p>This ConfigProperty can be used to set the properties of a plugin. The name of the property 
 * should follow JaveBean convention. That is, for every property, there should be a 
 * corresponding set method in the plugin class.
 * <p>
 * Users can use <code>com.ibm.websphere.objectgrid.config.ObjectGridConfigFactory.createConfigProperty(ConfigPropertyType, String, String)</code>
 * to create a ConfigProperty object.
 * <p>
 * <code>ConfigProperty evictorNameProp = ObjectGridConfigFactory.createConfigProperty(ConfigProperty.STRING_JAVA_LANG, "evictorName", "evictor1");</code> 
 * This creates a property "evictorName" with value "evictor1", and the type is <code>java.lang.String</code>.
 * Use the <code>com.ibm.websphere.objectgrid.config.Plugin#addConfigProperty(ConfigProperty)</code> method to attach
 * a ConfigProperty to a Plugin.  When the Plugin is created, each ConfigProperty will have its corresponding
 * set method called.  
 * <p>
 * Continuing with the example above, attach the ConfigProperty to an Evictor Plugin.
 * <code>
 * evictorPlugin.addConfigProperty(evictorNameProp);
 * </code>
 * <p>
 * When this Evictor Plugin is created, the <code>setEvictorName(String)<code> method will be called with the value "evictor1".
 * @since WAS XD 6.0.1.2, XC10
 */
public interface ConfigProperty extends Serializable {
	

    /**
     * Set the <code>ConfigPropertyType</code> for this object.  The Java primitives, their java.lang counterparts, and
     * java.lang.String are the supported ConfigPropertyTypes.
     * @param configPropType
     * @see ConfigPropertyType#INTEGER_JAVA_LANG
     * @see ConfigPropertyType#INT_PRIM
     * @see ConfigPropertyType#BOOLEAN_JAVA_LANG
     * @see ConfigPropertyType#BOOLEAN_PRIM
     * @see ConfigPropertyType#CHARACTER_JAVA_LANG
     * @see ConfigPropertyType#CHAR_PRIM
     * @see ConfigPropertyType#BYTE_JAVA_LANG
     * @see ConfigPropertyType#BYTE_PRIM
     * @see ConfigPropertyType#SHORT_JAVA_LANG
     * @see ConfigPropertyType#SHORT_PRIM
     * @see ConfigPropertyType#LONG_JAVA_LANG
     * @see ConfigPropertyType#LONG_PRIM
     * @see ConfigPropertyType#FLOAT_JAVA_LANG
     * @see ConfigPropertyType#FLOAT_PRIM
     * @see ConfigPropertyType#DOUBLE_JAVA_LANG
     * @see ConfigPropertyType#DOUBLE_PRIM
     * @see ConfigPropertyType#STRING_JAVA_LANG
     */
    public void setConfigPropertyType(ConfigPropertyType configPropType);
    
	/**
	 * Get the <code>ConfigPropertyType</code> of this object.
	 * @return  the ConfigPropertyType for this object
	 */
	public ConfigPropertyType getConfigPropertyType();

    /**
     * Set the value of this ConfigProperty.  This String value will be converted to the proper
     * type, based on <code>ConfigPropertyType</code> assigned to this ConfigProperty
     * @param value - will be converted to type and passed to the setter on the plugin
     */
    public void setValue(String value);
    
	/**
	 * Get the value that is assigned to this ConfigProperty.  This is the value that will be passed
	 * to the set method on the plugin.
	 * @return Returns the value.
	 */
	public String getValue();
	
	/**
	 * Set the name of this object.  The <code>Plugin</code> that this ConfigProperty is attached to should
	 * have a setter that corresponds to this name.  For example, if "size" is passed in as the name, then 
	 * the Plugin must have a "setSize" method.
	 * @param name - name of the property
	 */
	public void setName(String name);
    
    /**
     * Get the name of this object.  The name must have a corresponding set method on the Plugin.
     * @return Returns the name.
     */
    public String getName();	
	


}
