// IBM Confidential OCO Source Material
//
// 5724-J34 COPYRIGHT International Business Machines Corp. 2005, 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ConfigPropertyType.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason    Version   Date         Userid    Change Description
// --------- --------- ------------ --------- -----------------------------------------
// b896                05/01/2006   vandersn  open config APIs
// --------- --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.config;

import java.io.Serializable;

/**
 * ConfigPropertyType is used to set the type of an attribute on a Plugin.  The Java
 * primitives, their java.lang counterparts, and java.lang.String are the supported
 * types.
 *
 * @since WAS XD 6.0.1.2, XC10
 * @see ObjectGridConfigFactory#createConfigProperty(ConfigPropertyType, String, String)
 * @see com.ibm.websphere.objectgrid.config.Plugin#setPluginType(PluginType)
 */
public final class ConfigPropertyType implements Serializable {

	private static final long serialVersionUID = -8432973787899386907L;
    /**
     * ConfigPropertyType.INTEGER_JAVA_LANG can be used to set a property of
     * type <code>java.lang.Integer</code> on a plugin.
     */
	public final static ConfigPropertyType INTEGER_JAVA_LANG = new ConfigPropertyType((byte) 0);

    /**
     * ConfigPropertyType.INT_PRIM can be used to set a property of type <code>int</code>
     *  on a plugin.
     */
	public final static ConfigPropertyType INT_PRIM = new ConfigPropertyType((byte) 1);

    /**
     * ConfigPropertyType.BOOLEAN_JAVA_LANG can be used to set a property of type
     * <code>java.lang.Boolean</code> on a plugin.
     */
	public final static ConfigPropertyType BOOLEAN_JAVA_LANG = new ConfigPropertyType((byte) 2);

    /**
     * ConfigPropertyType.BOOLEAN_PRIM can be used to set a property of type
     * <code>boolean</code> on a plugin.
     */
	public final static ConfigPropertyType BOOLEAN_PRIM = new ConfigPropertyType((byte) 3);

    /**
     * ConfigPropertyType.CHARACTER_JAVA_LANG can be used to set a property of type
     * <code>java.lang.Character</code> on a plugin.
     */
	public final static ConfigPropertyType CHARACTER_JAVA_LANG = new ConfigPropertyType((byte) 4);

    /**
     * ConfigPropertyType.CHAR_PRIM can be used to set a property of type
     * <code>char</code> on a plugin.
     */
	public final static ConfigPropertyType CHAR_PRIM = new ConfigPropertyType((byte) 5);

    /**
     * ConfigPropertyType.BYTE_JAVA_LANG can be used to set a property of type
     * <code>java.lang.Byte</code> on a plugin.
     */
	public final static ConfigPropertyType BYTE_JAVA_LANG = new ConfigPropertyType((byte) 6);

    /**
     * ConfigPropertyType.BYTE_PRIM can be used to set a property of type
     * <code>byte</code> on a plugin.
     */
	public final static ConfigPropertyType BYTE_PRIM = new ConfigPropertyType((byte) 7);

    /**
     * ConfigPropertyType.SHORT_JAVA_LANG can be used to set a property of type
     * <code>java.lang.Short</code> on a plugin.
     */
	public final static ConfigPropertyType SHORT_JAVA_LANG = new ConfigPropertyType((byte) 8);

    /**
     * ConfigPropertyType.SHORT_PRIM can be used to set a property of type
     * <code>short</code> on a plugin.
     */
	public final static ConfigPropertyType SHORT_PRIM = new ConfigPropertyType((byte) 9);

    /**
     * ConfigPropertyType.LONG_JAVA_LANG can be used to set a property of type
     * <code>java.lang.Long</code> on a plugin.
     */
	public final static ConfigPropertyType LONG_JAVA_LANG = new ConfigPropertyType((byte) 10);

    /**
     * ConfigPropertyType.LONG_PRIM can be used to set a property of type
     * </code>long</code> on a plugin.
     */
	public final static ConfigPropertyType LONG_PRIM = new ConfigPropertyType((byte) 11);

    /**
     * ConfigPropertyType.FLOAT_JAVA_LANG can be used to set a property of type
     * <code>java.lang.Float</code> on a plugin.
     */
	public final static ConfigPropertyType FLOAT_JAVA_LANG = new ConfigPropertyType((byte) 12);

    /**
     * ConfigPropertyType.FLOAT_PRIM can be used to set a property of type
     * <code>float</code> on a plugin.
     */
	public final static ConfigPropertyType FLOAT_PRIM = new ConfigPropertyType((byte) 13);

    /**
     * ConfigPropertyType.DOUBLE_JAVA_LANG can be used to set a property of type
     * <code>java.lang.Double</code> on a plugin.
     */
    public final static ConfigPropertyType DOUBLE_JAVA_LANG = new ConfigPropertyType((byte) 14);

    /**
     * ConfigPropertyType.DOUBLE_PRIM can be used to set a property of type
     * <code>double</code> on a plugin.
     */
	public final static ConfigPropertyType DOUBLE_PRIM = new ConfigPropertyType((byte) 15);

    /**
     * ConfigPropertyType.STRING_JAVA_LANG can be used to set a property of type
     * <code>java.lang.String</code> on a plugin.
     */
	public final static ConfigPropertyType STRING_JAVA_LANG = new ConfigPropertyType((byte) 16);

    /**
     * Given the raw value of a ConfigPropertyType, this method returns a ConfigPropertyType object,
     * or null if the raw value does not match an existing type.  This method is used to deserialize
     * this object.
     *
     * @param id the raw value of a ConfigPropertyType
     * @return the ConfigPropertyType corresponding to the raw input value
     * @since 8.6
     */
    public final static ConfigPropertyType valueOf(byte id) {
    	switch(id) {
    		case 0: return INTEGER_JAVA_LANG;
    		case 1: return INT_PRIM;
    		case 2: return BOOLEAN_JAVA_LANG;
    		case 3: return BOOLEAN_PRIM;
    		case 4: return CHARACTER_JAVA_LANG;
    		case 5: return CHAR_PRIM;
    		case 6: return BYTE_JAVA_LANG;
    		case 7: return BYTE_PRIM;
    		case 8: return SHORT_JAVA_LANG;
    		case 9: return SHORT_PRIM;
    		case 10: return LONG_JAVA_LANG;
    		case 11: return LONG_PRIM;
    		case 12: return FLOAT_JAVA_LANG;
    		case 13: return FLOAT_PRIM;
    		case 14: return DOUBLE_JAVA_LANG;
    		case 15: return DOUBLE_PRIM;
    		case 16: return STRING_JAVA_LANG;
    		default: return null;
    	}
    }

    private byte ivId;

    private ConfigPropertyType()
    {
        super();
        ivId = 0;
    }


    private ConfigPropertyType( byte id )
    {
        super();
        ivId = id;
    }

    public boolean equals(Object o)
    {
    	ConfigPropertyType cpt = (ConfigPropertyType)o;
        return ivId == cpt.ivId;
    }

    public int hashCode() {
        return (int) ivId;
    }

    /**
     * Get the raw value of this ConfigPropertyType.  This method is used to serialize this object.
     * @return the raw value of this ConfigPropertyType.
     */
    public int getId() {
    	return ivId;
    }

    private Object readResolve () throws java.io.ObjectStreamException {
    	switch(ivId){
    		case 0: return INTEGER_JAVA_LANG;
    		case 1: return INT_PRIM;
    		case 2: return BOOLEAN_JAVA_LANG;
    		case 3: return BOOLEAN_PRIM;
    		case 4: return CHARACTER_JAVA_LANG;
    		case 5: return CHAR_PRIM;
    		case 6: return BYTE_JAVA_LANG;
    		case 7: return BYTE_PRIM;
    		case 8: return SHORT_JAVA_LANG;
    		case 9: return SHORT_PRIM;
    		case 10: return LONG_JAVA_LANG;
    		case 11: return LONG_PRIM;
    		case 12: return FLOAT_JAVA_LANG;
    		case 13: return FLOAT_PRIM;
    		case 14: return DOUBLE_JAVA_LANG;
    		case 15: return DOUBLE_PRIM;
    		case 16: return STRING_JAVA_LANG;
    		default: return null;
    	}
    }

    public String toString() {
        switch (ivId) {
        case 0: return "INTEGER_JAVA_LANG ConfigPropertyType";
        case 1: return "INT_PRIM ConfigPropertyType";
        case 2: return "BOOLEAN_JAVA_LANG ConfigPropertyType";
        case 3: return "BOOLEAN_PRIM ConfigPropertyType";
        case 4: return "CHARACTER_JAVA_LANG ConfigPropertyType";
        case 5: return "CHAR_PRIM ConfigPropertyType";
        case 6: return "BYTE_JAVA_LANG ConfigPropertyType";
        case 7: return "BYTE_PRIM ConfigPropertyType";
        case 8: return "SHORT_JAVA_LANG ConfigPropertyType";
        case 9: return "SHORT_PRIM ConfigPropertyType";
        case 10: return "LONG_JAVA_LANG ConfigPropertyType";
        case 11: return "LONG_PRIM ConfigPropertyType";
        case 12: return "FLOAT_JAVA_LANG ConfigPropertyType";
        case 13: return "FLOAT_PRIM ConfigPropertyType";
        case 14: return "DOUBLE_JAVA_LANG ConfigPropertyType";
        case 15: return "DOUBLE_PRIM ConfigPropertyType";
        case 16: return "STRING_JAVA_LANG ConfigPropertyType";
        default: return "UNKNOWN ConfigPropertyType";
        }
    }

}
