// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  : QueryConfig.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b834         6.1       04/26/2006   cdjohnson new part
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.config;

import java.io.Serializable;
import java.util.ArrayList;

import com.ibm.websphere.objectgrid.ObjectGrid;

/**
 * This QueryConfig represents a schema configuration for a QueryManager. A QueryConfig object 
 * contains a set of QueryMapping objects and a set of QueryRelationship objects.
 * <p>
 * Users use addQuerySchema method to add a QueryMapping object and use 
 * addRelationship method to add a QueryRelationship object into this QueryConfig object. 
 * 
 * @see ObjectGrid#setQueryConfig(QueryConfig)
 * @see QueryMapping
 * @see QueryRelationship
 * @since WAS XD 6.1, XC10
 * 
 */
public class QueryConfig implements Serializable {

    private static final long serialVersionUID = 1L;

    private ArrayList/*<QueryMapping>*/maps = new ArrayList/*<QueryMapping>*/();

    private ArrayList/*<QueryRelationship>*/relationships = new ArrayList/*<QueryQueryRelationship>*/();
    
    /**
     * Default constructor.
     */
    public QueryConfig() {
        super();
    }

    /**
     * Add a QueryMapping to this QueryConfig
     * @param mapping the QueryMapping to add.
     */
    public void addQueryMapping(QueryMapping mapping) {
        maps.add(mapping);
    }

    /**
     * Add a QueryRelationship to this QueryConfig
     * @param relation the QueryRelationship to add.
     */
    public void addQueryRelationship(QueryRelationship relation) {
        relationships.add(relation);
    }
    /**
     * Retrieve all added QueryMappings
     * @return array of QueryMapping
     */
    public QueryMapping[] getQueryMappings() {
        return (QueryMapping[]) maps.toArray(new QueryMapping[maps.size()]);
    }

    /**
     * Retrieve all added QueryRelationships
     * @return array of QueryRelationship
     */
    public QueryRelationship[] getQueryRelationships() {
        return (QueryRelationship[]) relationships.toArray(new QueryRelationship[relationships.size()]);
    }

}
