// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ContinuousQueryCache.java
//
// Source File Description:
//
// See javadoc for class.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.continuousquery;

import java.util.List;
import java.util.Set;

import com.ibm.websphere.objectgrid.SessionHandle;

/**
 * A Continuous Query Cache contains the keys and optionally the values that match a defined continuous query. The
 * contents of this cache arrive asynchronously from the grid for which the query is defined. If the query is defined
 * such that only the keys are stored in the continuous query cache then all operations that return a value will return
 * null.
 * 
 * @param <KeyType>  Type of the key object for the map being queried
 * @param <ValueType> Type of the value object for the map being queried
 * @since 8.6
 */
public interface ContinuousQueryCache<KeyType,ValueType> {

    /**
     * Returns a Set view of a snapshot of the keys contained in the cache.
     * 
     * @return a set view of the keys in the cache.
     */
    public Set<KeyType> keySnapshot();

    /**
     * Return the number of keys in the cache.
     * 
     * @return the number of keys in the cache.
     */
    public int size();

    /**
     * Returns the value for the given key.
     * 
     * @param key
     *            - the key whose associated value is to be returned
     * @return the value in the cache for the specified key or null if either the cache does not contain this key or if
     *         the cache is configured to not contain values.
     */
    public ValueType get(KeyType key);

    /**
     * Returns {@link SessionHandle} objects that can be used to retrieve
     * all query matches associated with this key.
     * @param key
     * 			- the key for which SessionHandles will be returned
     * @return a List of SessionHandles, one for each instance of this key in the grid.
     * 
     */
    public List<SessionHandle> getSessionHandles(KeyType key);
    /**
     * Returns true if the cache contains the given key.
     * 
     * @param key
     *            - key to check for
     * @return true if the specified key is in the cache at the time of the request.
     */
    public boolean containsKey(KeyType key);

    /**
     * Returns true if the cache contains the given value.
     * 
     * @param value
     *            - value to check for
     * @return true if the specified object is in the cache at the time of the request.
     */
    public boolean containsValue(ValueType value);
    
    /**
     * Returns true if this cache includes the values associated with the keys that have matched the query.
     * If true, {@link ContinuousQueryCache#containsValue(Object)} and {@link ContinuousQueryCache#get(Object)} will return usable values, otherwise they will return null.
     * @return true if this query cache contains values in addition to keys.
     * @see ContinuousQueryManager#defineContinuousQuery
     */
    public boolean includesValues();
}
