// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ContinuousQueryManagerFactory.java
//
// Source File Description:
//
// See javadoc for class.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.continuousquery;

import java.util.HashMap;
import java.util.Map;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.ws.objectgrid.continuousquery.ContinuousQueryManagerImpl;

/**
 * A factory class for obtaining the ContinuousQueryManager.
 * 
 * @since 8.6
 */
public class ContinuousQueryManagerFactory {
    private static Map<ObjectGrid, ContinuousQueryManager> cqmMap = new HashMap<ObjectGrid, ContinuousQueryManager>();

    /**
     * Factory that returns the single instance of a ContinuousQueryManager for the given object grid.
     * 
     * @param objGrid
     *            - the ObjectGrid this Continuous Query Manager will define continuous queries for.
     * @return the ContinuousQueryManager instance.
     */
    public static synchronized ContinuousQueryManager getManager(ObjectGrid objGrid) {
        ContinuousQueryManager cqm = cqmMap.get(objGrid);
        if (cqm == null) {
            cqm = new ContinuousQueryManagerImpl(objGrid);
            cqmMap.put(objGrid, cqm);
        }
        return cqm;
    }

}
