// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ContinuousQueryTopic.java
//
// Source File Description:
//
// See javadoc for class.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.continuousquery;

import java.util.Collection;

/**
 * A Continuous Query Topic embodies the client side view of a defined Continuous Query. The topic gives access to the
 * client-side cached results of the continuous query and allows for management of Continuous Query Listeners for this
 * topic.
 * 
 * @param <KeyType>
 *            Type of the key object for the map being queried
 * @param <ValueType>
 *            Type of the value object for the map being queried
 * 
 * @since 8.6
 */
public interface ContinuousQueryTopic<KeyType, ValueType> {

    /**
     * Add a Continuous Query Listener to this topic. The listener will be called when this topic receives updates to
     * the query. There is no guarantee of the order that listeners will be called. Multiple adds of the same listener
     * will result in only one call to the listener.
     * 
     * @param listener
     *            the listener to add to this continuous query's set of listeners.
     */
    public void addListener(ContinuousQueryListener<KeyType, ValueType> listener);

    /**
     * Remove the listener from this continuous query's the set of listeners. In flight notifications to this listener
     * may continue after this method returns.
     * 
     * @param listener
     *            - the continuous query listener to remove.
     * @return true if the listener was removed.
     */
    public boolean removeListener(ContinuousQueryListener<KeyType, ValueType> listener);

    /**
     * Return a continuous query cache object with methods to access the keys and values stored in the cache.
     * 
     * @return a Continuous Query Cache object for this topic.
     */
    public ContinuousQueryCache<KeyType, ValueType> getCache();

    /**
     * Returns the generated unique name for this continuous query.
     * 
     * @return a unique string.
     */
    public String getName();

    /**
     * Return true if the cache for this continuous query contains only keys. Return false if it also contains the value
     * associated with the key.
     * 
     * @return true if the cache does not contain values.
     */
    public boolean isKeysOnlyCache();

    /**
     * Return true if the no caching option is enabled for this Continuous Query.
     * 
     * @return true if no keys or values are being cached.
     */
    public boolean noCache();

    /**
     * Return true if this query will return keys/values in {@link SerializedEntry} format.
     * 
     * @return true if RAW format is used, false if NATIVE format is used.
     */
    public boolean isRawFormat();
    /**
     * Returns a shallow copy of the current set of listeners registered for this query topic.
     * 
     * @return a new collection object containing all currently registered listeners.
     */
    public Collection<ContinuousQueryListener<KeyType, ValueType>> getAllListeners();

    /**
     * Removes all registered listeners from this query topic. In flight notifications to listeners may continue after
     * this method returns.
     */
    public void removeAllListeners();
}
