// IBM Confidential OCO Source Material
//
// 5724-J34 COPYRIGHT International Business Machines Corp. 2005, 2006, 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  MapGridAgent.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason    Version   Date         Userid    Change Description
// --------- --------- ------------ --------- -----------------------------------------
// b9168        XD6.1     07/19/2007   cdjohnson Fix javadoc.
//
package com.ibm.websphere.objectgrid.datagrid;

import java.io.Serializable;
import java.util.Map;

import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.Session;

/**
 * A MapGridAgent is used to process operations against map entries in a remote (server-side) ObjectGrid.
 * An agent is submitted using an ObjectMap's {@link AgentManager} and is run on the server ObjectGrid
 * instances that host the data in the map.  The agent is routed to one or more server instances based on
 * the partitions that the optional keys resolve to.<p>
 *
 * If the map is associated with an entity, then the provided key should be an entity
 * instance and the keys returned from processAllEntries map should be
 * an entity with at least all the key attributes populated.  Agent implementations should implement
 * the EntityAgentMixin interface to allow automatic key to entity conversion.<p>
 *
 * MapGridAgents are invoked in a separate, independent transaction from the client's transaction.
 * A single client request may result in multiple transactions. This can happen if the ObjectGrid
 * processes a thousand keys using 10 transactions on 10 threads for example, 100 keys per thread.<p>
 *
 * Keys are processed in arbitrary order sequentially or in parallel and may be grouped into one
 * or more transactions.  Agents may run more than once for a key if a subsequent agent call fails
 * in the same transaction, however an agent will only commit a successful process call one time.<p>
 *
 * @see EntityAgentMixin
 * @since WAS XD 6.1
 */
public interface MapGridAgent extends Serializable
{
	/**
     * This method is called once per key per partition.
     *
     * @param s The Session to use for any work. Transactions are controlled by the caller.
	 * @param map The ObjectMap this agent was invoked for
	 * @param key The key of the entry to process.
	 * @return The result of the processing.
	 */
	Object process(Session s, ObjectMap map, Object key);

	/**
	 * This method is called once per partition.<p>
     *
     * The MapGridAgent instance may have additional state
     * to filter the set of entries usually using a query against the partition.
     *
	 * @param s The Session to use for any work. Transactions are controlled by the caller.
	 * @param map The ObjectMap this agent was invoked for.
	 * @return A map of processed values for keys
	 */
	Map/*<Object key, Object processedValue<*/ processAllEntries(Session s, ObjectMap map);
}
