//IBM Confidential OCO Source Material
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
//The source code for this program is not published or otherwise divested
//of its trade secrets, irrespective of what has been deposited with the
//U.S. Copyright Office.
//
//$Id$
//
//Module  :  ProjectorFactory.java
//
//Source File Description:
//
//    See javadoc.
//
//Change Activity:
//
//Reason       Version   Date         Userid    Change Description
//------------ --------- ------------ --------- -----------------------------------------
// b13144      6.1.0.5   09/18/2008   schmittm  new file
//------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.em;

import java.lang.reflect.InvocationTargetException;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;
import com.ibm.websphere.projector.Projector;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.xs.util.MethodHelper;

/**
 * A factory for creating or retrieving <tt>Projector</tt> instances.<p>
 *
 * Example:
 * <pre>
 * // Get a session and it's EntityManager instance.
 * Session s = grid.getSession();
 * EntityManager em = s.getEntityManager();
 *
 * // Get the Projector that is associated with the EntityManager.
 * Projector proj = ProjectorFactory.getProjector(em);
 *
 * // Retrieve the EntityMetadata for the "Person" entity type.
 * EntityMetadata emdPerson = s.getObjectMap("Person").getEntityMetadata();
 *
 * // Find a Person object
 * Person p = (Person) em.find(Person.class, "123-45-6789");
 *
 * // We can convert any entity to a Tuple.  It can be new, detached or managed.
 * Tuple tKey = proj.getTupleFromEntity(p, emdPerson, true);
 * Tuple tValue = proj.getTupleFromEntity(p, emdPerson, false);
 * </pre>
 *
 * @since WAS XD 6.1.0.5
 */
public class ProjectorFactory {

    private static final TraceComponent tc = Tr.register(
            ProjectorFactory.class,
            Constants.TR_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);


 /**
  * Retrieves the <code>Projector</code> associated with the specified <code>EntityManager</code>.
  *
  * @param em The instance of the EntityManager to access the Projector.
  * @return a <code>Projector</code> instance.
  */
    public static Projector getProjector(EntityManager em){
     if ( tc.isEntryEnabled())
         Tr.entry(tc, "getProjector");
    try {
        Projector projectorInstance = (Projector)MethodHelper.invoke(em, "getProjector", new Object[]{});
        if (tc.isEntryEnabled())
            Tr.exit(tc, "getProjector", projectorInstance);
        return projectorInstance;
    } catch (IllegalAccessException iae) {
        if (tc.isEntryEnabled()) {
            Tr.exit(tc, "IllegalAccessException " + iae);
         }
         throw new ObjectGridRuntimeException("IllegalAccessException", iae);
    } catch (NoSuchMethodException nsme) {
        if (tc.isEntryEnabled()) {
            Tr.exit(tc, "NoSuchMethodException " + nsme);
         }
         throw new ObjectGridRuntimeException("NoSuchMethodException", nsme);
    } catch (IllegalArgumentException iare) {
        if (tc.isEntryEnabled()) {
            Tr.exit(tc, "IllegalArgumentException" + iare);
         }
         throw new ObjectGridRuntimeException("IllegalArgumentException", iare);
    } catch (InvocationTargetException ite) {
        if (tc.isEntryEnabled()) {
            Tr.exit(tc, "InvocationTargetException" + ite);
         }
         throw new ObjectGridRuntimeException("InvocationTargetException", ite);
    }
 }
}