//IBM Confidential OCO Source Material
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
//The source code for this program is not published or otherwise divested
//of its trade secrets, irrespective of what has been deposited with the
//U.S. Copyright Office.
//
//$Id$
//
//Module  :  HttpSessionConfigurationUtility.java
//
//Source File Description:
//
//    See javadoc.
//
//Change Activity:
//
//Reason       Version   Date         Userid    Change Description
//------------ --------- ------------ --------- -----------------------------------------
// b13409      6.1.0.3   10/01/2008   bsandmann new file
//------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.httpsession;


/**
 * The HTTPSessioncConfigurationUtility is a singleton utility class that allows configuring or overriding
 * the ObjectGrid HTTP Session plugin options at the server scope, affecting all web applications for the server.  All
 * web applications using this utility must be configured share the same classloader.<p>
 *
 * This utility is to be used for all web applications deployed on web application servers other than WebSphere Application Server,
 * such as Apache Geronimo.  Web applications deployed on WebSphere Application Server do not require this utility.
 *
 * @since WAS XD 6.1.0.2
 */

public class HttpSessionConfigurationUtility {

	private static HttpSessionConfigurationUtility instance  = new HttpSessionConfigurationUtility();
	private String catalogServiceEndPoints = "";
	private boolean replicationDisabled = false;

	private HttpSessionConfigurationUtility() {

	}

	/**
	 * Returns the singleton instance of a HttpSessionConfigurationUtility.
	 *
	 * @return the singleton instance of this class
	 */

	public static HttpSessionConfigurationUtility getInstance() {
		return instance;
	}

	/**
	 * Returns a comma-separated, {@link String} representation of host:port tuples.
	 * Multiple tuples are realized when catalog servers are clustered for high
	 * availability reasons among others.
	 *
	 * @return the String of endpoint tuples
	 */

	public String getCatalogServiceEndPoints() {
		return catalogServiceEndPoints;
	}

	/**
	 * Sets the {@link String} which represents the endpoints for all available catalog services
	 *
	 * @param catalogServiceEndPoints a non-null, comma-separated, String representation of host:port tuples.
	 */

	public void setCatalogServiceEndPoints(String catalogServiceEndPoints) {
		if (catalogServiceEndPoints == null)
			throw new IllegalArgumentException("catalogServiceEndPoints can not be null");
		this.catalogServiceEndPoints = catalogServiceEndPoints;
	}

	/**
	 * Returns whether replication is currently disabled or not.
	 *
	 * @return true if replication is disabled, false otherwise
	 */

	public boolean isReplicationDisabled() {
		return replicationDisabled;
	}

	/**
	 * Sets whether or not replication is disabled for this server.
	 *
	 * @param replicationDisabled true if replication is to be disabled, false otherwise
	 */

	public void setReplicationDisabled(boolean replicationDisabled) {
		this.replicationDisabled = replicationDisabled;
	}
}
