// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Source File Description:
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d11207       WASX.XD   03/04/2008   bnewport  OpenJPA Load and Preload
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.jpa;

import com.ibm.websphere.objectgrid.Session;

/**
 *
 * A ClientLoadCallback instance receives notifications when a ClientLoader starts and
 * finishes.<p>
 *
 * Clients should implement this interface and pass it to the ClientLoader.load() method to be notified
 * when the client loader has started or completed loading the specified maps.
 *
 * @since WAS XD 6.1.0.3
 * @see com.ibm.websphere.objectgrid.jpa.ClientLoader
 */
public interface ClientLoadCallback {

    /**
     * This method is called before the client starts loading a map.
     *
     * @param session the session object used to preload the map
     * @param mapName the name of the map is being preloaded
     */
    void preStart(Session session, String mapName);

    /**
     * This method is called right after the client finishes loading a map.
     * <p>
     * The application could do some clean up work after the the client finishes loading the map. For example,
     * in the entity map case, the application could run a query to clean up all the orphaned entities.
     *
     * @param session the session object used to preload the map
     * @param mapName the name of the map is being preloaded
     * @param success indicates whether the preload is successful or not
     */
    void postFinish(Session session, String mapName, boolean success);

}
