// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Source File Description:
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d11207       WASX.XD   05/06/2008   jian.tang  OpenJPA Load and Preload
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.jpa;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.ws.objectgrid.SessionImpl;
import com.ibm.ws.objectgrid.SessionImpl.SessionAttribute;
import com.ibm.ws.objectgrid.jpa.ClientLoaderImpl;

/**
 * A factory used for client loader purpose. It can be used to get an instance of
 * client loader and make a {@link Session} object be able to drive transactions to a shard
 * in {@link com.ibm.websphere.objectgrid.AvailabilityState#PRELOAD}  state.
 *
 * @since WAS XD 6.1.0.3
 */
public class ClientLoaderFactory {

    /**
     * Get an instance of ClientLoader which can be used to load the distributed ObjectGrid
     * maps from an ObjectGrid client.
     * @return an instance of ClientLoader.
     */
    public static ClientLoader getClientLoader() {
        return ClientLoaderImpl.instance();
    }

    /**
     * A {@link Session} object directly retrieved from {@link ObjectGrid} can
     * not be used to drive transactions to a shard in
     * {@link com.ibm.websphere.objectgrid.AvailabilityState#PRELOAD} state.
     * This method makes a {@link Session} object be able to drive transactions to a shard
     * in {@link com.ibm.websphere.objectgrid.AvailabilityState#PRELOAD} state.
     * <p>
     * When this method is called, the session object can only used to drive transactions to a shard
     * in PRELOAD state. It cannot be used to drive transactions to a shard in
     * {@link com.ibm.websphere.objectgrid.AvailabilityState#ONLINE} state. Instead, retrieve
     * a different session object.
     * @param session the Session object retrieved from {@link ObjectGrid}
     * @return the session object which can be used to drive transactions to a shared in PRELOAD state
     */
    public static Session getClientLoaderSession(Session session) {
        SessionImpl s = (SessionImpl) session;
        if (s.getObjectGrid().getObjectGridType() != ObjectGrid.CLIENT) {
            throw new IllegalStateException("The session is not a client session.");
        }
        s.setSessionAttribute(SessionAttribute.CLIENT_LOAD_SESSION);
        return s;
    }
}
