/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008-2010
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 * xxxx          6.1      10/27/2007    haowang   catalog server control
 */

package com.ibm.websphere.objectgrid.management;

/**
 * This MBean interface allows user to manipulate the behaviors of heartbeat and leader manager. The object name pattern
 * for this MBean is:
 *
 * <pre>
 * com.ibm.websphere.objectgrid:type=CoreGroupService
 * </pre>
 *
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object
 * name.
 *
 * @since WAS XD 6.1.0.3
 *
 * @deprecated This MBean interface is deprecated in 7.1. Use the CatalogServiceManagementMBean MBean interface.
 * @see CatalogServiceManagementMBean
 */

public interface CoreGroupServiceMBean {

    /**
     * Constant representing a heartbeat level at a typical rate.
     * <p>
     *
     * A typical heartbeat frequency allows reasonable failover detection and resource utilization. This value is the
     * default.
     * @deprecated This MBean interface is deprecated in 7.1. Use the CatalogServiceManagementMBean MBean interface.
     */
    public static final int HEARTBEAT_FREQUENCY_LEVEL_TYPICAL = 0;

    /**
     * Constant representing a heartbeat level at an aggressive rate.
     * <p>
     *
     * An increased heartbeat frequency allows failures to be detected more quickly, but can also uses additional CPU
     * and network resources. This level is more sensitive to missing heartbeats when the server is stressed.
     * @deprecated This MBean interface is deprecated in 7.1. Use the CatalogServiceManagementMBean MBean interface.
     */
    public static final int HEARTBEAT_FREQUENCY_LEVEL_AGGRESSIVE = -1;

    /**
     * Constant representing a heartbeat level at relaxed rate.
     * <p>
     *
     * A decreased heartbeat frequency increases the time to detect failures, but also decreases CPU and network
     * utilization.
     * @deprecated This MBean interface is deprecated in 7.1. Use the CatalogServiceManagementMBean MBean interface.
     */
    public static final int HEARTBEAT_FREQUENCY_LEVEL_RELAXED = 1;

    /**
     * /** Sets the heartbeat frequency level. The heartbeat frequency level tells the catalog service how aggressively
     * to check for container failures. The default is 0, which is suitable for most environments.
     * <p>
     *
     * Valid values include:
     * <ul>
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_TYPICAL} (default)
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_RELAXED}
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_AGGRESSIVE}
     * </ul>
     *
     * @param frequencyLevel
     *            one of three values: -1, 0 or 1 as defined by the constants that begin with name
     *            <tt>HEARTBEAT_FREQUENCY_LEVEL</tt>.
     * @throws IllegalArgumentException
     *             if the frequencyLevel is out of range.
     * @deprecated This MBean interface is deprecated in 7.1.
     * @see CatalogServiceManagementMBean
     */
    public void setHeartBeatFrequencyLevel(int frequencyLevel);

    /**
     * Retrieves the heartbeat frequency level.
     * <p>
     *
     * Valid values include:
     * <ul>
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_TYPICAL}
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_RELAXED}
     * <li> {@link #HEARTBEAT_FREQUENCY_LEVEL_AGGRESSIVE}
     * </ul>
     *
     * @return the heartbeat frequency level: -1, 0 or 1 as defined by the constants that begin with name
     *         <tt>HEARTBEAT_FREQUENCY_LEVEL</tt>.
     * @deprecated This MBean interface is deprecated in 7.1.
     * @see CatalogServiceManagementMBean#getHeartBeatFrequencyLevel()
     */
    public int getHeartBeatFrequencyLevel();
}
