/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2010
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 * _____________________________________________________________________________________
 */
package com.ibm.websphere.objectgrid.management;

import java.util.List;

import javax.management.Notification;

/**
 * This MBean interface allows a client process to access JMX notification history broadcasted by the grid
 * catalog and/or server processes in a dynamic environment. The MBean dynamically re-broadcasts all notifications
 * generated by any of the MBean objects registered in the grid catalogs and servers in both its original format
 * and also a generalised "canonical" format. The object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=ManagementConcentrator
 * </pre>
 * <strong>The following notifications are available:</strong><p>
 * <p>
 * <code><strong>com.ibm.websphere.objectgrid.management.message</strong></code>
 * <table>
 * <tr>
 *     <td><strong>Description:</strong></td>
 *     <td>A notification received from any of the registered MBean objects in the grid catalogs and servers.</td>
 * </tr>
 * <tr> <td valign="top"><strong>UserData:</strong></td>
 *      <td>A CompositeData with the following items representing the generalised "canonical" format:<p>
 * <table>
 * <tr>
 *     <td><strong><u>Item Name</u></strong></td>
 *     <td><strong><u>Type</u></strong></td>
 *     <td><strong><u>Description</u></strong></td>
 *     <td><strong><u>Since</u></strong></td>
 * </tr>
 * <tr>
 *     <td>SourceServer</td>
 *     <td>String</td>
 *     <td>The name of the server that generated the notification.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>SeverityLevel</td>
 *     <td>String</td>
 *     <td>The level of severity associated with the notification.</td>
 *     <td>8.6</td>
 * </tr>
 * <tr>
 *     <td>OriginalType</td>
 *     <td>String</td>
 *     <td>The type of notification before the canonical formatting was applied.</td>
 *     <td>8.6</td>
 * </tr>
 * </table>
 * </td></tr>
 * <tr><td><strong>Message:</strong></td><td>The notification message.</td></tr>
 * <tr><td><strong>Since:</strong></td><td>8.6</td></tr>
 * </table>
 * @since 8.6
 */
public interface ManagementConcentratorMBean {
    /**
     * A constant identifying the canonical formatted notification type for the Management Concentrator.
     * @since 8.6
     */
	public static final String MESSAGE_HUB_NOTIFICATION = "com.ibm.websphere.objectgrid.management.message";

    /**
     * A constant identifying the Management Concentrator MBean name used during MBeanServer registration.
     * @since 8.6
     */
    public static final String MANAGEMENT_MBEAN_NAME    = "ManagementConcentrator";
    /**
     * A constant identifying the composite data name for the Management Concentrator notification.
     * @since 8.6
     */
	public static final String NOTIFICATION_DATA        = "com.ibm.websphere.objectgrid.management.NotificationData";
    /**
     * A constant identifying the composite data representing the source of notification.
     * @since 8.6
     */
    public static final String SOURCE_SERVER            = "SourceServer";
    /**
     * A constant identifying the composite data representing notification severity level.
     * @since 8.6
     */
	public static final String SEVERITY_LEVEL           = "SeverityLevel";
    /**
     * A constant identifying the composite data representing notification original type.
     * @since 8.6
     */
    public static final String ORIGINAL_TYPE            = "OriginalType";

    /**
     * Retrieves the most recent history list subset of notifications starting with the specified
     * <code>first</code> index and ending with the <code>last</code> index specified.
     * Some of the notifications specified by a certain index may no longer be available to return, depending on 
     * the timing of request relative to the rate of notifications being generated. In such case, the most recently
     * available notifications (subsequent to the <code>first</code> index specified) will be returned.
     * The notifications in the history list preserve the original content and formatting generated by the originating
     * MBean object registered in the grid catalog or server container.
     *  
     * @param first notification (sequence number) of the subset to be retrieved
     * @param last notification (sequence number) of the subset to be retrieved
     * @return notifications list history subset
     * @since 8.6
     */
    public List<Notification> getNotificationHistory( long first, long last );

    /**
     * Retrieves the most recent (last 1000) notifications list history subset available.
     * The notifications in the history list preserve the original content and formatting generated by the originating
     * MBean object registered in the grid catalog or server container.
     *  
     * @return notifications list history subset (last 1000)
     * @since 8.6
     */
    public List<Notification> getNotificationHistory();

    /**
	 * Retrieves the most recent history list subset of notifications in a generalised "canonical" form,
	 * starting with the specified <code>first</code> index and ending with the <code>last</code> index specified.
	 * Some of the notifications specified by a certain index may no longer be available to return, depending on 
	 * the timing of request relative to the rate of notifications being generated. In such case, the most recently
	 * available notifications (subsequent to the <code>first</code> index specified) will be returned.
     * 
     * <strong>The notifications in the history list have the following content:</strong><p>
     * <p>
     * <code><strong>com.ibm.websphere.objectgrid.management.message</strong></code>
     * <table>
     * <tr>
     *     <td><strong>Description:</strong></td>
     *     <td>A notification received from any of the registered MBean objects in the grid catalogs and servers.</td>
     * </tr>
     * <tr> <td valign="top"><strong>UserData:</strong></td>
     *      <td>A CompositeData with the following items representing the generalised "canonical" format:<p>
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong></td>
     *     <td><strong><u>Type</u></strong></td>
     *     <td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>SourceServer</td>
     *     <td>String</td>
     *     <td>The name of the server that generated the notification.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SeverityLevel</td>
     *     <td>String</td>
     *     <td>The level of severity associated with the notification.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>OriginalType</td>
     *     <td>String</td>
     *     <td>The type of notification before the canonical formatting was applied.</td>
     *     <td>8.6</td>
     * </tr>
     * </table>
     * </td></tr>
     * <tr><td><strong>Message:</strong></td><td>The notification message.</td></tr>
     * <tr><td><strong>Since:</strong></td><td>8.6</td></tr>
     * </table>
     * @param first notification (sequence number) of the subset to be retrieved
     * @param last notification (sequence number) of the subset to be retrieved
     * @return canonically formatted notifications list history subset
     * @since 8.6
	 */
	public List<Notification> getFormattedMessageHistory( long first, long last );

	/**
     * Retrieves the most recent history list subset (last 1000) containing notifications in a generalised
     * "canonical" form.
     * <strong>The notifications in the history list have the following content:</strong><p>
     * <p>
     * <code><strong>com.ibm.websphere.objectgrid.management.message</strong></code>
     * <table>
     * <tr>
     *     <td><strong>Description:</strong></td>
     *     <td>A notification received from any of the registered MBean objects in the grid catalogs and servers.</td>
     * </tr>
     * <tr> <td valign="top"><strong>UserData:</strong></td>
     *      <td>A CompositeData with the following items representing the generalised "canonical" format:<p>
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong></td>
     *     <td><strong><u>Type</u></strong></td>
     *     <td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>SourceServer</td>
     *     <td>String</td>
     *     <td>The name of the server that generated the notification.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>SeverityLevel</td>
     *     <td>String</td>
     *     <td>The level of severity associated with the notification.</td>
     *     <td>8.6</td>
     * </tr>
     * <tr>
     *     <td>OriginalType</td>
     *     <td>String</td>
     *     <td>The type of notification before the canonical formatting was applied.</td>
     *     <td>8.6</td>
     * </tr>
     * </table>
     * </td></tr>
     * <tr><td><strong>Message:</strong></td><td>The notification message.</td></tr>
     * <tr><td><strong>Since:</strong></td><td>8.6</td></tr>
     * </table>
     * @return canonically formatted notifications list history subset (last 1000)
     * @since 8.6
     */
    public List<Notification> getFormattedMessageHistory();
}
