// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  QuorumManagerMBean.java
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
//              XS 7.0    May 26, 2009 bsandman  new code drop
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.management;

/**
 * Each catalog service has a QuorumManager and an associated MBean.
 * The QuorumManager monitors and manages the quorum state of the catalog service grid.  When quorum is enabled,
 * the QuorumManager for each catalog service process detects when all catalog services in the grid have quorum or not.
 * This MBean allows querying the current quorum state and allows administrators to force quorum when there is a network failure.
 *
 * <pre>
 * com.ibm.websphere.objectgrid:type=QuorumManager
 * </pre>
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object name.
 *
 * @since 7.0, XC10
 */
public interface QuorumManagerMBean {
    public static final String QUORUM_LOST_NOTIFICATION = "com.ibm.websphere.objectgrid.quorum.lost";
    public static final String QUORUM_CHANGED_NOTIFICATION = "com.ibm.websphere.objectgrid.quorum.changed";

    /**
     * This operation forces surviving catalog service grid processes to
     * reestablish a quorum.<p>
     *
     * If a portion the catalog service grid fails or is divided due to a network failure, the grid will
     * lose quorum.  Once the administrator identifies the failure and the viable portion of the
     * grid, this operation can be invoked on any of the surviving catalog service processes to reestablish
     * a quorum.  Reestablishing a quorum will allow the catalog service to continue to react to
     * failures and topology changes.
     */
    public void overrideQuorum() throws Exception;

    /**
     * Retrieve the known number of active catalog service processes.
     * @return the known number of active catalog service processes.
     */
    public int getActiveCatalogServers();

    /**
     * Retrieve the number of catalog service processes required for quorum.
     * @return the number of catalog service processes required for quorum.
     */
    public int getQuorumCatalogServers();

    /**
     * Retrieves the names of the known active catalog service processes.
     * @return the names of the known active catalog service processes.
     * @since 7.1
     */
    public String[] getActiveCatalogServerNames();

}
