// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.4 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/management/ObjectGridMBean.java, WAS.objectgrid, WASX.XD, b0614.107 2/17/06 20:14:01
//
// Source File Description:
//
// Interface for the MBean operations and attributes related to ObjectGrids
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d327475      WASX.XD   Nov 29, 2005 joshuad     new code drop
// d329200      WASX.XD   12-6-05      joshuad   gateway enhancements
// d338619      WASX.XD   01/30/06     joshuad   add TPS
// d348005      WXD601.XD 02-16-2006   joshuad   add transaction type
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.management;

import com.ibm.websphere.objectgrid.stats.MapStatsModule;
import com.ibm.websphere.objectgrid.stats.SessionStatsModule;

/**
 * This MBean interface allows a client process to access different attributes
 * and statistical data about a specific session. The object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=Session,name=&lt;id&gt;,host=&lt;host&gt;,ogServerName=&lt;server&gt;
 * </pre>
 *
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object name.
 *
 * @since WAS XD 6.0.1, XC10
 */
public interface SessionMBean {

    /**
     * Gets the <code>SessionStatsModule</code> used to retrieve statistics
     * associated with the session for this MBean.
     *
     * @return an SessionStatsModule for statistics associated with this session
     *
     * @see SessionStatsModule
     */
    SessionStatsModule retrieveStatsModule();

    /**
     * Gets the ID of the session instance associated
     * with this MBean.
     *
     * @return the ID of the session instance associated
     *         with this MBean.
     */
    String getSessionID();

    /**
     * Gets a string representation of the <code>SessionStatsModule<code> attributes
     * loaded up by the <code>retrieveStatsModule()</code> method.
     *
     * @return String form of <code>SessionStatsModule</code>
     *
     * @see #retrieveStatsModule()
     * @see SessionStatsModule
     */
    String getSessionStatsModule();

    /**
     * Gets the map count attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return the number of entries in the map
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule#getNumEntries(boolean)
     */
    long getCreatedSessionsCount();

    /**
     * Gets invalidated sessions count
     * @return the count of invalidated Sessions
     */
    long getInvalidatedSessionsCount();

    /**
     * Gets active sessions count
     * @return the count of active Sessions
     */
    long getActiveSessionsCount();

    /**
     * Gets memory count
     * @return memory count
     */
    long getMemoryCount();

    /**
     * Gets cache discards count
     * @return cache discards count
     */
    long getCacheDiscardsCount();

    /**
     * Gets affinity breaks count
     * @return affinity breaks count
     */
    long getAffinityBreaksCount();

    /**
     * Gets invalidated by timeout count
     * @return count
     */
    long getInvalidatedByTimeoutCount();

    /**
     * Gets access to non-existent session count
     * @return count
     */
    long getAccessToNonExistentSessionCount();

    /**
     * Gets acessed sessions count
     * @return count
     */
    long getAccessedSessionsCount();
}
