/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008-2009
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 * xxxx          7.0 fix2  12/2/2009    joshuad   ThreadPoolMBean
 */

package com.ibm.websphere.objectgrid.management;

/**
 * This MBean interface allows user to access the thread pool properties. The object name pattern
 * for this MBean is:
 *
 * <pre>
 * com.ibm.websphere.objectgrid:type=ThreadPool
 * </pre>
 *
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object
 * name.
 *
 * @since 7.0.0.0 FIX2, XC10
 *
 */

public interface ThreadPoolMBean {

    /**
     * Sets the maximum thread pool size.
     * <p>
     *
     * @param size
     *            the maximum number of threads.
     */
    public void setMaximumSize(int size);

    /**
     * Retrieves the maximum number of threads in the pool.
     * <p>
     *
     * @return the maximum number of threads in the pool
     */
    public int getMaximumSize();

    /**
     * Sets the minimum thread pool size.
     * <p>
     *
     * @param size
     *            the minimum number of threads.
     */
    public void setMinimumSize(int size);

    /**
     * Retrieves the minimum number of threads in the pool.
     * <p>
     *
     * @return the minimum number of threads in the pool
     */
    public int getMinimumSize();

    /**
     * Retrieves the approximate number of active threads in the pool.
     * <p>
     *
     * @return the number of active threads in the pool in use
     */
    public int getActiveThreadCount();
    
    /**
     * Retrieves the name of the ThreadPool.
     * <p>
     *
     * @return the name of the ThreadPool
     */
    public String getName();
}
