// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $HeadURL$
// $Id$
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b21871       XS7.1.1   01/11/2011   jian.tang eXtreme Scale OSGi Support
// b21871       XS7.1.1   05/13/2011   pradine   eXtreme Scale OSGi Support
// b24535       XS7.1.1   08/31/2011   weaverm   remove OSGi support from XSAdmin
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.osgi;

import java.lang.reflect.Method;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.xs.util.ForNameClassStore;
import com.ibm.ws.xs.util.MethodHelper;

/**
 * A factory to get the following objects:
 * <UL>
 * <LI>{@link ObjectGridOSGiManager} for managing OSGi service for a local or
 * client ObjectGrid instance. </LI>
 * <LI>{@link OSGiServiceRepository} for querying OSGi service versions for an OSGi
 * runtime. </LI>
 * </UL>
 *
 * @since 7.1.1
 *
 */
public final class OSGiManagerFactory {

    final static private String CLASS_NAME = OSGiManagerFactory.class.getName();

    final static private OSGiManagerFactory singleton = new OSGiManagerFactory();

    static final private OSGiServiceRepository serviceRepository;

    static final private Method METHOD_GET_OBJECTGRID_OSGI_MANAGER;

    static {

        String className = "com.ibm.ws.xs.osgi.service.OSGiRepositoriesImpl";
        String methodName = "getInstance";

        Class<?> clazz = null;
        Method m = null;

        try {
            clazz = ForNameClassStore.forName(className, ForNameClassStore.DOPRIV_FOR_NAME);
            m = MethodHelper.getAccessibleMethod(clazz, methodName, new Class[] {});
            serviceRepository = (OSGiServiceRepository) m.invoke(null, new Object[] {});
        } catch (Exception e) {
            FFDCFilter.processException(e, CLASS_NAME + ".<init>", "60",
                    new Object[] { className, clazz, methodName, m });
            throw new ObjectGridRuntimeException(e);
        }
    }

    static {
        String className = "com.ibm.ws.xs.osgi.service.ClientActivator";
        String methodName = "getObjectGridOSGiManager";

        Class<?> clazz = null;
        Method m = null;

        try {
            clazz = ForNameClassStore.forName(className, ForNameClassStore.DOPRIV_FOR_NAME);
            m = MethodHelper.getAccessibleMethod(clazz, methodName, new Class[] { ObjectGrid.class });
        } catch (Exception e) {
            FFDCFilter.processException(e, CLASS_NAME + ".<init>", "80",
                    new Object[] { className, clazz, methodName, m });
            throw new ObjectGridRuntimeException(e);
        } finally {
            METHOD_GET_OBJECTGRID_OSGI_MANAGER = m;
        }
    }

    private OSGiManagerFactory() {
    }

    /**
     * Retrieve the singleton OSGiManagerFactory.
     * @return the singleton OSGiManagerFactory.
     */
    static public OSGiManagerFactory getInstance() {
        return singleton;
    }

    /**
    * Return the ObjectGridOSGiManager for this ObjectGrid instance.
    * <p>
    *
    * @param objectGrid the ObjectGrid instance. It could be a server-side ObjectGrid instance,
    * client ObjectGrid instance, or a local ObjectGrid instance.
    *
    * @return the ObjectGridOSGiManager for the specified ObjectGrid instance.
    */
    public ObjectGridOSGiManager getOSGiManager(ObjectGrid objectGrid) {
        try {
            return (ObjectGridOSGiManager) METHOD_GET_OBJECTGRID_OSGI_MANAGER.invoke(null, new Object[] { objectGrid });
        } catch (Exception e) {
            FFDCFilter.processException(e, CLASS_NAME + ".getOSGiManager", "60", new Object[] { this,
                    METHOD_GET_OBJECTGRID_OSGI_MANAGER, objectGrid });
            throw new ObjectGridRuntimeException(e);
        }
    }

    /**
     * Return the OSGiServiceRepository.
     *
     * @return the OSGiServiceRepository instance.
     */
    public OSGiServiceRepository getOSGiServiceRepository() {
        return serviceRepository;
    }

}
