// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $HeadURL$
// $Id$
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b21871       XS7.1.1   01/11/2011   jian.tang eXtreme Scale OSGi Support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.osgi;

import java.util.Map;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.ObjectGridManager;
import com.ibm.websphere.objectgrid.plugins.ObjectGridEventGroup.ShardEvents;

/**
 * An {@link ObjectGridOSGiManager} is responsible for querying and managing the
 * OSGi services currently used for an ObjectGrid instance.
 * <p>
 * This API is useful in the case that the user application can directly access
 * the ObjectGrid instances.  Users can directly access the client ObjectGrid instances
 * and local ObjectGrid instances using {@link ObjectGridManager} interface.
 * Users can also access server-side ObjectGrid instances using user plug-ins
 * running in the container server, for example, {@link ShardEvents}.
 * <p>
 * Use the
 * {@link com.ibm.websphere.objectgrid.osgi.OSGiManagerFactory#getOSGiManager(ObjectGrid objectGrid)}
 * method to get an instance of ObjectGridOSGiManager.
 *
 * @since 7.1.1
 *
 *
 */
public interface ObjectGridOSGiManager {

    /**
     * Get the current ranking for a specific service used by a grid
     *
     * @param serviceName the service name
     * @return the current used service ranking for that ObjectGrid
     */
    Integer getRanking(String serviceName);

    /**
     * Get the current service rankings for all services used by an ObjectGrid.
     *
     * @return a map of currently used service rankings. The key of the map is the
     * service name, and the value of the map is the service ranking.
     */
    Map<String, Integer> getRankings();

    // TODO we might remove these two useRanking methods
    /**
     * To use the service for a specific ranking.
     * <p>
     * This method can only be called when the ObjectGrid type is a CLIENT type or
     * a LOCAL type. You can get the ObjectGrid type using the
     * {@link ObjectGrid#getObjectGridType()} method.
     * <p>
     * This method might be removed when the API is finalized.
     *
     * @param serviceName the OSGi service name
     * @param ranking the service ranking to be used
     *
     * @throws ServiceNotAvailableException
     *         thrown when that specified service ranking of the service is not available
     * @throws ServiceUpdateException
     *         thrown when the specified OSGi service version is
     *         available, but the service update fails.
     */
    void useRanking(String serviceName, Integer ranking) throws ServiceNotAvailableException, ServiceUpdateException;

    /**
     * To use a set of services with specific rankings.
     * <p>
     * This method can only be called when the ObjectGrid type is a CLIENT type or
     * a LOCAL type. You can get the ObjectGrid type using the
     * {@link ObjectGrid#getObjectGridType()} method.
     * <p>
     * This method might be removed when the API is finalized.
     *
     * @param serviceRankings A map of the used service rankings to be used.
     * The key of the map is the service name, and value of the map is the service version to be used.
     *
     * @throws ServiceNotAvailableException
     *         thrown when that specified version of the service is not available
     * @throws ServiceUpdateException
     *         thrown when the specified OSGi service version is
     *         available, but the service update fails.
     */
    void useRanking(Map<String, Integer> serviceRankings) throws ServiceNotAvailableException,
            ServiceUpdateException;

}
