// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2010
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $HeadURL$
// $Id$
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b21871       XS7.1.1   11/12/2010   jian.tang eXtreme Scale OSGi Support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.osgi;

import java.util.HashMap;
import java.util.Map;

import com.ibm.websphere.objectgrid.ObjectGridException;

// TODO enhance the exception to contain failed partition id.
/**
 * A ServiceNotAvailableException indicates one or more specific
 * service rankings are not available.
 * <p>
 * Users can use {@link #getUnavailableServiceRankings()} to get the unavailable
 * service rankings.
 *
 * @since 7.1.1
 */
public class ServiceNotAvailableException extends ObjectGridException {
    /**
     * The <code>serialVersionUID</code> for this class.
     */
    private static final long serialVersionUID = 1L;

    final private Map<String, Integer> unavailableServiceRankings;

    /**
     * Constructs a new ServiceNotAvailableException with the specified detail message. The
     * cause is not initialized, and may subsequently be initialized by a call to
     * the <code>initCause</code> method.
     *
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     * @param service
     *           the OSGi service name
     * @param ranking
     *           the unavailable OSGi service ranking
     *
     * @see #initCause(Throwable)
     * @see #getMessage()
     */
    public ServiceNotAvailableException(String message, String service, Integer ranking) {
        super(message);
        if (service == null || ranking == null) {
            throw new IllegalArgumentException("The service or the rank cannot be null.");
        }
        unavailableServiceRankings = new HashMap<String, Integer>();
        unavailableServiceRankings.put(service, ranking);
    }

    /**
     * Constructs a new ServiceNotAvailableException with the specified detail message. The
     * cause is not initialized, and may subsequently be initialized by a call to
     * the <code>initCause</code> method.
     *
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     *
     * @param unavailableRankings
     *           the service rankings that are not available. The key of the map is the service name,
     *           and the value is the service ranking.
     *
     * @see #initCause(Throwable)
     * @see #getMessage()
     */
    public ServiceNotAvailableException(String message, Map<String, Integer> unavailableRankings) {
        super(message);
        this.unavailableServiceRankings = unavailableRankings;
    }

    /**
     * Constructs a new ServiceNotAvailableException with the specified detail message and
     * cause.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this ServiceNotAvailableException's detail
     * message.
     *
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). (A<code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown).
     * @param service
     *           the OSGi service name
     * @param ranking
     *           the unavailable OSGi service ranking
     *
     * @see #getCause()
     * @see #getMessage()
     */
    public ServiceNotAvailableException(String message, Throwable cause, String service, Integer ranking) {
        super(message, cause);
        if (service == null || ranking == null) {
            throw new IllegalArgumentException("The service or the ranking cannot be null.");
        }
        unavailableServiceRankings = new HashMap<String, Integer>();
        unavailableServiceRankings.put(service, ranking);
    }

    /**
     * Constructs a new ServiceNotAvailableException with the specified detail message and
     * cause.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this ServiceNotAvailableException's detail
     * message.
     *
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). (A<code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown).
     * @param serviceRankings
     *           the service rankings that are not available. The key of the map is the service name,
     *           and the value is the service ranking.
     *
     *
     * @see #getCause()
     * @see #getMessage()
     */
    public ServiceNotAvailableException(String message, Throwable cause, Map<String, Integer> serviceRankings) {
        super(message, cause);
        this.unavailableServiceRankings = serviceRankings;
    }

    /**
     * @return the unavailable service rankings.
     */
    public Map<String, Integer> getUnavailableServiceRankings() {
        return unavailableServiceRankings;
    }


}