// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $HeadURL$
// $Id$
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b21871       XS7.1.1   04/28/2011   jian.tang eXtreme Scale OSGi Support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.osgi;

import com.ibm.websphere.objectgrid.ObjectGridException;

/**
 * A ServiceUpdateException is thrown if an OSGi service
 * fails to update to a specific service ranking.
 * <p>
 * The method {@link #getServiceName()} and {@link #getServiceRanking()} can
 * be used to get the service name and the service ranking that it
 * fails to update to.
 *
 * @since 7.1.1
 */
public class ServiceUpdateException extends ObjectGridException {

    /**
     * The <code>serialVersionUID</code> for this class.
     */
    private static final long serialVersionUID = 1L;

    /**
     * The name of the OSGi service that fails to update
     */
    final private String serviceName;

    /**
     * The ranking of the OSGi service that fails to update
     */
    final private Integer serviceRanking;

    /**
     * Constructs a new ServiceUpdateException with <code>null</code> as its detail
     * message. The cause is not initialized, and may subsequently be initialized
     * by a call to the <code>initCause</code> method.
     *
     * @param serviceName the name of the OSGi service that fails to update
     * @param serviceRanking the ranking of the OSGi service that fails to update
     *
     * @see #initCause(Throwable)
     */
    public ServiceUpdateException(String serviceName, Integer serviceRanking) {
        super();
        this.serviceName = serviceName;
        this.serviceRanking = serviceRanking;
    }

    /**
     * Constructs a new ServiceUpdateException with the specified detail message. The
     * cause is not initialized, and may subsequently be initialized by a call to
     * the <code>initCause</code> method.
     *
     * @param serviceName the name of the OSGi service that fails to update
     * @param serviceRanking the ranking of the OSGi service that fails to update
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     *
     * @see #initCause(Throwable)
     * @see #getMessage()
     */
    public ServiceUpdateException(String serviceName, Integer serviceRanking, String message) {
        super(message);
        this.serviceName = serviceName;
        this.serviceRanking = serviceRanking;
    }

    /**
     * Constructs a new ServiceUpdateException with the specified detail message and
     * cause.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this ServiceUpdateException's detail
     * message.
     *
     * @param serviceName the name of the OSGi service that fails to update
     * @param serviceRanking the ranking of the OSGi service that fails to update
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). (A<code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown).
     *
     * @see #getCause()
     * @see #getMessage()
     */
    public ServiceUpdateException(String serviceName, Integer serviceRanking, String message, Throwable cause) {
        super(message, cause);
        this.serviceName = serviceName;
        this.serviceRanking = serviceRanking;
    }

    /**
     * Constructs a new ServiceUpdateException with a specified cause.
     * The cause and a detail message of <code>(cause==null ? null : cause.toString())</code>
     * is used (which typically contains the class and detail message of <code>cause</code>).
     * This constructor is useful for ServiceUpdateExceptions that are little more
     * than wrappers for other throwables.
     *
     * @param serviceName the name of the OSGi service that fails to update
     * @param serviceRanking the ranking of the OSGi service that fails to update
     * @param cause is the exception that caused this exception to be thrown,
     *        which is saved for later retrieval by the <code>getCause()</code> method.
     *        A <code>null</code> value is permitted and indicates that the cause
     *        is nonexistent or is unknown.
     *
     * @see #getCause()
     */
    public ServiceUpdateException(String serviceName, Integer serviceRanking, Throwable cause) {
        super(cause);
        this.serviceName = serviceName;
        this.serviceRanking = serviceRanking;
    }

    /**
     * @return the name of the service that fails to update
     */
    public String getServiceName() {
        return serviceName;
    }

    /**
     * @return the ranking of the service that fails to update
     */
    public Integer getServiceRanking() {
        return serviceRanking;
    }

}
