// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2010,2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  BackingMapPlugin.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b21148       7.1.1     11/30/2010   cdjohnson new part
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins;

import com.ibm.websphere.objectgrid.BackingMap;

/**
 * BackingMap plug-ins implement this interface to be invoked when
 * the plug-in should be initialized and destroyed.<p>
 *
 * BackingMap plug-ins are instantiated, state is injected into the plug-in,
 * it is then initialized and finally destroyed. The destroy method will only
 * be invoked for plug-ins that have successfully been initialized.
 * A plug-in cannot be initialized or destroyed more than once.<p>
 *
 * General lifecycle:
 * <ol>
 * <li>UserBackingMapPlugin plugin = new UserBackingMapPlugin();
 * <li>All setter methods are called to set the plugin's state.
 * <li>plugin.initialize();
 * <li>plugin.destroy();
 * <li>plugin = null;
 * </ol>
 *
 * Any exceptions thrown from the initialize method are fatal and result in the parent
 * BackingMap to fail to initialize.  The result of the exception depends on the
 * current state of the BackingMap.<p>
 *
 * Plug-ins can optionally register a BackingMapLifecycleListener to be notified of
 * other map-related events during the initialize method, or can directly implement
 * the BackingMapLifecycleListener interface which will automatically register the plug-in
 * as a listener.
 *
 * @see BackingMapLifecycleListener
 * @since 7.1.1
 */
public interface BackingMapPlugin extends Initializable, Destroyable {

	/**
	 * The BackingMap instance is at an undefined state, depending
     * on the type of plug-in and how it was registered with the map.  Use the {@link BackingMap#getState()} method
     * to determine what state the map is in.
	 * @param map the BackingMap associated with this plug-in instance.
	 */
	public void setBackingMap(BackingMap map);

	/**
	 * Retrieve the BackingMap instance associated with this plug-in.
     * <p>
     * During normal operation, this method may be called to confirm the correct 
     * operation of the BackingMapPlugin.
     * It must return the BackingMap instance previously set using 
     * the {@link BackingMapPlugin#setBackingMap(BackingMap)} method.
     *  
	 * @return the BackingMap instance.
	 */
	public BackingMap getBackingMap();

}
