// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2010,2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  Destroyable.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b21148       7.1.1     11/30/2010   cdjohnson new part
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins;

/**
 * Java bean plug-ins can implement this interface to allow destroying the plug-in
 * after it's state has been set.
 *
 * @since 7.1.1
 * @see BackingMapPlugin
 * @see ObjectGridPlugin
 */
public interface Destroyable {
	/**
	 * Invoked when the bean's state has been set and is ready to be destroyed.
     * The result should be that the bean is in the destroyed state.
	 */
	public void destroy();

	/**
	 * Answers true if the bean is in the destroyed state, false otherwise.
     * <p>
     * During normal operation, this method may be called to confirm the correct 
     * operation of the {@link Destroyable#destroy()} method.
     * 
	 * @return true if the bean is in the destroyed state.
	 */
	public boolean isDestroyed();
}
