// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2010
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.2 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/EvictorData.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:22:30
//
// Module  : EvictorData.java
//
// Source File Description: 
//
//    See javadoc for class.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d328136      WASX.XD   12/08/2005   kjlaw     externalize EvictorData.
// ------------ --------- ------------ --------- -----------------------------------------
//

package com.ibm.websphere.objectgrid.plugins; 


/**
 * This interface is optionally used by an implementator of the 
 * <code>Evictor</code> interface. Application changes applied to a 
 * <code>BackingMap</code> are asynchronous from the <code>Evictor</code>
 * activity. The <code>Evictor</code> is not notified of changes to 
 * the <code>BackingMap</code> until after application transactions are 
 * committed.  Consequently, if an <code>Evictor</code> decides to evict a map
 * entry, it is possible that the <code>BackingMap</code> could evict an entry
 * that was different from the original entry it was tracking.  For example, 
 * consider that an application could execute a transaction that removes a map 
 * entry.  Before the <code>Evictor</code> is notified of the remove, another
 * transaction inserts a new entry into the <code>BackingMap</code> for the same
 * key as the old entry. Consequently, the <code>Evictor</code> could evict the 
 * newly created entry when it meant to evict the old entry.  To help close this 
 * small timing window, the <code>Evictor</code> can use this interface to 
 * associate evictor specific data with a map entry.
 * The <code>Evictor</code> can then do the following:  
 * <ul>
 * <li>store the <code>EvictorData</code> object for a map entry by using the 
 * <code>EvictionEventCallback.setEvictorData(Object, EvictorData)</code> 
 * method.  
 * <li>retrieve the <code>EvictorData</code> object for a map entry by using
 * the <code>EvictionEventCallback.getEvictorData(Object)</code> method.  
 * <li>Conditionally evict a map entry if and only if the cache entry for a 
 * specified key has the exact same <code>EvictorData</code> object 
 * (the java == operator returns true) associated with it by using the  
 * <code>EvictionEventCallback.evictMapEntries(List)</code> method.
 * </ul> 
 * 
 * @since WAS XD 6.0.1, XC10
 * 
 * @see Evictor
 * @see EvictionEventCallback
 */
public interface EvictorData
{
    /**
     * Retrieves the key object for this <code>EvictorData</code> instance.
     * 
     * @return the same key object that was passed to the  
     *         <code>EvictionEventCallback.setEvictorData(Object, EvictorData)</code>
     *         method when this EvictorData was associated with the map entry
     *         with the given key.
     * 
     * @see EvictionEventCallback#setEvictorData(Object, EvictorData)
     */
    public Object getKey();

    /**
     * Special value class used for representing the key not being found in the 
     * <code>BackingMap</code>.
     * 
     * @since WAS XD 6.0.1
     */
    public static final class SpecialEvictorData implements EvictorData 
    {
        /** 
         * Dummy implementation method since this class will not be called.
         * @return null
         */
        public Object getKey()
        {     
            return null;
        } 
    }

    /** 
     * A special value indicating that the key was not found. 
     */
    public static final EvictorData KEY_NOT_FOUND = new SpecialEvictorData(); 
}


