// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.5 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/OptimisticCallback.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:23:15
//
// Module  :  OptimisticCallback.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   01/14/2005   sutter    new code drop
// XD60_M3_B.11 WASX.XD   04/11/2005   kjlaw     added updateVersionedObjectForValue method.
// 279166       WASX.XD   06/16/2005   sutter    Moved serialize/inflate versioned value methods to OptimisticCallback
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.LockStrategy;
import com.ibm.websphere.objectgrid.plugins.io.ValueDataSerializer.Versionable;
import com.ibm.websphere.objectgrid.plugins.io.ValueSerializerPlugin;

/**
 * An object implementing this interface is used to provide optimistic
 * comparison operations for the values of a Map.
 * <p>
 * An <code>OptimisticCallback</code> implementation that also implements the {@link BackingMapLifecycleListener}
 * interface will be automatically added as an {@link EventListener} on the {@link BackingMap} when the
 * callback set on the backing map.
 * <p>
 * An <code>OptimisticCallback</code> may also implement the {@link BackingMapPlugin} interface in order
 * to receive enhanced BackingMap plug-in lifecycle method calls. The plug-in is then
 * also required to correctly implement each of the bean methods related to
 * introspection of its state (for example <code>isInitialized()</code>,
 * <code>isDestroyed()</code>, etc).
 *
 * @since WAS XD 6.0
 *
 * @see BackingMap#addMapEventListener(EventListener)
 * @see BackingMap#setLockStrategy(LockStrategy)
 * @see BackingMap#setOptimisticCallback(OptimisticCallback)
 * @see LockStrategy#OPTIMISTIC
 * @deprecated This plug-in is deprecated in version 7.1.1.  Use the {@link ValueSerializerPlugin} with the {@link Versionable} mix-in interface.
 */
@Deprecated
public interface OptimisticCallback
{
    /**
     * Special value returned by getVersionObjectForValue if the default
     * OptimisticCallback is used by a BackingMap since the
     * <code>BackingMap.setOptimisticCallback(OptimisticCallback)</code>
     * method was never called.
     */
    public final static Byte NULL_OPTIMISTIC_VERSION = new Byte((byte) 0);

    /**
     * Returns the value (must be a copy) or it may return an attribute
     * of the value that can be used for versioning purposes.
     * <p>
     * This method is called whenever an object is associated with a transaction
     * so that at commit time, the ObjectGrid runtime can do qualified updates
     * using this value to detect if the value changed during the optimistic
     * transaction. The <code>Loader</code> also gets this value from the
     * <code>LogElement</code>.
     *
     * @param value The value to version
     *
     * @return Either a copy of the value or a copy of an attribute that can be
     *         used for versioning.
     *
     * @see Loader
     * @see LogElement#getVersionedValue()
     */
    Object getVersionedObjectForValue(Object value);

    /**
     * Called whenever a transaction has updated a value and a new versioned
     * object is needed.
     * <p>
     * If the <code>getVersionedObjectForValue(Object)</code> method returned an
     * attribute of the value, this method typically updates that value with a
     * new version object for the value. If the <code>getVersionedObjectForValue(Object)</code>
     * method returned a copy of the value, then this method typically does nothing.
     *
     * @param value The value whose version attribute must be updated.
     *
     * @see #getVersionedObjectForValue(Object)
     */
    void updateVersionedObjectForValue(Object value);

    /**
     * Writes the versioned value object to the specified stream.
     * <p>
     * Depending on the implementation, the versioned value can be used to
     * identify optimistic update collisions.  In the default implementation,
     * the versioned value is just a copy of the original value.  Other
     * implementations may have a sequence number or some other object to
     * indicate the version of the value.  Since the actual implementation is
     * unknown, this method is provided to perform the proper serialization.
     * The default implementation just does a <code>writeObject</code>.
     *
     * @param versionedValue the versioned value object
     * @param stream The stream to write the object to
     *
     * @throws IOException if an error occurs during processing of this method
     */
    void serializeVersionedValue(Object versionedValue, ObjectOutputStream stream)
        throws IOException;

    /**
     * Reads a versioned value object from the specified stream.
     * <p>
     * This method takes the Serializable version of the versioned value and
     * returns the actual versioned value object.
     * Depending on the implementation, the versioned value can be used to
     * identify optimistic update collisions.  In the default implementation,
     * the versioned value is just a copy of the original value.  Other
     * implementations may have a sequence number or some other object to
     * indicate the version of the value.  Since the actual implementation is
     * unknown, this method is provided to perform the proper de-serialization.
     * The default implementation just does a <code>readObject</code>.
     *
     * @param stream the <code>ObjectInputStream</code> object to read the
     *               versioned value object from
     *
     * @return de-serialized versioned value object.
     *
     * @throws IOException if an error occurs during processing of this method
     * @throws ClassNotFoundException if the versioned value class isn't found
     *
     * @see #serializeVersionedValue(Object, ObjectOutputStream)
     */
    Object inflateVersionedValue(ObjectInputStream stream)
        throws IOException, ClassNotFoundException;
}
