// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.5 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/ValueProxyInfo.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:23:36
//
// Module  :  ValueProxyInfo.java
//
// Source File Description: 
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   01/14/2005   sutter    new code drop
// 292337       WASX.XD   07/28/2005   kjlaw     add ibmClearDirtyAttributes.
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins;

import java.util.List;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.CopyMode;
import com.ibm.websphere.objectgrid.TxID;

/**
 * This interface can be used by value objects to inform a BackingMap or Loader 
 * which attribute(s) have been dirtied.  This mechanism allows the BackingMap 
 * and Loader to interrogate the set of changed attributes in the value object 
 * instead of just assuming the whole value object has been updated. For this 
 * to be useful, the application must only use the getter and setter methods defined
 * for the value object's interface.
 * 
 * @since WAS XD 6.0, XC10
 * 
 * @see CopyMode#COPY_ON_WRITE
 * @see BackingMap#setCopyMode(CopyMode, Class)
 * @see Loader#batchUpdate(TxID, LogSequence)
 */
public interface ValueProxyInfo 
{
    /**
     * Returns a list of dirty attributes based on the value interface set on
     * the map. 
     * <p>
     * The attribute name is always starts with an upper case letter. For 
     * example, if the setter for the attribute is setPrice then 'Price' is the 
     * string returned here. The runtime uses <code>substring(3)</code> of the
     * setter method name as the attribute name.
     * 
     * @return List of attribute names (Strings)
     */
    List /*<String>*/ ibmGetDirtyAttributes();

    /**
     * Returns the real value object this proxy represents.
     * <p>
     * Needed internally by the BackingMap to return a separate 
     * proxy for each transaction.
     * 
     * @return actual value object.
     */
    Object ibmGetRealValue();

    /**
     * Clears the list of dirty attributes.   
     */
    void ibmClearDirtyAttributes();
}
