// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  NoVersioningOptimisticCallback.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
//
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.builtins;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import com.ibm.websphere.objectgrid.plugins.OptimisticCallback;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer.DataAttributeInflatable;
import com.ibm.websphere.objectgrid.plugins.io.ValueDataSerializer.Versionable;;

/**
 * This built-in OptimisticCallback allows the user to use Optimistic locking without doing version checking. In the
 * 6.0.x release this was the default behavior to not actually have any version checking since the ObjectGrid runtime
 * does not know how to version an application's object. A default version mechanism is now used by default where the
 * entire application object is used as the version value to know if the initial version matches the current committed
 * value. If the application's object does not have an equals() method, the default versioning will not work.
 * <p>
 * A use case for using this built-in OptimisticCallback is the Loader is handling the version checking, but optimistic
 * locking is used to guarantee committed data is always returned on a read. If there is no contention between map
 * entries that would cause a collision with two threads modifying a value at the same time, then this built-in plugin
 * should also be used to avoid the overhead of optimistic version checking.
 *
 * @see OptimisticCallback
 *
 * @since WAS XD 6.1
 * @deprecated This plug-in is deprecated in version 7.1.1.  Use the {@link DataAttributeInflatable} plug-in with the {@link Versionable} mix-in interface.
 */
@Deprecated
public class NoVersioningOptimisticCallback implements OptimisticCallback {

    /**
     * @return a {@link OptimisticCallback#NULL_OPTIMISTIC_VERSION} to indicate no
     * versioning check is used.
     * @see OptimisticCallback#getVersionedObjectForValue(Object)
     */
    public Object getVersionedObjectForValue(Object value) {
        return OptimisticCallback.NULL_OPTIMISTIC_VERSION;
    }


    /**
     * This method does nothing because no version checking is used.
     * @see OptimisticCallback#updateVersionedObjectForValue(Object)
     */
    public void updateVersionedObjectForValue(Object value) {
        // do nothing since by default we are just returning NULL_OPTIMISTIC_VERSION.
    }

    /**
     * This method does nothing because no version checking is used.
     * @see OptimisticCallback#serializeVersionedValue(Object, ObjectOutputStream)
     */
    public void serializeVersionedValue(Object versionedValue, ObjectOutputStream stream) {
        // do nothing since by default we are just using NULL_OPTIMISTIC_VERSION.
    }

    /**
     * @return a {@link OptimisticCallback#NULL_OPTIMISTIC_VERSION} to indicate no
     * versioning check is used.
     * @see OptimisticCallback#inflateVersionedValue(ObjectInputStream)
     */
    public Object inflateVersionedValue(ObjectInputStream stream) {
        // do nothing since by default we are just returning NULL_OPTIMISTIC_VERSION.
        return OptimisticCallback.NULL_OPTIMISTIC_VERSION;
    }
}
