// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.10 XD/ws/code/objectgrid.impl/src/com/ibm/websphere/objectgrid/plugins/builtins/TranPropListener.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 19:05:51
//
// Module  :  TranPropListener.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason    Version   Date         Userid    Change Description
// --------- --------- ------------ --------- -----------------------------------------
// dxxxxxxx  WASX      May 25, 2005 pdykes    New copyright/prologue
// d279166   WASX      Jun 06, 2005 pdykes    Enhance property usage
// 284443    WASX.XD   06/23/2005   sutter    Modify initialize signature to provide a Session
// --------- --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins.builtins;

import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.zip.CRC32;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.ObjectGridManager;
import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.plugins.DistributionMode;
import com.ibm.websphere.objectgrid.plugins.LogSequence;
import com.ibm.websphere.objectgrid.plugins.LogSequenceFilter;
import com.ibm.websphere.objectgrid.plugins.LogSequenceTransformer;
import com.ibm.websphere.objectgrid.plugins.ObjectGridEventListener;
import com.ibm.ws.bootstrap.WSLauncher;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;
import com.ibm.ws.objectgrid.server.plugins.TranCommitLogSeqFilter;
import com.ibm.ws.objectgrid.server.plugins.TranPropSvcEndPoint;
import com.ibm.ws.objectgrid.runtime.RuntimeInfo;
import com.ibm.ws.objectgrid.server.DependencyProvider;
import com.ibm.ws.objectgrid.server.event.TranPropReq;
import com.ibm.ws.xs.NLSConstants;
import com.ibm.ws.xs.util.zip.Compressor;
import com.ibm.wsspi.hamanager.GroupManager;
import com.ibm.wsspi.hamanager.GroupName;
import com.ibm.wsspi.hamanager.HAGroup;
import com.ibm.wsspi.hamanager.corestack.CoreStack;
import com.ibm.wsspi.hamanager.datastack.MsgQoS;
import com.ibm.wsspi.hamanager.policy.NOOPPolicy;

/**
 * This implementation of the <code>ObjectGridEventListener</code> interface
 * allows users to propagate successful ObjectGrid transaction commits to other
 * WebSphere Application Servers hosting the same ObjectGrid instance (based
 * upon the ObjectGrid name). The sole attribute to distinguish each ObjectGrid
 * is its name. If the names are the same, and both are configured with this
 * listener on separate WebSphere Application Server coregroup members (can
 * span a WebSphere cluster) non-readonly changes will be propagated
 * depending on the options the user selects.
 * <p>
 * Four options for distribution are provided. They include: push, push with
 * versioning (the default), invalidate and invalidate with versioning. The
 * various options are described in the <code>DistributionMode</code> class.
 * <p>
 * In addition to the distribution mode, the user can configure if the listener
 * should only propagate transactions to other clusters members, only receive
 * transaction commits from other similarly configured ObjectGrid instances, or
 * do both in each application server instance (the default). This support
 * provides for optimistic, asynchronous distributed transaction commit support
 * only.
 * <p>
 * Finally, the customer can configure if compression should be enabled.
 * Generally for typical transaction commit streams, the compression support
 * will use some local JVM cpu and memory footprint to create a compressed
 * stream for transmission to other instances of the ObjectGrid enabled with
 * this listener support located in other WebSphere core group members. However,
 * users should be aware of a common compression concern. For the scenarios when
 * there are usually very short messages, the overhead maybe more than the
 * computing resources saved by doing the compression. If that scenario is a
 * problem, compression can be disabled (the default value).
 * <p>
 * This class will only initialize correctly within a WebSphere runtime
 * environment and requires the High Availability Manager to successful function.
 * <p>
 * Users must have each ObjectGrid instance configured with this listener (see
 * the programming guide) that they wish to send and receive distributed
 * transactions. This support has worker thread support to ensure all
 * transaction content is processed in the background (asynchronous support).
 * The thread that processes this work is normally started during the initial
 * ObjectGrid instance initialization (ObjectGridManager will cache this for a
 * single JVM. Users are encouraged to use the ObjectGridManager interface to
 * consistently manage a single ObjectGrid instance within a single JVM). Server
 * shutdown will cause the worker thread to exit. If a single JVM has multiple
 * instances of a specific ObjectGrid instance with this listener support
 * configured, each instance will work independently in the same JVM, and cause
 * performance overhead because each will duplicate the same processing.
 * <p>
 * For users who wish to control the worker thread support themselves methods
 * are provided for this option, but this should be done carefully. The
 * anticipated use of this support is to start the server, use the
 * ObjectGridManager to create the initial instance, have all in JVM users
 * lookup the existing instance of the ObjectGrid from the ObjectGridManager
 * and finally at server shutdown the threads handling the distribution
 * transaction support will be cleaned up properly and in an orderly manner
 * without programmer or administrator intervention.
 * <p>
 * Each application using the ObjectGrid in a WebSphere Application Server that
 * is sharing commit information must be in the same HA Manager coregroup.
 * <p>
 * The following example illustrates how this class can be configured for a
 * specific ObjectGrid:
 *
 * <pre>
 *
 *           &lt;bean id=&quot;ObjectGridEventListener&quot; className=&quot;com.ibm.websphere.objectgrid.plugins.builtins.TranPropListener&quot; &gt;
 *
 *               &lt;property name=&quot;propagateService&quot; type=&quot;java.lang.String&quot; value=&quot;all&quot;
 *                    description=&quot;Option all includes propagating and receiving transaction commits.
 *                    (propagate) and (receive) other options.&quot; /&gt;
 *
 *               &lt;property name=&quot;propagateMode&quot; type=&quot;java.lang.String&quot; value=&quot;update&quot;
 *                    description=&quot;Propagate value (update), or just convert all
 *                    updates to invalidates in remote cache (invalidate).&quot; /&gt;
 *
 *               &lt;property name=&quot;propagateVersionOption&quot; type=&quot;java.lang.String&quot; value=&quot;enable&quot;
 *                    description=&quot;Enable the use of versioning (enable) or disable (disable).&quot; /&gt;
 *
 *               &lt;property name=&quot;compressionMode&quot; type=&quot;java.lang.String&quot; value=&quot;disable&quot;
 *                    description=&quot;Enable the use of stream compression for transmission (enable) or disable (disable).&quot; /&gt;
 *
 *            &lt;/bean&gt;
 *
 * </pre>
 *
 * <p>
 * The example above simply can be achieved by setting no properties, as the
 * defaults are illustrated. For settings other than the defaults, see the
 * description text above for each property.
 *
 * @author pdykes
 * @since WAS XD 6.0
 *
 * @see ObjectGridManager
 * @see DistributionMode
 * @deprecated This plug-in has been deprecated in version 7.1.1.  See the {@link JMSObjectGridEventListener} plug-in for equivalent function.
 */
@Deprecated
public class TranPropListener implements ObjectGridEventListener {

    private static final String CLASS_NAME = TranPropListener.class.getName();

    private static final TraceComponent tc = Tr.register(
            CLASS_NAME, Constants.TR_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);

    private TranPropSvcEndPoint ogTransactionCommitServiceEndPoint = null;

    private String objectGridName = null;

    private GroupName groupName = null;

    private HAGroup group = null;

    private final MsgQoS defaultMsgQos = MsgQoS.NORMAL_NOSELF;

    // Configuration properties for this listener

    //  value reflects if invalidate or updates should be used, see
    // DistributionMode
    private String propagateMode = Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_DEFAULT_KEY;

    //  value reflects if version enforcement should be used, see
    // DistributionMode
    private String propagateVersionOption = Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_VERS_KEY;

    //  value determines if listener only propagates, recieves or does both
    private String propagateService = Constants.OBJECTGRID_TRAN_PROPAGATION_SVC_DEFAULT_KEY;

    // compression enabled

    private String compressionMode = Constants.OBJECTGRID_TRAN_PROPAGATION_COMPRESS_DISABLED;

    // dervied attributes from configuration properties

    private boolean compressionEnabled = true;

    private DistributionMode propagateOperations = DistributionMode.PUSH_CONDITIONAL; // default

    private int propagateFunction = Constants.OBJECTGRID_TRAN_PROPAGATION_PROPAGATE_COMMITS_AND_APPLY_COMMITS; // default

    /**
     * Creates a new TranPropListener.
     *
     * @throws UnsupportedOperationException if this listener is added to an
     *         ObjectGrid instance in a non WebSphere environment or using it
     *         in the WebSphere z/OS environment
     */
    public TranPropListener() {
        if (RuntimeInfo.instance().isWASServerProcess() == false || WSLauncher.isZOS()) {
            Tr.error(tc, NLSConstants.TRANPROPLISTENER_UNSUPPORTED_CWOBJ1220);
            throw new UnsupportedOperationException("TranPropListener is not supported in this configuration");
        }
    }

    /**
     * Gets the propagate mode.
     *
     * @return the propagate mode
     *
     * @see #setPropagateMode(String)
     */
    public String getPropagateMode() {
        return propagateMode;
    }

    /**
     * Sets the propagate mode.
     * <p>
     * This method allows a user to set the propagate mode for this listener
     * either programatically or using the XML support. Propagation
     * is enabled to either propagate all changes as updates or converts
     * updates to invalidates (See <code>DistributionMode</code> for more details).
     * <p>
     * Valid values are either "update" or "invalidate", with the default set to
     * "update" if not specified.
     *
     * @param propagateOperation the propagateMode to set.
     *
     * @throws IllegalArgumentException if propagateOperation is <code>null</code>
     *         or is not one of the valid propagate modes
     */
    public void setPropagateMode(String propagateOperation) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "setPropagateMode:" + propagateOperation);
        }

        if (propagateOperation == null) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "setPropagateMode-IllegalArgumentException");
            }

            throw new IllegalArgumentException("parameter cannot be null");
        }
        if ((Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_DEFAULT_KEY
                .equalsIgnoreCase(propagateOperation))
                || (Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_INVALID_KEY
                        .equalsIgnoreCase(propagateOperation))) {

            this.propagateMode = propagateOperation.toLowerCase();
        } else {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "setPropagateMode-IllegalArgumentException");
            }
            throw new IllegalArgumentException("Provided invalid value ["
                    + propagateOperation + "] Valid options:"
                    + Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_DEFAULT_KEY
                    + " (default) or "
                    + Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_INVALID_KEY
                    + ".");
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "propagateMode service mapping:" + propagateMode);
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "setPropagateMode");
        }
    }

    /**
     * Gets the propagate version option.
     *
     * @return the propagate version option.
     *
     * @see #setPropagateVersionOption(String)
     */
    public String getPropagateVersionOption() {
        return propagateVersionOption;
    }

    /**
     * Sets the propagate version option.
     * <p>
     * This method allows a user to set the propagate version option for this
     * listener either programatically or using the XML support
     * Propagation is enabled to either propagate version information or not.
     * (See <code>DistributionMode</code> for more detail).
     * <p>
     * Valid values are either "enable" or "disable", with the default set to
     * "enable" if not specified.
     *
     * @param propagateOption the propagate version option to set.
     *
     * @throws IllegalArgumentException if propagateOption is <code>null</code>
     *         or is not one of the valid propagate version options
     */
    public void setPropagateVersionOption(String propagateOption) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "setPropagateVersionOption:" + propagateOption);
        }

        if (propagateOption == null)
            throw new IllegalArgumentException("parameter cannot be null");

        if ((Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_VERS_KEY
                .equalsIgnoreCase(propagateOption))
                || (Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_NOVERS_KEY
                        .equalsIgnoreCase(propagateOption))) {

            this.propagateVersionOption = propagateOption.toLowerCase();
        } else {
            throw new IllegalArgumentException("Provided invalid value ["
                    + propagateOption + "] Valid options:"
                    + Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_VERS_KEY
                    + " (default) or "
                    + Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_NOVERS_KEY
                    + ".");
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "setPropagateVersionOption service mapping:"
                    + propagateVersionOption);
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "setPropagateVersionOption");
        }
    }

    /**
     * Gets the propagate service
     *
     * @return the propagate service.
     *
     * @see #setPropagateService(String)
     */
    public String getPropagateService() {
        return propagateService;
    }

    /**
     * Sets the propagate service.
     * <p>
     * This method allows a user to set the propagate service option for this
     * listener either programatically or using the XML support. By default,
     * propagation is enabled to propagate to and receive transaction commit
     * information from other WebSphere AppServer cluster members in the same
     * core group and initializing a specific ObjectGrid instance of a certain
     * name and configuring this built in ObjectGrid listener. In some cases,
     * users may want a second cluster or specific application server to only
     * listen to the changes for reading purposes.
     * <p>
     * Valid values are either "all", "propagate" or "receive", with the default
     * set to "all" if not specified.
     *
     * @param propagateService the propagate service to set
     *
     * @throws IllegalArgumentException if propagateService is <code>null</code>
     *         or is not one of the valid propagate service options
     */
    public void setPropagateService(String propagateService) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "setPropagateService:" + propagateService);
        }

        if (propagateService == null)
            throw new IllegalArgumentException("parameter cannot be null");

        // since this value describes all options, we can throw an immediate
        // exception
        // here and set the final value of propagateFunction

        if (Constants.OBJECTGRID_TRAN_PROPAGATION_SVC_DEFAULT_KEY
                .equalsIgnoreCase(propagateService)) {
            propagateFunction = Constants.OBJECTGRID_TRAN_PROPAGATION_PROPAGATE_COMMITS_AND_APPLY_COMMITS;

        } else if (Constants.OBJECTGRID_TRAN_PROPAGATION_SVC_PROPAGATE_KEY
                .equalsIgnoreCase(propagateService)) {
            propagateFunction = Constants.OBJECTGRID_TRAN_PROPAGATION_PROPAGATE_COMMITS;
        } else if (Constants.OBJECTGRID_TRAN_PROPAGATION_SVC_RECIEVE_KEY
                .equalsIgnoreCase(propagateService)) {
            propagateFunction = Constants.OBJECTGRID_TRAN_PROPAGATION_APPLY_COMMITS;
        } else {
            throw new IllegalArgumentException("Provided invalid value ["
                    + propagateService + "] Valid options:"
                    + Constants.OBJECTGRID_TRAN_PROPAGATION_SVC_DEFAULT_KEY
                    + " (default), "
                    + Constants.OBJECTGRID_TRAN_PROPAGATION_SVC_PROPAGATE_KEY
                    + " or "
                    + Constants.OBJECTGRID_TRAN_PROPAGATION_SVC_RECIEVE_KEY
                    + ".");
        }

        this.propagateService = propagateService.toLowerCase();

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "progration service mapping:" + propagateFunction);
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "setPropagateService");
        }
    }

    /**
     * Users do not call this method directly, an ObjectGrid instance's
     * <code>initalize()</code> methods will call this method at the appropriate
     * time.
     *
     * @see ObjectGrid#initialize
     * @see ObjectGridEventListener#initialize(Session)
     */
    public void initialize(Session session) {

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "initialize");
        }

        if (session == null)
            throw new IllegalArgumentException(
            "Session parameter cannot be null.");

        objectGridName = session.getObjectGrid().getName();

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc,
                    "ObjectGrid Transaction Propagation Listener attached to:"
                    + objectGridName);
        }

        // process the distribution mode that should be used

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "Configured values :" + propagateMode
                    + " propagateVersionOption:" + propagateVersionOption);
        }

        // assume these values are good, checked by setter code, or simply the
        // defaults defined above

        // Customer chose update/enable
        if (Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_DEFAULT_KEY
                .equalsIgnoreCase(propagateMode)
                && Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_VERS_KEY
                .equalsIgnoreCase(propagateVersionOption)) {
            propagateOperations = DistributionMode.PUSH_CONDITIONAL;
        }

        // Customer chose update/disable
        if (Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_DEFAULT_KEY
                .equalsIgnoreCase(propagateMode)
                && Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_NOVERS_KEY
                .equalsIgnoreCase(propagateVersionOption)) {
            propagateOperations = DistributionMode.PUSH;
        }

        // Customer chose invalidate/enable
        if (Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_INVALID_KEY
                .equalsIgnoreCase(propagateMode)
                && Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_VERS_KEY
                .equalsIgnoreCase(propagateVersionOption)) {
            propagateOperations = DistributionMode.INVALIDATE_CONDITIONAL;
        }

        // Customer chose invalidate/disable
        if (Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_INVALID_KEY
                .equalsIgnoreCase(propagateMode)
                && Constants.OBJECTGRID_TRAN_PROPAGATION_MODE_NOVERS_KEY
                .equalsIgnoreCase(propagateVersionOption)) {
            propagateOperations = DistributionMode.INVALIDATE;
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "progateOperations:" + propagateOperations);
        }

        Tr.info(tc, NLSConstants.TP_CWOBJ1215, new Object[] { objectGridName });

        // prepare endpoint, must always either send or recieve transaction
        // commit content (otherwise
        // they should not configure this listener)

        ogTransactionCommitServiceEndPoint = new TranPropSvcEndPoint();
        ogTransactionCommitServiceEndPoint.initialize(session.getObjectGrid(),
                propagateFunction, compressionEnabled);

        /*
         * EndPoint is the HA Manager interface implementation for this support.
         *
         */

        /*
         * Acquire the group manager and corestack support. At the current time,
         * the Agent interface will not be used.
         */

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "ObjectGrid Listener attaching to HA Manager");
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "ObjectGrid Listener acquring HA Manager CoreStack");
        }

        /*
         * join ha group group name, set of attributes:
         * Constants.OBJECTGRID_WEBSPHERE_HAMGROUP_NAME objectGridName
         *
         * Note: there is a benefit to keeping these strings small, but unique
         * as ha manager infrastructure transmits as required.
         */

        Map groupProps = new HashMap();

        // TODO: make these contstants as well...

        // infrastructure for objectgrid ha gruops = objg-gt
        groupProps.put(Constants.OBJECTGRID_WEBSPHERE_HAMGROUP_NAME_KEY,
                Constants.OBJECTGRID_WEBSPHERE_HAMGROUP_NAME_VALUE);
        // datagrid name attribute = objg-dg
        groupProps.put(
                Constants.OBJECTGRID_WEBSPHERE_HAMGROUP_OBJECTGRID_NAME_KEY,
                objectGridName);
        // Attached the default NOOP policy attributes, this is the default in
        // any event
        // but making it explicit is always good. This avoid any need for
        // external configuration
        // of the policy.
        groupProps.put(NOOPPolicy.defaultPolicyKey,
                NOOPPolicy.defaultPolicyValue);

        CoreStack coreStack = (CoreStack) DependencyProvider.getServiceFromProvider(CoreStack.class);
        GroupManager gm = null;
        if (coreStack != null) {
            gm = coreStack.getGroupManager();
        } else {
            Tr.error(tc, NLSConstants.STANDLAONE_HAMANAGER_NOT_STARTED_CWOBJ1712);
        }

        try {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr
                .debug(
                        tc,
                        "ObjectGrid Listener joining HA Manager Group, properties:",
                        groupProps);
            }

            groupName = gm.createGroupName(groupProps);
        } catch (Throwable t) {
            FFDCFilter.processException(t, CLASS_NAME + ".initialize", "152",
                    this);

            // Since any Throwable is handled exactly the same, we might as well
            // catch all Throwables and make code more robust.
            Tr.error(tc, NLSConstants.TP_CWOBJ1218, new Object[] { objectGridName, t });
            throw new ObjectGridRuntimeException(
                    "exception while creating group name", t);
        }

        /*
         * These properties are currently not required, so stamp with a version
         * id, and provide initialzation api what is required.
         */

        Map memberProperties = new HashMap();
        memberProperties.put(Constants.OBJECTGRID_WEBSPHERE_HAMGROUP_NAME_KEY,
                objectGridName);

        try {

            // TODO: Perry bug, if SEP not created, this will throw exception...
            group = gm.joinGroup(groupName, memberProperties,
                    ogTransactionCommitServiceEndPoint);
        } catch (Throwable t) {
            FFDCFilter.processException(t, CLASS_NAME + ".initialize", "172",
                    this);

            // Since any Throwable is handled exactly the same, we might as well
            // catch all Throwables and make code more robust.

            Tr.error(tc, NLSConstants.TP_CWOBJ1218, new Object[] { objectGridName, t });
            throw new ObjectGridRuntimeException(
                    "exception while joining HA Group", t);
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "initialize");
        }

        Tr.info(tc, NLSConstants.TP_CWOBJ1216, new Object[] { objectGridName });
    }

    /**
     * @see ObjectGridEventListener#transactionEnd(String, boolean, boolean, Collection)
     */
    public void transactionEnd(String txid, boolean isWriteThroughEnabled,
            boolean committed, Collection changes) {

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "transactionEnd - transaction progagation commit event received:" + txid
                    + ": wt:" + isWriteThroughEnabled + " commited:"
                    + committed);
        }

        // if the transaction is not committed, it should not be propagated
        if (!committed) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr
                .exit(tc, "transactionEnd - transaction not commited returning, changes not propagated");
            }
            return;
        }

        // if the transaction has write through disabled, the contract
        // is this commit should not be replicated

        if (!isWriteThroughEnabled) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr
                .exit(
                        tc,
                "transactionEnd - transaction has not specified write through, returning, changes not propagated");
            }
            return;
        }

        // if only a reciever, skip request
        if (propagateFunction == Constants.OBJECTGRID_TRAN_PROPAGATION_APPLY_COMMITS) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled())
                Tr
                .exit(
                        tc,
                        "transactionEnd -transaction commit ["
                        + txid
                        + "] is not propagated, this listener only applies changes from other remote requests.");
            return;
        }

        // if the logsequene is empty, no reason to propagate
        if (changes.size() == 0) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "transactionEnd - transaction has no changes to propagate");
            }
            return;
        }
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "transaction has " + changes.size()
                    + " non-optimized change(s) to propagate");
        }

        // walk the incoming logsequence set of changes, ensure only transmit
        // logsequences that resulted in updates of some form

        Iterator iter = changes.iterator();

        boolean remoteTransmissionRequired = false;

        Collection dirtyLogSequences = new ArrayList();

        try {

            while (iter.hasNext()) {

                LogSequence logSeq = (LogSequence) iter.next();

                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                    Tr.debug(tc, "Processing Log Sequence for Map:"
                            + logSeq.getMapName());
                }

                // do not include read only transactions (ever)
                if (logSeq.isDirty()) {

                    dirtyLogSequences.add(logSeq);

                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "Map " + logSeq.getMapName()
                                + " has changes, will propagate");
                    }

                    remoteTransmissionRequired = true;
                }
            }

            // check to see if we have any log sequences that really need to
            // be propgated (must consist of more than gets for example)

            // add check, return if tx only results in gets

            if (!remoteTransmissionRequired) {
                if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                    Tr
                    .exit(tc, "transactionEnd",
                    "Propagation skipped - no log sequences required transmisssion");
                }
                return;
            }

            // use collection created above of dirtied maps

            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
            ObjectOutputStream os = new ObjectOutputStream(byteStream);
            LogSequenceFilter filter = new TranCommitLogSeqFilter();

            // Create the byte array to propagate to other severs
            LogSequenceTransformer.serialize(dirtyLogSequences, os, filter,
                    propagateOperations);

            // store byte array for this logsequence set in message
            // for distribution

            os.close();

            // stream all logsequence content to a bytearray for remote
            // transmission
            byte[] outGoingByteArray = byteStream.toByteArray();

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc,
                        "Processed Log Sequence into stream byte array size:"
                        + outGoingByteArray.length);
            }

            // create out going message and set critical values...
            TranPropReq tranPropagateRequestMessage = new TranPropReq();

            tranPropagateRequestMessage.setObjectGridName(objectGridName);
            tranPropagateRequestMessage.setTxID(txid);
            tranPropagateRequestMessage.setLogSequenceCount(dirtyLogSequences
                    .size());
            tranPropagateRequestMessage
            .setTranLogSequenceGroupByteArray(outGoingByteArray);

            // propagation content now complete, prepare to send across to other
            // cluster members....

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr
                .debug(tc,
                "Preparing stream for full message propagation to DCS/HA Manager");
            }

            ByteArrayOutputStream msgByteStream = new ByteArrayOutputStream();

            // convert object to byte array
            ObjectOutputStream msgOutputStream = new ObjectOutputStream(
                    msgByteStream);
            msgOutputStream.writeObject(tranPropagateRequestMessage);
            msgOutputStream.flush(); // ensure all bytes to output stream
            msgOutputStream.close();

            byte[] sendBuf = msgByteStream.toByteArray();

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {

                CRC32 chkBytes = new CRC32();
                chkBytes.update(sendBuf);

                Tr.debug(tc,
                        "commit propagation preparing to send message, details for tx:"
                        + txid + " ByteCount:" + sendBuf.length
                        + " CRC32:"
                        + Long.toString(chkBytes.getValue()));
            }

            // compress message content if enabled
            byte[] compressedSendBuf = null;
            if (compressionEnabled) {
                compressedSendBuf = Compressor.compress(sendBuf, 0);

                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {

                    CRC32 chkBytes = new CRC32();
                    chkBytes.update(compressedSendBuf);

                    Tr.debug(tc,
                            "commit propagation preparing to send compressed message, details for tx:"
                            + txid + " ByteCount:"
                            + compressedSendBuf.length + " CRC32:"
                            + Long.toString(chkBytes.getValue()));

                }

                group.sendMessage(defaultMsgQos, compressedSendBuf);
            } else {
                group.sendMessage(defaultMsgQos, sendBuf);
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr
                .debug(
                        tc,
                        "transaction propagation successfully released propagation request to coregroup peers for tx:"
                        + txid);
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "transactionEnd - transaction commit progration processing complete for tx:"
                        + txid);
            }

        } catch (Throwable t) {
            FFDCFilter.processException(t, CLASS_NAME + ".transactionEnd",
                    "400", this);

            // Since any Throwable is handled exactly the same, we might as well
            // catch all Throwables and make code more robust.
            Tr.error(tc, NLSConstants.TP_CWOBJ1218, new Object[] { objectGridName, t });
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "transactionEnd", t);
            }
            throw new ObjectGridRuntimeException(
                    "Exception encountered while preparing to propagate commit for Txid:"
                    + txid, t);
        }
    }

    /**
     * @see ObjectGridEventListener#transactionBegin(String, boolean)
     */
    public void transactionBegin(String txid, boolean isWriteThroughEnabled) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc,
                    "transaction propagate objectgrid listener notified of begin for:"
                    + txid);
        }
    }

    /**
     * Stops the transaction receiver background thread.
     * <p>
     * This method is used to control the background thread processing
     * transactions received from other ObjectGrid instances with same "name"
     * and configured with the builtin distributed transaction propagation
     * support. This method is generally not a method most developers will
     * require.
     * <p>
     * Generally, the work thread lifecycle for each ObjectGrid instance is managed
     * automatically at ObjectGrid initialization and server instance shutdown.
     * <p>
     * This method stops the distributed listener thread which handles all
     * transaction commits propagated to this JVM's ObjectGrid instances. Users
     * should be advised, stopping this thread is something to be done only for
     * a short amount of time (the server shutdown will handle server stop
     * scenarios). If the queueTransactionCommitRequests parameter is
     * <code>false</code>, the safest approach, transaction commits on the
     * other ObjectGrid instance of the same name will be received by the
     * listener, but the request will be thrown away. For optimistic updates
     * this maybe acceptable, but one users should consider. If the
     * queueTransactionCommitRequests parameter is set to <code>true</code>,
     * the programmer should ensure the worker thread is restarted quickly, as
     * a busy system may queue many commits very quickly, and each one takes
     * memory in the server. Without the work thread processing these commits,
     * the queue will grow unbounded.
     *
     * @param queueTransactionCommitRequests
     *      whether remote requests should be queued up while the receiving
     *      commit worker thread is disabled.
     */
    public void stopTransactionReceiver(boolean queueTransactionCommitRequests) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "stopTransactionReceiver" +  queueTransactionCommitRequests);
        }
        if ((propagateFunction != Constants.OBJECTGRID_TRAN_PROPAGATION_PROPAGATE_COMMITS)
                && (ogTransactionCommitServiceEndPoint != null)) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, "stopTransactionReceiver requested stop.");
            }
            ogTransactionCommitServiceEndPoint.stopWorkerThread(queueTransactionCommitRequests);
        }
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "stopTransactionReceiver");
        }
    }

    /**
     * Stops the transaction receiver background thread.
     * <p>
     * This method is used to control the background thread processing
     * transactions received from other ObjectGrid instances with same "name"
     * and configured with the builtin distributed transaction propagation
     * support. This method is generally not a method most developers will require.
     * <p>
     * Generally, the work thread lifecycle for each ObjectGrid instance is managed
     * automatically at ObjectGrid initialization and server instance shutdown.
     */
    public void startTransactionReceiver() {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "startTransactionReceiver");
        }
        if ((propagateFunction != Constants.OBJECTGRID_TRAN_PROPAGATION_PROPAGATE_COMMITS)
                && (ogTransactionCommitServiceEndPoint != null)) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, "startTransactionReceiver requested start.");
            }
            ogTransactionCommitServiceEndPoint.startWorkerThread();
        }
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "startTransactionReceiver");
        }
    }

    /**
     * Gets the compression mode.
     *
     * @return the compression mode.
     *
     * @see #setCompressionMode(String)
     */
    public String getCompressionMode() {

        return compressionMode;
    }

    /**
     * Sets the compression mode.
     * <p>
     * This method allows a user to set the compression mode for this listener
     * either programatically or using the XML support. By default, compression
     * is enabled as it reduces the overall network traffic within the cluster,
     * at a cost in the local JVM to compress/uncompress. The acceptable values
     * are either "enable" or "disable", with "disable" being the default if not
     * specified.
     *
     * @param compressionEnabledOption the compression mode to set.
     *
     * @throws IllegalArgumentException if compressionEnabledOption is
     *         <code>null</code> or is not one of the valid compression modes
     */
    public void setCompressionMode(String compressionEnabledOption) {

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "setCompressionMode:"
                    + compressionEnabledOption);
        }

        if (compressionEnabledOption == null) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc,
                "setCompressionMode-IllegalArgumentException");
            }

            throw new IllegalArgumentException("parameter cannot be null");
        }
        if ((Constants.OBJECTGRID_TRAN_PROPAGATION_COMPRESS_DISABLED
                .equalsIgnoreCase(compressionEnabledOption))
                || (Constants.OBJECTGRID_TRAN_PROPAGATION_COMPRESS_ENABLED
                        .equalsIgnoreCase(compressionEnabledOption))) {

            this.compressionMode = compressionEnabledOption
            .toLowerCase();

        } else {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc,
                "setCompressionMode-IllegalArgumentException");
            }
            throw new IllegalArgumentException("Provided invalid value ["
                    + compressionEnabledOption + "] Valid options:"
                    + Constants.OBJECTGRID_TRAN_PROPAGATION_COMPRESS_DISABLED
                    + " (default) or "
                    + Constants.OBJECTGRID_TRAN_PROPAGATION_COMPRESS_ENABLED
                    + ".");
        }

        // have a proper value, set the actual flag...

        if (Constants.OBJECTGRID_TRAN_PROPAGATION_COMPRESS_DISABLED
                .equalsIgnoreCase(compressionEnabledOption)) {
            compressionEnabled = false;
        } else if (Constants.OBJECTGRID_TRAN_PROPAGATION_COMPRESS_ENABLED
                .equalsIgnoreCase(compressionEnabledOption)) {
            compressionEnabled = true;
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            Tr.debug(tc, "compressionMode service mapping:"
                    + compressionEnabledOption);
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "setCompressionMode");
        }
    }

    /**
     * Called when the ObjectGrid instance itself is being destroyed.
     * <p>
     * At this point, the reciever cannot be restarted as the ObjectGrid
     * instance is going away permanently.
     *
     * @see ObjectGrid#destroy()
     * @see ObjectGridEventListener#destroy()
     */
    public void destroy() {
        stopTransactionReceiver(false); // going down, and will not be coming
        // back so do not queue requests
    }
}