// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Source File Description:
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// t9225		8.6       05/18/2012   cheng1    
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins.index;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

/**
 * This interface is an abstract index that can be created on a Map. The index
 * can be used to perform efficient look ups on attributes in a Map.
 * 
 * @param <KeyType> The map key type.
 * @param <ValueType> The map value type.
 * @param <AttributeType> The indexed attribute type.
 * 
 * @since WXS 8.6
 * 
 */
public interface MapGlobalIndex<KeyType,ValueType,AttributeType>  {

	/**
	 * Returns the partitions contain the entries that have the specified
	 * attribute values.
	 * <p>
	 * 
	 * @param attributeValues
	 *            a reference to the attribute values to search for.
	 * @return the partitions contain the entries that have the specified
	 *         attribute values.
	 * @throws GlobalIndexException
	 */
	public Collection<Integer> findPartitions(AttributeType... attributes);

	/**
	 * Returns a map of partitions contain the entries for specified attribute
	 * values.
	 * <p>
	 * 
	 * @param attributeValues
	 *            a reference to the attribute values to search for.
	 * @return A map of partitions contain the entries for the specified
	 *         attribute values.
	 * @throws GlobalIndexException
	 */	
	public Map<AttributeType, Collection<Integer>> findPartitionsByAttribute(AttributeType... attributes);

	/**
	 * Returns the keys for the entries that have the specified attribute
	 * values.
	 * <p>
	 * 
	 * @param attributeValues
	 *            a reference to the attribute values to search for.
	 * @return Returns the keys for the entries that have the specified
	 *         attribute values.
	 * @throws GlobalIndexException
	 */
	public Set<KeyType> findKeys(AttributeType... attributes);
	
	/**
	 * Returns a map of the keys of the entries for the specified attribute
	 * values.
	 * <p>
	 * 
	 * @param attributeValues
	 *            a reference to the attribute values to search for.
	 * @return a map of the keys of the entries for the specified attribute
	 * @throws GlobalIndexException
	 */	
	public Map<AttributeType, Set<KeyType>> findKeysByAttribute(AttributeType... attributes);

	/**
	 * Returns the entries that have the specified attribute values.
	 * <p>
	 * 
	 * @param attributes
	 *            the attribute values to search for.
	 * @return the entries that have the specified attribute values.
	 * @throws GlobalIndexException
	 */
	public Set<ValueType> findValues(AttributeType... attributes);

	/**
	 * Returns a map of the entries for the specified attribute values.
	 * <p>
	 * 
	 * @param attributes
	 *            the attribute values to search for.
	 * @return a map of the entries for the specified attribute values.
	 * @throws GlobalIndexException
	 */
	public Map<AttributeType, Set<ValueType>> findValuesByAttribute(AttributeType... attributes);

	/**
	 * Returns the keys and entries that have the specified attribute values.
	 * <p>
	 * 
	 * @param attributes
	 *            the attribute values to search for.
	 * @return the keys and entries that have the specified attribute values.
	 * @throws GlobalIndexException
	 */
	public Map<KeyType,ValueType> findEntries(AttributeType... attributes);

	/**
	 * Returns a map of the keys and entries for the specified attribute values.
	 * <p>
	 * 
	 * @param attributes
	 *            the attribute values to search for.
	 * @return the keys and entries that have the specified attribute values.
	 * @throws GlobalIndexException
	 */
	public Map<AttributeType, Map<KeyType,ValueType>> findEntriesByAttribute(AttributeType... attributes);
	
	/**
	 * Returns the number of entries that have the specified attribute values.
	 * <p>
	 * 
	 * @param attributeValues
	 *            a reference to the attribute values to search for.
	 * @return the number of entries that have the specified attribute values.
	 * @throws GlobalIndexException
	 */
	public int findSize (AttributeType... attributes);
	
	/**
	 * Returns a map of the number of entries for specified attribute values.
	 * <p>
	 * 
	 * @param attributeValues
	 *            a reference to the attribute values to search for.
	 * @return a map of the number of entries for specified attribute values.
	 * @throws GlobalIndexException
	 */
	public Map<AttributeType, Integer> findSizesByAttribute(AttributeType... attributes);
}
