// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  MapIndexInfo.java
//
// Source File Description: 
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M2_B.2  WASX.XD   03/04/2005   newport   new code drop
// XD60_M3_B.10 WASX.XD   04/07/2005   sutter    Re-factor datagrid/datagrid.impl
// 271122       WASX.XD   04/30/2005   kjlaw     cleanup javadoc and overload getTransactionChanges.
// 285430       WASX.XD   06/21/2005   pdykes    Pre-GM changes to remove non-supported public interfaces
// b1944        XD6.0.2   07/26/2006   jhanders  Add getBackingMap method
// b993         XD 6.1    08/09/2006   cheng1    getTransactionLogSequence method
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.index;

import java.util.List;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;
import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.OutputFormat;
import com.ibm.websphere.objectgrid.plugins.LogElement;
import com.ibm.websphere.objectgrid.plugins.LogSequence;

/**
 * This interface is used to provide an index with detailed change information
 * for a specific Map in a transaction.
 *
 * @since WAS XD 6.0.1
 * 
 * @see MapIndexPlugin#getIndexProxy(MapIndexInfo)
 */
public interface MapIndexInfo 
{
    /**
     * Returns the <code>ObjectMap</code> the index is associated with.
     * 
     * @return the ObjectMap this index is associated with.
     * 
     * @see ObjectMap
     */
    ObjectMap getMap();

    /**
     * Returns the set of all changes made by the current transaction to the
     * ObjectMap that is returned by the <code>getMap()</code> method.<p>  
     * 
     * This method provides the same results as {@link #getTransactionChanges(boolean, OutputFormat, OutputFormat)}
     * using {@link OutputFormat#UNDEFINED}.
     * 
     * @param includeRemoved must be set to <code>true</code> to include 
     *        <code>LogElement.DELETE</code> types in the list returned by this method.
     * 
     * @return a <code>List</code> of <code>LogElement</code>s created for each 
     *         ObjectMap entry that was either inserted, updated, or removed by
     *         the current transaction.  
     * 
     * @throws ObjectGridRuntimeException if an error occurs during processing
     * 
     * @see #getMap()
     * @see LogElement#DELETE
     */
    List /*<LogElement>*/ getTransactionChanges(boolean includeRemoved) throws ObjectGridRuntimeException;

    /**
     * Returns the set of all changes made by the current transaction to the
     * ObjectMap that is returned by the <code>getMap()</code> method.<p>
     * 
     * The OutputFormat specifies the data format to be returned by each LogElement.
     * 
     * @param includeRemoved must be set to <code>true</code> to include 
     *        <code>LogElement.DELETE</code> types in the list returned by this method.
     * 
     * @param keyOutputFormat identifies the data format to be returned each of the LogElement keys.
     * @param valueOutputFormat identifies the data format to be returned each of the LogElement values.
     * 
     * @return a <code>List</code> of <code>LogElement</code>s created for each 
     *         ObjectMap entry that was either inserted, updated, or removed by
     *         the current transaction.  
     * 
     * @throws ObjectGridRuntimeException if an error occurs during processing
     * 
     * @see #getMap()
     * @see LogElement#DELETE
     * @since 8.6
     */
    List<LogElement> getTransactionChanges(boolean includeRemoved, OutputFormat keyOutputFormat, OutputFormat valueOutputFormat);

    /**
     * Returns the set of changes as they apply to a particular set of keys
     * in the current transaction for the ObjectMap that is returned by the 
     * <code>getMap()</code> method.  If a key has not been referenced
     * in the transaction then <code>null</code> is returned.<p>
     * 
     * This method provides the same results as {@link #getTransactionChanges(List, OutputFormat, OutputFormat)}
     * using {@link OutputFormat#UNDEFINED}.
     * 
     * @param keys The list of keys for which the data is required.
     * 
     * @return A <code>List</code> of <code>LogElement</code>s corresponding to 
     *         the keys specified.  A <code>null</code> entry in the list
     *         indicates the key was not referenced.
     * 
     * @throws ObjectGridRuntimeException if an error occurs during processing
     * 
     * @see #getMap()
     * @see LogElement
     * @see ObjectMap
     */
    List/*<LogElement>*/ getTransactionChanges(List/*<Object>*/ keys) throws ObjectGridRuntimeException;

    /**
     * Returns the set of changes as they apply to a particular set of keys
     * in the current transaction for the ObjectMap that is returned by the 
     * <code>getMap()</code> method.  If a key has not been referenced
     * in the transaction then <code>null</code> is returned.<p>
     * 
     * The OutputFormat specifies the data format to be returned by each LogElement.
     * 
     * @param keys The list of keys for which the data is required.
     * @param keyOutputFormat identifies the data format to be returned each of the LogElement keys.
     * @param valueOutputFormat identifies the data format to be returned each of the LogElement values.
     * 
     * @return A <code>List</code> of <code>LogElement</code>s corresponding to 
     *         the keys specified.  A <code>null</code> entry in the list
     *         indicates the key was not referenced.
     * 
     * @throws ObjectGridRuntimeException if an error occurs during processing
     * 
     * @see #getMap()
     * @see LogElement
     * @see ObjectMap
     */
    List<LogElement> getTransactionChanges(List/*<Object>*/ keys, OutputFormat keyOutputFormat, OutputFormat valueOutputFormat);
    
    /**
     * Returns the <code>BackingMap</code> the index is associated with.
     * 
     * @return the BackingMap this index is associated with.
     * 
     * @see BackingMap
     * @since WAS XD 6.1
     */
    BackingMap getBackingMap();
    
    /**
     * Returns the set of all changes made by the current transaction to the
     * ObjectMap that is returned by the <code>getMap()</code> method.  
     * 
     * @throws ObjectGridRuntimeException if an error occurs during processing
     * 
     * @see #getMap()
     * @see LogElement#DELETE
     * @since WAS XD 6.1
     */
    LogSequence /*<LogElement>*/ getTransactionLogSequence() throws ObjectGridRuntimeException;

    /**
     * Returns the set of all changes made by the current transaction to the
     * ObjectMap that is returned by the <code>getMap()</code> method. <p>
     * 
     * The OutputFormat specifies the data format to be returned by each LogElement. 
     * 
     * @param keyOutputFormat identifies the data format to be returned each of the LogElement keys.
     * @param valueOutputFormat identifies the data format to be returned each of the LogElement values.
     * 
     * @throws ObjectGridRuntimeException if an error occurs during processing
     * 
     * @see #getMap()
     * @see LogElement#DELETE
     * @since WAS XD 6.1
     */
    LogSequence getTransactionLogSequence(OutputFormat keyOutputFormat, OutputFormat valueOutputFormat);

}
