// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  SerializerAccessor.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io;

import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.plugins.BackingMapLifecycleListener;
import com.ibm.websphere.objectgrid.plugins.EventListener;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.DataObjectContext;

/**
 * Each BackingMap has a SerializerAccessor associated with it, which includes
 * references to custom DataSerializers that eXtreme Scale uses to serialize, inflate
 * and query data.<p>
 *
 * The MapSerializerPlugin BackingMap plug-in is the BackingMap's main interface which provides
 * metadata and a DataSerializer for both the map's key and value.<p>
 *
 * The DataObjectContext associated with a SerializerAccessor provides methods for
 * creating and converting key and value data objects.<p>
 *
 * A MapSerializerPlugin can be registered programmatically using the <code>registerMapSerializer</code>
 * when using a local, in-memory ObjectGrid.  For a distributed ObjectGrid, a MapSerializer
 * is configured using the ObjectGrid descriptor XML file as a BackingMap plug-in.<p>
 *
 * Example MapSerializerPlugin configuration:
 * <pre>
 * &lt;backingMapPluginCollections&gt;
 *   &lt;backingMapPluginCollection id="plugin_Order"&gt;
 *     &lt;bean
 *        id="MapSerializerPlugin"
 *        className="com.ibm.websphere.objectgrid.plugins.io.BasicMapSerializer"&gt;
 *
 *       &lt;property
 *          name="keyDataSerializerClass" type="java.lang.String"
 *          value="com.ibm.websphere.samples.xs.serializer.proto.ProtoKeySerializer" /&gt;
 *       &lt;property
 *          name="keyProperties" type="java.lang.String"
 *          value="type=com.ibm.websphere.samples.xs.serializer.proto.DataObjects$OrderKey" /&gt;
 *
 *       &lt;property
 *          name="valueDataSerializerClass" type="java.lang.String"
 *          value="com.ibm.websphere.samples.xs.serializer.proto.ProtoValueSerializer" /&gt;
 *       &lt;property
 *          name="valueProperties" type="java.lang.String"
 *          value="type=com.ibm.websphere.samples.xs.serializer.proto.DataObjects$Order" /&gt;
 *     &lt;/bean&gt;
 * ...
 * </pre>
 *
 * @since 7.1.1
 */
public interface SerializerAccessor {

    /**
     * Register a new MapSerializerPlugin for the cache entry key for a local, in-memory
     * ObjectGrid instance.<p>
     *
     * A MapSerializerPlugin can only be registered before the BackingMap has been initialized.
     *
     * A MapSerializerPlugin that implements the {@link BackingMapLifecycleListener} is automatically added
     * as if the {@link BackingMap#addMapEventListener(EventListener)} method was called. Any previous
     * MapSerializer which implements {@link BackingMapLifecycleListener} is removed as if
     * the {@link BackingMap#removeMapEventListener(EventListener)} method was called.
     *
     * @param mapSerializerPlugin the MapSerializerPlugin plug-in.
     */
    public void registerMapSerializerPlugin(MapSerializerPlugin mapSerializerPlugin);

    /**
     * Remove the MapSerializerPlugin instance.
     *
     * If the MapSerializerPlugin implements {@link BackingMapLifecycleListener}, it is removed as if
     * the {@link BackingMap#removeMapEventListener(EventListener)} method was called.
     * @return the MapSerializerPlugin that was removed.
     */
    public MapSerializerPlugin unregisterMapSerializerPlugin();

    /**
     * Retrieve the MapSerializerPlugin instance.
     *
     * @return the MapSerializerPlugin if present.
     */
    public MapSerializerPlugin getMapSerializerPlugin();

    /**
     * Retrieve the BackingMap associated with this SerializerAccessor.
     * @return the BackingMap instance.
     */
    public BackingMap getBackingMap();

    /**
     * Retrieve the default DataObjectContext associated with this SerializerAccessor
     * @return the default DataObjectContext, or null if there is no MapSerializer defined.
     */
    public DataObjectContext getDefaultContext();

}
