// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  DataDescriptor.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io.datadescriptor;

import java.util.Map;

import com.ibm.websphere.objectgrid.plugins.Initializable;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer.DataAttributeInflatable;
import com.ibm.websphere.objectgrid.plugins.io.KeyDataSerializer;
import com.ibm.websphere.objectgrid.plugins.io.ValueDataSerializer;

/**
 * Identifies accessible attributes that the {@link DataAttributeInflatable} can
 * retrieve.  {@link KeyDataSerializer} implementations expose a {@link KeyDataDescriptor} and
 * {@link ValueDataSerializer} implementations expose a {@link ValueDataDescriptor}.
 * @since 7.1.1
 */
public interface DataDescriptor extends Initializable {

    /**
     * The default path separator to use when not overridden with the
     * {@link #setPathSeparator(String)} method.
     */
    public String DEFAULT_PATHSEPARATOR = ".";

    /**
     * Returns the string to use to use for attribute navigation for embedded attributes.er.f<p>
     * @return the path separator string.  The default is {@link #DEFAULT_PATHSEPARATOR}
     */
    public String getPathSeparator();

    /**
     * The string to use to use for attribute navigation for embedded attributes.  The string must not include
     * a valid attribute name that the DataDescriptor exposes, nor the underscore "_" character.
     * @param pathSeparator the path separator string.
     * @throws IllegalStateException if the descriptor has been initialized.
     */
    public void setPathSeparator(String pathSeparator);

    /**
     * Retrieve all Attributes defined for this DataDescriptor.
     * @return a Map of attribute names to the Attribute definition.
     * The attribute map will be read-only if the descriptor has been initialized.
     */
    public Map<String, Attribute> getAttributes();

    /**
     * Set the Map of attribute names to the Attribute definition.  An attribute defines
     * what elements are exposed to the {@link DataAttributeInflatable} interface.  Elements can be nested
     * using the {@link AttributeType#EMBEDDED} attributes.
     * @param attributes the Map of attribute names to the Attribute definition.
     * @throws IllegalStateException if the descriptor has been initialized.
     */
    public void setAttributes(Map<String, Attribute> attributes);

}
