// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  DataObjectContext.java
//
// Source File Description: Reference JavaDoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b22183       7.1.1      2/22/2011   cdjohnson Creation, new code.
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.io.dataobject;

import com.ibm.websphere.objectgrid.io.XsDataStreamManager;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer;
import com.ibm.websphere.objectgrid.plugins.io.SerializerAccessor;

/**
 * Context for for interacting with a data object.  A DataObjectContext can
 * be retrieved from a SerializerAccessor using the
 * {@link SerializerAccessor#getDefaultContext()} method.
 * It is also passed into each {@link DataSerializer} method.
 * @since 7.1.1
 */
public interface DataObjectContext {

    /**
     * Retrieve the SerializerAccessor associated with this DataObjectContext
     * @return the SerializerAccessor instance
     */
    public SerializerAccessor getSerializerAccessor();

    /**
     * Return the data stream manager associated with this DataObjectContext
     * @return the XsDataStreamManager instance
     */
    public XsDataStreamManager getDataStreamManager();
    /**
     * Returns a KeyFactory used to create Key objects.
     * @return a KeyFactory
     */
    public DataObjectKeyFactory getKeyFactory();

    /**
     * Returns a ValueFactory used to create Value objects.
     * @return a ValueFactory
     */
    public DataObjectValueFactory getValueFactory();
}