package com.ibm.websphere.objectgrid.plugins.io.dataobject;

import com.ibm.websphere.objectgrid.plugins.io.SerializerAccessor;

/**
 * This interface is implemented by DataObjectEntry implementations or sub-interfaces
 * to identify that they are associated with a {@link DataObjectContext}.
 * @since 7.1.1
 * @see SerializerAccessor#getDefaultContext()
 */
public interface DataObjectContextAware {
    /**
     * Retrieves the current DataObjectContext associated with this object, or null if not associated
     * with a context.
     * @return the DataObjectContext or null.
     */
    public DataObjectContext getContext();

    /**
     * Apply a non-null DataObjectContext to this object.
     * @param ctx the DataObjectContext to associate with the object.
     * @throws IllegalArgumentException if the context is incompatible or is already set.
     */
    public void applyContext(DataObjectContext ctx);
}
