// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2010
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Source File Description: See javadoc
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- ------------------------------------------
// bug 18270     7.1       03/22/2 010  jpcammar  Creation.
// ------------ --------- ------------ --------- ------------------------------------------

package com.ibm.websphere.objectgrid.rest;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.ObjectGridManagerFactory;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;
import com.ibm.ws.xs.rest.RestConstants;
import com.ibm.ws.xs.rest.resources.NLSConstants;

/**
 * The WebSphere eXtreme Scale REST Service MBean implementation.
 * 
 * @see RestServiceMBean
 * 
 * @author jpcammar
 * 
 */
public class RestService implements RestServiceMBean {

    /**
     * RAS trace handle.
     */
    private static final TraceComponent tc = Tr.register(RestService.class.getName(), RestConstants.TR_GROUP_NAME,
            RestConstants.TR_RESOURCE_BUNDLE_NAME);

    /**
     * Constants.
     */
    private static final String TRUE = "true";
    private static final String FALSE = "false";

    /**
     * Local ObjectGrid manager reference.
     */
    private ObjectGridManagerImpl objectGridManager;

    /**
     * Constructor.
     */
    public RestService() {
        this.objectGridManager = (ObjectGridManagerImpl) ObjectGridManagerFactory.getObjectGridManager();
    }

    /**
     * @see RestServiceMBean#setTraceEnabled(String)
     */

    public void setTraceEnabled(String enableTrace) {

        // Validate arguments first.
        if (enableTrace == null || enableTrace.length() == 0) {
            if (tc.isWarningEnabled())
                Tr.warning(tc, NLSConstants.RESTSERVICE_MBEAN_INCORRECT_OPERATION_PARAM_CWOBJ4027W, new Object[] {
                        "setTraceEnabled", enableTrace });
            throw new IllegalArgumentException("Argument must have a value of true or false.");
        }

        // if args are good, set it.
        if (enableTrace.equalsIgnoreCase(TRUE)) {
            objectGridManager.setTraceEnabled(true);

        } else if (enableTrace.equalsIgnoreCase(FALSE)) {
            objectGridManager.setTraceEnabled(false);

        }

        if (tc.isInfoEnabled())
            Tr.info(tc, NLSConstants.RESTSERVICE_MBEAN_TRACE_DYNAMIC_CWOBJ4028I, enableTrace);

    }

    /**
     * @see RestServiceMBean#setTraceSpec(String)
     */
    public void setTraceSpec(String traceString) {

        // Validate arguments.
        if (traceString == null || traceString.length() == 0) {
            if (tc.isWarningEnabled())
                Tr.warning(tc, NLSConstants.RESTSERVICE_MBEAN_INCORRECT_TRACESPEC_CWOBJ4029W, new Object[] {
                        "setTraceSpec", traceString });
            throw new IllegalArgumentException("argument must not be null or zero length.");
        }

        // If args are good, set it.
        objectGridManager.setTraceSpecification(traceString);
        
        // No Tr.info needed here because WAS RAS does the Tr.info message.

    }
}
