/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2010
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * Module  :  CollisionData.java
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- -----------------------------------------
 * ____________ _________ ____________ _________ _________________________________________
 */
package com.ibm.websphere.objectgrid.revision;

import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.OutputFormat;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedKey;

/**
 * Holds data necessary for arbitrating collisions between entries.
 *
 * @author dcberg
 * @since 7.1
 */
public interface CollisionData {
    /**
     * Retrieves the session that this revision transaction is operating under.
     *
     * @return The active revision session.
     */
    public Session getSession();

    /**
     * Retrieves the map that changes for this revision transaction will be committed under.
     *
     * @return The active revision map.
     */
    public ObjectMap getMap();

    /**
     * Retrieves the key to the cache entry that has the collision.
     *
     * For a cache entry in an {@link ObjectMap} that is configured to use {@link OutputFormat#RAW} for the keys,
     * the key object returned will be a {@link SerializedKey} object. If required, you
     * can use the {@link SerializedKey#getObject()} method to inflate the bytes to the key object.
     *
     * @return The application key or {@link SerializedKey} object as described.
     */
    public Object getKey();

    /**
     * Retrieves the revision element wrapping the value that is currently in the cache.
     *
     * @return The entry that is currently in the cache.
     */
    public RevisionElement getExisting();

    /**
     * Retrieves the revision element wrapping the value that has collided with the entry in the cache.
     *
     * @return The entry that is pending arbitration.
     */
    public RevisionElement getCollision();
}
