// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.5 XD/ws/code/objectgrid.client/src/com/ibm/websphere/objectgrid/security/config/ClientSecurityConfigurationFactory.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 18:21:56
//
// Module  :  ClientSecurityConfigurationFactory.java
//
// Source File Description:
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.13 WASX.XD   04/08/2005   jitang    ObjectGrid - JAAS / Security - Drop 1 - Basic Support
// XD601_M2_B.x WASX.XD   09/20/2005   jitang    ObjectGrid - 6.0.1 M2 security
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.security.config;

import java.util.Properties;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;
import com.ibm.ws.objectgrid.security.config.ClientSecurityConfigurationImpl;
import com.ibm.ws.objectgrid.security.config.SSLConfigurationImpl;

/**
 * This class is a factory class for creating <code>ClientSecurityConfiguration</code>
 * and <code>SSLConfiguration</code> objects.
 *
 * @since WAS XD 6.0.1, XC10
 *
 * @see ClientSecurityConfiguration
 * @see SSLConfiguration
 */
public class ClientSecurityConfigurationFactory {

    private static final TraceComponent TC = Tr.register(
            ClientSecurityConfigurationFactory.class,
            Constants.TR_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);

    /**
     * Creates a new SSL configuration with all its attributes set to
     * their default values.
     *
     * @return a new <code>SSLConfiguration</code> instance
     *
     * @see ClientSecurityConfiguration#setSSLConfiguration(SSLConfiguration)
     * @see SSLConfiguration
     */
    public static SSLConfiguration getSSLConfiguration() {
        return new SSLConfigurationImpl();
    }

    /**
     * Creates a new SSL configuration using the specified attributes.
     *
     * @param alias the key alias in the key store
     * @param jsseProvider the JSSE provider
     * @param keyStoreName the key store file path name
     * @param keyStoreType format of the key store
     * @param keyStorePassword the password to the key store
     * @param trustStoreName the trust store file path name
     * @param trustStoreType the format of the trust store
     * @param trustStorePassword the password to the trust store
     * @param protocol the SSL protocol
     *
     * @return a new <code>SSLConfiguration</code> instance
     *
     * @throws IllegalArgumentException if keyStoreName or trustStoreName are
     *         <code>null</code> or a zero length String
     *
     * @see ClientSecurityConfiguration#setSSLConfiguration(SSLConfiguration)
     * @see SSLConfiguration
     */
    public static SSLConfiguration getSSLConfiguration(String alias, String jsseProvider,
            String keyStoreName, String keyStoreType, String keyStorePassword,
            String trustStoreName, String trustStoreType, String trustStorePassword,
            String protocol) {
        return new SSLConfigurationImpl(alias, jsseProvider, keyStoreName, keyStoreType,
                keyStorePassword, trustStoreName, trustStoreType, trustStorePassword, protocol);
    }

    /**
     * Creates a new client security configuration with all its attributes set to
     * their default values.
     *
     * @return a new <code>ClientSecurityConfiguration</code> instance
     *
     * @see ClientSecurityConfiguration
     */
    public static ClientSecurityConfiguration getClientSecurityConfiguration() {
        if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
            Tr.entry(TC, "getClientSecurityConfiguration");
        ClientSecurityConfiguration config = new ClientSecurityConfigurationImpl();
        if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
            Tr.exit(TC, "getClientSecurityConfiguration", config);
        return config;
    }

    /**
     * Creates a new client security configuration based on a client security
     * property file.
     *
     * @param fileName a security property file name
     *
     * @return a new <code>ClientSecurityConfiguration</code> instance
     *
     * @throws IllegalArgumentException if there are attributes in the Properties
     *         object with invalid values
     *
     * @see ClientSecurityConfiguration
     */
    public static ClientSecurityConfiguration getClientSecurityConfiguration(String fileName) {
        return new ClientSecurityConfigurationImpl(fileName);
    }

    /**
     * Creates a new client security configuration based on a <code>Properties</code>
     * object.
     *
     * @param props a <code>Properties</code> object
     *
     * @return a new <code>ClientSecurityConfiguration</code> instance
     *
     * @throws IllegalArgumentException if there are attributes in the Properties
     *         object with invalid values
     *
     * @see ClientSecurityConfiguration
     */
    public static ClientSecurityConfiguration getClientSecurityConfiguration(Properties props) {
        return new ClientSecurityConfigurationImpl(props);
    }
}