// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.4 XD/ws/code/objectgrid.impl/src/com/ibm/websphere/objectgrid/security/plugins/builtins/LDAPAuthenticatorHelper.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 19:06:07
//
// Module  :  LDAPAuthenticatorHelper.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD601_M2_Bx  WASX.XD   10/03/2005   jitang    ObjectGrid security support
// d343450      WXD601.XD 02/02/2006   jitang    Deal with hrdepartment field not existing in the LDAP server
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.security.plugins.builtins;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;

/**
 * This class is an LDAP Authenticator helper class used to connect to an 
 * LDAP Server using a user name and password. If the LDAP server can be connected 
 * to using the user name and password, the user name and password are considered 
 * valid.
 * 
 * @since WAS XD 6.0.1
 * @deprecated This plug-in is deprecated in version 8.6.
 */
public class LDAPAuthenticatorHelper {
    private static final TraceComponent TC = Tr.register(
            LDAPAuthenticatorHelper.class,
            Constants.TR_SECURITY_GROUP_NAME,
            Constants.TR_RESOURCE_BUNDLE_NAME);

    private static final String UID = "uid";

    private static final String HR_DEPT = "hrdepartment";

    private String providerURL = null;

    private String factoryClass = null;

    /**
     * Creates a new LDAPAuthenticatorHelper
     * 
     * @param _providerURL the providerURL of the LDAP server
     * @param _factoryClass the implementation class of the LDAP 
     *        initial context factory.
     */
    public LDAPAuthenticatorHelper(String _providerURL, String _factoryClass) {
        providerURL = _providerURL;
        factoryClass = _factoryClass;
    }

    /**
     * Authenticates the user to the LDAP directory.
     * 
     * @param user the user name, e.g., uid=xxxxxx,c=us,ou=bluepages,o=ibm.com
     * @param pwd the password
     * @return an array of the user attributes. The first element of the array 
     * is the value of "uid" in the directory, and the second element is the value 
     * of "hrdepartment" in the directory. If the directory doesn't have "hrdepartment",
     * the second element will be null.
     *   
     * @throws NamingException if a Naming Exception occured during initializing the  
     * naming context or during user look up.   
     */
    public String[] authenticate(String user, String pwd) throws NamingException {
        if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
            Tr.entry(TC, "authenticate", new Object[] { user, "xxxxxx", this });
        Hashtable env = new Hashtable();
        env.put(Context.INITIAL_CONTEXT_FACTORY, factoryClass);
        env.put(Context.PROVIDER_URL, providerURL);
        env.put(Context.SECURITY_PRINCIPAL, user);
        env.put(Context.SECURITY_CREDENTIALS, pwd);
        env.put(Context.SECURITY_AUTHENTICATION, "simple");

        InitialContext initialContext = new InitialContext(env);

        // Look up for the user
        DirContext dirCtx = (DirContext) initialContext.lookup(user);

        String uid = null;
        int iComma = user.indexOf(",");
        int iEqual = user.indexOf("=");
        if (iComma > 0 && iComma > 0) {
            uid = user.substring(iEqual + 1, iComma);
        }
        else {
            uid = user;
        }

        Attributes attributes = dirCtx.getAttributes("");
        //System.out.println(attributes);

        // Check the UID
        String thisUID = (String) (attributes.get(UID).get());

        String thisDept = attributes.get(HR_DEPT) != null ? (String) (attributes.get(HR_DEPT)
                .get()) : null;

        if (thisUID.equals(uid)) {
            if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
                Tr.exit(TC, "authenticate", new Object[] { thisUID, thisDept });
            return new String[] { thisUID, thisDept };
        }

        if (ObjectGridManagerImpl.isTraceEnabled && TC.isEntryEnabled())
            Tr.exit(TC, "authenticate", null);
        return null;
    }
}