// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.3 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/stats/CountStatistic.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:25:03
//
// Module  :  CountStatistic.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.9  WASX.XD   04/14/2005   jitang    ObjectGrid - Metric enablement - Drop 1
// XD601_M2_B.x WASX.XD   09/08/2005   jitang    ObjectGrid - Statistics enablement for 6.0.1 
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.stats;

/**
 * This interface represents statistic with a standard count measurement.
 * 
 * @since WAS XD 6.0.1
 */
public interface CountStatistic extends Statistic {

    /**
     * Gets the count since the last reset.
     * 
     * @return the count 
     */
    long getCount();

    /**
     * Returns a new copy of this statistic
     * 
     * @return a copy of this object
     */
    public CountStatistic copy();

    /**
     * Calculates the rate of change between two CountStatistics.
     * <p>
     * The provided <code>CountStatistic</code> must have the same statistic
     * ID and type.  The returned CountStatistic object is calculated 
     * by the count by the duration time difference of update time between the 
     * two statistics. 
     * 
     * @param otherStatistic a count statistic to compare with
     * 
     * @return a <code>CountStatistic</code> with the rate of change
     *         between this object and the specified object
     *         or <code>null</code> if the types or IDs don't match
     *         or the update time between the two Statistics is zero
     * 
     * @see #getId()
     * @see #getLastSampleTime()
     */
    public CountStatistic rateOfChange(CountStatistic otherStatistic);

    /**
     * Gets the minimum count value.
     * 
     * @return the minimum count value 
     */
    public long getMin();

    /**
     * Gets the maximum count value
     * 
     * @return the maximum count value 
     */
    public long getMax();
}