// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2010
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.4 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/stats/MapStatsModule.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:25:06
//
// Module  :  MapStatsModule.java
//
// Source File Description:
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.9  WASX.XD   05/06/2005   jitang    ObjectGrid - Metric enablement - Drop 1
// b18300       XS7.1     02/11/2010   jaredp    Updated Javadoc for getHitRate
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.stats;

import com.ibm.websphere.objectgrid.TxID;
import com.ibm.websphere.objectgrid.management.MapMBean;
import com.ibm.websphere.objectgrid.plugins.Loader;
import com.ibm.websphere.objectgrid.plugins.LogSequence;

/**
 * <p>This interface represents a statistic module for map. A map
 * statistic module contains 3 statistics:
 *
 * <ol>
 * <li>number of entries
 * <li>the map hit rate, which includes map get count and hit count
 * <li>the loader batch update time
 * </ol>
 *
 * The path to a map statistic module is the ObjectGrid name,
 * concatenated with "/", and then the map name. You can also use an array
 * to represent the path, with the first element being the ObjectGrid name,
 * and the second element being the map name.
 *
 * @since WAS XD 6.0.1
 *
 * @see MapMBean#retrieveStatsModule()
 */
public interface MapStatsModule extends StatsModule {

    /**
     * Gets the number of entries statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of entries count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getNumEntries(boolean copyMode);

    /**
     * Gets the number of bytes in use by the map.
     * <p>
     * The used bytes statistics are accurate only when you are using simple objects
     * or the COPY_TO_BYTES copy mode.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the number of bytes used count statistic
     *
     * @see ActiveCountStatistic
     */
    ActiveCountStatistic getUsedBytes(boolean copyMode);

    /**
     * Gets the map hit rate statistic.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * <p>
     * This statistic also contains the map get count and hit count from which
     * the hit rate is derived.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the map hit rate statistic
     *
     * @see PercentageStatistic
     */
    PercentageStatistic getHitRate(boolean copyMode);

    /**
     * Gets the map's loader batch update time.
     * <p>
     * If the copyMode is set to <code>true</code>, a copy is returned. It is
     * recommended to set the copy mode to <code>true</code> when this method is
     * called in the same JVM as the JVM in which the statistics are gathered.
     *
     * @param copyMode if set to <code>true</code>, a copy is returned.
     *
     * @return the map's loader batch update time statistic
     *
     * @see ActiveTimeStatistic
     * @see Loader#batchUpdate(TxID, LogSequence)
     */
    ActiveTimeStatistic getBatchUpdateTime(boolean copyMode);
}