// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.3 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/stats/Statistic.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:25:21
//
// Module  :  Statistic.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.9  WASX.XD   04/14/2005   jitang    ObjectGrid - Metric enablement - Drop 1
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.stats;

import java.io.Serializable;

/**
 * This inteface represents statistical data. It is a generic statistic 
 * interface.
 * 
 * @since WAS XD 6.0.1
 */
public interface Statistic extends Serializable {

    /** 
     * Gets the name of this statistic.
     *  
     * @return the name of this statistic
     */
    public String getName();

    /** 
     * Gets the unit of measurement for this statistic.
     * 
     * @return the unit of measurement for this statistic 
     */
    public String getUnit();

    /** 
     * Gets the description of this statistic.
     * 
     * @return the description of this statistic 
     */
    public String getDescription();

    /**
     * Gets the time the first measurement was taken.
     * 
     * @return the start time represented as a long
     */
    public long getStartTime();

    /** 
     * Gets the time the most recent measurement was taken.
     * 
     * @return the last sample time represented as a long. 
     */
    public long getLastSampleTime();

    /**
     * Gets the statistic identifier.
     *  
     * @return the statistic ID
     */
    public int getId();

    /** 
     * Gets whether this statistic is being monitored.
     * 
     * @return <code>true</code> if monitoring for this statistic is enabled 
     */
    public boolean isEnabled();

    /**
     * Updates this statistic with the given value.
     *  
     * @param newStatistic must have the same statistic ID and type
     */
    public void update(Statistic newStatistic);

    /**
     * Gets the difference between two statistics.
     * <p>
     * The provided <code>Statistic</code> must have the same statistic
     * ID and type.  The returned Statistic value is calculated 
     * by subtracting the provided statistic from this object.
     *  
     * @param otherStatistic a statistic to compare with
     * 
     * @return the difference between this statistic and the specified parameter
     *         or <code>null</code> if the ID and type do not match
     */
    public Statistic delta(Statistic otherStatistic);

    /**
     * Combines the value of the provided statistic with this object.
     * <p>
     * The provided <code>Statistic</code> must have the same statistic
     * ID and type.  If they ID and type do not match this method will
     * do nothing.
     * 
     * @param otherStatistic the statistic to combine with this object
     */
    public void combine(Statistic otherStatistic);

    /**
     * Resets the statistic to zero. 
     * <p>
     * Typically, this method is not called by the application.  
     */
    public void reset(); 

    /** 
     * Gets the String representation of this statistic.
     * 
     * @return a String represenation of this object 
     */
    public String toString();
}