// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.2 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/stats/TimeStatistic.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:25:30
//
// Module  :  TimeStatistic.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.13 WASX.XD   04/01/2005   jitang    DataGrid - JAAS / Security - Drop 1 - Basic Support
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.stats;

/**
 * This interface represents a standard time measurement.
 * 
 * @since WAS XD 6.0.1
 */
public interface TimeStatistic extends Statistic {

    /**
     * Gets the count since the last reset.
     * 
     * @return the count 
     */
    long getCount();

    /**
     * Gets the total time since the beginning of the measurement. 
     *  
     * @return the sum total of time 
     */
    public long getTotalTime();

    /**
     * Gets the minimum time value since the beginning of the measurement.
     *  
     * @return the minimum time 
     */
    public long getMinTime();

    /**
     * Gets the maximum time value since the beginning of the measurement.
     *  
     * @return the maximum time 
     */
    public long getMaxTime();

    /**
     * Gets the mean or average of the time values.
     * <p>
     * The average time is calculated by dividing <code>getTotalTime()</code>
     * by <code>getCount()</code>
     *  
     * @return the mean time 
     */
    public double getMeanTime();

    /** 
     * Gets the sum of squares of the time values.
     * 
     * @return the sum of squares of the time values  
     */
    public long getSumOfSquares();

    /** 
     * Adds a time value to this statistic.
     *  
     * @param time the time duration to add 
     */
    public void add(long time);

    /**
     * Returns a new copy of this statistic.
     * 
     * @return a copy of this object
     */
    public TimeStatistic copy();
}