// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b13153       6.1.0.5   10/06/2008   jian.tang FetchPlan support
// ---------------------------------------------------------------------------------------
package com.ibm.websphere.projector;

/**
 * A FetchPlan object allows you to customize how to fetch relations. It can customize the maximum 
 * depth of the eager relations to be fetched.
 *  
 * @since WAS XD 6.1.0.5
 */
public interface FetchPlan {

    /**
     * The default maximum fetch depth. If the maximum fetch depth is set to {@link #DEPTH_INFINITE}, 
     * eager relations of all levels will be fetched. 
     */
    public int DEPTH_INFINITE = -1;

    /**
     * Set the maximum fetch depth of eager relationships to retrieve.
     * <p>
     * For example, if the maximum fetch depth is set to 1, then only the first-level eager relations
     * will be fetched with the fetch operations. If it is set to 0, then no relations will be fetched.
     * <p>
     * By default the max fetch depth is {@link #DEPTH_INFINITE}, which means eager relations
     * of all levels will be fetched. 
     * 
     * @param depth the maximum fetch depth
     **/
    FetchPlan setMaxFetchDepth(int depth);

    /**
     * Get the maximum fetch depth of eager relationships to retrieve.
     * 
     * @return the maximum fetch depth of eager relationships to retrieve
     */
    int getMaxFetchDepth();

}
