// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  MetadataException.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b2203        6.1       08/24/2006   cdjohnson M5 feature additions
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector;



/**
 * Base exception for all entity metadata errors.
 * @since WAS XD 6.1
 */
public class MetadataException extends ProjectorException
{
    private String entityName;
    private String entityClassName;
    private String attributeName;
    
    /**
     * The <code>serialVersionUID</code> for this class.
     */
    private static final long serialVersionUID = -4906618749247294317L;

    /**
     * @see ProjectorException#ProjectorException()
     */
    public MetadataException() {
        super();
    }

    /**
     * @see ProjectorException#ProjectorException(String)
     */
    public MetadataException(String message) {
        super(message);
    }

    /**
     * @see ProjectorException#ProjectorException(Throwable)
     */
    public MetadataException(Throwable cause) {
        super(cause);
    }

    /**
     * @see ProjectorException#ProjectorException(String, Throwable)
     */
    public MetadataException(String message, Throwable cause) {
        super(message, cause);
    }

    /**
     * @see ProjectorException#ProjectorException(String)
     */
    public MetadataException(String message, String entityName, String entityClassName, String attributeName) {
        super(message);
        this.entityName = entityName;
        this.entityClassName = entityClassName;
        this.attributeName = attributeName;
    }

    /**
     * @see ProjectorException#ProjectorException(String, Throwable)
     */
    public MetadataException(String message, String entityName, String entityClassName, String attributeName, Throwable cause) {
        super(message, cause);
        this.entityName = entityName;
        this.entityClassName = entityClassName;
        this.attributeName = attributeName;
    }
    
    /**
     * The name of the attribute that this exception refers to or null if the exception
     * doesn't refer to an attribute.
     * @return the attributeName
     */
    public String getAttributeName() {
        return attributeName;
    }

    /**
     * The name of the attribute that this exception refers to or null if the exception
     * doesn't refer to an attribute.
     * @param attributeName the attributeName to set
     */
    public void setAttributeName(String attributeName) {
        this.attributeName = attributeName;
    }

    /**
     * The name of the entity metadata class that this exception refers to.
     * @return the entityClassName
     */
    public String getEntityClassName() {
        return entityClassName;
    }

    /**
     * The name of the entity metadata class that this exception refers to.
     * @param entityClassName the entityClassName to set
     */
    public void setEntityClassName(String entityClassName) {
        this.entityClassName = entityClassName;
    }

    /**
     * The name of the entity that this exception refers to.
     * @return the entityName
     */
    public String getEntityName() {
        return entityName;
    }

    /**
     * The name of the entity that this exception refers to.
     * @param entityName the entityName to set
     */
    public void setEntityName(String entityName) {
        this.entityName = entityName;
    }

}