// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  : OrderBy.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b995         6.1       08/31/2006   rajeshk   New part.
// b2404        XD 6.1    09/20/2006   cdjohnson Feature updates for M6
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.annotations;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * Specifies the ordering of the elements of a collection valued association at the
 * point when the association is retrieved.<p>
 * 
 * The syntax of the value ordering element is an orderby_list, as follows:
 * <pre>
 * orderby_list::= orderby_item [,orderby_item]*
 * orderby_item::= property_or_field_name [ASC | DESC]
 * </pre><p>
 * 
 * If ASC or DESC is not specified, ASC (ascending order) is assumed.<p>
 * 
 * If the ordering element is not specified, ordering by the primary key of the associated entity is assumed.
 * The property or field name must correspond to that of a persistent property or field of the associated
 * class. The properties or fields used in the ordering must correspond to columns for which comparison
 * operators are supported.
 * 
 * @since WAS XD 6.1
 */
@Target({FIELD, METHOD})
@Retention(RUNTIME)
public @interface OrderBy {
    /**
     * The ordering element string.
     * Example:
     * <pre>
     * "salary DESC, dept ASC"
     * </pre>
     * @return the ordering element string.
     */
	public String value() default "";
}


