// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2006-2007
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  TupleMetadata.java
//
// Source File Description:
//
//    See javadoc.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// b995         6.1       05/22/2006   cdjohnson New part.
// b1591        6.1       06/27/2006   cdjohnson M4 feature additions
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.projector.md;

import com.ibm.websphere.projector.Tuple;

/**
 * This describes the metadata for a Tuple.<p>
 *
 * @since WAS XD 6.1
 * @see EntityMetadata
 */
public interface TupleMetadata
{
	/**
	 * The number of basic attributes in this tuple
	 * @return the number of basic attributes defined for this tuple.
	 */
	int getNumAttributes();

	/**
	 * Retrieve the attribute metadata at the given position.
	 * @param position the position of the attribute.  The first attribute is in position 0;
	 * @return the attribute metadata.
	 */
	TupleAttribute getAttribute(int position);

	/**
	 * The number of associations this tuple has to other entities.
	 * @return the number of associations defined for this tuple.
	 */
	int getNumAssociations();

	/**
	 * Retrieve the association metadata at the given position.
	 * @param position the position of the association.   The first association is in position 0;
	 * @return the association metadata
	 */
	TupleAssociation getAssociation(int position);

    /**
     * Create a new Tuple associated with this metadata.
     * @return a new, empty Tuple instance
     */
    Tuple createTuple();

    /**
     * Retrieve the position of the named attribute.
     * @param name the name of the attribute.
     * @return the position of the attribute or -1 if not found.
     */
    int getAttributePosition(String name);

    /**
     * Retrieve the position of the named association.
     * @param name the name of the association.
     * @return the position of the association or -1 if not found.
     */
    int getAssociationPosition(String name);

    /**
     * Retrieve the position of the version attribute.
     * @return the position of version attribute. It will return -1
     * if TupleMetadata doesn't contain a Version attribute.
     * @see com.ibm.websphere.projector.annotations.Version
     */
    int getVersionAttributePosition();

}
