// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.

package com.ibm.websphere.xs.rest;

import java.util.HashMap;
import java.util.Map;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.xs.rest.RestValueImpl;

/**
 * This RestValueFactory class is used to create object that implements RestValue interface.
 * 
 * 
 * @since 8.6
 * @see com.ibm.websphere.xs.rest.RestValue
 * 
 */
final public class RestValueFactory {
    private final static String CLASS_NAME = RestValueFactory.class.getName();
    private final static TraceComponent tc = Tr.register(CLASS_NAME, Constants.TR_REST_GROUP_NAME, Constants.TR_RESOURCE_BUNDLE_NAME);

    // Singleton
    static private RestValueFactory restValueFactoryInstance = null;

    // Supported content types
    private Map<String, Boolean> supportedContentTypes = null;
    static {
        restValueFactoryInstance = new RestValueFactory();
        restValueFactoryInstance.supportedContentTypes = new HashMap<String, Boolean>(3);
        restValueFactoryInstance.supportedContentTypes.put(RestValue.CONTENT_TYPE_APP_XML, Boolean.TRUE);
        restValueFactoryInstance.supportedContentTypes.put(RestValue.CONTENT_TYPE_APP_JSON, Boolean.TRUE);
        restValueFactoryInstance.supportedContentTypes.put(RestValue.CONTENT_TYPE_APP_OCTET, Boolean.TRUE);
    };

    /**
     * Constructor
     * 
     */
    private RestValueFactory() {
    }

    /**
     * Create an object instance that implements the RestValue interface.  
     * Returns NULL if the given content type is not supported.
     * 
     * @since 8.6
     * 
     * @param contentType
     * 		an content type string used in HTML headers.  See RestValue interface for the 
     * 		list of supported content type.
     * @param valueBytes
     * 		array of byte for the data to be wrapped by the RestValue object
     * 
     * @return
     * 		an object instance that implements the RestValue interface
     * 
     * @see com.ibm.websphere.xs.rest.RestValue
     */
    public RestValue createRestValue(String contentType, byte[] valueBytes) {

        RestValue aRestValue = null;

        if (isSupported(contentType)) {
            aRestValue = new RestValueImpl(valueBytes, contentType);
        }
        if (TraceComponent.isAnyTracingEnabled() && tc.isDebugEnabled()) {
            Tr.debug(tc,  "createRestvalue", new Object[]{contentType, valueBytes, aRestValue});
        }
        return aRestValue; 
    }

    /**
     * Return the singleton instance of RestValueFactory
     * 
     * @since 8.6
     * 
     * @return
     * 		the instance of RestValueFactory
     */
    static public RestValueFactory instance() {

        if (restValueFactoryInstance == null) {
            restValueFactoryInstance = new RestValueFactory();
        }

        return restValueFactoryInstance;
    }

    /**
     * Check if the given content type is supported
     * 
     * @param aContentType
     * @return
     */
    private boolean isSupported(String aContentType) {

        boolean isSupported = false;

        if (supportedContentTypes.get(aContentType) != null) {
            isSupported = true;
        }

        if (TraceComponent.isAnyTracingEnabled() && tc.isDebugEnabled()) {
            Tr.debug(tc,  "isSupported", new Object[]{aContentType, isSupported});
        }
        return isSupported;
    }
}
