// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module : HashIndex.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version     Date       Userid       Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M2_B.2  WASX.XD   02/24/2005  newport    new code drop
// XD60_M3_B.10 WASX.XD   04/07/2005  sutter     Re-factor datagrid/datagrid.impl
// 267386       WASX.XD   04/18/2005  jhanders   Add exception handling for ObjectMap
// 267383       WASX.XD   04/20/2005  jhanders   touch support
// 271122       WASX.XD   04/28/2005  kjlaw      fix various defects and add tracing.
// 258492       WASX.XD   05/06/2005  kjlaw      undoBatchUpdate changes.
// XD60_M4_B.21 WASX.XD   05/23/2005  sutter     Client/server optimizations
// 285430       WASX.XD   06/21/2005  pdykes     Pre-GM changes to remove non-supported public interfaces
// 280613       WASX.XD   07/27/2005  kjlaw      fix NPE in findLessEqual method.
// 327907       WASX.XD   12/05/2005  cheng1     the 1st char of the attribute in the getAttributeMethod() shouls be upper case always.
// b1153        XD6.1     05/30/2006  cheng1     B25: Indexing enhancement
// b1153        XD6.1     06/12/2006  cheng1     B25: add findAll() returns all keys
// b2609        XD6.1     10/01/2006  cheng1     support field access attribute
// b2400        XD6.1     11/22/2006  cheng1     add findRange with includes flags
// b8888        XD6.1.x   06/18/2007  cheng1     performance enhancement
// b10896       6.1.0.1   12/14/2007  cheng1     Index / query forUpdate intent support
// b11149       6.1.0.1   01022/2008  cdjohnson  Allow attributes on the object's superclass.
// b12468       6.1.0.3   06/09/2008  cheng1     POJO key attribute indexing
// b12769       6.1.0.3   07/03/2008  cheng1     Handle potential NPE during iterate through index iterator
// b12684       6.1.0.3   07/07/2008   cheng1    handle TransactionTimeoutException
// b13025       6.1.0.3   07/25/2008   cheng1    POJO key attribute indexing should handle null value
// b13077       6.1.0.3   08/12/2008   cheng1    Use LinkedHashMap to avoid potential deadlock
// b12849       XD 6104   08/19/2008   cdjohnson Performance improvements for queries
// b13289       XD 6104   08/25/2008   cheng1    handle null returned by invalidSet method
// b13150       XD 6105   10/06/2008   cheng1    relationship indexing support
// b13150       XD 6105   10/17/2008   cheng1    collection element indexing and size indexing
// b13527       XD 7.0    01/12/2009   cheng1    composite index support
// b13527       XD 7.0    03/25/2009   cheng1    mixed attributes from key and value, including toOne relationships
// b18113       7004      12/10/2009   cheng1    include DELETE log element during applying transaction changes
// b18178       XS7.1     02/02/2010   jhanders  Add TxID to methods
// b18180       XS7.1     02/08/2010   bparees   Convert index queries to go to DB via diskhashtable
// b19053       XS7.1     03/26/2010   bparees   Ignore evicted log entries
// b18918       XS7.1     04/09/2010   cdjohnson  Add support for dynamic statistics enable/disable
// b19919       XS7.1     05/11/2010   cheng1    handle 1st transaction has insert scenario
// b19978       XS7.1     05/13/2010   jhanders  Fix the same problem in 19919 except in the disk path
// b21170       XS7.1.x   09/10/2010   cheng1    use java.util.ConcurrentSkipListMap as range index structure
// b22228       7.1.1     1/10/2010    kulack    Changes in support for byte array data serializer
// b23308       7.1.0.3   05/02/2011   jhanders  Update undoBatchUpdate to make sure checkExecutionInfo is complete.
// b23798       7.1.1     05/27/2011   cheng1    Collection index serializer support
// b23798       7.1.1     06/06/2011   cheng1    Multi-type indexing for Serializer support
// b23904       7.1.1     06/08/2011   jhanders  Update Index to not deserialize value when doing key only operations and removes
// b24865       7.1.1     10/26/2011   cheng1    support embedded object attribute navigation composite index
// t9392        8.6       08/23/2012   boggara   Added support for InverseRangeIndex
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.plugins.index;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.security.AccessController;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.ConcurrentModificationException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.Set;
import java.util.SortedMap;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentSkipListMap;

import com.ibm.ejs.ras.Tr;
import com.ibm.ejs.ras.TraceComponent;
import com.ibm.websphere.objectgrid.BackingMap;
import com.ibm.websphere.objectgrid.NoActiveTransactionException;
import com.ibm.websphere.objectgrid.ObjectGridException;
import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;
import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.OutputFormat;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.TransactionException;
import com.ibm.websphere.objectgrid.TxID;
import com.ibm.websphere.objectgrid.io.XsDataInputStream;
import com.ibm.websphere.objectgrid.plugins.BackingMapLifecycleListener;
import com.ibm.websphere.objectgrid.plugins.BackingMapPlugin;
import com.ibm.websphere.objectgrid.plugins.CacheEntry;
import com.ibm.websphere.objectgrid.plugins.LifecycleFailedException;
import com.ibm.websphere.objectgrid.plugins.LogElement;
import com.ibm.websphere.objectgrid.plugins.LogElement.Type;
import com.ibm.websphere.objectgrid.plugins.LogSequence;
import com.ibm.websphere.objectgrid.plugins.index.InverseRangeIndex;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer;
import com.ibm.websphere.objectgrid.plugins.io.DataSerializer.DataAttributeInflatable;
import com.ibm.websphere.objectgrid.plugins.io.KeySerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.MapSerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.ValueSerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.annotations.PluginOutputFormat;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.Attribute;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.DataDescriptor;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.KeyDataDescriptor;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.MapDataDescriptor;
import com.ibm.websphere.objectgrid.plugins.io.datadescriptor.ValueDataDescriptor;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.DataObjectContext;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedEntry;
import com.ibm.websphere.projector.MetadataException;
import com.ibm.websphere.projector.Tuple;
import com.ibm.websphere.projector.md.AssociationType;
import com.ibm.websphere.projector.md.EntityMetadata;
import com.ibm.websphere.projector.md.TupleAssociation;
import com.ibm.websphere.projector.md.TupleMetadata;
import com.ibm.ws.ffdc.FFDCFilter;
import com.ibm.ws.objectgrid.BackingMapExtensions;
import com.ibm.ws.objectgrid.Constants;
import com.ibm.ws.objectgrid.DiffMapValue;
import com.ibm.ws.objectgrid.ObjectGridManagerImpl;
import com.ibm.ws.objectgrid.ObjectMapImpl;
import com.ibm.ws.objectgrid.Storage;
import com.ibm.ws.objectgrid.index.CompositeValue;
import com.ibm.ws.objectgrid.index.GlobalIndexImpl;
import com.ibm.ws.objectgrid.index.InternalMapIndex;
import com.ibm.ws.objectgrid.index.inverserange.CompositeRangeAttribute;
import com.ibm.ws.objectgrid.index.inverserange.InverseRangeComparator;
import com.ibm.ws.objectgrid.index.inverserange.CompositeRangeSearchAttribute;
import com.ibm.ws.objectgrid.index.inverserange.InverseRangeSearchKey;
import com.ibm.ws.objectgrid.index.inverserange.KeyOverlapError;
import com.ibm.ws.objectgrid.map.BaseMap;
import com.ibm.ws.objectgrid.map.ClearUndoData;
import com.ibm.ws.objectgrid.plugins.BackingMapLifecycleListenerInternal;
import com.ibm.ws.objectgrid.plugins.GlobalIndexMessage;
import com.ibm.ws.objectgrid.plugins.ProxyComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyEQComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyEQNullComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyGEComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyGTComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyInverseRangeIndexComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyLEComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyLTComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyRangeComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyRangeExcludesBothEndComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyRangeIncludesBothEndComparator;
import com.ibm.ws.objectgrid.plugins.ProxyComparator.ProxyRangeIncludesHighEndComparator;
import com.ibm.ws.objectgrid.plugins.SystemCacheEntry;
import com.ibm.ws.objectgrid.plugins.io.SerializerAccessorImpl;
import com.ibm.ws.objectgrid.plugins.io.dataobject.SerializedEntryExtensions;
import com.ibm.ws.objectgrid.plugins.io.dataobject.keys.KeyConfig;
import com.ibm.ws.objectgrid.pmi.HashIndexModule;
import com.ibm.ws.objectgrid.runtime.RuntimeInfo;
import com.ibm.ws.objectgrid.stats.HashIndexStatsModuleImpl;
import com.ibm.ws.objectgrid.stats.HashIndexStatsProvider;
import com.ibm.ws.objectgrid.util.CollectionIterator;
import com.ibm.ws.objectgrid.util.IndexableHashtable;
import com.ibm.ws.objectgrid.util.ObjectGridUtil;
import com.ibm.ws.objectgrid.util.TransactionQueryMatcher;
import com.ibm.ws.projector.TupleAttributeInternal;
import com.ibm.ws.projector.TypeBasic;
import com.ibm.ws.projector.resources.Messages;
import com.ibm.ws.xs.NLSConstants;
import com.ibm.ws.xs.io.streams.XsDataStreamPool;
import com.ibm.ws.xs.util.LifecycleState;
import com.ibm.ws.xs.util.LifecycleState.Operation;
import com.ibm.ws.xs.util.dopriv.DoPrivUtil;

/**
 * HashIndex is a built-in index implementation of a <code>MapIndexPlugin</code> and can be used for typical indexing operations.
 *  This index plug-in supports both the <code>MapIndex</code> and <code>MapRangeIndex</code> interfaces and can be used for maps 
 *  that store native Java Objects, EntityManager API entities or when a DataSerializer plug-in is used.
 * <P>
 * Client applications should not extend this class.
 * <P>
 *
 * When used with Java Objects (POJO), HashIndex uses Java reflection to dynamically introspect objects stored in a map when they are updated.
 * It can index any individual fields or properties in the key or value portion of a map.  The index implementation uses
 * a synchronized HashMap to index the attributes for a non-range mode or a synchronized
 * TreeMap for range mode.
 * <P>
 * 
 * When used with the {@link KeySerializerPlugin} and {@link ValueSerializerPlugin} BackingMap plug-ins,  
 * the {@linkplain  DataAttributeInflatable#inflateDataObjectAttributes(DataObjectContext, XsDataInputStream, Object)} is used
 * to inflate and introspect the cache data.  HashIndex does not require that the attributes be defined in the {@linkplain DataDescriptor}.<p>
 * 
 * When used with a BackingMap associated with an EntityManager API entity, the attributes are fetched directly from the underlying
 * {@linkplain Tuple}.  HashIndex validates configured index attributes against the associated EntityMetadata.<p>
 *
 * The following properties can be used to configure the index using the ObjectGrid
 * deployment descriptor XML file or programmatically:
 * <table>
 * <tr>
 *   <td valign="top"><b><u>Property Name</u></b></td><td valign="top"><b><u>Applicability</u></b></td><td valign="top"><b><u>Description</u></b></td>
 * <tr>
 *   <td valign="top"><b>Name</b></td><td valign="top">POJO, Entity, Serializer</td><td valign="top">The name of the index.  The name must be unique for each map.</td>
 * </tr>
 * <tr>
 *   <td valign="top"><b>AttributeName</b></td><td valign="top">POJO, Entity, Serializer</td><td valign="top">The comma-delimited names of the attributes to index.<p>
 *   For field-access indexes, the attribute names are equivalent to the field names.<br>
 *   For property-access indexes, the attribute names is the JavaBean-compatible property names.<br>
 *   For indexes associated with a {@linkplain MapSerializerPlugin} configured BackingMap, the attribute name is a path to the attribute, 
 *   which can be in the KeySerializerPlugin or ValueSerializerPlugin.  See the {@link #setAttributeName(String)} method for details.  
 *   </td>
 * </tr>
 * <tr>
 *   <td valign="top"><b>FieldAccessAttribute</b></td><td valign="top">POJO</td><td valign="top">
 *   If true, the object is accessed using the fields directly.  If not specified or false, the
 *   attribute's getter method is used to access the data.</td>
 * </tr>
 * <tr>
 *   <td valign="top"><b>POJOKeyIndex</b></td><td valign="top">POJO</td><td valign="top">
 *   If true, the index will introspect the object in the key part of the map.  This is useful
 *   when the key is a composite key and the value does not have the key embedded within it.  If not specified
 *   or false, then the index will introspect the object in the value part of the map.</td>
 * </tr>
 * <tr>
 *   <td valign="top"><b>RangeIndex</b></td><td valign="top">POJO, Entity, Serializer</td><td valign="top">If true, range indexing is enabled.</td>
 * </tr>
 * <tr>
 *   <td valign="top"><b>GlobalIndexEnabled</b></td><td valign="top">POJO, Entity, Serializer</td><td valign="top">If true, global index support is enabled.</td>
 * </tr>
 * </table>
 *
 * <P>
 * The following is an ObjectGrid descriptor XML file example that defines the built-in HashIndex MapIndexPlugin for BackingMap:
 * <Pre>
 *  &lt;?xml version="1.0" encoding="UTF-8"?&gt;
 *  &lt;objectGridConfig xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
 *   xsi:schemaLocation="http://ibm.com/ws/objectgrid/config ../objectGrid.xsd"
 *   xmlns="http://ibm.com/ws/objectgrid/config"&gt;
 *      &lt;objectGrids&gt;
 *          &lt;objectGrid name="indexSampleGrid"&gt;
 *              &lt;backingMap name="person" pluginCollectionRef="person" readOnly="false"
 *                             preloadMode="false" lockStrategy="OPTIMISTIC" copyMode="COPY_ON_READ_AND_COMMIT" /&gt;
 *          &lt;/objectGrid&gt;
 *      &lt;/objectGrids&gt;
 *      &lt;backingMapPluginCollections&gt;
 *          &lt;backingMapPluginCollection id="person"&gt;
 *              &lt;bean id="MapIndexPlugin" className="com.ibm.websphere.objectgrid.plugins.index.HashIndex"&gt;
 *                 &lt;property name="Name" type="java.lang.String" value="person.employeeCode" description="index name" /&gt;
 *                 &lt;property name="AttributeName" type="java.lang.String" value="employeeCode" description="attribute name" /&gt;
 *                 &lt;property name="RangeIndex" type="boolean" value="true" description="true for MapRangeIndex" /&gt;
 *                 &lt;property name="FieldAccessAttribute" type="boolean" value="false" description="property-access" /&gt;
 *                 &lt;property name="POJOKeyIndex" type="boolean" value="false" description="Attribute is in the value (not the key)" /&gt;
 *              &lt;/bean&gt;
 *          &lt;/backingMapPluginCollection&gt;
 *      &lt;/backingMapPluginCollections&gt;
 *   &lt;/objectGridConfig&gt;
 * </Pre>
 * <p>
 *
 * These properties can also be set programmatically, using the JavaBean pattern. For example, the
 * setName(String) method is for setting Name property:
 * <pre>
 *      BackingMap personBackingMap = ivObjectGrid.getMap("person");
 *      //use the built-in HashIndex class as the index plug-in class.
 *      HashIndex mapIndexPlugin = new HashIndex();
 *      mapIndexPlugin.setName("person.employeeCode");
 *      mapIndexPlugin.setAttributeName("employeeCode");
 *      mapIndexPlugin.setRangeIndex(true);
 *      mapIndexPlugin.setFieldAccessAttribute(true);
 *      mapIndexPlugin.setPOJOKeyIndex(true);
 *      personBackingMap.addMapIndexPlugin(mapIndexPlugin);
 * </pre>
 * <P>
 *
 * @since WAS XD 6.0.1
 *
 * @see MapIndexPlugin
 * @see MapIndex
 * @see MapRangeIndex
 */
@PluginOutputFormat(keyFormat=OutputFormat.RAW, valueFormat=OutputFormat.RAW)
public /*final*/ class HashIndex implements MapIndexPlugin, BackingMapPlugin {
    /*
     * Serializer Notes:
     * For POJO and Entity indexes, the schema is fixed.  When the EntityMetadata is set, or when
     * the first object is inserted or queried, the metadata for that object is read, and cached.
     * For example:  Is it a key or value, a composite value, or a Collection...
     *
     * The Serializer doesn't cache any metadata.  An index can hold many different data types.
     * Note:  This is a problem for Range indexes since most attribute compareTo() operations
     * will throw a ClassCastExeption when comparing dissimilar types.
     */


    final static String CLASS_NAME = HashIndex.class.getName();

    final static TraceComponent tc = Tr
            .register(CLASS_NAME, Constants.TR_GROUP_NAME, Constants.TR_RESOURCE_BUNDLE_NAME);

    final static Object[] emptyArray = new Object[0];

    final static int defaulSizeOfCollectionSizeIndex = 100;

    private final static XsDataStreamPool xsDataStreamPool = XsDataStreamPool.getInstance();

    /**
     * Name of attribute this index is for.
     */
    String attributeName;
    
    private boolean isAttributeNameSet = false;

    /**
     * Attribute data descriptor.
     */
    Attribute   attributeDD;
    /**
     * Entity: The type
     */
    TypeBasic attributeType;

    HashIndexStatsProvider statsProvider = null;
    //HashIndexModule statsModule = null;

    /**
     * POJO: java reflection Method object for calling "get" + attributeName method. This method is used to extract the
     * attribute value from a BackingMap entry.
     */
    private Method getAttributeMethod;

    /**
     * POJO: java reflection Field object for extract the field value. This Field is used to extract the attribute value from
     * a BackingMap entry.
     */
    private Field attributeField;

    /**
     * The synchronized Map used as the index implementation where the attribute value object is the key and a java Set
     * is the value. A Set member is the key of a BackingMap entry that matches attribute value.
     */
    Map<Object, Set> index;

    /**
     * Name assigned to this index object.
     */
    String indexName;

    /**
     * Set to true if index supports MapRangeIndex.
     */
    private boolean ivRangeIndex;

    /**
     * Entity: The ivEntityMetadata is required to support Tuple-base map
     */

    private EntityMetadata ivEntityMetadata;

    /**
     * Entity: The ivTupleValueIndex is used to extract the attribute value from a Tuple value object. If the ivTupleValueIndex
     * is the default -1, implies the owner BackingMap is not Tuple-based.
     */
    private int ivTupleValueIndex = -1;

    /**
     * Use the keysWithNullAttribute set to hold keys with null attribute value of the indexed attribute.
     */
    Set keysWithNullAttribute;

    /**
     * POJO: The default attribute value access is property access (via getter). Users can change to field access by set the
     * ivFieldAccessAttribute to true.
     */
    private boolean ivFieldAccessAttribute;

    /**
     * Entity: The ivKeyTupleAttributeIndex indicates if this is indexing on an attribute of Key Tuple. It will be determined
     * during setEntityMetadata().
     */
    boolean ivKeyTupleAttributeIndex;

    // Entity: only relevant for single attribute indexes, set to true if the attribute
    // indexed is an association
    boolean ivAssociationTupleIndex;

    // Entity: only relevant for single attribute indexes, set to true if the
    // association attribute indexed has a toMany relationship
    // POJO:  Valid for any attribute that is of type collection
    boolean isCollectionAttribute = false;

    //POJO: If true, it's a single-attribute key index, applicable
    // to both the serializer and POJO.
    boolean ivPOJOKeyIndex;

    // POJO: is this an index based on EntityMetadata or a POJO
    // gets toggled to false when setEntityMetadata is called.
    boolean ivPOJOIndex=true;

    // POJO, Entity: For initialization.
    private volatile boolean hasCachedObjectInfo;

    // POJO, Entity
    boolean isComparable = false;

    // POJO, Entity
    boolean useEqualsMethodForComparison = false;

    // POJO or Serializer
    // If true, we are indexing an attribute in a complex key/value type.
    // If false, we are indexing the entire key or value.
    private boolean isAttributeIndex = false;

    @SuppressWarnings("unused")
    private final static Method getPmiModuleMethod;

    /**
     * Support collection size indexing in additional collection indexing Each size number will be in its corresponding
     * index in the List e.g. size=0 will be in the position of 0 in the list.
     *
     * There is a difference between entity and POJO in collection indexing. In entity, empty or null collection from
     * application's view is not distinguished during indexing. Both cases become null attribute. The impact is the
     * size(n.businessUnitType)=0 will include both null and empty collection cases. in entity query. In POJO, null and
     * empty collection is distinguished, so is the size(n.businessUnitType)=0 will not include null collection case
     */
    Set[] collectionSizeIndex = null;

    HashIndexModule indexPmi = null;

    // for composite index support
    boolean isComposite;

    /**
     * Name of attributes this composite index is for.
     * When used with a serializer, the key prefix is stripped off and they are all
     * as-is.  The ivPOJOKeyAttributeIndexes identifies which attributes are keys.
     */
    String[] attributeNamesArray;

    /**
     * To support composite index for query engine in serializer case, we need to reserve the original attribute names for key attributes.
     * The point is to keep the "key." prefix in order to distinguish key or value attribute.
     * For example, if both key and value have idString attribute and a composite index is defined like "key.idString,idString",
     * after strip off "key.", it will become "idString,idString" and query engine has no way to tell who (key or value) owns idString.
     *
     */
    String[] originalAttributeNamesArray;

    TypeBasic[] attributeTypesArray;

    // POJO:
    private Method[] getAttributeMethods;

    // POJO:
    private Field[] attributeFields;

    // POJO:
    private boolean areAttributeFieldsAndMethodsReady;

    // Entity:
    private int[] ivTupleValueIndexes;

    // POJO/Serializer:
    boolean[] ivPOJOKeyAttributeIndexes;

    // POJO:
    boolean[] ivPOJOCollectionAttributeChecks;
    volatile boolean ivCollectionAttributeChecksComplete = false;

    // Entity:
    byte[] ivAttributeTypeIndexes=null;

    final static Collection USE_EQUALS_CLASSES;

    // have we created the derby indexes to support this index yet
    volatile boolean ivIndexCreated=false;

    // heap vs derby(disk) backed cache
    boolean ivDiskBacked=false;

    // reference to the derby index, if any
    Object ivTableIndex=null;

    // can't create indexes until getProxy is called at least once, used to pull backing info from the map
    boolean ivInitialized=false;
    IndexableHashtable ivDiskHash;

    // The state of this plug-in.
    private LifecycleState state = new LifecycleState();

    // for diskbacked indexes, certain index definitions are invalid and need to be rejected.
    boolean isIndexValidForDiskBacked=true;

    long countCME_index = 0;
    long countCME_attSet = 0;

    // Identifies the data type that we are using:
    // Serializer, EntityMetadata or POJO
    boolean useSerializer = false;
    boolean isKeySerializer = false;  // Will flip on this value if attributeName is key.xxx

    MapSerializerPlugin mapSerializer;

    BackingMap backingMap;

    KeyConfig  keyConf;

    /**
     * The serializer attribute context to be used for retrieving values from the key or
     * value serializer.  Composite indexes may have both set.
     */
    Object serAttributeContextKey = null;
    Object serAttributeContextValue = null;

    // in serializer case, if the configure attributeName is not found in metadata,
    // the attribute type is open and we have to support multiple types.
    // For example, atributeName is id and cannot be found in metadata,  the id could be int or String.
    // and we have to maintain multiple type index structures.
    // This flag indicates this is multi-type index.
    boolean isMultiTypes;

    /**
     * The type indexes map holds all types indexes to support multi types indexing for serializer.
     */
    Map /* <String typeName, Map typeIndex> */ typeIndexesMap;

    /**
     * Returned from getAttribute() when the attribute is not found in the index.
     */
    static final Object ATTRIBUTE_NOT_FOUND = new Object();

    private final BMapLifecycleListener bmapLifecycleListener = new BMapLifecycleListener();
    
    boolean globalIndexEnabled;
    
    GlobalIndexImpl globalIndex;
        
	String globalIndexMapName = Constants.GLOBAL_INDEX_BACKINGMAP_NAME;
    
    boolean debug = false; // internal for test debug
    
    boolean globalIndexAsyncUpdate = true; // internal for now, unless we consider to support sync mode

    // BEGIN - Variables for InverseRangeIndex
    
    InverseRangeIndex inverseRangeIndex = null; // Holds a reference to caller object - InverseRangeIndex
    
    String searchAttributeName; // Search attribute name string as in configuration
    
    String[] searchAttributeNamesArray; // searchAttributeName string parsed into an array of names
    
    String[] originalSearchAttributeNamesArray; // Backup for original names

    private Field[] searchAttributeFields;
    
    private Method[] searchAttributeMethods;
    
    private boolean areSearchAttributeFieldsAndMethodsReady;
    
    boolean inverseRangeIndexEnabled = false; //true for InverseRangeIndex
    
    boolean[] ivPOJOSearchKeyAttributeIndexes;
    
    // END - Variables for InverseRangeIndex
    
    static {
        Method getModuleMethod = null;
        RuntimeInfo info = RuntimeInfo.instance();
        if (info.isWASApplicationServer()) {
            boolean pmiEnabled = false;
            try {
                Class clazz = DoPrivUtil.forName("com.ibm.wsspi.pmi.factory.StatsFactory");
                Object sf = clazz.newInstance();
                Method isPMIEnabledMethod = clazz.getMethod("isPMIEnabled", new Class[0]);
                pmiEnabled = ((Boolean) isPMIEnabledMethod.invoke(sf, new Object[0])).booleanValue();

                if (tc.isDebugEnabled()) {
                    Tr.debug(tc, "PMI is " + (pmiEnabled ? "enabled" : "disabled"));
                }

                if (pmiEnabled) {
                    Class class2 = DoPrivUtil.forName("com.ibm.ws.objectgrid.pmi.HashIndexPMIModule");

                    getModuleMethod = class2.getMethod("getHashIndexPMIModule", new Class[] { String.class,
                            String.class });
                }
            } catch (Throwable e) {
                FFDCFilter.processException(e, "com.ibm.websphere.objectgrid.plugins.index.HashIndex.<cinit>", "260");
                Tr.info(tc, NLSConstants.PMI_NOT_FOUND);
            }
        }
        getPmiModuleMethod = getModuleMethod;

        Collection types = new HashSet(16);
        types.add(String.class);
        types.add(Boolean.class);
        types.add(Byte.class);
        types.add(Character.class);
        types.add(Short.class);
        types.add(Integer.class);
        types.add(Long.class);
        types.add(Double.class);
        types.add(Float.class);

        types.add(java.util.Date.class);
        types.add(java.util.Calendar.class);
        types.add(java.sql.Date.class);
        types.add(java.sql.Time.class);
        types.add(java.sql.Timestamp.class);
        USE_EQUALS_CLASSES = Collections.unmodifiableCollection(types);
    }

    /**
     * Creates a new HashIndex object.
     */
    public HashIndex() {
        index = new ConcurrentHashMap();
        ivRangeIndex = false;
        keysWithNullAttribute = Collections.synchronizedSet(new LinkedHashSet());
    }

    /**
     * Called via reflection from the parent BaseMap when the statistics are
     * enabled or disabled for this index.
     */
    @SuppressWarnings("unused")
    private void setStatsProvider(HashIndexStatsProvider statsProvider) {
        this.statsProvider = statsProvider;
    }

    /**
     * Called via reflection from the parent BaseMap.
     */
    @SuppressWarnings("unused")
    private HashIndexStatsProvider getStatsProvider() {
        return statsProvider;
    }

    private HashIndexStatsModuleImpl getStatsModule() {
        if(statsProvider == null)
            return null;
        return (HashIndexStatsModuleImpl) statsProvider.getStatsModule();
    }

    /**
     * Called via reflection from the parent BaseMap
     * @param module
     */
    @SuppressWarnings("unused")
    private void setHashIndexPmiModule(HashIndexModule module) {
        indexPmi = module;
    }

    /**
     * Setter for the fieldAccessAttribute property.<p>
     * 
     * This property does not apply BackingMaps that have an EntityManager API entity associated or if the 
     * BackingMap has a {@linkplain MapSerializerPlugin} configured.
     *
     * @param isFieldAccessAttribute
     */
    public void setFieldAccessAttribute(boolean isFieldAccessAttribute) {
        ivFieldAccessAttribute = isFieldAccessAttribute;
    }

    /**
     * return the value of FieldAccessAttribute property
     *
     * @return fieldAccessAttribute property
     */
    public boolean isFieldAccessAttribute() {
        return ivFieldAccessAttribute;
    }

    /**
     * Setter for the POJOKeyIndex property.<p>
     * 
     * This property does not apply BackingMaps that have an EntityManager API entity associated or if the 
     * BackingMap has a {@linkplain MapSerializerPlugin} configured.
     *
     * @param isPOJOKeyIndex
     */
    public void setPOJOKeyIndex(boolean isPOJOKeyIndex) {
        ivPOJOKeyIndex = isPOJOKeyIndex;
    }

    /**
     * return the value of POJOKeyIndex property
     *
     * @return ivPOJOKeyIndex property
     */
    public boolean isPOJOKeyIndex() {
        return ivPOJOKeyIndex;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.websphere.objectgrid.plugins.index.MapIndexPlugin#setProperties(java.util.Properties)
     */
    public void setProperties(Properties properties) {

        String ivRangeIndexString = properties.getProperty("rangeIndex");
        if (ivRangeIndexString != null && ivRangeIndexString.equals("true")) {
            setRangeIndex(true);
        }
        setName(properties.getProperty("indexName"));
        setAttributeName(properties.getProperty("attributeName"));

        String ivFieldAccessAttributeString = properties.getProperty("fieldAccessAttribute");
        if (ivFieldAccessAttributeString != null && ivFieldAccessAttributeString.equals("true")) {
            setFieldAccessAttribute(true);
        }

        String ivPOJOKeyIndexString = properties.getProperty("POJOKeyIndex");
        if (ivPOJOKeyIndexString != null && ivPOJOKeyIndexString.equals("true")) {
            setPOJOKeyIndex(true);
        }
        
        String globalIndexEnabledString = properties.getProperty("globalIndexEnabled");
        if (globalIndexEnabledString != null && globalIndexEnabledString.equals("true")) {
        	setGlobalIndexEnabled(true);
        }

        // the isComposite is set by setAttribute() method, no need to get it here.

    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.websphere.objectgrid.plugins.index.MapIndexPlugin#getProperties()
     */
    public Properties getProperties() {
        Properties p = new Properties();
        // check is not null, to avoid NPE.
        if (indexName != null) {
            p.put("indexName", indexName);
        }
        if (attributeName != null) {
            p.put("attributeName", attributeName);
        }
        p.put("rangeIndex", ivRangeIndex ? "true" : "false");
        p.put("fieldAccessAttribute", ivFieldAccessAttribute ? "true" : "false");
        p.put("POJOKeyIndex", ivPOJOKeyIndex ? "true" : "false");
        p.put("globalIndexEnabled", globalIndexEnabled ? "true" : "false");

        return p;
    }

    /**
     * Sets the name of this index.
     *
     * @param name
     *            the name of the index
     */
    public void setName(String name) {
        indexName = name;
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
            if (ivRangeIndex) {
                Tr.debug(tc, "HashIndex using TreeMap as an index: " + indexName);
            } else {
                Tr.debug(tc, "HashIndex using HashMap as an index: " + indexName);
            }
        }
    }

    /**
     * Sets whether to create a <code>MapRangeIndex</code> or a <code>MapIndex</code>.
     * <p>
     * If this method is never called, a <code>MapIndex</code> is used as the default implementation.
     *
     * @param rangeIndex
     *            <code>true</code> for a MapRangeIndex, <code>false</code> for MapIndex
     *
     * @see MapIndex
     * @see MapRangeIndex
     * @see ObjectMap#getIndex(String)
     */
    public void setRangeIndex(boolean rangeIndex) {
        if(isComposite && rangeIndex==true) {
            Tr.warning(tc, NLSConstants.RANGE_INDEX_NOT_SUPPORTED_CWOBJ0041, indexName);
            return;
        }

        // there is a problem when this method is called before determine isComposite
        // If this method got called first, the index = Collections.synchronizedSortedMap(new TreeMap()); will run
        // need to change the index type back when the isComposite is determined
        // that is in setAttributeNamesArray() method.

        ivRangeIndex = rangeIndex;
        if (rangeIndex && !isComposite) {
            index = new ConcurrentSkipListMap();
            // indexing object/attribute must be comparable if range indexing
            isComparable = true;
        }
    }
    
	/**
	 * return the value of globalIndexEnabled property
	 * 
	 * @return globalIndexEnabled property
	 */
    public boolean isGlobalIndexEnabled() {
		return globalIndexEnabled;
	}

	/**
	 * Sets whether to enable global index and create a
	 * <code>MapGlobalIndex</code>.
	 * 
	 * @param globalIndexEnabled
	 *            <code>true</code> for support of MapGlobalIndex,
	 *            <code>false</code> for no support of MapGlobalIndex.
	 * 
	 * @see MapGlobalIndex
	 * @see ObjectMap#getIndex(String)
	 */
	public void setGlobalIndexEnabled(boolean globalIndexEnabled) {
		this.globalIndexEnabled = globalIndexEnabled;
	}

	// disbale globalIndexAsyncUpdate property.
    private void setGlobalIndexAsyncUpdate(boolean globalIndexAsyncUpdate) {
        this.globalIndexAsyncUpdate = globalIndexAsyncUpdate;
    }
    
    /**
     * Only InverseRangeIndex will call this during initialization. This method will set appropriate variables for InverseRangeIndex.
     * 
     * @param inverseRangeIndex the inverseRangeIndex to set
     */
    void setInverseRangeIndex(InverseRangeIndex inverseRangeIndex) {
        this.inverseRangeIndex = inverseRangeIndex;
        this.inverseRangeIndexEnabled = true;
        setIsMultiTypes(true);
    }

    // the debug controls global index messaging debug module
    // keep it internal for testing purpose.
    private void setDebug(boolean debug) {
        this.debug = debug;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.websphere.objectgrid.plugins.index.MapIndexPlugin#isRangeIndex()
     */
    public boolean isRangeIndex() {
        return ivRangeIndex;
    }

    /**
     * Retrieves the names of each attribute included in the index
     * that was set with the {@link #setAttributeName(String)} method.
     *
     * @return an array of attribute names as specified as an argument to the
     *         <code>setAttributeName(String)</code> method. Answers
     *         <code>null</code> if the <code>setAttributeName</code> method was
     *         not previously called.
     *
     * @see #setAttributeName(String)
     * @since 7.0
     */

    public String[] getAttributeNames() {
    	if(mapSerializer != null) {
    		// external view of atrtibute names of composite index.
    		// this is to support query engine for key embedded attribute navigation
    		// in order to distinguish key or value own the property.
    		return this.originalAttributeNamesArray;
    	} else {
    		return this.attributeNamesArray;
    	}
    }

    private String getListString(List list){
        if(list == null) {
            return null;
        } else if(list.size() == 0){
            return "[]";
        }

        int size = list.size();

        String listString = null;
        if (size == 1) {
            listString = "[" + list.get(0) + "]";
        } else {
            for (int i = 0; i < size; i++) {
                if (i == 0)
                    listString = "[" + list.get(i) + ", ";
                else if (i == size - 1)
                    listString = listString + list.get(i) + "]";
                else
                    listString = listString + list.get(i) + ", ";
            }
        }
        return listString;
    }

    private void setAttributeNamesArray(String attributeNames) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "setAttributeNamesArray called for index name = "+ indexName + " , attributeNames = " + attributeNames);
        }

        this.isComposite = true;

        if(ivRangeIndex) {
            Tr.warning(tc, NLSConstants.RANGE_INDEX_NOT_SUPPORTED_CWOBJ0041, indexName);

            // indicates the setRangeIndex() has been called to set to true.
            // re-set it back.
            index = new ConcurrentHashMap();
            ivRangeIndex = false;
            isComparable = false;
        }

        // process the attributeNames string and convert to String[]
        // need to verify each attribute and throw error if no such attribute occurs.

        List validTokenPositionList = new ArrayList();
        List invalidTokenPositionList = new ArrayList();
        if (attributeNames != null && attributeNames.trim().length() > 0) {
            String[] attributeTokens = attributeNames.split(",");
            for (int i = 0; i < attributeTokens.length; i++) {
                String attributeToken = attributeTokens[i].trim();
                if(attributeToken != null && attributeToken.trim().length() > 0) {
                    validTokenPositionList.add(attributeToken);
                } else {
                    invalidTokenPositionList.add(attributeToken);
                    // output warning ...
                }
            }

            if(validTokenPositionList.size() == attributeTokens.length){
                this.attributeNamesArray = attributeTokens;
            } else if(validTokenPositionList.size() > 0){
                int size = validTokenPositionList.size();
                this.attributeNamesArray = new String[size];
                for(int x=0; x< size; x++){
                    this.attributeNamesArray[x] = (String) validTokenPositionList.get(x);
                }
            }
        }

        this.originalAttributeNamesArray = new String[this.attributeNamesArray.length];
        for(int i=0; i<this.attributeNamesArray.length; i++) {
        	this.originalAttributeNamesArray[i] = this.attributeNamesArray[i];
        }

        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            String attributeNamesArrayString = ObjectGridUtil.getObjectArrayString(this.attributeNamesArray);
            Tr.exit(tc, "setAttributeName returning for index name = "+ indexName + " , attributeNames = " + attributeNamesArrayString);
        }
    }


    /*
     * (non-Javadoc)
     *
     * @see com.ibm.websphere.objectgrid.plugins.index.MapIndexPlugin#setEntityMetadata(com.ibm.websphere.projector.md.EntityMetadata)
     */
    public void setEntityMetadata(EntityMetadata entityMetadata) {
        // Composite index:
        // In the case of composite index, there will be a list of attribute names set on HashIndex
        // Need to find corresponding position indexes for each attribute.
        // need support mix attributes from key and value tuple

        ivEntityMetadata = entityMetadata;
        if (ivEntityMetadata != null) {
            // if entitymetadata is being set, then this is not a POJO index.
            ivPOJOIndex=false;
            if(isComposite) {
                ivTupleValueIndexes = new int[attributeNamesArray.length];
                ivAttributeTypeIndexes = new byte[attributeNamesArray.length];
                attributeTypesArray=new TypeBasic[attributeNamesArray.length];
                for(int i=0; i< ivTupleValueIndexes.length; i++){
                    // initialized as -1 indicates not set yet.
                    ivTupleValueIndexes[i] = -1;
                    ivAttributeTypeIndexes[i] = -1;
                }

                for (int x = 0; x < attributeNamesArray.length; x++) {
                    // value tuple
                    TupleMetadata valueMetadata = ivEntityMetadata.getValueMetadata();
                    int numAttributes = valueMetadata.getNumAttributes();
                    for (int i = 0; i < numAttributes; i++) {
                        String tupleAttributeName = valueMetadata.getAttribute(i).getName();
                        if (ivTupleValueIndexes[x] == -1 && attributeNamesArray[x].equals(tupleAttributeName)) {
                            ivTupleValueIndexes[x] = i;
                            ivAttributeTypeIndexes[x] = Constants.ATTRIBUTE_TYPE_VALUE_TUPLE;
                            attributeTypesArray[x]=((TupleAttributeInternal)valueMetadata.getAttribute(i)).getAttributeType();
                            break;
                        }
                    }

                    if (ivTupleValueIndexes[x] == -1) {
                        // key tuple
                        TupleMetadata keyMetadata = ivEntityMetadata.getKeyMetadata();
                        numAttributes = keyMetadata.getNumAttributes();
                        for (int i = 0; i < numAttributes; i++) {
                            String tupleAttributeName = keyMetadata.getAttribute(i).getName();
                            if (ivTupleValueIndexes[x] == -1 && attributeNamesArray[x].equals(tupleAttributeName)) {
                                ivTupleValueIndexes[x] = i;
                                ivAttributeTypeIndexes[x] = Constants.ATTRIBUTE_TYPE_KEY_TUPLE;
                                attributeTypesArray[x]=((TupleAttributeInternal)keyMetadata.getAttribute(i)).getAttributeType();
                                break;
                            }
                        }
                    }

                    // value associations
                    if (ivTupleValueIndexes[x] == -1) {
                        int numAssociations = valueMetadata.getNumAssociations();
                        for (int i = 0; i < numAssociations; i++) {
                            TupleAssociation assoc = valueMetadata.getAssociation(i);
                            String tupleAttributeName = assoc.getName();
                            if (ivTupleValueIndexes[x] == -1 && attributeNamesArray[x].equals(tupleAttributeName)) {
                                AssociationType assocCardinality = assoc.getAssociationType();
                                // cheng1: only allow toOne relationship
                                if (assocCardinality == AssociationType.ManyToMany
                                        || assocCardinality == AssociationType.OneToMany) {
                                    throw new MetadataException(Messages.getString("COLLECTION_ATTRIBUTE_NOT_SUPPORTED_CWPRJ1031E", new Object[]{tupleAttributeName, indexName, entityMetadata.getName()}));                                }
                                ivTupleValueIndexes[x] = i;
                                ivAttributeTypeIndexes[x] = Constants.ATTRIBUTE_TYPE_VALUE_ASSOCIATION;
                                // diskbacked indexes don't support associations
                                isIndexValidForDiskBacked=false;
                                break;

                            }
                        }
                    }

                    // key associations
                    if (ivTupleValueIndexes[x] == -1) {
                        TupleMetadata keyMetadata = ivEntityMetadata.getKeyMetadata();
                        int numAssociations = keyMetadata.getNumAssociations();
                        for (int i = 0; i < numAssociations; i++) {
                            TupleAssociation assoc = keyMetadata.getAssociation(i);
                            String tupleAttributeName = assoc.getName();
                            if (ivTupleValueIndexes[x] == -1 && attributeNamesArray[x].equals(tupleAttributeName)) {
                                AssociationType assocCardinality = assoc.getAssociationType();
                                // cheng1: only allow toOne relationship
                                if (assocCardinality == AssociationType.ManyToMany
                                        || assocCardinality == AssociationType.OneToMany) {
                                    throw new MetadataException(Messages.getString("COLLECTION_ATTRIBUTE_NOT_SUPPORTED_CWPRJ1031E", new Object[]{tupleAttributeName, indexName, entityMetadata.getName()}));
                                }
                                ivTupleValueIndexes[x] = i;
                                ivAttributeTypeIndexes[x] = Constants.ATTRIBUTE_TYPE_KEY_ASSOCIATION;
                                // diskbacked indexes don't support associations
                                isIndexValidForDiskBacked=false;
                                break;

                            }
                        }
                    }
                }

                // check if TupleValueIndexes is complete
                boolean isTupleValueIndexesComplete = true;
                List invalidAttributeNames = new ArrayList();
                for (int x = 0; x < ivTupleValueIndexes.length; x++) {
                    if (ivTupleValueIndexes[x] == -1) {
                        isTupleValueIndexesComplete = false;
                        invalidAttributeNames.add(attributeNamesArray[x]);
                    }
                }

                if(!isTupleValueIndexesComplete) {
                    throw new ObjectGridRuntimeException("Composite index: Invalid attributeNames: " + getListString(invalidAttributeNames) + " included in attributeNames: " + ObjectGridUtil.getObjectArrayString(attributeNamesArray) + ", Entity: "
                            + ivEntityMetadata.getName());
                }

            } else {
                // not composite index for entity
                // this is a tuple map
                TupleMetadata valueMetadata = ivEntityMetadata.getValueMetadata();
                int numAttributes = valueMetadata.getNumAttributes();
                for (int i = 0; i < numAttributes; i++) {
                    String tupleAttributeName = valueMetadata.getAttribute(i).getName();
                    if (attributeName.equals(tupleAttributeName)) {
                        ivTupleValueIndex = i;
                        attributeType=((TupleAttributeInternal)valueMetadata.getAttribute(i)).getAttributeType();
                        break;
                    }
                }

                if (ivTupleValueIndex == -1) {
                    // did not find the attribute in value tuple, try to find it on key tuple.
                    // if found on key tuple, implies key indexing on one of tuple key attributes.
                    TupleMetadata keyMetadata = ivEntityMetadata.getKeyMetadata();
                    numAttributes = keyMetadata.getNumAttributes();
                    for (int i = 0; i < numAttributes; i++) {
                        String tupleAttributeName = keyMetadata.getAttribute(i).getName();
                        if (attributeName.equals(tupleAttributeName)) {
                            ivTupleValueIndex = i;
                            ivKeyTupleAttributeIndex = true;
                            attributeType=((TupleAttributeInternal)keyMetadata.getAttribute(i)).getAttributeType();
                            break;
                        }
                    }


                    if (ivTupleValueIndex == -1) {
                        // Test for Value Association
                        int numAssociations = valueMetadata.getNumAssociations();
                        for (int i = 0; i < numAssociations; i++) {
                            TupleAssociation assoc = valueMetadata.getAssociation(i);
                            String tupleAttributeName = assoc.getName();
                            if (attributeName.equals(tupleAttributeName)) {
                                AssociationType assocCardinality = assoc.getAssociationType();
                                // cheng1: allow toMany indexing
                                if (assocCardinality == AssociationType.ManyToMany
                                        || assocCardinality == AssociationType.OneToMany) {
                                    // throw new ObjectGridRuntimeException("Multi-valued associations cannot be indexed:
                                    // attributeName: " + attributeName + ", Entity: " + ivEntityMetadata.getName());
                                    isCollectionAttribute = true;
                                    initializeCollectionSizeIndex();
                                }
                                ivTupleValueIndex = i;
                                ivAssociationTupleIndex = true;
                                // diskbacked indexes don't support associations
                                isIndexValidForDiskBacked=false;
                                break;
                            }
                        }

                        if (ivTupleValueIndex == -1) {
                            // Test for key association.
                            numAssociations = keyMetadata.getNumAssociations();
                            for (int i = 0; i < numAssociations; i++) {
                                TupleAssociation assoc = keyMetadata.getAssociation(i);
                                String tupleAttributeName = assoc.getName();
                                if (attributeName.equals(tupleAttributeName)) {
                                    AssociationType assocCardinality = assoc.getAssociationType();
                                    // allow toMany indexing
                                    if (assocCardinality == AssociationType.ManyToMany
                                            || assocCardinality == AssociationType.OneToMany) {
                                        // throw new ObjectGridRuntimeException("Multi-valued associations cannot be
                                        // indexed: attributeName: " + attributeName + ", Entity: " +
                                        // ivEntityMetadata.getName());
                                        isCollectionAttribute = true;
                                        initializeCollectionSizeIndex();
                                    }
                                    ivTupleValueIndex = i;
                                    ivAssociationTupleIndex = true;
                                    ivKeyTupleAttributeIndex = true;
                                    // diskbacked indexes don't support associations
                                    isIndexValidForDiskBacked=false;
                                    break;
                                }
                            }
                        }
                    }
                }

                if (ivTupleValueIndex == -1) {
                    // if entityMetadata is not null and we could not find the attributeName in entityMetadata, this is an
                    // error
                    throw new ObjectGridRuntimeException("Invalid attributeName: " + attributeName + ", Entity: "
                            + ivEntityMetadata.getName());
                }
            } // not composite
        }

        ivCollectionAttributeChecksComplete = true;

        /*
        if (false && isComposite) {
            System.out.println("setEntityMetadata() -> indexName = " + indexName);
            System.out.println("setEntityMetadata() -> ivTupleValueIndex = " + ivTupleValueIndex);
            System.out.println("setEntityMetadata() -> ivTupleValueIndexes = " + ivTupleValueIndexes);
            System.out.println("setEntityMetadata() -> ivKeyTupleAttributeIndex = " + ivKeyTupleAttributeIndex);
            System.out.println("setEntityMetadata() -> attributeNames = "
                    + ObjectGridUtil.getObjectArrayString(attributeNamesArray));
            if (ivTupleValueIndexes != null) {
                Object[] valueIndexes = new Object[ivTupleValueIndexes.length];
                Object[] attributeType = new Object[ivTupleValueIndexes.length];
                for (int i = 0; i < ivTupleValueIndexes.length; i++) {
                    valueIndexes[i] = new Integer(ivTupleValueIndexes[i]);
                    attributeType[i] = new Byte(ivAttributeTypeIndexes[i]);
                }
                System.out.println("setEntityMetadata() -> ivTupleValueIndexes        = "
                        + ObjectGridUtil.getObjectArrayString(valueIndexes));
                System.out.println("setEntityMetadata() -> ivKeyTupleAttributeIndexes = "
                        + ObjectGridUtil.getObjectArrayString(attributeType));
            }
        }
        */
    }

    private Method[] getCompositeAttributeMethods(final Object key, final Object value) throws SecurityException, NoSuchMethodException {
    	String originalAttributeName = this.attributeName;
    	try {
    		// To accomodate AddressableKeyName property for InverseRangeIndex, without impacting existing code. 
    		if(inverseRangeIndexEnabled) return getInverseRangeCompositeAttributeMethods(key,value);

    		if(!areAttributeFieldsAndMethodsReady) {
    			synchronized (attributeNamesArray) {
    				ivPOJOKeyAttributeIndexes = new boolean[attributeNamesArray.length];
    				ivPOJOCollectionAttributeChecks = new boolean[attributeNamesArray.length];

    				if(getAttributeMethods == null) {
    					getAttributeMethods = new Method[attributeNamesArray.length];
    				}

    				this.attributeName = null;
    				this.getAttributeMethod = null;

    				getAttributeMethods = new Method[attributeNamesArray.length];
    				for (int i = 0; i < attributeNamesArray.length; i++) {
    					// temp: use the attributeName to get method

    					this.attributeName = this.attributeNamesArray[i];
    					this.getAttributeMethod = null;

    					if(value != null) {
    						try {
    							// try on value object first
    							getAttributeMethods[i] = getAttributeMethod(value);
    						} catch (NoSuchMethodException e) {
    							// try on key
    							try {
    								getAttributeMethods[i] = getAttributeMethod(key);
    								ivPOJOKeyAttributeIndexes[i] = true;
    							} catch (NoSuchMethodException ex) {
    								this.attributeName = null;
    								this.attributeField = null;
    								if(value!=null) {
    									throw ex;
    								}
    							}
    						}
    					} else {
    						try {
    							getAttributeMethods[i] = getAttributeMethod(key);
    							ivPOJOKeyAttributeIndexes[i] = true;
    						} catch (NoSuchMethodException ex) {
    							this.attributeName = null;
    							this.attributeField = null;
    							if(value!=null) {
    								throw ex;
    							}
    						}
    					}
    				}

    				this.attributeName = null;
    				this.getAttributeMethod = null;

    				areAttributeFieldsAndMethodsReady = true;
    				for (int i = 0; i < attributeNamesArray.length; i++) {
    					if(getAttributeMethods[i] == null) {
    						areAttributeFieldsAndMethodsReady = false;
    						break;
    					}
    				}

    				/*
                if (false && isComposite) {
                    System.out.println("getCompositeAttributeMethods() -> indexName = " + indexName);
                    System.out.println("getCompositeAttributeMethods() -> key class = " + key.getClass().getName());
                    System.out.println("getCompositeAttributeMethods() -> key = " + key);
                    System.out.println("getCompositeAttributeMethods() -> attributeNames = "
                            + ObjectGridUtil.getObjectArrayString(attributeNamesArray));
                    Object[] isPOJOKeyAttributeIndexes = new Object[attributeNamesArray.length];
                    Object[] getMethods = new Object[attributeNamesArray.length];
                    for (int i = 0; i < attributeNamesArray.length; i++) {
                        isPOJOKeyAttributeIndexes[i] = new Boolean(ivPOJOKeyAttributeIndexes[i]);
                    }
                    System.out.println("getCompositeAttributeMethods() -> isPOJOKeyAttributeIndexes = "
                            + ObjectGridUtil.getObjectArrayString(isPOJOKeyAttributeIndexes));
                    System.out.println("getCompositeAttributeMethods() -> getAttributeMethods = "
                            + ObjectGridUtil.getObjectArrayString(getAttributeMethods));

                }
    				 */
    			}
    		}
    	} finally {
    		this.attributeName = originalAttributeName;
    	}
        return this.getAttributeMethods;
    }

    /*
     * To accomodate AddressableKeyName property without impacting existing code, this new method was added.
     */
    private Method[] getInverseRangeCompositeAttributeMethods(final Object key, final Object value)
            throws SecurityException, NoSuchMethodException {
        if (!areAttributeFieldsAndMethodsReady) {
            
            synchronized (attributeNamesArray) {
                
                ivPOJOKeyAttributeIndexes = new boolean[attributeNamesArray.length];
                ivPOJOCollectionAttributeChecks = new boolean[attributeNamesArray.length];
                
                this.attributeName = null;
                this.getAttributeMethod = null;
                getAttributeMethods = new Method[attributeNamesArray.length];

                for (int i = 0; i < attributeNamesArray.length; i++) {
                    // temp: use the attributeName to get method

                    this.attributeName = this.attributeNamesArray[i];
                    this.getAttributeMethod = null;
                    Object objectForMethodSearch = null;

                    if (this.attributeName.startsWith(inverseRangeIndex.getAddressableKeyName() + ".")) {
                        this.attributeName = this.attributeName.substring(inverseRangeIndex.getAddressableKeyName().length()+1);
                        this.attributeNamesArray[i] = this.attributeName;
                        ivPOJOKeyAttributeIndexes[i] = true;
                        objectForMethodSearch = key;
                    } else {
                        ivPOJOKeyAttributeIndexes[i] = false;
                        objectForMethodSearch = value;
                    }

                    try {
                        getAttributeMethods[i] = getAttributeMethod(objectForMethodSearch);
                    } catch (NoSuchMethodException e) {
                        this.attributeName = null;
                        this.getAttributeMethod = null;
                        throw e;
                    }
                }
                this.attributeName = null;
                this.getAttributeMethod = null;
            }
            areAttributeFieldsAndMethodsReady = true;
        }
        return this.getAttributeMethods;
    }    

    /*
     * To accomodate AddressableKeyName property without impacting existing code, this new method was added.
     */
    private Field[] getInverseRangeCompositeAttributeFields(final Object key, final Object value)
            throws SecurityException, NoSuchFieldException {
        if (!areAttributeFieldsAndMethodsReady) {
            
            synchronized (attributeNamesArray) {
                
                ivPOJOKeyAttributeIndexes = new boolean[attributeNamesArray.length];
                ivPOJOCollectionAttributeChecks = new boolean[attributeNamesArray.length];
                
                attributeFields = new Field[attributeNamesArray.length];

                this.attributeName = null;
                this.attributeField = null;

                for (int i = 0; i < attributeNamesArray.length; i++) {

                    this.attributeName = this.attributeNamesArray[i];
                    this.attributeField = null;
                    Object objectForFieldSearch = null;

                    if (this.attributeName.startsWith(inverseRangeIndex.getAddressableKeyName() + ".")) {
                        this.attributeName = this.attributeName.substring(inverseRangeIndex.getAddressableKeyName().length()+1);
                        this.attributeNamesArray[i] = this.attributeName;
                        ivPOJOKeyAttributeIndexes[i] = true;
                        objectForFieldSearch = key;
                    } else {
                        ivPOJOKeyAttributeIndexes[i] = false;
                        objectForFieldSearch = value;
                    }

                    try {
                        attributeFields[i] = getAttributeField(objectForFieldSearch);
                    } catch (NoSuchFieldException e) {
                        this.attributeName = null;
                        this.attributeField = null;
                        throw e;
                    }
                }
                this.attributeName = null;
                this.attributeField = null;
            }
            areAttributeFieldsAndMethodsReady = true;
        }
        return this.attributeFields;
    }    
        
    /**
     * This will return the array of Methods for search attributes for findAll() operation from Proxy when using InverseRangeIndex
     * 
     * @param key
     * @param value
     * @return
     * @throws SecurityException
     * @throws NoSuchMethodException
     */
    private Method[] getCompositeSearchAttributeMethods(final Object key) throws SecurityException, NoSuchMethodException {
        if(!areSearchAttributeFieldsAndMethodsReady) {
            synchronized (searchAttributeNamesArray) {
                ivPOJOSearchKeyAttributeIndexes = new boolean[searchAttributeNamesArray.length];

                this.attributeName = null;
                this.getAttributeMethod = null;

                searchAttributeMethods = new Method[searchAttributeNamesArray.length];
                for (int i = 0; i < searchAttributeNamesArray.length; i++) {
                    this.attributeName = this.searchAttributeNamesArray[i];

                    //If exists, discard AddressableKeyName prepended to search attribute names array
                    if(attributeName.startsWith(inverseRangeIndex.getAddressableKeyName() + ".")) {
                        this.attributeName = this.attributeName.substring(inverseRangeIndex.getAddressableKeyName().length()+1);
                        this.searchAttributeNamesArray[i] = this.attributeName;      
                    }
                    this.getAttributeMethod = null;

                    try {
                        searchAttributeMethods[i] = getAttributeMethod(key);
                        ivPOJOSearchKeyAttributeIndexes[i] = true;
                    } catch (NoSuchMethodException ex) {
                        this.attributeName = null;
                        this.attributeField = null;
                        throw ex;
                    }

                }
                this.attributeName = null;
                this.getAttributeMethod = null;
                areSearchAttributeFieldsAndMethodsReady = true;
            }
        }

        return this.searchAttributeMethods;
    }

    /**
     * Gets the Method object to use for extracting attribute value from a BackingMap entry.
     *
     * @param value
     *            is the BackingMap entry
     *
     * @return the java Method object for getting attribute value from value.
     *
     * @throws SecurityException
     * @throws NoSuchMethodException
     */
    private Method getAttributeMethod(Object value) throws SecurityException, NoSuchMethodException {
        if (getAttributeMethod == null) {
            // Note, if LogElement is for LogElement.CODE_DELETE, newValue will be null.
            // In that case, we need to use the oldValue (last committed value).
            if (value != null) {
                Class[] noParms = new Class[0];

                int attributeNameLength = attributeName.length();
                StringBuffer sb = new StringBuffer(attributeNameLength + 3);
                sb.append("get");
                if (attributeNameLength == 1) {
                    sb.append(attributeName.toUpperCase());
                } else {
                    sb.append(Character.toUpperCase(attributeName.charAt(0)));
                    sb.append(attributeName.substring(1));
                }

                String methodName = new String(sb);

                try {
                    getAttributeMethod = value.getClass().getMethod(methodName, noParms);

                } catch (NoSuchMethodException e1) {
                    // no such method
                    // if the getAttributeMethod is null and the attribute type is boolean,
                    // try to find isAttributeName method on the current class any any base class
                    boolean isBooleanAttribute = false;
                    Field field = null;
                    Class currentClass = value.getClass();
                    while (true) {
                        try {
                            field = currentClass.getDeclaredField(attributeName);
                            if (field != null) {
                                Class fieldType = field.getType();
                                if (fieldType == boolean.class || fieldType == Boolean.class) {
                                    isBooleanAttribute = true;
                                    if (attributeName.length() == 1) {
                                        methodName = "is" + attributeName.toUpperCase();
                                    } else {
                                        char upper1stChar = Character.toUpperCase(attributeName.charAt(0));
                                        methodName = "is" + upper1stChar + attributeName.substring(1);
                                    }
                                    break;
                                } else {
                                    throw e1;
                                }
                            } else {
                                throw e1;
                            }
                        } catch (NoSuchFieldException e2) {
                            currentClass = currentClass.getSuperclass();
                            if (currentClass == Object.class) {
                                throw e1;
                            }
                        }
                    }

                    if (isBooleanAttribute) {
                        getAttributeMethod = value.getClass().getMethod(methodName, noParms);
                    }

                }

            }
        }
        return getAttributeMethod;
    }

    private Field[] getCompositeAttributeFields(final Object key, final Object value) throws SecurityException, NoSuchFieldException {
    	String originalAttributeName = this.attributeName;
    	try {

    		// To accomodate AddressableKeyName property for InverseRangeIndex, without impacting existing code.
    		if(inverseRangeIndexEnabled) return getInverseRangeCompositeAttributeFields(key,value);

    		if(!areAttributeFieldsAndMethodsReady) {
    			synchronized (attributeNamesArray) {
    				ivPOJOKeyAttributeIndexes = new boolean[attributeNamesArray.length];
    				ivPOJOCollectionAttributeChecks = new boolean[attributeNamesArray.length];

    				if(attributeFields == null) {
    					attributeFields = new Field[attributeNamesArray.length];
    				}

    				this.attributeName = null;
    				this.attributeField = null;
    				for (int i = 0; i < attributeNamesArray.length; i++) {
    					if(attributeFields[i] == null) {
    						// temp: use the attributeName to get method
    						this.attributeName = this.attributeNamesArray[i];
    						this.attributeField = null;

    						if(value != null) {
    							try {
    								// try on value object first
    								attributeFields[i] = getAttributeField(value);
    							} catch (NoSuchFieldException e) {
    								// try on key
    								try {
    									attributeFields[i] = getAttributeField(key);
    									ivPOJOKeyAttributeIndexes[i] = true;
    								} catch (NoSuchFieldException ex) {
    									this.attributeName = null;
    									this.attributeField = null;
    									if(value!=null) {
    										throw ex;
    									}
    								}
    							}
    						} else {
    							try {
    								attributeFields[i] = getAttributeField(key);
    								ivPOJOKeyAttributeIndexes[i] = true;
    							} catch (NoSuchFieldException ex) {
    								this.attributeName = null;
    								this.attributeField = null;
    								if(value!=null) {
    									throw ex;
    								}
    							}
    						}
    					}
    				}

    				this.attributeName = null;
    				this.attributeField = null;

    				areAttributeFieldsAndMethodsReady = true;
    				for (int i = 0; i < attributeNamesArray.length; i++) {
    					if(attributeFields[i] == null) {
    						areAttributeFieldsAndMethodsReady = false;
    						break;
    					}
    				}
    			}
    		}
    	} finally {
    		this.attributeName = originalAttributeName;
    	}
        return attributeFields;
    }

    /**
     * This will return the array of Fields for search attributes for findAll() operation from Proxy when using InverseRangeIndex
     * 
     * @param key
     * @param value
     * @return
     * @throws SecurityException
     * @throws NoSuchFieldException
     */
    private Field[] getCompositeSearchAttributeFields(final Object key) throws SecurityException, NoSuchFieldException {
        if (!areSearchAttributeFieldsAndMethodsReady) {
            synchronized (searchAttributeNamesArray) {
                ivPOJOSearchKeyAttributeIndexes = new boolean[searchAttributeNamesArray.length];

                searchAttributeFields = new Field[searchAttributeNamesArray.length];

                this.attributeName = null;
                this.attributeField = null;
                for (int i = 0; i < searchAttributeNamesArray.length; i++) {
                    if (searchAttributeFields[i] == null) {

                        this.attributeName = this.searchAttributeNamesArray[i];

                        //If exists, discard AddressableKeyName prepended to search attribute names array
                        if(attributeName.startsWith(inverseRangeIndex.getAddressableKeyName() + ".")) {
                            this.attributeName = this.attributeName.substring(inverseRangeIndex.getAddressableKeyName().length()+1);
                            this.searchAttributeNamesArray[i] = this.attributeName;      
                        }
                        
                        this.attributeField = null;

                        try {
                            searchAttributeFields[i] = getAttributeField(key);
                            ivPOJOSearchKeyAttributeIndexes[i] = true;
                        } catch (NoSuchFieldException ex) {
                            this.attributeName = null;
                            this.attributeField = null;
                            throw ex;
                        }
                    }
                }

                this.attributeName = null;
                this.attributeField = null;

                areSearchAttributeFieldsAndMethodsReady = true;
            }
        }

        return searchAttributeFields;
    }

    /**
     * Gets the Field object to use for extracting attribute value from a BackingMap entry.
     *
     * Retrieves any field of the specified object's class or superclasses.
     *
     * @param value
     *            is the BackingMap entry
     *
     * @return the java Field object for getting attribute value from value.
     *
     * @throws SecurityException
     * @throws NoSuchMethodException
     */
    private Field getAttributeField(final Object value) throws SecurityException, NoSuchFieldException {
        if (attributeField == null) {
            // Note, if LogElement is for LogElement.CODE_DELETE, newValue will be null.
            // In that case, we need to use the oldValue (last committed value).
            if (value != null) {
                try {
                    attributeField = (Field) AccessController.doPrivileged(new PrivilegedExceptionAction() {
                        public Object run() throws Exception {
                            Class currentClass = value.getClass();
                            while (true) {
                                try {
                                    // The class may have a base class. If we get a NoSuchFieldException,
                                    // try the base class.
                                    Field field = currentClass.getDeclaredField(attributeName);
                                    field.setAccessible(true);
                                    return field;
                                } catch (NoSuchFieldException e) {
                                    currentClass = currentClass.getSuperclass();
                                    if (currentClass == Object.class) {
                                        throw e;
                                    }
                                }
                            }
                        }
                    });
                } catch (PrivilegedActionException pae) {
                    Throwable causeBy = pae.getException();
                    if (causeBy instanceof NoSuchFieldException) {
                        throw (NoSuchFieldException) causeBy;
                    }
                    throw new ObjectGridRuntimeException(causeBy);
                }
            }
        }

        return attributeField;
    }

    void validateExecutionInfo(LogSequence sequence) {
        // The following block only executes once to determine runtime information, such as
        // 1. if need to add comparator into TreeMap.
        // 2. if POJOKeyINdex, check if attribute name is valid
        // 3. if this is collection indexing.
    	
    	if(globalIndexEnabled && globalIndex==null && backingMap.getMapType() != BackingMap.CLIENT) {
    		createGlobalIndex();
    	}

        // no need to check on composite index or when using a Serializer
        int size = sequence.size();

        if (!useSerializer && !isComposite && !hasCachedObjectInfo) {
            Iterator iterator = sequence.getAllChanges();
            for (int i = size - 1; i >= 0 && !hasCachedObjectInfo; --i) {
                LogElement elem = (LogElement) iterator.next();
                Object newAttribute = null;
                Object oldAttribute = null;

                if (ivKeyTupleAttributeIndex) {
                    // in the case of key tuple indexing, the key won't change value
                    newAttribute = getAttribute(elem.getKey());
                } else {
                    if (ivPOJOKeyIndex) {
                        newAttribute = elem.getKey();
                        if(isAttributeNameSet) {
                            newAttribute = getAttribute(newAttribute);
                        }
                    } else {
                        LogElement.Type doType = elem.getType();
                        if (doType == LogElement.INSERT) {
                            newAttribute = elem.getAfterImage();
                            if(isAttributeNameSet) {
                                newAttribute = getAttribute(newAttribute);
                            }
                        } else if (doType == LogElement.UPDATE) {
                            newAttribute = elem.getAfterImage();
                            oldAttribute = elem.getBeforeImage();
                            if(isAttributeNameSet) {
                                newAttribute = getAttribute(newAttribute);
                                oldAttribute = getAttribute(oldAttribute);
                            }
                        } else if (doType == LogElement.DELETE) {
                            oldAttribute = elem.getBeforeImage();
                            if(isAttributeNameSet) {
                                oldAttribute = getAttribute(oldAttribute);
                            }
                        } else if (doType == LogElement.EVICT) {
                            Object beforeImage = elem.getBeforeImage();
                            if (beforeImage != null) {
                                oldAttribute = beforeImage;
                                if(isAttributeNameSet) {
                                    oldAttribute = getAttribute(oldAttribute);
                                }
                            }
                        }
                    }
                }

                if (newAttribute != null) {
                    cacheObjectInfo(newAttribute);
                } else if (oldAttribute != null) {
                    cacheObjectInfo(oldAttribute);
                }

            }
        } else if(!useSerializer && isComposite && !ivCollectionAttributeChecksComplete) {
            Iterator iterator = sequence.getAllChanges();
            for (int i = size - 1; i >= 0 && !ivCollectionAttributeChecksComplete; --i) {
                LogElement elem = (LogElement) iterator.next();
                Object key = elem.getKey();
                Object newValue = null;
                Object oldValue = null;

                LogElement.Type doType = elem.getType();
                if (doType == LogElement.INSERT) {
                    newValue = elem.getAfterImage();
                } else if (doType == LogElement.UPDATE) {
                    newValue = elem.getAfterImage();
                    oldValue = elem.getBeforeImage();
                } else if (doType == LogElement.DELETE) {
                    oldValue = elem.getBeforeImage();
                } else if (doType == LogElement.CLEAR) {
                    oldValue = elem.getBeforeImage();
                } else if (doType == LogElement.EVICT) {
                    Object beforeImage = elem.getBeforeImage();
                    if (beforeImage != null) {
                        oldValue = beforeImage;
                    }
                }

                if (newValue != null) {
                    Object[] compositeValule;
                    if(inverseRangeIndexEnabled) {
                        compositeValule = (Object[]) getCompositeAttribute(key, newValue);
                    } else {
                        compositeValule = ((CompositeValue)getCompositeAttribute(key, newValue)).getAttributeValues();
                    }
                    checkCollectionAttributeIncluded(compositeValule);
                } else if (oldValue != null) {
                    Object[] compositeValule;
                    if(inverseRangeIndexEnabled) {
                        compositeValule = (Object[]) getCompositeAttribute(key, oldValue);
                    } else {
                        compositeValule = ((CompositeValue)getCompositeAttribute(key, oldValue)).getAttributeValues();
                    }
                    checkCollectionAttributeIncluded(compositeValule);
                }
            }
        }
    }

    /**
     * @see MapIndexPlugin#doBatchUpdate(TxID,LogSequence)
     */
    public void doBatchUpdate(TxID txid, LogSequence sequence) throws ObjectGridRuntimeException {
        final boolean entryEnabled = ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled();
        if (entryEnabled) {
            Tr.entry(tc, "batchUpdate called for index name = " + indexName);
        }

        validateExecutionInfo(sequence);

        Iterator iter = sequence.getAllChanges();

        int size = sequence.size();
        for (int i = size - 1; i >= 0; --i) {
            LogElement elem = (LogElement) iter.next();
            LogElement.Type doType = elem.getType();
            if (doType == LogElement.INSERT) {
                Object key = elem.getKey();
                if(!ivDiskBacked)
                    insertIntoIndex(key, elem.getAfterImage());
                // once we see an insert event, we know the underlying base table has been created
                // try to get the index from diskhash.
                if(ivTableIndex==null && ivDiskBacked && ivDiskHash!=null) {
                    ivTableIndex=ivDiskHash.getIndex(indexName);
                }

            } else if (doType == LogElement.UPDATE) {
                // once we see an update event, we know the underlying base table has been created
                // try to get the index from diskhash
                if(ivTableIndex==null && ivDiskBacked && ivDiskHash!=null) {
                    ivTableIndex=ivDiskHash.getIndex(indexName);
                }

                if (ivPOJOKeyIndex || ivKeyTupleAttributeIndex || isKeySerializer) {
                    // in the case of key indexing, the update on the value object that the key is associated with
                    // has no impact on the key index structure.
                    continue;
                }
                if(!ivDiskBacked) {
                    Object key = elem.getKey();
                    updateIndex(key, elem.getBeforeImage(), elem.getAfterImage());
                }
            } else if (doType == LogElement.DELETE) {
                if(!ivDiskBacked) {
                    Object key = elem.getKey();
                    removeFromIndex(key, elem.getBeforeImage());
                }
            } else if (doType == LogElement.CLEAR) {
                // TODO Jerry
                // Two concurrent problems here:
                // 1) If the clear thread comes here and replace the index map,
                // and another thread is doing insert or
                // other write operations, the other thread will update the old index map.
                //
                // 2) Also if the clear fails, we change the index data back to the old
                // index data, which will not have all the new updates between the time
                // the new index data is created and the time we undo the clear operation.

                // need to new up the index and save old index for undo
                 if(!ivDiskBacked) {
                	if(globalIndexEnabled) {
                		clearGlobalIndex();
                	} 
                	
            		// HashIndex
                    Map oldIndex = null;
                    if(!isMultiTypes) {
                        oldIndex = index;
                    } else {
                        oldIndex = typeIndexesMap;
                    }
                    Set oldKeysWithNullAttribute = keysWithNullAttribute;
                    ClearUndoData undoData = (ClearUndoData) ((SystemCacheEntry) ((DiffMapValue) elem).getRawBeforeImage())
                            .getValue();
                    if (undoData != null) {
                        undoData.indexData.put(indexName, new Object[] { oldIndex, oldKeysWithNullAttribute,
                                collectionSizeIndex });
                    }

                    initializeCollections();

                    // index.clear();
                    final boolean debugEnabled = ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled();
                    if (debugEnabled) {
                        Tr.debug(tc, "cleared all keys from index");
                    }
                }
            } else if (doType == LogElement.EVICT) {
                if(!ivDiskBacked){
                    Object beforeImage = elem.getBeforeImage();
                    if (beforeImage != null) {
                        Object key = elem.getKey();
                        removeFromIndex(key, beforeImage);
                    }
                }
            }
        }

        if (indexPmi != null) {
            indexPmi.incrementBatchUpdateCount(indexName, 1);
        }
        HashIndexStatsModuleImpl statsModule = getStatsModule();
        if (statsModule != null) {
            statsModule.incrementBatchUpdateCount(indexName, 1);
        }

        if (entryEnabled) {
            Tr.exit(tc, "batchUpdate returning for index name = " + indexName);
        }

    }

    /**
     * Initialize the collections that we require for the
     * index, including the main index, the null attribute
     * value index, and the collection size index.
     */
    private void initializeCollections() {
        if(!isMultiTypes) {
            if (ivRangeIndex) {
                index = new ConcurrentSkipListMap();
            } else {
                index = new ConcurrentHashMap();
            }
        } else {
            this.typeIndexesMap = new ConcurrentHashMap();
        }
        keysWithNullAttribute = Collections.synchronizedSet(new LinkedHashSet());

        // re-initialize collectionSizeIndex
        initializeCollectionSizeIndex();
    }
    /**
     * @see MapIndexPlugin#undoBatchUpdate(TxID, LogSequence)
     */
    public void undoBatchUpdate(TxID txid, LogSequence sequence) throws ObjectGridException {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "undoUpdate called for index name = " + indexName);
        }

        // if doBatchUpdate isn't called, we need to check the execution info since it wasn't done
        // already.
        validateExecutionInfo(sequence);

        int errors = 0;
        // for disk backed, undo is handled in derby.
        if(!ivDiskBacked) {
            Iterator iter = sequence.getAllChanges();
            for (int i = sequence.size() - 1; i >= 0; --i) {
                try {
                    LogElement elem = (LogElement) iter.next();
                    LogElement.Type undoType = elem.getUndoType();
                    if (undoType == LogElement.INSERT) {

                        // if undoing a clear, restore the index and keysWithNullAttribute
                        // from the CacheEntry
                        if (elem.getType() == LogElement.CLEAR) {
                            SystemCacheEntry entry = (SystemCacheEntry) ((DiffMapValue) elem).getRawBeforeImage();

                            ClearUndoData undoData = (ClearUndoData) entry.getValue();
                            if (undoData != null) {
                                Object[] indexCollections = (Object[]) undoData.indexData.get(indexName);
                                keysWithNullAttribute = (Set) indexCollections[1];
                                if(!isMultiTypes) {
                                    index = (Map) indexCollections[0];
                                } else {
                                    typeIndexesMap = (Map) indexCollections[0];
                                }
                                collectionSizeIndex = (Set[]) indexCollections[2];
                            }
                        } else {
                            Object key = elem.getKey();
                            insertIntoIndex(key, elem.getBeforeImage());
                        }
                    } else if (undoType == LogElement.UPDATE) {
                        if (ivPOJOKeyIndex || ivKeyTupleAttributeIndex || isKeySerializer) {
                            // in the case of key indexing, the update on the value object that the key is assocaited with
                            // has no impact on the key index structure.
                            continue;
                        }
                        Object key = elem.getKey();
                        updateIndex(key, elem.getAfterImage(), elem.getBeforeImage());
                    } else if (undoType == LogElement.DELETE) {
                        Object key = elem.getKey();
                        removeFromIndex(key, elem.getAfterImage());
                    }
                } catch (Throwable t) {
                    // FFDCFilter.processException( t, CLASS_NAME + ".undoBatchUpdate", "268" );
                    // As required in programming contract of this method in MapIndexPlugin
                    // interface, we need to log warning message and continue to process
                    // the LogSequence. The exception is more than likely from the getAttributeName
                    // method on the value object or the attribute returned is not a Comparable
                    // object as required to be for the index.
                    Tr.warning(tc, NLSConstants.GENERAL_EXCEPTION_WARNING_CWOBJ0006, t);
                    ++errors;
                }
            }
        }
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.exit(tc, "undoUpdate returning for index name = " + indexName);
        }

        if (errors > 0) {
            throw new ObjectGridException(errors
                    + " exceptions occured during rollback of index changes. See warning messages log for details.");
        }
    }

    private Object getAttribute(Object key, Object value) throws ObjectGridRuntimeException {
        if(useSerializer) {
            try {
                return getSerializerAttribute(key, value);
            } catch (IOException e) {
                FFDCFilter.processException(e, CLASS_NAME + ".getAttribute", "1637");
                throw new ObjectGridRuntimeException(e);
            }
        } else if(ivPOJOKeyIndex || ivKeyTupleAttributeIndex) {
            return getAttribute(key);
        } else {
            return getAttribute(value);
        }
    }

    /**
     * Extracts the attribute from a specified object. If the index is for an attribute
     * extracted by a {@link KeySerializerPlugin} or {@link ValueSerializerPlugin}, then the
     * object passed in should be the key or value object as appropriate for the index.
     *
     * @param valueOrKey
     *            The value or key object.
     *
     * @throws ObjectGridRuntimeException
     *             is thrown if any exception occurs attempting to extract the attribute value from the value or key Object.
     */
    public Object getAttribute(Object valueOrKey) throws ObjectGridRuntimeException {
        if ((hasCachedObjectInfo && !isAttributeIndex) || !isAttributeNameSet) {
            if(useSerializer)
                // Return the the entire value/key to index if they didn't specify an attribute name.
                return ((SerializedEntry) valueOrKey).getObject();
            else
                return valueOrKey;
        }

        if (useSerializer) {
            return getAttribute(valueOrKey, valueOrKey);
        }

        try {
            Object attribute = null;
            if (valueOrKey != null) {
                // b1153: handle Tuple value if ivTupleValueIndex != -1
                // if ivEntityMetadata == null, means POJO case
                if (ivEntityMetadata==null) {
                    // regular value
                    if (ivFieldAccessAttribute) {
                        attribute = (attributeField == null ? getAttributeField(valueOrKey) : attributeField).get(valueOrKey);
                    } else {
                        attribute = (getAttributeMethod == null ? getAttributeMethod(valueOrKey) : getAttributeMethod)
                                .invoke(valueOrKey, emptyArray);
                    }
                } else {
                    // Tuple value
                    if (ivAssociationTupleIndex) {
                        if (isCollectionAttribute) {
                            // X-to-Many
                            attribute = ((Tuple) valueOrKey).getAssociations(ivTupleValueIndex);
                        } else {
                            // N-to-One, so just get the first index. There are no more.
                            attribute = ((Tuple) valueOrKey).getAssociation(ivTupleValueIndex, 0);
                        }
                    } else {
                        attribute = ((Tuple) valueOrKey).getAttribute(ivTupleValueIndex);
                    }
                }
            }

            return attribute;
        } catch (InvocationTargetException e) {
            FFDCFilter.processException(e, CLASS_NAME + ".getAttribute", "290");
            Throwable t = e.getTargetException();
            throw new ObjectGridRuntimeException(
                    "Caught unexpected Throwable during index update processing, index name = " + indexName + ": " + t,
                    t);
        } catch (Throwable t) {
            FFDCFilter.processException(t, CLASS_NAME + ".getAttribute", "297");
            throw new ObjectGridRuntimeException(
                    "Caught unexpected Throwable during index update processing, index name = " + indexName + ": " + t,
                    t);
        }
    }

    /**
     * Retrieve a single attribute from the specified object using the key and value serializers.
     * This automatically searches the value and key objects (if specified) for the index.
     */
    private Object getSerializerAttribute(Object keySerialized, Object valueSerialized) throws IOException {
        Object result = null;

        // result must stay null if the value is actually null
        if(valueSerialized != null || (isKeySerializer && keySerialized != null)) {
            result = ATTRIBUTE_NOT_FOUND;
            final DataObjectContext serCtx = ((BackingMapExtensions)backingMap).getSerializerContext();
            XsDataInputStream is = null;
            try {
                is = ((SerializedEntryExtensions)keySerialized).getPooledInputStream();
                if (isKeySerializer) {
                    if(isAttributeNameSet) {
                        result = ObjectGridUtil.toSingleObject(mapSerializer.getKeySerializerPlugin().inflateDataObjectAttributes(serCtx, is, serAttributeContextKey));
                    } else {
                        result = mapSerializer.getKeySerializerPlugin().inflateDataObject(serCtx, is);
                    }
                } else {
                    is = ((SerializedEntryExtensions)valueSerialized).getPooledInputStream();
                    if(isAttributeNameSet) {
                        result = ObjectGridUtil.toSingleObject(mapSerializer.getValueSerializerPlugin().inflateDataObjectAttributes(serCtx, is, serAttributeContextValue));
                    } else {
                        result = mapSerializer.getValueSerializerPlugin().inflateDataObject(serCtx, is);
                    }
                }
            }
            finally {
                if (is != null) {
                    xsDataStreamPool.returnInputStream(is); // Return stream
                }
            }
        }

        if(result == DataSerializer.SpecialValue.NOT_FOUND) {
            return ATTRIBUTE_NOT_FOUND;
        }
        return result;
    }

    /**
     * Extracts the attribute from both key and value
     * This is to support composite index that allows mix attributes from both key and value.
     * The list of attribute values are retrieved from both key and value and construct CompositeValue object to return
     *
     * @param key
     *            The key Object of cached entry.
     * @param value
     *            The value Object of cached entry.
     *
     * @return compositeAttribute
     *            The type of CompositeValue consists of all attribute values of a composite index
     *
     * @throws ObjectGridRuntimeException
     *             is thrown if any exception occurs attempting to extract the attribute value from the value Object.
     */
    Object getCompositeAttribute(Object key, Object value) throws ObjectGridRuntimeException {
        try {
            Object attribute = null;
            // if ivEntityMetadata == null, means POJO case
            if (ivEntityMetadata==null) {
                Object[] compositeValue;
                if(useSerializer) {
                    compositeValue = getSerializerCompositeAttribute(key, value);
                } else {
                    compositeValue = new Object[attributeNamesArray.length];
                    if (ivFieldAccessAttribute) {
                        if(attributeFields == null) {
                            getCompositeAttributeFields(key, value);
                        }
                        for(int i=0; i< attributeNamesArray.length; i++){
                            if(ivPOJOKeyAttributeIndexes[i]) {
                                compositeValue[i] = attributeFields[i].get(key);
                            } else {
                                if(value == null) {
                                    compositeValue[i] = null;
                                } else {
                                    compositeValue[i] = attributeFields[i].get(value);
                                }
                            }
                        }

                    } else {
                        if(getAttributeMethods == null) {
                            getCompositeAttributeMethods(key, value);
                        }
                        for(int i=0; i< attributeNamesArray.length; i++){
                            if(ivPOJOKeyAttributeIndexes[i]) {
                                compositeValue[i] = getAttributeMethods[i].invoke(key, emptyArray);
                            } else {
                                if(value == null) {
                                    compositeValue[i] = null;
                                } else {
                                    compositeValue[i] = getAttributeMethods[i].invoke(value, emptyArray);
                                }
                            }

                        }
                    }
                }
                if(inverseRangeIndexEnabled) {
                    attribute = compositeValue;
                } else {
                    attribute = new CompositeValue(compositeValue);
                }
                //if(indexName.indexOf("key1") > -1)
                //    System.out.println("getAttribute() -> attribute = "+attribute);

            } else {
                // entity case
                Object[] compositeValue = new Object[attributeNamesArray.length];
                for(int i=0; i< attributeNamesArray.length; i++){
                    switch (ivAttributeTypeIndexes[i]) {
                    case Constants.ATTRIBUTE_TYPE_VALUE_TUPLE:
                        compositeValue[i] = ((Tuple) value).getAttribute(ivTupleValueIndexes[i]);
                        break;
                    case Constants.ATTRIBUTE_TYPE_KEY_TUPLE:
                        compositeValue[i] = ((Tuple) key).getAttribute(ivTupleValueIndexes[i]);
                        break;
                    case Constants.ATTRIBUTE_TYPE_VALUE_ASSOCIATION:
                        // only allow toOne
                        compositeValue[i] = ((Tuple) value).getAssociation(ivTupleValueIndexes[i], 0);
                        break;
                    case Constants.ATTRIBUTE_TYPE_KEY_ASSOCIATION:
                        // only allow toOne
                        compositeValue[i] = ((Tuple) key).getAssociation(ivTupleValueIndexes[i], 0);
                        break;
                    default:
                        break;
                    }
                    /*
                    if(ivAttributeTypeIndexes[i] == ATTRIBUTE_TYPE_VALUE_TUPLE) {
                        compositeValue[i] = ((Tuple) value).getAttribute(ivTupleValueIndexes[i]);
                    } else if(ivAttributeTypeIndexes[i] == ATTRIBUTE_TYPE_KEY_TUPLE) {
                        compositeValue[i] = ((Tuple) key).getAttribute(ivTupleValueIndexes[i]);
                    } else if(ivAttributeTypeIndexes[i] == ATTRIBUTE_TYPE_VALUE_ASSOCIATION) {
                        // only allow toOne
                        compositeValue[i] = ((Tuple) value).getAssociation(ivTupleValueIndexes[i], 0);
                    } else if(ivAttributeTypeIndexes[i] == ATTRIBUTE_TYPE_KEY_ASSOCIATION) {
                        // only allow toOne
                        compositeValue[i] = ((Tuple) key).getAssociation(ivTupleValueIndexes[i], 0);
                    }
                    */
                }
                attribute = new CompositeValue(compositeValue);

                //if(indexName.indexOf("parentNode-rootNode") > -1)
                //    System.out.println("getAttribute() -> attribute = "+attribute);

            }

            return attribute;
        } catch (InvocationTargetException e) {
            FFDCFilter.processException(e, CLASS_NAME + ".getAttribute", "290");
            Throwable t = e.getTargetException();
            throw new ObjectGridRuntimeException(
                    "Caught unexpected Throwable during index update processing, index name = " + indexName + ": " + t,
                    t);
        } catch (Throwable t) {
            FFDCFilter.processException(t, CLASS_NAME + ".getAttribute", "297");
            throw new ObjectGridRuntimeException(
                    "Caught unexpected Throwable during index update processing, index name = " + indexName + ": " + t,
                    t);
        }
    }

    /**
     * This will return the ConpositeValue for search attributes when using InverseRangeIndex
     * 
     * @param key
     * @param value
     * @return
     * @throws ObjectGridRuntimeException
     */
    Object getCompositeSearchAttribute(Object key) throws ObjectGridRuntimeException {
        try {
            Object attribute = null;
            // if ivEntityMetadata == null, means POJO case
            Object[] compositeValue;
            if(useSerializer) {
                compositeValue = getSerializerCompositeAttribute(key, null);
            } else {
                compositeValue = new Object[searchAttributeNamesArray.length];
                if (ivFieldAccessAttribute) {
                    if(searchAttributeFields == null) {
                        getCompositeSearchAttributeFields(key);
                    }
                    for(int i=0; i< searchAttributeNamesArray.length; i++){
                            compositeValue[i] = searchAttributeFields[i].get(key);
                    }
                } else {
                    if(searchAttributeMethods == null) {
                        getCompositeSearchAttributeMethods(key);
                    }
                    for(int i=0; i< searchAttributeNamesArray.length; i++){
                            compositeValue[i] = searchAttributeMethods[i].invoke(key, emptyArray);
                    }
                }
            }
            return compositeValue;
        } catch (InvocationTargetException e) {
            FFDCFilter.processException(e, CLASS_NAME + ".getCompositeSearchAttribute", "390");
            Throwable t = e.getTargetException();
            throw new ObjectGridRuntimeException(
                    "Caught unexpected Throwable while setting search attribute, index name = " + indexName + ": " + t, t);
        } catch (Throwable t) {
            FFDCFilter.processException(t, CLASS_NAME + ".getCompositeSearchAttribute", "391");
            throw new ObjectGridRuntimeException(
                    "Caught unexpected Throwable while setting search attribute = " + indexName + ": " + t, t);
        }
    }
    
    /**
     * For the given key and value byte array, retrieve the values for the composite index.
     * The index values may be in the key and/or value in any order.
     * The index values may even have missing or null values.
     * For example: the composite index may be:<br>
     *   key.name, value.title, key.company
     *   value.name, value.title, key.company
     * The resulting Object[] will return the results in the order they were found.
     * @param key the SerializedKey or null
     * @param value the SerializedValue or null
     */
    private Object[] getSerializerCompositeAttribute(Object key, Object value) throws IOException {
        final DataObjectContext serCtx = ((BackingMapExtensions) backingMap).getSerializerContext();

        Object[] compositeKey = null;
        if (serAttributeContextKey != null) {
            XsDataInputStream is = null;
            try {
                is = ((SerializedEntryExtensions) key).getPooledInputStream();
                compositeKey = (Object[]) mapSerializer.getKeySerializerPlugin().inflateDataObjectAttributes(serCtx, is, serAttributeContextKey);
            }
            finally {
                if (is != null) {
                    xsDataStreamPool.returnInputStream(is); // Return stream
                }
            }
        }

        // Values may be null if the configuration allows it.
        Object[] compositeValue = null;
        if(value != null) {
            XsDataInputStream   is = null;
            if (serAttributeContextValue != null) {
                is = ((SerializedEntryExtensions) value).getPooledInputStream();
                try {
                    compositeValue = (Object[]) mapSerializer.getValueSerializerPlugin().inflateDataObjectAttributes(serCtx,is, serAttributeContextValue);
                }
                finally {
                    if (is != null) {
                        xsDataStreamPool.returnInputStream(is); // Return stream
                    }
                }
            }
        }

        // Merge the two together;
        Object[] result = new Object[attributeNamesArray.length];
        for (int i = 0, valIdx = 0, keyIdx = 0; i < attributeNamesArray.length; i++) {
            if (ivPOJOKeyAttributeIndexes[i]) {
                result[i] = compositeKey[keyIdx++];
            } else {
                // If the compositeValue/Value is null, then keep the null in the result.
                if(compositeValue != null)
                    result[i] = compositeValue[valIdx++];
            }
        }

        return result;
    }

    /**
     * In the case that we're using a serializer. A collection attribute
     * may be an array element as well.
     * @param attribute
     * @return
     */
    private final boolean isCollectionAttribute(Object attribute) {
        if (useSerializer) {
            // By flagging this attribute as a collection with the isCollectionAttribute
            // collections from serializers can processed without having to register the 
            // field in the descriptor list.
            return isCollectionAttribute = ( attribute instanceof Collection) ;
        } else {
            return isCollectionAttribute;
        }
    }


    private void updateIndex(Object key, Object oldValue, Object newValue) {
        if (isNonCompositeKeyIndex()) {
            // in the case of key indexing and no valid key attribute, the update on the value object that the key is
            // assocaited with
            // has no impact on the key index structure.
            return;
        }

        final boolean debugEnabled = ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled();

        Object oldAttribute = null;
        Object newAttribute = null;
        if(isComposite) {
            oldAttribute = getCompositeAttribute(key, oldValue);
            newAttribute = getCompositeAttribute(key, newValue);

            // When using InvreseRangeIndex; oldAttribute and newAttribute derived from getCompositeAttribute() will be replaced with InverseRangeKey
            if (inverseRangeIndexEnabled) {
                Object[] oldInverseRangeKey = inverseRangeIndex.getInverseRangeKey(oldAttribute);
                oldAttribute = oldInverseRangeKey;

                Object[] newInverseRangeKey = inverseRangeIndex.getInverseRangeKey(newAttribute);
                newAttribute = newInverseRangeKey;
            }
        } else {
            oldAttribute = getAttribute(key, oldValue);
            newAttribute = getAttribute(key, newValue);
        }

        boolean isOldCollectionAttr = isCollectionAttribute(oldAttribute);
        boolean isNewCollectionAttr = isCollectionAttribute(newAttribute);
        if (useSerializer) {
            if (oldAttribute != null && oldAttribute.getClass().isArray()) {
                // We don't let this go through
                throw new ObjectGridRuntimeException(new IllegalArgumentException("The attribute value array " + oldAttribute.getClass().getName() + " is not supported for " +
                        "the index [" + indexName + "]."));
            }
            if (newAttribute != null && newAttribute.getClass().isArray()) {
                // We don't let this go through
                throw new ObjectGridRuntimeException(new IllegalArgumentException("The attribute value array " + newAttribute.getClass().getName() + " is not supported for " +
                        "the index [" + indexName + "]."));
            }
        }

        // Was attributed changed by the update?
        if (isOldCollectionAttr && isNewCollectionAttr) {
            if (ivAssociationTupleIndex) {
                if (ObjectGridUtil.arrayEqualsIgnoreOrder((Object[]) oldAttribute, (Object[]) newAttribute)) {
                    if (debugEnabled) {
                        Tr.debug(tc, "update did not change attribute value");
                    }
                    return;
                }
            } else {
                // POJO
                if (debugEnabled) {
                    Tr.debug(tc, "update did not change attribute value");
                }
                if (ObjectGridUtil.isCollectionEqual((Collection) oldAttribute, (Collection) newAttribute)) {
                    return;
                }
            }

        } else {
            if ( (newAttribute != null && oldAttribute != null && oldAttribute.equals(newAttribute)) ||
                    (oldAttribute == null && newAttribute == null)) {
                // Nope, then nothing needs to be changed in index.
                if (debugEnabled) {
                    Tr.debug(tc, "update did not change attribute value");
                }
                return;
            }
        }

        if (isOldCollectionAttr || isNewCollectionAttr) {
            // handling oldAttribute
            if (isOldCollectionAttr && oldAttribute != null) {
                int oldSize = -1;
                Object[] oldCollectionElements = new Object[0];
                if (ivAssociationTupleIndex) {
                    oldCollectionElements = (Object[]) oldAttribute;
                    oldSize = oldCollectionElements.length;
                } else {
                    Collection collection = (Collection) oldAttribute;
                    oldSize = collection.size();
                    if (oldSize > 0) {
                        oldCollectionElements = ((Collection) oldAttribute).toArray();
                    }
                }

                // old size processing
                processRemoveFromCollectionSizeIndex(key, oldSize, "update", debugEnabled);

                // old elements processing
                for (int i = 0; i < oldSize; i++) {
                    processRemoveFromIndex(key, oldCollectionElements[i], "update", debugEnabled);
                }

            } else {
                // oldAttribute == null
                processRemoveFromIndex(key, null, "update", debugEnabled);
            }

            // handling newAttribute
            if (isNewCollectionAttr && newAttribute != null) {
                int newSize = -1;
                Object[] newCollectionElements = new Object[0];
                if (ivAssociationTupleIndex) {
                    newCollectionElements = (Object[]) newAttribute;
                    newSize = newCollectionElements.length;
                } else {
                    Collection collection = (Collection) newAttribute;
                    newSize = collection.size();
                    if (newSize > 0) {
                        newCollectionElements = ((Collection) newAttribute).toArray();
                    }
                }

                // new size processing
                processInsertIntoCollectionSizeIndex(key, newSize, "update", debugEnabled);

                // new elements processing
                for (int i = 0; i < newSize; i++) {
                    processInsertIntoIndex(key, newCollectionElements[i], "update", debugEnabled);
                }

            } else {
                // newAttribute == null
                processInsertIntoIndex(key, null, "insert", debugEnabled);
            }
            return; // if collection
        }

        processRemoveFromIndex(key, oldAttribute, "update", debugEnabled);

        processInsertIntoIndex(key, newAttribute, "update", debugEnabled);

    }

    private void processRemoveFromIndex(Object key, Object oldAttribute, String operation, boolean debugEnabled) {
        // Unless we restrict Loader to only access tables with non-nullable columns,
        // we have to handle the possibility that the attribute is null.

        if(oldAttribute == ATTRIBUTE_NOT_FOUND)
            return;

        Set oldKeys = null;
        if (oldAttribute != null) {
            if (useSerializer && oldAttribute.getClass().isArray()) {
                // We don't let this go through
                throw new ObjectGridRuntimeException(new IllegalArgumentException("The attribute value array " + oldAttribute.getClass().getName() + " is not supported for " +
                        "the index [" + indexName + "]."));
            }

            Map currentIndex = null;

            //InverseRangeIndex: key for currentindex will be only identifier portion of inverseRangeKey. 
            if(inverseRangeIndexEnabled) {
                
                Object[] inverseRangeKey = (Object[]) oldAttribute;
                Object identifier = inverseRangeKey[0];
                Object compositeRangeAttribute = inverseRangeKey[1];
                
                oldAttribute = compositeRangeAttribute;
                
                currentIndex = getCurrentIndex(identifier);
                
                try {
                    oldKeys = (Set) currentIndex.get(oldAttribute);
                } catch (KeyOverlapError k) {
                    // ignore this error, as we do not care if there is an overlap. oldKeys will be null in this case.
                }
            } else {
                currentIndex = getCurrentIndex(oldAttribute);
                // Remove oldAttribute from index entry.
                oldKeys = (Set) currentIndex.get(oldAttribute);
            }
            
            if (oldKeys != null) {
                oldKeys.remove(key);
                if (debugEnabled) {
                    Tr.debug(tc, operation + " removed key = " + key + " from keyset for attribute value = "
                            + oldAttribute);
                }
                if (oldKeys.isEmpty()) {
                    currentIndex.remove(oldAttribute);
                    if (debugEnabled) {
                        Tr.debug(tc, operation + " removed from index attribute value = " + oldAttribute);
                    }
                    
                    if(globalIndexEnabled) {
                    	globalIndex.update(oldAttribute, GlobalIndexMessage.GLOBAL_INDEX_OPERATION_TYPE_REMOVE);
                    }
                }                
            }
        } else {
            // handle oldAttribute==null case
            keysWithNullAttribute.remove(key);
            if (debugEnabled) {
                Tr.debug(tc, operation + " removed key = " + key + " from keysWithNullAttribute for attribute value = "
                        + oldAttribute);
            }
        }
    }

    private void insertIntoIndex(Object key, Object value) {
        final boolean debugEnabled = ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled();
        Object indexingAttribute;
        if (isNonCompositeKeyIndex()) {
            indexingAttribute = getAttribute(key, value);
        } else {
            if(isComposite) {
                indexingAttribute = getCompositeAttribute(key, value);
                // InverseRangeIndex: incoming indexingAttribute will be transformed into return Object array from getInverseRangeKey 
                if(inverseRangeIndexEnabled) {
                    Object[] inverseRangeKey = inverseRangeIndex.getInverseRangeKey(indexingAttribute);
                    indexingAttribute = inverseRangeKey;
                }
            } else {
                indexingAttribute = getAttribute(key, value);
            }


            if (isCollectionAttribute(indexingAttribute)) {
                // In the case of collection indexing, need to iterate through all elements and build size index too.
                // The getAttribute(value) will return either a collection (POJO case) or Tuple[] (entity case)
                // indexingAttribute = getAttribute(value);
                int size = -1;
                Object[] collectionElements = new Object[0];

                if (indexingAttribute != null) {
                    if (ivAssociationTupleIndex) {
                        // entity toMany association indexing
                        // Entity: in both empty and null collection case, the indexingAttribute will be null
                        collectionElements = (Object[]) indexingAttribute;
                        size = collectionElements.length;
                    } else {
                        // POJO association index or Serializer collection
                        // POJO: empty collection will be size = 0, null collection is null.
                        Collection collection = (Collection) indexingAttribute;
                        size = collection.size();
                        if (size > 0) {
                            collectionElements = ((Collection) indexingAttribute).toArray();
                        }
                    }

                    processInsertIntoCollectionSizeIndex(key, size, "insert", debugEnabled);

                    for (int i = 0; i < size; i++) {
                        // Note: this assume there is no "null" elements in the collectionElements
                        // otherwise, need to handle it
                        processInsertIntoIndex(key, collectionElements[i], "insert", debugEnabled);
                    }

                } else {
                    // need to handle null indexingAttribute case, this means a null toMany association.
                    // indexingAttribute == null
                    processInsertIntoIndex(key, null, "insert", debugEnabled);
                }

                return;
            }
        }

        processInsertIntoIndex(key, indexingAttribute, "insert", debugEnabled);
    }

    private void processInsertIntoIndex(Object key, Object indexingAttribute, String operation, boolean debugEnabled) {
        // Unless we restrict Loader to only access tables with non-nullable columns,
        // we have to handle the possibility that the attribute is null.
        //if(isComposite){
        //    System.out.println("processInsertIntoIndex -> indexName="+indexName + ", indexingAttribute="+indexingAttribute + ", attributeNames=" + ObjectGridUtil.getObjectArrayString(attributeNamesArray) + ", pid="+backingMap.getPartitionId());
        //}

        if(indexingAttribute == ATTRIBUTE_NOT_FOUND)
            return;

        if (indexingAttribute != null) {
            if (useSerializer && indexingAttribute.getClass().isArray()) {
                // We don't let this go through
                throw new ObjectGridRuntimeException(new IllegalArgumentException("The attribute value array " + indexingAttribute.getClass().getName() + " is not supported for " +
                        "the index [" + indexName + "]."));
            }

            Map currentIndex = null;
            Set keys = null;

            // InverseRangeIndex: incoming indexingAttribute will be Object array from getInverseRangeKey. identifier and CompositeRangeAttribute
            // will be split and finally indexingAttribute will be transformed into CompositeRangeAttribute value only for further processing.
            
            if(inverseRangeIndexEnabled) {
                
                Object[] inverseRangeKey = (Object[]) indexingAttribute;
                Object identifier = inverseRangeKey[0];
                Object compositeRangeAttribute = inverseRangeKey[1];
                indexingAttribute = compositeRangeAttribute;
                
                currentIndex = getCurrentIndex(identifier);
                try {
                    // Overlap can only be detected on a get(), but on a put() into currentIndex. 
                    // This is because compare() function will be invoked on get().
                    keys = (Set) currentIndex.get(indexingAttribute);
                    if(keys != null) {
                        throw new ObjectGridRuntimeException("put() failed with an error, overlapping key found", 
                                new KeyOverlapError("Existing Key with overlapping index values is : " + keys.toString()
                                + ". Index attributes from current key "+ key + " are : [Identifier " + identifier.toString() + "] " +
                                Arrays.toString(((CompositeRangeAttribute) indexingAttribute).getRangeValues())));
                    }
                } catch (KeyOverlapError k) {
                    // We do not allow overlaping range keys. So, if an OverlapError occur, this will be re-thrown as ObjectGridRuntimeException.
                    FFDCFilter.processException(k, CLASS_NAME + ".processInsertIntoIndex", "392");
                    throw new ObjectGridRuntimeException(
                            "put() failed with an error, overlapping key found.", k);
                }
                
            } else {
                currentIndex = getCurrentIndex(indexingAttribute);
                keys = (Set) currentIndex.get(indexingAttribute);
            }
            if (keys == null) {
                // Since different transactions can be updating different
                // Map entries and multiple map entries can have same attribute
                // value, we need to use a synchronized Set object to ensure only
                // one transaction at a time can make changes to the Set.
                keys = Collections.synchronizedSet(new LinkedHashSet());
                
                // the currentIndex could be either ConcurrentHashMap or ConcurrentSkipListMap
                // different trans may run into this block concurrently, and some may lost update.
                Set oldSet = null;
                if(currentIndex instanceof ConcurrentHashMap) {
                	oldSet = (Set) ((ConcurrentHashMap)currentIndex).putIfAbsent(indexingAttribute, keys);
                } else {
                	// must be ConcurrentSkipListMap
                	oldSet = (Set) ((ConcurrentSkipListMap)currentIndex).putIfAbsent(indexingAttribute, keys);
                }
                
            	if(oldSet != null) {
               		// another thread successfully insert new key set, this thread lost.
            		// use the old key set returned from putIfAbsent call.
            		// This also indicates, this thread did not create the key set for this attribute.
            		// It was done in another thread.
                	keys = oldSet;
                	if (debugEnabled) {
                        Tr.debug(tc, "index[" + indexName + "]: " + operation + " tried to create attribute key set but did not add attribute to index becasue another thread just did, attribute value = " + indexingAttribute);
                    } 
                } else {
                	// this thread successfullly inserts the created attribute key set into current index.
                	 if (debugEnabled) {
                         Tr.debug(tc, "index[" + indexName + "]: " + operation + " added attribute to index, attribute value = " + indexingAttribute);
                     } 
                     
                     if(globalIndexEnabled) {
                     	globalIndex.update(indexingAttribute, GlobalIndexMessage.GLOBAL_INDEX_OPERATION_TYPE_PUT);
                     }
                }
            }
 
            // Add key for this map entry to the Set of keys for the new attribute value.
            keys.add(key);
            
            if (debugEnabled) {
                Tr.debug(tc, "index[" + indexName + "]: " + operation + " added key = " + key + " to keyset for attribute value = "
                        + indexingAttribute);
            }
        } else {
            // handle null value. The TreeMap will run into NPE if the key is null.
            keysWithNullAttribute.add(key);
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, "index[" + indexName + "]: " + operation + " added key = " + key + " to keysWithNullAttribute for attribute value = "
                        + indexingAttribute);
            }
        }
    }

    private void processInsertIntoCollectionSizeIndex(Object key, int size, String operation, boolean debugEnabled) {
        // check if the size is in range
        initializeCollectionSizeIndex();
        if (size >= collectionSizeIndex.length) {
            // grow the collectionSizeIndex by every 100 until the size is in range
            synchronized (collectionSizeIndex) {
                if (size >= collectionSizeIndex.length) {
                    int oldSize = collectionSizeIndex.length;
                    int newSize = size + defaulSizeOfCollectionSizeIndex;
                    Set[] newCollectionSizeIndex = new Set[newSize];

                    if (newSize > 1) {
                        System.arraycopy(collectionSizeIndex, 0, newCollectionSizeIndex, 0, oldSize);
                    }

                    if (debugEnabled) {
                        Tr.debug(tc, indexName + ".collectionSizeIndex: growing collectionSizeIndex from " + oldSize
                                + " to " + newSize);
                    }

                    collectionSizeIndex = newCollectionSizeIndex;

                    collectionSizeIndex[size] = Collections.synchronizedSet(new LinkedHashSet());
                }
            }
        } else if (size < 0) {
            // this is unlikely occur; unless there is a error.
            throw new ObjectGridRuntimeException("Collection size indexing can not process size " + size
                    + " that is less than 0. indexName = " + indexName);
        }

        Set keys = collectionSizeIndex[size];
        if (keys == null) {
            // there may be multiple transaction run into this situation
            // need to ensure only one set created during contention.
            synchronized (collectionSizeIndex) {
                // try again, may be it is created by other tran.
                keys = collectionSizeIndex[size];
                if (keys == null) {
                    keys = Collections.synchronizedSet(new LinkedHashSet());
                    collectionSizeIndex[size] = keys;
                    if (debugEnabled) {
                        Tr.debug(tc, "collectionSizeIndex: " + operation
                                + " added key set to collectionSizeIndex, size = " + size);
                    }
                }
            }
        }

        // Add key for this map entry to the Set of keys for the new attribute value.
        keys.add(key);

        if (debugEnabled) {
            Tr.debug(tc, "collectionSizeIndex: " + operation + " added key = " + key + " to keyset for size = " + size);
        }
    }

    private void processRemoveFromCollectionSizeIndex(Object key, int size, String operation, boolean debugEnabled) {
        // during remove, the collectionSizeIndex[size] should not be null and the remove will not remove empty key set.
        // no need to synchronized on this method.
        Set keys = collectionSizeIndex[size];
        keys.remove(key);
        if (debugEnabled) {
            Tr.debug(tc, "collectionSizeIndex: " + operation + " removed key = " + key + " from keyset for size = "
                    + size);
        }

        // In the size indexing case, the set is likely to be created again
        // to avoid contention on creating a new set during insertion, just leave it alone.
        //
        // if (keys.isEmpty()) {
        // collectionSizeIndex[size] = null;
        // if (debugEnabled) {
        // Tr.debug(tc,"collectionSizeIndex: " + operation + " removed keyset from collectionSizeIndex size = " + size);
        // }
        // }
    }

    private void removeFromIndex(Object key, Object value) {

        Object indexingAttribute;

        if (isNonCompositeKeyIndex()) {
            indexingAttribute = getAttribute(key, value);
        } else {
            if(isComposite) {
                indexingAttribute = getCompositeAttribute(key, value);
                //InverseRangeIndex: Tranform indexingAttribute to an Object array of InverseRangeKey
                if (inverseRangeIndexEnabled) {
                    Object[] inverseRangeKey = inverseRangeIndex.getInverseRangeKey(indexingAttribute);
                    indexingAttribute = inverseRangeKey;
                }                
            } else {
                indexingAttribute = getAttribute(key, value);
            }
        }

        final boolean debugEnabled = ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled();

        if (isCollectionAttribute(indexingAttribute)) {
            Object oldAttribute = indexingAttribute;
            int oldSize = -1;
            Object[] oldCollectionElements = new Object[0];

            // handling oldAttribute
            if (oldAttribute != null) {
                if (ivAssociationTupleIndex) {
                    oldCollectionElements = (Object[]) oldAttribute;
                    oldSize = oldCollectionElements.length;
                } else {
                    Collection collection = (Collection) oldAttribute;
                    oldSize = collection.size();
                    if (oldSize > 0) {
                        oldCollectionElements = ((Collection) oldAttribute).toArray();
                    }
                }

                // old size processing
                processRemoveFromCollectionSizeIndex(key, oldSize, "remove", debugEnabled);

                // old elements processing
                for (int i = 0; i < oldSize; i++) {
                    processRemoveFromIndex(key, oldCollectionElements[i], "remove", debugEnabled);
                }

            } else {
                // oldAttribute == null
                processRemoveFromIndex(key, null, "remove", debugEnabled);
            }

            return;
        }

        processRemoveFromIndex(key, indexingAttribute, "remove", debugEnabled);
    }

    /*
     * (non-Javadoc)
     *
     * @see com.ibm.websphere.objectgrid.plugins.index.MapIndexPlugin#getName()
     */
    public String getName() {
        return indexName;
    }

    /**
     * Retrieves the attribute name or comma-delimited list
     * of attribute names that was set with the
     * {@link #setAttributeName(String)} method.
     *
     * @return the name specified as an argument to the
     *         <code>setAttributeName(String)</code> method or <code>null</code>
     *         if the <code>setAttributeName</code> method was not previously
     *         called or if more than one attribute is defined.
     *
     * @see #setAttributeName(String)
     * @see #getAttributeNames()
     *
     */

    public String getAttributeName() {
        return attributeName;
    }
    
    /**
     * Sets the comma-delimited set of attribute names to include in the index.<p>
     * 
     * The attribute name format is dependent on how the associated BackingMap is configured:
     * <ul>
     * <li><b>POJO</b> - A map that stores Java Objects without an EntityManager API entity or MapSerializerPlugin configured.  The HashIndex 
     * uses reflection to interact with the data in the cache objects.
     * <li><b>Entity</b> - A map that has an EntityManager API entity associated.  
     * <li><b>MapSerializerPlugin</b> - A map that has a MapSerializerPlugin configured.
     * </ul><p>
     * 
     * For POJO maps, attributes are accessed using reflection, using the field name or the property name as defined by the 
     * JavaBean specification.  The {@linkplain #setPOJOKeyIndex(boolean)} property identifies if the attribute is in the 
     * key or value of the cache entry.  The {@linkplain #setFieldAccessAttribute(boolean)} identifies how to access the cache object:  
     * by field or by getter method. <p>
     * 
     * For entity maps, attributes are accessed directly from the {@linkplain Tuple} object stored in the map.  The attribute name
     * must match the attribute name assocated in the EntityMetadata.<p>
     * 
     * For MapSerializerPlugin maps, the attribute names are passed directly to the {@linkplain DataAttributeInflatable#getAttributeContexts(String...)} method
     * and the attribute values are fetched using the 
     * {@linkplain DataAttributeInflatable#inflateDataObjectAttributes(DataObjectContext, XsDataInputStream, Object)}
     * method. The attribute name is a path to an attribute within the key or value.  
     * A key attribute is specified by including the "key" path prefix, or the prefix specified by the {@link MapDataDescriptor#getAddressableKeyName()} method. 
     * The path delmiter is specified in the {@link DataDescriptor#getPathSeparator()} for the DataDescriptor for each KeyDataSerializer or ValueSerializer.<p>
     * 
     * MapSerializerPlugin index attribute examples:<br>
     * (assuming the default addressible key name of "key" and path separator of ".")
     * <ul>
     * <li><code>name</code> - Indexes the "name" attribute in the value.
     * <li><code>key.ssn</code> - Indexes the "ssn" attribute in the key.
     * <li><code>address.country.code</code> - Indexes the "code" attribute from the "country" embedded type from the "address" embedded type in the value. 
     * </ul>
     * 
     *    
     * @see MapIndexPlugin#setAttributeName(String)
     */
    public void setAttributeName(String attributeName) {
        this.attributeName = attributeName;

        if (attributeName != null && attributeName.trim().indexOf(",") > 0) {
            setAttributeNamesArray(attributeName);
        } else {
            this.attributeNamesArray = new String[1];
            this.attributeNamesArray[0] = attributeName;
        }
    }

    /**
     * @see MapIndexPlugin#getIndexProxy(MapIndexInfo)
     */
    public Object getIndexProxy(MapIndexInfo map) {
        return getIndexProxy(map,false);
    }

    /**
     * Gets an index proxy object for performing index lookup operations, optionally
     * requesting upgradeable (U) locks when accessing maps during validation.
     * <p>
     * The caller must cast the object returned to either a <code>MapIndex</code> or
     * <code>MapRangeIndex</code> to perform the lookup operations.
     *
     * @param map the MapIndexInfo object required for maintaining the index.
     * @param forUpdate if true, request upgradeable (U) locks when validating
     * results, otherwise request shareable (S) locks.
     * @return a proxy to either an object that implements MapIndex or MapRangeIndex.
     *
     * @see MapIndex
     * @see MapRangeIndex
     * @see ObjectMap#getIndex(String)
     * @see MapIndexPlugin#getIndexProxy(MapIndexInfo)
     */
    public Object getIndexProxy(MapIndexInfo map, boolean forUpdate) {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.entry(tc, "getIndexProxy");
            Tr.exit(tc, "getIndexProxy");
        }

        if(!ivInitialized)
        {
            // check if this is a disk-backed map, if not, don't create derby indexes
            ivDiskBacked = ((BaseMap)map.getBackingMap()).getStorage() == Storage.DISK;
            if(indexName.equals(Constants.DC_DEPENDENCY_ID_INDEX) || indexName.equals(Constants.DC_TEMPLATE_INDEX)) {
                ivDiskBacked=false;
            }
            if(ivDiskBacked)
            {
                if(!isIndexValidForDiskBacked)
                {
                    throw new ObjectGridRuntimeException("Index "+indexName+" cannot be supported for a grid running in disk offload mode.");
                }
                Object cache=((ObjectMapImpl)map).getBaseMap().getCacheEntries();
                ivDiskHash=(IndexableHashtable)cache;
                if(ivTableIndex==null && ivDiskHash!=null) {
                    ivTableIndex=ivDiskHash.getIndex(indexName);
                }

            }
            ivInitialized=true;
        }
        Proxy indexProxy = (ivRangeIndex && !isComposite) ? new RangeProxy(map) : new Proxy(map);
        indexProxy.setForUpdate(forUpdate);

        return indexProxy;
    }

    /**
     * Introspect the indexed attribute and determine how to index it, based
     * on it's type, and cache the results.
     */
    private synchronized void cacheObjectInfo(Object o) throws ObjectGridRuntimeException {
        if (hasCachedObjectInfo) {
            return;
        }

        // check if comparable object.
        if (o instanceof Comparable) {
            isComparable = true;
        }
        if (o.getClass().isArray()) {
            isComposite = false;
        }

        if (o != null && USE_EQUALS_CLASSES.contains(o.getClass())) {
            useEqualsMethodForComparison = true;
        }

//        if (ivPOJOKeyIndex) {
//            try {
                if (isAttributeNameSet) {
//                    getAttribute(o);
                    // if no error occur, implies the key attribute is valid.
                    isAttributeIndex = true;
                }
//            } catch (ObjectGridRuntimeException e) {
//                throw e;
//            }
//        }

        // POJO relationship indexing support for "IS EMPTY" query.
        // Note: Entity case must be set in SetEntityMetadata() method
        if (o != null && o instanceof Collection) {
            isCollectionAttribute = true;
            initializeCollectionSizeIndex();
        }
        hasCachedObjectInfo = true;
    }

    private synchronized void checkCollectionAttributeIncluded(Object[] compositeValule) throws ObjectGridRuntimeException {
        for(int i=0; i< compositeValule.length; i++) {
            if(!ivPOJOCollectionAttributeChecks[i] && compositeValule[i] != null) {
                ivPOJOCollectionAttributeChecks[i] = true;
                
                if(!inverseRangeIndexEnabled && compositeValule[i] instanceof Collection) {
                    throw new ObjectGridRuntimeException("Multi-valued association: " + attributeNamesArray[i] + " is not supported in composite index: " + indexName);
                }
            }
        }

        boolean allChecksComplete = true;
        for(int i=0; i< ivPOJOCollectionAttributeChecks.length; i++) {
            if(!ivPOJOCollectionAttributeChecks[i]) {
                allChecksComplete = false;
            }
        }
        if (allChecksComplete) {
            ivCollectionAttributeChecksComplete = true;
        }
    }

    private final void initializeCollectionSizeIndex() {
        if(collectionSizeIndex == null) {
            Set[] s = collectionSizeIndex = new Set[defaulSizeOfCollectionSizeIndex];

            // initialize set on 0-10 that are most common cases.
            // this can reduce the chance of contention during insert
            for (int i = 0; i <= 10; i++) {
                s[i] = Collections.synchronizedSet(new LinkedHashSet());
            }

            synchronized(this) {
                if(collectionSizeIndex == null) {
                    collectionSizeIndex = s;
                }
            }
        }
    }

    /**
     * A class used to return a proxy to this map index so that applications can perform query operations using MapIndex
     * interface.
     */
    private class Proxy implements InternalMapIndex, TransactionQueryMatcher {
        /**
         * The MapIndexInfo object associated with this index proxy object.
         */
        MapIndexInfo ivMap;

        /**
         * Maximum number of retries when concurrent transactions modify index during a query operation.
         */
        static final int RETRY_LIMIT = 10;

        // this is a hint for index query optimization
        // if true:
        // when pesimistic map,
        // acquire U lock
        // otherwise acquire S lock
        // For optimistic map, always lock/release as before.

        private boolean forUpdate = false;

        final BaseMap ivBaseMap;

        private boolean doValidation;

        final private ObjectMapImpl ivObjectMap;

        /**
         * Construct a proxy object for a given ObjectMap.
         *
         * @param map
         *            is the MapIndexInfo object.
         */
        Proxy(MapIndexInfo map) {
            ivMap = map;
            if (map != null) {
                ivBaseMap = (BaseMap) map.getBackingMap();
                ivObjectMap = (ObjectMapImpl) map.getMap();
                // Don't validate if READ_UNCOMMITTED or READ_COMMITTED or if the map is read-only
                doValidation = !(ivObjectMap.getTransactionIsolationLevel() <= Session.TRANSACTION_READ_UNCOMMITTED || ivBaseMap
                        .getReadOnly());
            } else {
                // Static client-side index
                ivBaseMap = null;
                ivObjectMap = null;
                doValidation = true;
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, "doValidation = " + doValidation);
            }
        }

        public void setForUpdate(boolean isForUpdate) {
            this.forUpdate = isForUpdate;
        }

        /**
         * The doValidation flag is determined when creating Proxy object based on isolation level and isReadOnlyMap.
         * Because Query Engine would like to conditionally skip invalidation during index query execution. This method
         * allows it to turn on/off invalidation process. The setting of doValidation is proxy object scope. When this
         * method is used, the caller has to always use this method to ensure the doValidation value is desired.
         *
         *
         * @param doValidation
         *
         * @see InternalMapIndex#setDoValidation(boolean)
         */
        public void setDoValidation(boolean doValidation) {
            if (!(ivObjectMap.getTransactionIsolationLevel() <= Session.TRANSACTION_READ_UNCOMMITTED || ivBaseMap
                    .getReadOnly())) {
                this.doValidation = doValidation;
                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                    Tr.debug(tc, "set doValidation to " + doValidation);
                }
            } else {
                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                    Tr.debug(tc, "Can not override doValidation that is " + doValidation);
                }
            }
        }

        /**
         *
         * @see InternalMapIndex#isCollectionIndex()
         */
        public boolean isCollectionIndex() {
            // TODO DS: Can be both a collection and non-collection index.
            if(useSerializer)
                return true;

            return isCollectionAttribute;
        }

        /**
         *
         * @see InternalMapIndex#getCollectionSizeIndex()
         */
        public MapRangeIndex getCollectionSizeIndex() {
            return new SizeRangeProxy(ivMap);
        }


        /**
         * Check if a current transaction change should be included in the current query.  Note
         * that transaction changes corresponding to deletes/removes should not be passed to this
         * method as they should never be included in a query, so this method does not handle them.
         * @param logElement
         *          a log entry from the current transaction
         * @return
         *        true if the current change fits the query parameters, false otherwise
         */
        public boolean tranChangeMatchesQuery(Object logElement,Object comp)
        {
            ProxyComparator comparator=(ProxyComparator)comp;
            LogElement le=(LogElement)logElement;
            // Now fetch each map entry that was updated, inserted, deleted, or fetched from the loader
            // by current transaction and is not currently in the index
            // and determine if it should be included in result list.
            // DELETE log elements are handled elsewhere, they obviously will never match the current
            // query.
            Object key = le.getKey();
            try {
                if (comparator == null) {
                    // no range to check, just include it
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "including current tran change, entry key = " + key
                                    + " that is not indexed yet: " + indexName);
                    }
                    return true;
                } else {
                    // Now get the map entry for current key and verify it
                    // still matches the attributeValue argument.
                    Object attribute = null;

                    if (isNonCompositeKeyIndex()) {
                        if (!hasCachedObjectInfo) {
                            // key won't be null;
                            cacheObjectInfo(key);
                        }
                        // if this is a key index, should get the key attribute value
                        attribute = getAttribute(key, null);
                    } else {
                        // Object value = ivMap.getMap().get(key);
                        Object value = le.getCurrentValue();

                        if (!hasCachedObjectInfo && value!=null) {
                            cacheObjectInfo(value);
                        }

                        if(isComposite) {
                            attribute = getCompositeAttribute(key, value);
                            // InverseRangeIndex: Evaluate CompositeValue based on the SearchAttributes and transform attributeValue to InverseRangeSearchKey.
                            if (inverseRangeIndexEnabled) {
                                Object[] inverseRangeAttributes =  inverseRangeIndex.getInverseRangeKey(attribute);
                                attribute = inverseRangeAttributes;
                            }
                        } else {
                            attribute = getAttribute(key, value);
                        }
                    }
                    // Attribute in range?
                    if (comparator.inRange(attribute)) {
                        // Yep, include this change

                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc, "including current tran change entry (key,attribute) = (" + key + ","
                                    + attribute + ") that is not indexed yet: " + indexName);
                        }
                        return true;
                    } else {
                        // the updated value is not in range, removed from result set
                        return false;
                    }
                }

            } catch (Throwable e) {
                FFDCFilter.processException(e, CLASS_NAME + "$Proxy.applyingTransactionalChanges", "1603");
                Tr.warning(tc, NLSConstants.GENERAL_EXCEPTION_WARNING_CWOBJ0006, e);
                /*
                throw new FinderException(
                        "unexpected Throwable caught during index query processing, index name = " + indexName
                                + ": " + e, e);
                                */
                return false;
            }
        }


        /**
         *
         * @see MapIndex#findAll()
         */
        public Iterator findAll() throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "findAll");
            }

            long start=0;
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();


            if(ivDiskBacked)
            {
                TxID tranID;
                try {
                    tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                } catch (NoActiveTransactionException nate) {
                    throw new FinderException(nate);
                }
                // get an iterator which will return all derby rows not touched by the current XS transaction plus matching rows from the current XS transaction
                Iterator iterator=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindAll(tranID,this,
                                                                                                 ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),
                                                                                                 ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW));
                // TODO DS: The getTransactionChanges() method isn't fixed yet. It returns byte[] LogElements (user level keys). This plugin uses KeyBytess

                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }

                return iterator;
            }
            else
            {
                Set resultSet = Collections.EMPTY_SET;
                int retryCount = 0;
                boolean retry;
                boolean force = false;
                do {
                    // Variables that need to be re-initialize each time thru loop.
                    retry = false;

                    // should include keys with attribute value == null
                    Set s = null;//getAllIndexedKeys(null, force, index, true);
                    if(!isMultiTypes) {
                        s = getAllIndexedKeys(null, force, index, true);
                    } else {
                        s = getAllIndexedKeysFromTypeIndex(null, force, index, true);
                    }

                    Set keySet = null;

                    if (s != null) {
                        // Returns a copy of the processed set.
                        keySet = processSet(s,force);
                    }

                    if (keySet != null) {
                        resultSet = keySet;
                        continue;
                    } else {
                        // Whoops, another transaction modified Set obtained from index
                        // while the above was iterating over the Set to perform the
                        // addAll operation. Therefore, we need to retry by starting
                        // over beginning with getting Set from index to pickup changes
                        // from the transaction that just modified the Set.
                        ++retryCount;
                        if (retryCount >= RETRY_LIMIT) {
                            if (!force){
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "findAll reached retry limit and is locking to complete lookup: " + indexName);
                                }
                                force = true;
                            }else{
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "findAll reached retry limit and is throwing FinderException: " + indexName);
                                }
                                throw new FinderException(QUERY_RETRY_LIMIT_EXCEEDED);
                            }
                        }

                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc, "findAll retrying query since another tran modified index during query: "
                                    + indexName);
                        }
                        retry = true;
                    }
                } while (retry);

                // Return iterator for result list created by above loop.
                Iterator result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);

                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }

                if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                    Tr.exit(tc, "findAll returning: " + result);
                }

                return result;
            }

        } // end findAll
        
        /**
         * This internal method is to support multi-types mode, to get all keys for a given attribute type.
         * Used in testcase only.
         *
         * @param attributeTypeClass
         *
         * @see InternalMapIndex#findAll(Class)
         */
        public Iterator findAll(Class attributeTypeClass) throws FinderException {        	
            if(inverseRangeIndexEnabled) {throw new UnsupportedOperationException("findAll by attribute type is not suppported in InverseRangeIndex mode.");}
            if(!isMultiTypes) {throw new UnsupportedOperationException("findAll by attribute type is not suppported in non multi-type mode.");}
        	
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "findAll", attributeTypeClass);
            }

            long start=0;
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();


            if(ivDiskBacked)
            {
            	// note: this part still return all data (all types)
            	// TODO: 
            	// for now, for test purpose, only in memory mode need this function.
            	
                TxID tranID;
                try {
                    tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                } catch (NoActiveTransactionException nate) {
                    throw new FinderException(nate);
                }
                // get an iterator which will return all derby rows not touched by the current XS transaction plus matching rows from the current XS transaction
                Iterator iterator=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindAll(tranID,this,
                                                                                                 ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),
                                                                                                 ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW));
                // TODO DS: The getTransactionChanges() method isn't fixed yet. It returns byte[] LogElements (user level keys). This plugin uses KeyBytess

                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }

                return iterator;
            }
            else
            {
                Set resultSet = Collections.EMPTY_SET;
                int retryCount = 0;
                boolean retry;
                boolean force = false;
                do {
                    // Variables that need to be re-initialize each time thru loop.
                    retry = false;

                    // should include keys with attribute value == null
                    Set s = null;//getAllIndexedKeys(null, force, index, true);
                    
                    // the isMultiTypes shoudl be true.
                    Set completeSet = new LinkedHashSet();
                    Map currentTypeIndex = (Map) typeIndexesMap.get(attributeTypeClass.getName());
                    // when calling getAllIndexedKeys method, passed false includeKeysWithNullAttribute
                    // null is not a type, exclude.
                    if(currentTypeIndex!=null && currentTypeIndex.size()>0) {
                    	Set currentTypeIndexSet = getAllIndexedKeys(null, force, currentTypeIndex, false);
                    	completeSet = copySnapshotFromSet(completeSet, currentTypeIndexSet);                   
                    }
                    s = completeSet;
                    
                    Set keySet = null;

                    if (s != null) {
                        // Returns a copy of the processed set.
                        keySet = processSet(s,force);
                    }

                    if (keySet != null) {
                        resultSet = keySet;
                        continue;
                    } else {
                        // Whoops, another transaction modified Set obtained from index
                        // while the above was iterating over the Set to perform the
                        // addAll operation. Therefore, we need to retry by starting
                        // over beginning with getting Set from index to pickup changes
                        // from the transaction that just modified the Set.
                        ++retryCount;
                        if (retryCount >= RETRY_LIMIT) {
                            if (!force){
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "findAll reached retry limit and is locking to complete lookup: " + indexName);
                                }
                                force = true;
                            }else{
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "findAll reached retry limit and is throwing FinderException: " + indexName);
                                }
                                throw new FinderException(QUERY_RETRY_LIMIT_EXCEEDED);
                            }
                        }

                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc, "findAll retrying query since another tran modified index during query: "
                                    + indexName);
                        }
                        retry = true;
                    }
                } while (retry);

                // Return iterator for result list created by above loop.
                Iterator result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);

                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }

                if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                    Tr.exit(tc, "findAll returning: " + result);
                }

                return result;
            }
        }

        /**
         *
         * @see MapIndex#findAll(Object)
         */
        public Iterator findAll(Object attributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                if(isComposite && attributeValue!=null && attributeValue instanceof Object[]){
                    Tr.entry(tc, "findAll for composite value = " + ObjectGridUtil.getObjectArrayString((Object[]) attributeValue) + ", indexName = "
                            + indexName + ", attributeNames = " + ObjectGridUtil.getObjectArrayString(attributeNamesArray));
                } else {
                    Tr.entry(tc, "findAll for attributeValue = " + attributeValue + ", indexName = " + indexName + ", isComposite = " + isComposite);
                }
            }

            // setup the comparator object regardless of whether this is a disk backed or heap index
            ProxyComparator comparator=null;
            ProxyEQComparator eqComparator=new ProxyEQComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
            long start=0;
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();

            boolean isCompositeIndexPartialSearch = false;
            if(isComposite) {
                // InverseRangeIndex: Evaluate CompositeValue based on the SearchAttributes and transform attributeValue to InverseRangeSearchKey.
                if (inverseRangeIndexEnabled) {
                    InverseRangeSearchKey inverseRangeSearchKey = inverseRangeIndex.getInverseRangeSearchKey(getCompositeSearchAttribute(attributeValue));
                    attributeValue = inverseRangeSearchKey;
                    ProxyInverseRangeIndexComparator proxyInverseRangeIndexComparator = new ProxyInverseRangeIndexComparator();
                    proxyInverseRangeIndexComparator.ivInverseRangeSearchKey = inverseRangeSearchKey;
                    comparator = proxyInverseRangeIndexComparator;
                } else {
                    // handle composite index scenario ...
                    if(attributeValue == null) {
                        // in composite index case, the expected attributeValue can not be null
                        throw new IllegalArgumentException("The attributeValue cannot be null for the composite index [" + indexName + "].");

                    } else if(attributeValue instanceof Object[]) {
                        Object[] attributeValues = (Object[]) attributeValue;
                        if(attributeValues.length != attributeNamesArray.length) {
                            throw new IllegalArgumentException("The passed Object[] parameter length for the composite index [" + indexName + "] is " + attributeValues.length + ", but the  expected length is " + attributeNamesArray.length + ".");
                        } else {
                            for(int i=0; i< attributeValues.length; i++) {
                                if(attributeValues[i] != null && attributeValues[i].equals(MapIndex.EMPTY_VALUE)) {
                                    isCompositeIndexPartialSearch = true;
                                    break;
                                }
                            }

                            if(isCompositeIndexPartialSearch) {
                                eqComparator.isCompositeIndexPartialSearch = true;
                                eqComparator.attributeValues = attributeValues;
                            } else {
                                attributeValue = new CompositeValue((Object[]) attributeValue);
                            }
                        }

                    } else {// could be other type is future ...

                        // if unknown type, throw exception.
                        throw new IllegalArgumentException("The attributeValue class [" + attributeValue.getClass().getName() + "] is not supported for the composite index [" + indexName + "].");
                    }
                }
            }
            else {
                if (useSerializer) {
                    // We don't let this go through, it'll blow up anyway
                    if (attributeValue != null && attributeValue.getClass().isArray()) {
                        // if unknown type, throw exception.
                        throw new IllegalArgumentException("The attribute value array " + attributeValue.getClass().getName() + " is not supported for " +
                                "the index [" + indexName + "].");
                    }
                }
            }

            if(isCompositeIndexPartialSearch) {
                // ivEQ should have been set.
                // perform index scan ..
            } else {
                // unset composite index partial search related variables
                // otherwsie, the next findAll(Object)  may have error because user may or may not use Criteria
                eqComparator.isCompositeIndexPartialSearch = false;
                eqComparator.attributeValues = null;

                // Note: the only method supports non comparable key object is findAll()
                // to support non-comparable object in findAll() method, need a way to distinguish
                // comparable and non-comparable in ProxyEQComparator.
                // The isComparable has been checked once in checkExcutionInfo() method.
                // use the isComparable flag to determine either ivAttribute or nonComparableObject should be set.
                if (isComparable) {
                    eqComparator.ivAttribute = (Comparable) attributeValue;
                } else {
                    eqComparator.nonComparableObject = attributeValue;
                }

                if (isCollectionAttribute) {
                    eqComparator.nonComparableObject = attributeValue;
                }
                if(attributeValue==null)
                {
                    comparator=new ProxyEQNullComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                }
            }
            // if we didn't set the comparator to an EQNull comparator above, set it to the
            // regular EQ comparator now.
            if(comparator==null) {
                comparator=eqComparator;
            }

            Iterator result=null;
            if(ivDiskBacked)
            {
                if(isComposite)
                {
                    TxID tranID;
                    try {
                        tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                    } catch (NoActiveTransactionException nate) {
                        throw new FinderException(nate);
                    }
                    Iterator keyIter;
                    if(isCompositeIndexPartialSearch)
                    {
                        keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindAll(tranID,ivTableIndex,(Object[])attributeValue,attributeTypesArray,isCompositeIndexPartialSearch,this,ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW),comparator);
                        // TODO DS: The getTransactionChanges() method isn't fixed yet. It returns byte[] LogElements (user level keys). This plugin uses KeyBytess
                    }
                    else
                    {
                        keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindAll(tranID,ivTableIndex,((CompositeValue)attributeValue).getAttributeValues(),attributeTypesArray,isCompositeIndexPartialSearch,this,ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW),comparator);
                        // TODO DS: The getTransactionChanges() method isn't fixed yet. It returns byte[] LogElements (user level keys). This plugin uses KeyBytess
                    }
                    result=keyIter;
                }
                else
                {
                    TxID tranID;
                    try {
                        tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                    } catch (NoActiveTransactionException nate) {
                        throw new FinderException(nate);
                    }

                    Iterator keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindAll(tranID,ivTableIndex,attributeValue,attributeType,this,ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW),comparator);
                    // TODO DS: The getTransactionChanges() method isn't fixed yet. It returns byte[] LogElements (user level keys). This plugin uses KeyBytess
                    result=keyIter;
                }
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }

            }
            else
            {
                Set resultSet = Collections.EMPTY_SET;
                int retryCount = 0;
                boolean retry;
                boolean force = false;
                
                //InverseRangeIndex: As the Attribute value will be transformed in subsequent code, backup is needed for reties.
                Object backupAttributeValue=attributeValue;
                
                do {
                    // Variables that need to be re-initialize each time thru loop.
                    retry = false;
                    attributeValue=backupAttributeValue;

                    // Use index to obtains the Set of keys for map entries that
                    // contain the specified attribute value.
                    Set s = null;
                    Set keySet = null;
                    if (attributeValue == null) {
                        s = keysWithNullAttribute;
                        keySet = processSet(s, comparator,force);
                    } else {
                        if(isCompositeIndexPartialSearch) {
                            // perform index scan ..
                            s = getAllIndexedKeys(comparator, force, index, true);
                        } else {
                            Map currentIndex = null;
                            // InverseRangeIndex: Gets current index based on identifier then do further processing 
                            // to find the cache key that is in range with lookup key
                            if(inverseRangeIndexEnabled) {
                                currentIndex = getCurrentIndex(((InverseRangeSearchKey)attributeValue).getIdentifier());
                                CompositeRangeSearchAttribute compositeRangeSearchAttribute = ((InverseRangeSearchKey)attributeValue).getRangeSearchValues();
                                attributeValue = compositeRangeSearchAttribute;
                                ConcurrentSkipListMap concurrentMap = (ConcurrentSkipListMap) currentIndex;
                                Set cmset = concurrentMap.keySet();
                                CompositeRangeAttribute rangeKey = (CompositeRangeAttribute) concurrentMap.floorKey(attributeValue);
                                if((rangeKey == null)) {
                                    s = null;
                                } else if (rangeKey.isInRange(attributeValue)){
                                    s = (Set) currentIndex.get(rangeKey);
                                } else {
                                    s = null;
                                }
                            } else {
                                currentIndex = getCurrentIndex(attributeValue);
                                s = (Set) currentIndex.get(attributeValue);
                            }
                            
                        }

                        keySet = processSet(s, comparator, force);
                    }

                    if (keySet != null) {
                        resultSet = keySet;
                        continue;
                    } else {
                        // Whoops, another transaction modified Set obtained from index
                        // while the above was iterating over the Set to perform the
                        // addAll operation. Therefore, we need to retry by starting
                        // over beginning with getting Set from index to pickup changes
                        // from the transaction that just modified the Set.
                        ++retryCount;
                        if (retryCount >= RETRY_LIMIT) {
                            if (!force){
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "findAll reached retry limit and locking to complete lookup: " + indexName);
                                }
                                force = true;
                            }else{
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "findAll reached retry limit and is throwing FinderException: " + indexName);
                                }
                                throw new FinderException(QUERY_RETRY_LIMIT_EXCEEDED);
                            }
                        }

                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc, "findAll retrying query since another tran modified index during query: "
                                    + indexName);
                        }
                        retry = true;
                    }
                } while (retry);

                
                if(inverseRangeIndexEnabled && resultSet.size() > 1) {
                    throw new FinderException("Overlapping entries in InverseRangeIndex un-commited entries." + resultSet.toString());
                }
                // Return iterator for result list created by above loop.
                result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "findAll returning: " + result);
            }
            return result;
        } // end findAll

        /**
         * Determine if another transaction may have modified index causing attribute to no longer be within range.
         *
         * @param keyset
         *            is the keyset from the index.
         * @param c
         *            is the ProxyComparator that was used to make range check.
         * @param key
         *            is the key from the keyset.
         * @param attribute
         *            is the attribute value that was checked.
         *
         * @return boolean true if another transaction modified index.
         */
        private boolean concurrentModificationCheck(Set keyset, ProxyComparator c, Object key, Object attribute) {
            if (!keyset.contains(key)) {
                // key no longer in keyset, so another transaction changed keyset.
                return true;
            }

            if (c instanceof ProxyGTComparator && !isCollectionAttribute) {
                // greater than comparator was used. Since tailMap includes low value
                // rather than excluding, the out of range is not the result of concurrent
                // modification of another tran if attribute = low value of range.
                ProxyEQComparator ivEQ = new ProxyEQComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                ProxyGTComparator ivGT=(ProxyGTComparator)c;
                ivEQ.ivAttribute = ivGT.ivAttribute;
                return !(ivEQ.inRange(attribute));
            }

            // greater than comparator was used. so must be out of range
            // as result of some other transaction modifying the attribute value and index.
            return true;
        }

        /**
         * Process a Set obtained from index to determine if which of the keys are for map entries that meet the query
         * select criteria.
         *
         * @param s
         *            is the Set of key values for entries in BackingMap this index is built over. A null reference
         *            indicates only changes from current transaction needs to be processed.
         *
         * @param comparator
         *            is the comparator to use for making range check.
         *
         * @return Set of keys that met the select criteria or a null reference if a ConcurrentModificationException
         *         occurs while iterating over the Set.
         *
         * @throws FinderException
         *             if an error condition prevents processing of Set from being performed.
         */
        Set processSet(Set s, ProxyComparator comparator,boolean force) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "processSet, index = " + indexName + ", Set = " + s);
            }

            Set resultSet = invalidateSet(s, comparator, null,force);

            if (resultSet == null) {
                // The invalidateSet failed, need re-try,
                // return null, so the caller knows to re-try.
                return null;
            }

            applyingTransactionalChanges(s, resultSet, comparator);

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "processSet returning: " + resultSet);
            }
            return resultSet;
        }

        /**
         * Process a Set obtained from index to determine if which of the keys are for map entries that meet the query
         * select criteria.
         *
         * @param s
         *            is the Set of key values for entries in BackingMap this index is built over. A null reference
         *            indicates only changes from current transaction needs to be processed.
         *
         * @return Set of keys (a COPY) that met the select criteria or a null reference if a
         *         ConcurrentModificationException occurs while iterating over the Set.
         *
         * @throws FinderException
         *             if an error condition prevents processing of Set from being performed.
         */
        private Set processSet(Set s, boolean force) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "processSet, index = " + indexName + ", Set = " + s);
            }

            Set resultSet = invalidateSet(s, null, null, force);

            if (resultSet == null) {
                // The invalidateSet failed, need re-try,
                // return null, so the caller knows to re-try.
                return null;
            }

            applyingTransactionalChanges(s, resultSet, null);

            return resultSet;
        }

        /**
         * Process a Set obtained from index to determine if which of the keys are for map entries that meet the query
         * select criteria. This method is to support executeQueryForEndValue method.
         *
         * @param s
         *            is the Set of key values for entries in BackingMap this index is built over. A null reference
         *            indicates only changes from current transaction needs to be processed.
         *
         * @param eqComparator
         *            is the comparator to use for making range check on equal.
         *
         * @param tailerComparator
         *            is the comparator to use for making range check on the tailer side.
         *
         * @return Set of keys (a COPY) that met the select criteria or a null reference if a
         *         ConcurrentModificationException occurs while iterating over the Set.
         *
         * @throws FinderException
         *             if an error condition prevents processing of Set from being performed.
         */
        Set processSet(Set s, ProxyComparator eqComparator, ProxyComparator tailerComparator,ProxyEQComparator ivEQ,ProxyGTComparator ivGT,ProxyLTComparator ivLT,boolean force) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "processSet, index = " + indexName + ", Set = " + s);
            }
            Set resultSet = invalidateSet(s, eqComparator, tailerComparator,force);

            if (resultSet == null) {
                // The invalidateSet failed, need re-try,
                // return null, so the caller knows to re-try.
                return null;
            }

            if (resultSet.size() == 0) {
                // if the result set is empty, then need to re-set the attribute and comparator.
                ivEQ.ivAttribute = null;
                ivGT.ivAttribute = null;
                ivLT.ivAttribute = null;
            }

            // Now fetch each map entry that was updated, inserted, deleted, or fetched from the loader
            // by current transaction and determine if it should be included in result list.
            List diffs = ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW);
            Iterator iter = diffs.iterator();
            for (int i = diffs.size() - 1; i >= 0; --i) {
                LogElement le = (LogElement) iter.next();
                Object key = le.getKey();
                try {
                    if(le.getType() == LogElement.DELETE || le.getType()==LogElement.EVICT) {
                        // if deleted, remove from resultSet.
                        // it does not have impact on the current attributeValue
                        // if the resultSet becomes empty, it is fine, the caller will go to re-fetch next candidate logic
                        resultSet.remove(key);
                    } else {
                        // Now get the map entry for current key and verify it
                        // still matches the attributeValue argument.
                        Object attribute = null;

                        if (!isComposite && (ivKeyTupleAttributeIndex || ivPOJOKeyIndex ||  isKeySerializer)) {
                            // if this is a key index, should get the key attribute value
                            attribute = getAttribute(key, null);
                        } else {
                            // Object value = ivMap.getMap().get(key);
                            Object value = le.getCurrentValue();
                            if(isComposite) {
                                attribute = getCompositeAttribute(key, value);
                            } else {
                                attribute = getAttribute(key, value);
                            }
                        }

                        if (ivEQ.ivAttribute == null) {
                            // the current result set is empty, need to find the max/min from diffs.
                            // re-set on the 1st LogElement
                            ivEQ.ivAttribute = (Comparable) attribute;
                            ivGT.ivAttribute = (Comparable) attribute;
                            ivLT.ivAttribute = (Comparable) attribute;

                            // the above comparators are teh same references as eqComparator and tailerComparator
                            // the re-set also re-set both eqComparator and tailerComparator
                        }

                        // Attribute in range?
                        if (eqComparator.inRange(attribute)) {
                            // Yep, add this key to the results being returned.
                            resultSet.add(key);
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc, "including current tran change that is not indexed yet: " + indexName);
                            }
                        } else {
                            if (tailerComparator.inRange(attribute)) {
                                // Note: if the attribute value is greater than the current min/max value,
                                // need to update the current min and max value, clear result set,
                                // and add the current key to result set.
                                // note: the current tailerComparator could be ivGT or ivLT, update both.
                                ivEQ.ivAttribute = (Comparable) attribute;
                                ivGT.ivAttribute = (Comparable) attribute;
                                ivLT.ivAttribute = (Comparable) attribute;

                                resultSet.clear();
                                resultSet.add(key);
                            } else {
                                // the updated value is not in range, removed from result set
                                resultSet.remove(key);
                            }
                        }
                    }
                } catch (Throwable e) {
                    FFDCFilter.processException(e, CLASS_NAME + "$Proxy.processSet", "1274");
                    Tr.warning(tc, NLSConstants.GENERAL_EXCEPTION_WARNING_CWOBJ0006, e);
                    throw new FinderException(
                                              "unexpected Throwable caught during index query processing, index name = " + indexName
                                              + ": " + e, e);
                }
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "processSet returning: " + resultSet);
            }

            // At the end, if the result set is empty, implies there might be a need to re-fetch key sets
            // from index by the range of the transactional changes.
            // Another challenges:
            // 1. if the transactional changes is empty
            // 2. if empty set return from index when re-fetching, given the range of the transactional changes.
            // In the above cases, need to find a better way to fetch the the next min or max.

            return resultSet;
        }

        /**
         * get the keys array from the target index
         */
        Set getAllIndexedKeys(ProxyComparator comparator, boolean force, Map targetIndex, boolean includeKeysWithNullAttribute) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "getAllIndexedKeys");
            }

            Set s = new LinkedHashSet();
            int retryKeysInIndexCount = 0;
            boolean retryKeysInIndex = true;
            while (retryKeysInIndex && retryKeysInIndexCount < RETRY_LIMIT) {
                try {
                    if(comparator == null) {
                        // should include keys with attribute value == null
                        if(force) {
                            if(includeKeysWithNullAttribute) {
                                synchronized (keysWithNullAttribute) {
                                    s.addAll(keysWithNullAttribute);
                                }
                            }

                            synchronized (targetIndex) {
                                // add all keys from the index.
                                Collection valueCollection = targetIndex.values();
                                if (valueCollection != null) {
                                    Iterator iter = valueCollection.iterator();
                                    for (int i = valueCollection.size() - 1; i >= 0; --i) {
                                        Set currentSet = (Set) iter.next();
                                        synchronized (currentSet) {
                                            s.addAll(currentSet);
                                        }
                                    }
                                }
                            }
                        } else {
                            if(includeKeysWithNullAttribute) {
                                s = copySnapshotFromSet(s, keysWithNullAttribute);
                            }

                            // add all keys from the index.
                            // Note: during iteration, there could be unexpected exception
                            // e.g. NoSuchElement, NPE
                            Collection valueCollection = targetIndex.values();
                            if (valueCollection != null) {
                                Iterator iter = valueCollection.iterator();
                                for (int i = valueCollection.size() - 1; i >= 0; --i) {
                                    Set currentSet = (Set) iter.next();
                                    s = copySnapshotFromSet(s, currentSet);
                                }
                            }
                        }

                        retryKeysInIndex = false;
                    } else {
                        // use comparator to do filteration
                        // does not need to deal with keysWithNullAttribute for now
                        // in composite index case, the indexedAttribute should be CompositeValue type
                        if (force) {
                            synchronized (targetIndex) {
                                Set indexedAttributeSet = targetIndex.keySet();
                                if (indexedAttributeSet != null) {
                                    Iterator iter = indexedAttributeSet.iterator();
                                    for (int i = indexedAttributeSet.size() - 1; i >= 0; --i) {
                                        Object indexedAttribut = iter.next();
                                        if (comparator.inRange(indexedAttribut)) {
                                            Set currentSet = (Set) targetIndex.get(indexedAttribut);
                                            synchronized (currentSet) {
                                                s.addAll(currentSet);
                                            }
                                        }
                                    }
                                }
                            }
                        } else {
                            Set indexedAttributeSet = targetIndex.keySet();
                            if (indexedAttributeSet != null) {
                                Iterator iter = indexedAttributeSet.iterator();
                                for (int i = indexedAttributeSet.size() - 1; i >= 0; --i) {
                                    Object indexedAttribut = iter.next();
                                    if (comparator.inRange(indexedAttribut)) {
                                        Set currentSet = (Set) targetIndex.get(indexedAttribut);
                                        s = copySnapshotFromSet(s, currentSet);
                                    }
                                }
                            }
                        }

                        retryKeysInIndex = false;
                    }

                } catch (ConcurrentModificationException e) {
                    // This exception occurs when the toArray method attempts to
                    // interate over the Set of keys being added and another
                    // transaction commits and modifies the Set object during
                    // commit processing. Therefore, we need to return a null
                    // reference to inform caller to retry the query since a
                    // concurrent modification occured on the Set.
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "getAllIndexedKeys 1331: ConcurrentModificationException occured for index: "
                                + indexName);
                    }

                    retryKeysInIndexCount++;
                } catch (Throwable e) {
                    // Could be NPE when other tran also modifying index
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "getAllIndexedKeys 1442: unexpected error occured for index: " + indexName
                                + ", error: " + e.getClass().getName() + ": " + e.getMessage());
                    }

                    retryKeysInIndexCount++;
                }
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "getAllIndexedKeys partition " + backingMap.getPartitionId() + " returning " + s);
            }
            return s;
        }

        Set getAllIndexedKeysFromTypeIndex(ProxyComparator comparator, boolean force, Map<Object, Set> targetIndex,
                                           boolean includeKeysWithNullAttribute) {
            Set completeSet = new LinkedHashSet();
            Collection valueCollection = typeIndexesMap.values();
            if (valueCollection != null && valueCollection.size() != 0) {
                Iterator iter = valueCollection.iterator();
                for (int i = valueCollection.size() - 1; i >= 0; --i) {
                    Map currentTypeIndex = (Map) iter.next();
                    // when calling getAllIndexedKeys method, passed false includeKeysWithNullAttribute
                    // then later only need to add KeysWithNullAttribute set once.
                    Set s = getAllIndexedKeys(null, force, currentTypeIndex, false);
                    completeSet = copySnapshotFromSet(completeSet, s);
                }
            }

            if(includeKeysWithNullAttribute) {
                 completeSet = copySnapshotFromSet(completeSet, keysWithNullAttribute);
            }

             return completeSet;
        }

        /**
         * This method should be called by processSet. Its purpose is to verify if the keys have been altered by other
         * trans. If so, return null so that the caller knows the fact and re-try. The caller has to handle returned
         * null that indicates the invalidateSet failed and need re-try.
         *
         * @param s
         *            the set of keys to invalidate.
         * @return a COPY of of the input set of keys.
         * @throws FinderException
         */
        Set invalidateSet(Set s, ProxyComparator comparator, ProxyComparator additionalComparator, boolean force)
                throws FinderException {

            ArrayList keys = null;
            CacheEntry[] cacheEntries = null;
            int size = 0;
            try {
                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                    Tr.debug(tc, "Performing Invalidation: forUpdate=" + forUpdate + ", numKeys="
                            + (s == null ? 0 : s.size()));
                }

                keys = new ArrayList();

                // Add all of the keys in the Set to the ArrayList to
                // be processed by this method.
                if (s != null) {
                    if (force){
                        synchronized(s){
                            keys.addAll(s);
                        }
                    }else{
                        keys.addAll(s);
                    }

                }

                if (!doValidation) return new LinkedHashSet(keys);

                size = keys.size();
                if (size > 0) {
                    cacheEntries = ivObjectMap.getAllFromBaseMap(keys, forUpdate);
                }

            } catch (ConcurrentModificationException e) {
                // FFDCFilter.invalidateSet(e, CLASS_NAME +
                // "$Proxy.processSet", "517" );

                // This exception occurs when the addAll method attempts to
                // interate over the Set of keys being added and another
                // transaction commits and modifies the Set object during
                // commit processing. Therefore, we need to return a null
                // reference to inform caller to retry the query since a
                // concurrent modification occured on the Set.
                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                    Tr.debug(tc, "invalidateSet 1391: ConcurrentModificationException occured for index: " + indexName);
                }
                return null;
            } catch (TransactionException e) {
                Throwable rootCause = ObjectGridUtil.findRootException(e);
                throw new FinderException("TransactionException occurred during index query processing, index name = "
                        + indexName + ": " + e + ", root cause: " + rootCause.getClass().getName(), e);
            } catch (ObjectGridException e) {
                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                    Tr.debug(tc, "invalidateSet 1396: ObjectGridException occured for index: " + indexName);
                }
                return null;
            }

            // List cachedDiffsNoRemove = ivMap.getTransactionChanges(false);
            List/* <LogElement> */diffs = ivMap.getTransactionChanges(keys, OutputFormat.RAW, OutputFormat.RAW);
            Iterator iter = diffs.iterator();

            // Fetch map entry for key obtained from index and verify the attribute
            // in map entry matches attribute value. We want to avoid the situation
            // where index returned a key which meets the query but the value is then
            // modified by another transaction. When the application gets the key,
            // it'll see a value that doesn't meet the query. If another transaction
            // did modify the map entry so that it no longer has correct attribute value,
            // then we need restart the query from the beginning to pickup committed changes
            // applied to the index. Note, this algorithm is similar to what is done
            // by DBMS products such as DB2 on iSeries. The idea is to avoid using
            // transaction locks on the index entries since that was shown in the past
            // to result in very poor performance.

            Set resultSet = new LinkedHashSet();

            for (int i = 0; i < size; ++i) {
                // Get current transaction LogElement for the current key being processed.
                // Note, LogElement is null if current transaction has not touched
                // the map entry associated with current key.
                Object key = keys.get(i);
                LogElement diff = (LogElement) iter.next();
                // Warning: This LogElement will return SerializedKey for getKey() in KEY_TO_BYTES mode.

                try {
                    // Now get the map entry for current key
                    // Object value = ivMap.getMap().get(key);
                    // CacheEntry cacheEntry = ivBaseMap.getRaw(key);
                    CacheEntry cacheEntry = cacheEntries[i];

                    if (cacheEntry == null) {
                        // in key indexing case, value == null doesn't mean it is removed.

                        // Was value removed by current transaction?
                        if (diff == null) {
                            if (ivPOJOKeyIndex) {
                                // the current tran doesn't touch this key entry
                                resultSet.add(key);
                                continue;
                            }

                            // No LogElement for current transaction, which implies another
                            // transaction may have modified map entry for this key.
                            if (!s.contains(key)) {
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                                    Tr.exit(tc,
                                            "query must be retried, another tran removed an index entry being processed: "
                                                    + indexName);
                                }
                                return null;
                            }
                            // TODO DS: I think we need to add the key to the resultSet here (Kulack added March 1st, 2011)
                            resultSet.add(key);
                        } else {
                            Type type = diff.getType();
                            if (type == LogElement.DELETE || type == LogElement.EVICT || type == LogElement.CLEAR) {
                                // the current tran has removed this entry, but not commit yet.
                                // skip the invalidation on the current key.
                                continue;
                            }
                        }
                    } else {
                        // when getting value from BaseMap, it does not consider the current tran may remove/update
                        // the entry.
                        Type type = null;
                        if (diff != null) {
                            type = diff.getType();
                            if (type == LogElement.DELETE || type == LogElement.EVICT
                                    || type == LogElement.CLEAR) {
                                // the current tran has removed this entry, but not commit yet.
                                // skip the invalidation on the current key.
                                continue;
                            }
                        }

                        if (comparator == null) {
                            // the caller must be processSet(Set), that only supports findAll() method
                            // as long as the cacheEntry != null, the key should be included.
                            resultSet.add(key);
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc, "including map entry currently in index: " + indexName);
                            }
                        } else {
                            // Get attribute from value and verify it is within range.
                            Object attribute = null;

                            if (isNonCompositeKeyIndex()) {
                                // if this is a key index, should get the key attribute value
                                attribute = getAttribute(key, null);
                            } else {
                                Object value = null;
                                if (type == LogElement.UPDATE) {
                                    // update to the current value of the current tran's view
                                    value = diff.getCurrentValue();
                                } else {
                                    value = cacheEntry.getCommittedValue();
                                }
                                if(isComposite) {
                                    attribute = getCompositeAttribute(key, value);
                                    if(inverseRangeIndexEnabled) {
                                        Object[] inverseRangeKey = inverseRangeIndex.getInverseRangeKey(attribute);
                                        attribute = inverseRangeKey;
                                    }
                                } else {
                                    attribute = getAttribute(key, value);
                                }
                            }

                            // Attribute still in range?
                            if (comparator.inRange(attribute)) {
                                // Yep, add this key to the results being returned.
                                resultSet.add(key);
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "comparator.inRange()=true, including map entry (key,attribute) = ("
                                            + key + "," + attribute + ") currently in index: " + indexName);
                                }
                            } else {
                                if (additionalComparator != null) {
                                    if (additionalComparator.inRange(attribute)) {
                                        // The additionalComparator will not be null in findMax/Min case.
                                        // Called by processSet(Set s, ProxyComparator eqComparator, ProxyComparator
                                        // tailerComparator)
                                        resultSet.add(key);
                                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                            Tr.debug(tc,
                                                    "additionalComparator.inRange()=true, including map entry (key,attribute) = ("
                                                            + key + "," + attribute + ") currently in index: "
                                                            + indexName);
                                        }
                                    }
                                } else {
                                    // nope, not in range check using both comparator and additionalComparator
                                    // Determine whether some other transaction modified the attribute for this map
                                    // entry.
                                    if (diff == null) {
                                        // No DiffMapValue for this key, so we know that this transaction
                                        // did not modify attribute value. Determine whether another
                                        // transaction may have modified the attribute.
                                        if (concurrentModificationCheck(s, comparator, key, attribute)) {
                                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                                                Tr.exit(tc,
                                                        "query must be retried, another tran modified index entry being processed: "
                                                                + indexName + ", current (key,attribute) = (" + key
                                                                + "," + attribute + ")");
                                            }
                                            return null;
                                        }
                                    } else {
                                        // We have a LogElement for this key, so the current transaction
                                        // modified the attribute value. We know its changes are not applied
                                        // to index until it commits, so just leave this key out of the
                                        // results returned to caller and continue current loop.
                                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                            Tr.debug(tc,
                                                    "index entry ignored, current tran modified index entry being processed: "
                                                            + indexName + ", current (key,attribute) = (" + key + ","
                                                            + attribute + ")");
                                        }
                                    }
                                }

                            }
                        }
                    } // end else
                } catch (Throwable e) {
                    FFDCFilter.processException(e, CLASS_NAME + "$Proxy.invalidateSet", "1852");
                    Tr.warning(tc, NLSConstants.GENERAL_EXCEPTION_WARNING_CWOBJ0006, e);
                    throw new FinderException(
                            "unexpected Throwable caught during index query processing, index name = " + indexName
                                    + ": " + e, e);
                }

            } // end for

            return resultSet;

        } // end method

        /**
         * Apply transactional changes on the possible resultSet that will be returned to caller This is teh post
         * processing of the resultSet.
         *
         * @param originalSet
         * @param resultSet
         * @param comparator
         * @throws FinderException
         */
        void applyingTransactionalChanges(Set originalSet, Set resultSet, ProxyComparator comparator)
                throws FinderException {
            // Now fetch each map entry that was updated, inserted, deleted, or fetched from the loader
            // by current transaction and is not currently in the index
            // and determine if it should be included in result list.
            // Note: as the doValidation may be false, need to include DELETE log elements in transactional changes
            //       and process every log element
            List diffs = ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW);
            Iterator iter = diffs.iterator();
            for (int i = diffs.size() - 1; i >= 0; --i) {
                LogElement le = (LogElement) iter.next();
                Object key = le.getKey();
                if(le.getType() == LogElement.DELETE || le.getType()==LogElement.EVICT) {
                    resultSet.remove(key);
                } else {  //if (originalSet == null || (!originalSet.contains(key))) {
                    try {
                        if (comparator == null) {
                            // called from processSet(Set), no need to check if in range
                            // Just add to the result set becasue the diffs does not have remove element
                            resultSet.add(key);
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc, "including current tran change, entry key = " + key
                                        + " that is not indexed yet: " + indexName);
                            }
                        } else {
                            // Now get the map entry for current key and verify it
                            // still matches the attributeValue argument.
                            Object attribute = null;

                            if (isNonCompositeKeyIndex()) {
                                if (!hasCachedObjectInfo) {
                                    // key won't be null;
                                    cacheObjectInfo(key);
                                }

                                // if this is a key index, should get the key attribute value
                                attribute = getAttribute(key, null);
                            } else {
                                // Object value = ivMap.getMap().get(key);
                                Object value = le.getCurrentValue();

                                if (!hasCachedObjectInfo && value!=null) {
                                    cacheObjectInfo(value);
                                }

                                if(isComposite) {
                                    attribute = getCompositeAttribute(key, value);
                                    // InverseRangeIndex: Evaluate CompositeValue based on the SearchAttributes and transform attributeValue to InverseRangeSearchKey.
                                    if (inverseRangeIndexEnabled) {
                                        Object[] inverseRangeAttributes =  inverseRangeIndex.getInverseRangeKey(attribute);
                                        attribute = inverseRangeAttributes;
                                    }
                                } else {
                                    attribute = getAttribute(key, value);
                                }

                            }

                            // Attribute in range?
                            if (comparator.inRange(attribute)) {
                                // Yep, add this key to the results being returned.
                                resultSet.add(key);
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "including current tran change entry (key,attribute) = (" + key + ","
                                            + attribute + ") that is not indexed yet: " + indexName);
                                }
                            } else {
                                // the updated valule is not in range, removed from result set
                                resultSet.remove(key);
                            }
                        }

                    } catch (Throwable e) {
                        FFDCFilter.processException(e, CLASS_NAME + "$Proxy.applyingTransactionalChanges", "1603");
                        Tr.warning(tc, NLSConstants.GENERAL_EXCEPTION_WARNING_CWOBJ0006, e);
                        throw new FinderException(
                                "unexpected Throwable caught during index query processing, index name = " + indexName
                                        + ": " + e, e);
                    }
                }
            }
        }

        public boolean isCollectionSizeIndex() {
            // TODO Auto-generated method stub
            return false;
        }

        Set copySnapshotFromSet(Set completeSet, Set s) {
            if(s==null) return completeSet;

            int retrySetCount = 0;
            int maxRetrySetCount = 3;
            boolean retrySet;
            boolean lockSet = false;
            do {
                retrySet = false;
                try {
                    if(lockSet) {
                        synchronized (s) {
                            completeSet.addAll(s);
                        }
                    } else {
                        completeSet.addAll(s);
                    }
                } catch (ConcurrentModificationException e) {
                    countCME_attSet++;
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "getSnapshotFromSet 3893: attribute set ConcurrentModificationException occured for index: "
                                        + indexName + ", countCME_index = " + countCME_index + ", countCME_attSet = " + countCME_attSet);
                    }

                    if(retrySetCount==(maxRetrySetCount-1)) {
                        retrySetCount++;
                        retrySet = true;
                        lockSet = true;
                    } else if(retrySetCount==maxRetrySetCount) {
                        // indicates unable to get a complete set.
                        completeSet = null;
                        // exist the loop, otherwise will cause
                        // infinite loop.
                        retrySet = false;
                        break;
                    } else {
                        retrySetCount++;
                        retrySet = true;
                    }
                }
            } while (retrySet);
            return completeSet;
        }


    } // end class Proxy

    /**
     * A class used to return a proxy to this map index so that applications can perform query operations using
     * MapRangeIndex interface.
     */
    private class RangeProxy extends Proxy implements MapRangeIndex {

        /**
         * Index is a synchronized SortedMap.
         */
        /*final*/ private SortedMap ivIndexSortedMap;

        final private boolean ivSizeRangeIndex;

        /**
         * Construct a MapRangeIndex proxy.
         */
        RangeProxy(MapIndexInfo map) {
            super(map);
            ivIndexSortedMap = (SortedMap) index;
            ivSizeRangeIndex = false;
        }

        /**
         * Construct a MapRangeIndex proxy.
         */
        RangeProxy(MapIndexInfo map, boolean sizeRangeIndex) {
            super(map);
            ivIndexSortedMap = null;
            ivSizeRangeIndex = sizeRangeIndex;
        }

        private SortedMap getIndexSortedMap (Object attributeObejct) {
            if(!isMultiTypes) {
                //if the RangeIndex proxy was obtainbed before CLEAR command,
                //the ivIndexSortedMap will point to "CLEARED" index instance.
                // so need to always return the index from HashIndex instance.
                //return ivIndexSortedMap;

                return (SortedMap) index;
            } else {
                return getCurrentRangeIndex(attributeObejct);
            }
        }

        /**
         * Execute query operation on a specified Map and ProxyComparator object.
         *
         * @param map
         *            is a subset of the index to perform finder operation on.
         * @param proxyComparator
         *            is a comparator used to perform range check on attribute value.
         *
         * @return Set of keys required to be returned by finder method.
         *
         * @throws FinderException
         *             is any failure occurs during execution of the query.
         */
        private Set executeQuery(Map map, ProxyComparator proxyComparator) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "executeQuery");
            }

            Set resultSet = Collections.EMPTY_SET;
            int retryCount = 0;
            boolean retry;
            boolean force = false;
            do {
                // Variables that need to be re-initialize each time thru loop.
                retry = false;

                // Use index to obtains the Set of keys for map entries that
                // contain the specified attribute value.
                Collection values = map.values();
                int size = values.size();
                if (size == 0) {
                    // Nothing in range currently in index, so we only
                    // need to check changes from current transaction.
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "executeQuery found no entries in index in range, checking current tran changes: "
                                + indexName);
                    }
                    Set keySet = processSet(null, proxyComparator,force);
                    if (keySet != null) {
                        resultSet = keySet;
                        continue;
                    }
                } else {
                    // Index does contains some keys in range, so we need to query
                    // both index entries as well as current transaction changes.
                    Set completeSet = copySnapshotFromSubMap(map, force);
                    Set keySet = null;

                    if (completeSet != null) {
                        try {
                            keySet = processSet(completeSet, proxyComparator,force);
                        } catch (ConcurrentModificationException e) {
                            // FFDCFilter.processException( e, CLASS_NAME + "$RangeProxy.executeQuery", "625" );
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc, "executeQuery 1702: ConcurrentModificationException occured for index: "
                                        + indexName);
                            }

                            // Indicate unable to get keySet.
                            keySet = null;
                        }
                    }

                    if (keySet != null) {
                        resultSet = keySet;
                        continue;
                    } else {
                        // Whoops, another transaction modified Set obtained from index
                        // while the above was iterating over the Set to perform the
                        // addAll operation. Therefore, we need to retry by starting over
                        // beginning with getting Set from index to pickup changes from
                        // the transaction that just modified the Set.
                        if (indexPmi != null) {
                            indexPmi.incrementFindCollisionCount(indexName, 1);
                        }
                        HashIndexStatsModuleImpl statsModule = getStatsModule();
                        if (statsModule != null) {
                            statsModule.incrementFindCollisionCount(indexName, 1);
                        }
                        ++retryCount;
                        if (retryCount >= RETRY_LIMIT) {
                            if (!force){
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "executeQuery reached retry limit and is locking to finish lookup: "
                                            + indexName);
                                }
                                force = true;
                            }else{
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "executeQuery reached retry limit and is throwing FinderException: "
                                            + indexName);
                                }
                                if (indexPmi != null) {
                                    indexPmi.incrementFindFailureCount(indexName, 1);
                                }
                                if (statsModule != null) {
                                    statsModule.incrementFindFailureCount(indexName, 1);
                                }
                                throw new FinderException(QUERY_RETRY_LIMIT_EXCEEDED);
                            }
                        }

                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc, "executeQuery retrying query since another tran modified index during query: "
                                    + indexName);
                        }
                        retry = true;
                    }
                }

            } while (retry);

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "executeQuery returning: " + resultSet);
            }

            return resultSet;
        }

        private Set executeQuery(Map map, ProxyComparator proxyComparator, Set includedSet, Set excludedSet)
                throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "executeQuery");
            }

            Set resultSet = null;
            int retryCount = 0;
            boolean retry;
            boolean force = false;
            do {
                // Variables that need to be re-initialize each time thru loop.
                retry = false;
                resultSet = new LinkedHashSet();

                // Use index to obtains the Set of keys for map entries that
                // contain the specified attribute value.
                Collection values = map.values();
                int size = values.size();
                if (size == 0) {
                    // Nothing in range currently in index, so we only
                    // need to check changes from current transaction.
                    if (includedSet == null) {
                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc,
                                    "executeQuery found no entries in index in range, checking current tran changes: "
                                            + indexName);
                        }
                        Set keySet = processSet(null, proxyComparator,force);
                        if (keySet != null) {
                            resultSet.addAll(keySet);
                        }

                    } else {
                        Set keySet = processSet(includedSet, proxyComparator, force);
                        if (keySet != null) {
                            resultSet.addAll(keySet);
                        }
                    }

                    // if values isEmpty, no need to deal with excludedSet.

                } else {
                    // Index does contains some keys in range, so we need to query
                    // both index entries as well as current transaction changes.
                    Set completeSet = copySnapshotFromSubMap(map, force);
                    Set keySet = null;

                    if (completeSet != null) {
                        try {
                            if (includedSet != null) {
                                completeSet = copySnapshotFromSet(completeSet, includedSet);
                            }

                            if (excludedSet != null) {
                                if(force) {
                                    synchronized (includedSet) {
                                        completeSet.removeAll(excludedSet);
                                    }
                                } else {
                                    completeSet.removeAll(excludedSet);
                                }
                            }

                            keySet = processSet(completeSet, proxyComparator,force);
                        } catch (ConcurrentModificationException e) {
                            // FFDCFilter.processException( e, CLASS_NAME + "$RangeProxy.executeQuery", "625" );
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc, "executeQuery 1824: ConcurrentModificationException occured for index: "
                                        + indexName);
                            }

                            // Indicate unable to get keySet.
                            keySet = null;
                        }
                    }

                    if (keySet != null) {
                        resultSet.addAll(keySet);
                    } else {
                        // Whoops, another transaction modified Set obtained from index
                        // while the above was iterating over the Set to perform the
                        // addAll operation. Therefore, we need to retry by starting over
                        // beginning with getting Set from index to pickup changes from
                        // the transaction that just modified the Set.
                        if (indexPmi != null) {
                            indexPmi.incrementFindCollisionCount(indexName, 1);
                        }
                        HashIndexStatsModuleImpl statsModule = getStatsModule();
                        if (statsModule != null) {
                            statsModule.incrementFindCollisionCount(indexName, 1);
                        }
                        ++retryCount;
                        if (retryCount >= RETRY_LIMIT) {
                            if (!force){
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "executeQuery reached retry limit and is locking to finish lookup: "
                                            + indexName);
                                }
                                force=true;
                            }else{
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "executeQuery reached retry limit and is throwing FinderException: "
                                            + indexName);
                                }
                                if (indexPmi != null) {
                                    indexPmi.incrementFindFailureCount(indexName, 1);
                                }
                                if (statsModule != null) {
                                    statsModule.incrementFindFailureCount(indexName, 1);
                                }
                                throw new FinderException(QUERY_RETRY_LIMIT_EXCEEDED);
                            }
                        }

                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc, "executeQuery retrying query since another tran modified index during query: "
                                    + indexName);
                        }
                        retry = true;
                    }
                }

            } while (retry);

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "executeQuery returning: " + resultSet);
            }

            return resultSet;
        }

        private Set copySnapshotFromSubMap(Map map, boolean force) {
            Set completeSet = new LinkedHashSet();
            if(force) {
                Set attributeKeySet = new LinkedHashSet();

                // quickly copy key set from sub map first to avoid conflict during iteration pf keys.
                try {
                    // In the last try, lock the index structure to prevent structure modification on index.
                    // then, get the snapshop of attribute key set for further processing.
                    synchronized (ivIndexSortedMap) {
                        Set keySet = map.keySet();
                        attributeKeySet.addAll(keySet);
                    }

                }  catch (ConcurrentModificationException e) {
                    countCME_index++;
                    // this should not occur.
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "copySnapshotFromSubMap 3780: index structure ConcurrentModificationException occured for index: "
                                + indexName + ", force="+force + ", countCME_index = " + countCME_index + ", countCME_attSet = " + countCME_attSet);
                    }
                    // indicates unable to get a complete set.
                    completeSet = null;
                }

                // optional lock on each set to mitigate impact on each set.
                if(completeSet != null) {
                    int attributeKeySetSize = attributeKeySet.size();
                    Iterator attributeIter = attributeKeySet.iterator();

                    for (int i = attributeKeySetSize - 1; i >= 0; --i) {
                        Object attributeValue = attributeIter.next();
                        Set s = (Set) ivIndexSortedMap.get(attributeValue);
                        completeSet = copySnapshotFromSet(completeSet, s);
                    }
                }
            } else {

                Iterator iter = map.values().iterator();
                while (iter.hasNext()) {
                    try {
                        Set s = (Set) iter.next();
                        completeSet = copySnapshotFromSet(completeSet, s);
                    } catch (ConcurrentModificationException e) {
                        countCME_index++;
                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc, "copySnapshotFromSubMap 3808: index structure ConcurrentModificationException occured for index: "
                                    + indexName + ", force="+force + ", countCME_index = " + countCME_index + ", countCME_attSet = " + countCME_attSet);
                        }

                        // indicates unable to get a complete set.
                        completeSet = null;
                        // exist the loop, otherwise will cause infinite loop.
                        break;
                    } catch (NoSuchElementException e) {
                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc, "executeQuery 3818: NoSuchElementException occured for index: "
                                    + indexName + "force="+force);
                        }

                        // element has been removed during iteration
                        // iteration has no more elements
                        break;
                    }
                }
            }
            return completeSet;
        }

        /**
         * @see MapRangeIndex#findGreater(Object)
         */
        public Iterator findGreater(Object attributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "findGreater attributeValue = " + attributeValue);
            }

            if (attributeValue == null) {
                throw new IllegalArgumentException(
                        "the attributeValue argument of findGreater method of MapRangeIndex interface must be a non null reference");
            }

            long start=0;
            ProxyGTComparator comparator=new ProxyGTComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();
            // Use the greater than comparator for range check.
            comparator.ivAttribute = (Comparable) attributeValue;
            Iterator result=null;
            if(ivDiskBacked)
            {
                TxID tranID;
                try {
                    tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                } catch (NoActiveTransactionException nate) {
                    throw new FinderException(nate);
                }

                Iterator keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindGreaterOrLesser(tranID,ivTableIndex,attributeValue,attributeType,true,false,this,ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW),comparator);
                result=keyIter;

                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }

            }
            else
            {
                ivIndexSortedMap = getIndexSortedMap(attributeValue);

                SortedMap tailMap = ivIndexSortedMap.tailMap(attributeValue);
                Set resultSet = null;
                // as the invalidation process becomes conditional, need to handle whether to exclude lower end up front.
                Set excludedEndKeySet = (Set) tailMap.get(attributeValue);
                if (excludedEndKeySet != null) {
                    resultSet = executeQuery(tailMap, comparator, null, excludedEndKeySet);
                } else {
                    resultSet = executeQuery(tailMap, comparator);
                }
                result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);

                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }
            }
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "findGreater returning: " + result);
            }

            return result;
        }

        /**
         * @see MapRangeIndex#findGreaterEqual(Object)
         */
        public Iterator findGreaterEqual(Object attributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "findGreaterEqual attributeValue = " + attributeValue);
            }

            if (attributeValue == null) {
                throw new IllegalArgumentException(
                        "the attributeValue argument of findGreaterEqual method of MapRangeIndex interface must be a non null reference");
            }

            long start=0;
            ProxyGEComparator comparator=new ProxyGEComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();
            // Use the greater/equal comparator for range check.
            comparator.ivAttribute = (Comparable) attributeValue;
            Iterator result=null;
            if(ivDiskBacked)
            {
                TxID tranID;
                try {
                    tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                } catch (NoActiveTransactionException nate) {
                    throw new FinderException(nate);
                }
                Iterator keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindGreaterOrLesser(tranID,ivTableIndex,attributeValue,attributeType,true,true,this,ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW),comparator);

                result=keyIter;
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }
            }
            else
            {
                ivIndexSortedMap = getIndexSortedMap(attributeValue);

                SortedMap tailMap = ivIndexSortedMap.tailMap(attributeValue);
                Set resultSet = executeQuery(tailMap, comparator);
                result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }
            }
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "findGreaterEqual returning: " + result);
            }

            return result;
        }

        /**
         * @see MapRangeIndex#findLess(Object)
         */
        public Iterator findLess(Object attributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "findLess attributeValue = " + attributeValue);
            }

            if (attributeValue == null) {
                throw new IllegalArgumentException(
                        "the attributeValue argument of findLess method of MapRangeIndex interface must be a non null reference");
            }

            long start=0;
            ProxyLTComparator ivLT=new ProxyLTComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();
            // Use the less than comparator for range check.
            ivLT.ivAttribute = (Comparable) attributeValue;
            Iterator result=null;
            if(ivDiskBacked)
            {
                TxID tranID;
                try {
                    tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                } catch (NoActiveTransactionException nate) {
                    throw new FinderException(nate);
                }
                Iterator keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindGreaterOrLesser(tranID,ivTableIndex,attributeValue,attributeType,false,false,this,ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW),ivLT);

                result=keyIter;
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }
            }
            else
            {
                ivIndexSortedMap = getIndexSortedMap(attributeValue);

                SortedMap headMap = ivIndexSortedMap.headMap(attributeValue);
                Set resultSet = executeQuery(headMap, ivLT);
                result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);

                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }
            }
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "findLess returning: " + result);
            }

            return result;
        }

        /**
         * @see MapRangeIndex#findLessEqual(Object)
         */
        public Iterator findLessEqual(Object attributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "findLessEqual attributeValue = " + attributeValue);
            }

            if (attributeValue == null) {
                throw new IllegalArgumentException(
                        "the attributeValue argument of findLessEqual method of MapRangeIndex interface must be a non null reference");
            }

            long start=0;
            ProxyLEComparator ivLE=new ProxyLEComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();
            // Use the less/equal comparator for range check.
            ivLE.ivAttribute = (Comparable) attributeValue;
            Iterator result=null;
            if(ivDiskBacked)
            {
                TxID tranID;
                try {
                    tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                } catch (NoActiveTransactionException nate) {
                    throw new FinderException(nate);
                }

                Iterator keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindGreaterOrLesser(tranID,ivTableIndex,attributeValue,attributeType,false,true,this,ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW),ivLE);
                result=keyIter;
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }
            }
            else
            {
                Set resultSet;
                int retryCount = 0;
                boolean retry;
                boolean force = false;
                do {
                    // re-initialize for each retry that occurs.
                    retry = false;

                    ivIndexSortedMap = getIndexSortedMap(attributeValue);

                    SortedMap headMap = ivIndexSortedMap.headMap(attributeValue);
                    resultSet = executeQuery(headMap, ivLE);
                    Set s = (Set) ivIndexSortedMap.get(attributeValue);
                    ProxyEQComparator ivEQ = new ProxyEQComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                    ivEQ.ivAttribute = (Comparable) attributeValue;
                    Set equalSet = processSet(s, ivEQ,force);
                    if (equalSet != null) {
                        if (!equalSet.isEmpty()) {
                            resultSet.addAll(equalSet);
                        }
                    } else {
                        // Whoops, another transaction modified index while processSet
                        // was executing. Therefore, we need to retry the entire query.
                        ++retryCount;
                        retry = true;
                        if (retryCount >= RETRY_LIMIT) {
                            if (!force){
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "findLessEqual reached retry limit and is locking to finish the lookup: "
                                            + indexName);
                                }
                                force = true;
                            }else{
                                if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                    Tr.debug(tc, "findLessEqual reached retry limit and is throwing FinderException: "
                                            + indexName);
                                }
                                throw new FinderException(QUERY_RETRY_LIMIT_EXCEEDED);
                            }
                        }

                        if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                            Tr.debug(tc, "findLessEqual retrying query since another tran modified index during query: "
                                    + indexName);
                        }
                    }
                } while (retry);

                // Return iterator for result list created by above loop.
                result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);

                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }
            }
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "findLessEqual returning: " + result);
            }

            return result;
        }

        /**
         * @see MapRangeIndex#findRange(Object, Object)
         */
        public Iterator findRange(Object lowAttributeValue, Object highAttributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "findRange lowAttributeValue = " + lowAttributeValue + ", highAttributeValue = "
                        + highAttributeValue);
            }

            if (lowAttributeValue == null) {
                throw new IllegalArgumentException(
                        "the lowAttributeValue argument of findRange method of MapRangeIndex interface must be a non null reference");
            }

            if (highAttributeValue == null) {
                throw new IllegalArgumentException(
                        "the highAttributeValue argument of findRange method of MapRangeIndex interface must be a non null reference");
            }

            // Use the range comparator for range check.
            long start=0;
            ProxyRangeComparator ivRange=new ProxyRangeComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();
            ivRange.ivLowAttribute = (Comparable) lowAttributeValue;
            ivRange.ivHighAttribute = (Comparable) highAttributeValue;
            Iterator result;
            if(ivDiskBacked)
            {
                TxID tranID;
                try {
                    tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                } catch (NoActiveTransactionException nate) {
                    throw new FinderException(nate);
                }

                Iterator keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindRange(tranID,ivTableIndex,lowAttributeValue,highAttributeValue,attributeType,true,false,this,ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW),ivRange);
                result=keyIter;
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }
            }
            else
            {
                ivIndexSortedMap = getIndexSortedMap(null);
                // the typeIndex could be null, if the passed parameter is null
                // because the typeIndex may not be created.
                Map subMap = null;
                if(ivIndexSortedMap == null){
                    subMap = Collections.EMPTY_MAP;
                } else {
                    subMap = ivIndexSortedMap.subMap(lowAttributeValue, highAttributeValue);
                }

                //SortedMap subMap = ivIndexSortedMap.subMap(lowAttributeValue, highAttributeValue);
                Set resultSet = executeQuery(subMap, ivRange);
                result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "findRange returning: " + result);
            }

            return result;
        }

        /**
         * @see MapRangeIndex#findRangeInclusive(Object, Object)
         */
        public Iterator findRange(Object lowAttributeValue, Object highAttributeValue,
                boolean includesLowAttributeValue, boolean includesHighAttributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "findRange lowAttributeValue = " + lowAttributeValue + ", highAttributeValue = "
                        + highAttributeValue + ", includesLowAttributeValue = " + includesLowAttributeValue
                        + ", includesHighAttributeValue = " + includesHighAttributeValue);
            }

            // determine the comparator
            long start=0;
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();
            ProxyRangeComparator proxyRangeComparator = null;
            if (includesLowAttributeValue) {
                if (includesHighAttributeValue) {
                    ProxyRangeIncludesBothEndComparator ivRangeIncludesBothEnd = new ProxyRangeIncludesBothEndComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                    proxyRangeComparator = ivRangeIncludesBothEnd;
                } else {
                    // Note: the includesHighAttributeValue==false case is equivalent to findRange(Object, Object)
                    Iterator result = findRange(lowAttributeValue, highAttributeValue);
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                        Tr.exit(tc, "findRange returning: " + result);
                    }

                    return result;
                }

            } else {
                if (includesHighAttributeValue) {
                    ProxyRangeIncludesHighEndComparator ivRangeIncludesHighEnd = new ProxyRangeIncludesHighEndComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                    proxyRangeComparator = ivRangeIncludesHighEnd;
                } else {
                    ProxyRangeExcludesBothEndComparator ivRangeExcludesBothEnd = new ProxyRangeExcludesBothEndComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                    proxyRangeComparator = ivRangeExcludesBothEnd;
                }
            }

            // check is valid parameters
            if (lowAttributeValue == null) {
                throw new IllegalArgumentException(
                        "the findRange argument of findRangeInclusive method of MapRangeIndex interface must be a non null reference");
            }

            if (highAttributeValue == null) {
                throw new IllegalArgumentException(
                        "the findRange argument of findRangeInclusive method of MapRangeIndex interface must be a non null reference");
            }

            proxyRangeComparator.ivLowAttribute = (Comparable) lowAttributeValue;
            proxyRangeComparator.ivHighAttribute = (Comparable) highAttributeValue;

            Iterator result;
            if(ivDiskBacked)
            {
                TxID tranID;
                try {
                    tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                } catch (NoActiveTransactionException nate) {
                    throw new FinderException(nate);
                }

                Iterator keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindRange(tranID,ivTableIndex,lowAttributeValue,highAttributeValue,attributeType,includesLowAttributeValue,includesHighAttributeValue,this,ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW),ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW),proxyRangeComparator);
                result=keyIter;
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }

            }
            else
            {
                ivIndexSortedMap = getIndexSortedMap(null);

                SortedMap subMap = ivIndexSortedMap.subMap(lowAttributeValue, highAttributeValue);

                // The subMap is backed by the ivIndexSortedMap and it includes lowEnd, but exclude high end.
                // Need to deal with include/exclude lowEnd and highEnd.
                Set includedEndKeySet = null;
                Set excludedEndKeySet = null;
                if (!includesLowAttributeValue) {
                    // need to remove low end keys from the subMap.
                    excludedEndKeySet = (Set) ivIndexSortedMap.get(lowAttributeValue);
                }

                if (includesHighAttributeValue) {
                    includedEndKeySet = (Set) ivIndexSortedMap.get(highAttributeValue);
                }

                Set resultSet = executeQuery(subMap, proxyRangeComparator, includedEndKeySet, excludedEndKeySet);

                result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "findRange returning: " + result);
            }

            return result;
        }

        /**
         * retrieve the maximum value from a log of transaction changes.
         * @param logKeys an empty set which will be populated with the keys that match the max value found
         * @return the maximum value found in the log
         */
        private Comparable getMaxFromLog(Set logKeys,ProxyGTComparator ivGT,ProxyEQComparator ivEQ)
        {
            Iterator logIter=ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW).iterator();
            while(logIter.hasNext())
            {
                LogElement le=(LogElement)logIter.next();
                Object attribute;
                if (isNonCompositeKeyIndex()) {
                    // if this is a key index, should get the key attribute value
                    attribute = getAttribute(le.getKey(), null);
                } else {
                    // is composite even valid for a range query?  other index code implies not
                    if(isComposite) {
                        attribute = getCompositeAttribute(le.getKey(), le.getCurrentValue());
                    } else {
                        attribute = getAttribute(le.getKey(), le.getCurrentValue());
                    }
                }
                if(ivGT.ivAttribute==null)
                {
                    // this is our new high value, we didn't have one previously.
                    // update the comparators
                    ivGT.ivAttribute=(Comparable)attribute;
                    ivEQ.ivAttribute=(Comparable)attribute;
                    logKeys.add(le.getKey());
                }
                else if(ivGT.inRange(attribute))
                {
                    // we found a new high, clear the set and add this entry
                    // and update the comparators
                    ivGT.ivAttribute=(Comparable)attribute;
                    ivEQ.ivAttribute=(Comparable)attribute;
                    logKeys.clear();
                    logKeys.add(le.getKey());
                }
                else if(ivEQ.inRange(attribute))
                {
                    // this matches the existing high, add this entry
                    logKeys.add(le.getKey());
                }
            }
            return ivEQ.ivAttribute;

        }

        /**
         * retrieve the minimum value from a log of transaction changes.
         * @param logKeys an empty set which will be populated with the keys that match the min value found
         * @return the minimum value found in the log
         */
        private Comparable getMinFromLog(Set logKeys,ProxyLTComparator ivLT,ProxyEQComparator ivEQ)
        {
            Iterator logIter=ivMap.getTransactionChanges(false, OutputFormat.RAW, OutputFormat.RAW).iterator();
            while(logIter.hasNext())
            {
                LogElement le=(LogElement)logIter.next();
                Object attribute;
                if (isNonCompositeKeyIndex()) {
                    // if this is a key index, should get the key attribute value
                    attribute = getAttribute(le.getKey(), null);
                } else {
                    // is composite even valid for a range query?  other index code implies not
                    if(isComposite) {
                        attribute = getCompositeAttribute(le.getKey(), le.getCurrentValue());
                    } else {
                        attribute = getAttribute(le.getKey(), le.getCurrentValue());
                    }
                }
                if(ivLT.ivAttribute==null)
                {
                    // this is our new low value, we didn't have one previously.
                    // update the comparators
                    ivLT.ivAttribute=(Comparable)attribute;
                    ivEQ.ivAttribute=(Comparable)attribute;
                    logKeys.add(le.getKey());
                }
                else if(ivLT.inRange(attribute))
                {
                    // we found a new low, clear the set and add this entry
                    // and update the comparators
                    ivLT.ivAttribute=(Comparable)attribute;
                    ivEQ.ivAttribute=(Comparable)attribute;
                    logKeys.clear();
                    logKeys.add(le.getKey());
                }
                else if(ivEQ.inRange(attribute))
                {
                    // this matches the existing low, add this entry
                    logKeys.add(le.getKey());
                }
            }
            return ivEQ.ivAttribute;

        }

        /**
         * @see com.ibm.websphere.objectgrid.plugins.index.MapRangeIndex#findMax()
         */
        public Iterator findMax() throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "findMax");
            }

            long start=0;
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();
            Iterator result=null;

            if(ivDiskBacked)
            {
                TxID tranID;
                try {
                    tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                } catch (NoActiveTransactionException nate) {
                    throw new FinderException(nate);
                }
                Set logKeys=new HashSet();
                ProxyGTComparator ivGT = new ProxyGTComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                ProxyEQComparator ivEQ = new ProxyEQComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                ivGT.ivAttribute=null;
                ivEQ.ivAttribute=null;
                Comparable logMax=getMaxFromLog(logKeys,ivGT,ivEQ);
                // ivGT, ivEQ are now set to the max value found in the log entries.
                // next find the max value in derby.
                // additional results from transaction changes accounted for via set passed in
                Iterator keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindMax(tranID,ivTableIndex,logMax,logKeys,ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW));
                result=keyIter;
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }
            }
            else
            {
                Set resultSet = executeQueryForEndValue(true);
                result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "findMax returning: " + result);
            }

            return result;
        }

        /**
         * @see com.ibm.websphere.objectgrid.plugins.index.MapRangeIndex#findMin()
         */
        public Iterator findMin() throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "findMin");
            }
            long start=0;
            HashIndexStatsModuleImpl statsModule = getStatsModule();
            if(statsModule != null || indexPmi != null) start = System.currentTimeMillis();
            Iterator result=null;
            if(ivDiskBacked)
            {
                TxID tranID;
                try {
                    tranID = ((ObjectMapImpl)ivMap).getSession().getTxID();
                } catch (NoActiveTransactionException nate) {
                    throw new FinderException(nate);
                }
                Set logKeys=new HashSet();
                ProxyLTComparator ivLT = new ProxyLTComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                ProxyEQComparator ivEQ = new ProxyEQComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                ivLT.ivAttribute=null;
                ivEQ.ivAttribute=null;
                Comparable logMin=getMinFromLog(logKeys,ivLT,ivEQ);
                // ivGT, ivEQ are now set to the min value found in the log entries.
                // next find the min value in derby.
                // additional results from transaction changes accounted for via set passed in
                Iterator keyIter=((IndexableHashtable)ivBaseMap.getCacheEntries()).indexFindMin(tranID,ivTableIndex,logMin,logKeys,ivMap.getTransactionChanges(true, OutputFormat.RAW, OutputFormat.RAW));
                result=keyIter;
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                }
            }
            else
            {
                Set resultSet = executeQueryForEndValue(false);

                result = new CollectionIterator(resultSet, ivBaseMap.getSerializerContext(), ivBaseMap.getKeyOutputFormat() == OutputFormat.NATIVE);
                if (indexPmi != null) {
                    indexPmi.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    indexPmi.incrementFindCount(indexName, 1);
                    indexPmi.incrementFindResultCount(indexName, resultSet.size());
                }
                if (statsModule != null) {
                    statsModule.recordFindDurationTime(indexName, System.currentTimeMillis() - start);
                    statsModule.incrementFindCount(indexName, 1);
                    statsModule.incrementFindResultCount(indexName, resultSet.size());
                }
            }
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "findMin returning: " + result);
            }

            return result;
        }

        /**
         * support both findMax and findMin methods.
         *
         * @param isHighEnd
         *            True indicates the high end that is corresponding to findMax. False indicates the low end that is
         *            corresponding to findMin.
         *
         * @return Iterator of keys that have the end value.
         *
         * @throws FinderException
         *             is any failure occurs during execution of the query.
         */

        private Set executeQueryForEndValue(boolean isHighEnd) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "executeQueryForEndValue , isHighEnd = " + isHighEnd);
            }

            Object attributeValue = null;
            ProxyComparator tailerComparator = null;

            ProxyEQComparator ivEQ = new ProxyEQComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);

            ProxyGTComparator ivGT = new ProxyGTComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);

            ProxyLTComparator ivLT = new ProxyLTComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);

            if (isHighEnd) {
                tailerComparator = ivGT;
            } else {
                tailerComparator = ivLT;
            }
            attributeValue = getEndKeyAttributeValue(isHighEnd);

            Set resultSet = null;
            int retryCount = 0;
            boolean retry;
            int retryReFetchLimit = 100;
            int retryReFetch = 0;
            int reFetchOffset = 0;
            // temporary use the Object[] which is a static view,
            // later, may need to resolve concurrently modification issue.
            Object[] keysInIndex = null;
            boolean force = false;
            do {
                // Variables that need to be re-initialize each time through loop.
                retry = false;
                resultSet = new LinkedHashSet();
                Set keySet = null;

                if (attributeValue == null) {
                    ivEQ.ivAttribute = null;
                    ivGT.ivAttribute = null;
                    ivLT.ivAttribute = null;
                    keySet = processSet(null, ivEQ, tailerComparator,ivEQ,ivGT,ivLT,force);
                } else {
                    // attributeValue != null
                    //Set s = (Set) ivIndexSortedMap.get(attributeValue);
                    Set s = null;
                    ivIndexSortedMap = getIndexSortedMap(null);
                    if(ivIndexSortedMap == null) {
                        s = Collections.EMPTY_SET;
                    } else {
                        s = (Set) ivIndexSortedMap.get(attributeValue);
                    }

                    ivEQ.ivAttribute = (Comparable) attributeValue;
                    ivGT.ivAttribute = (Comparable) attributeValue;
                    ivLT.ivAttribute = (Comparable) attributeValue;

                    keySet = processSet(s, ivEQ, tailerComparator,ivEQ,ivGT,ivLT,force);

                    if (keySet != null) {
                        if (keySet.size() > 0) {
                            // verify the returned keySet first, if it is not valid, re-fetch
                            // need to check if the original attribute value has been changed.
                            // the final end attribute value is held by ivEQ.
                            if (!ivEQ.inRange(attributeValue) && tailerComparator.inRange(attributeValue)) {
                                // the original end attribute value is changed.
                                // the tailerComparator is ivGT when isHighEnd = true,
                                // or is ivLT when isHighEnd = false.
                                // the tailerComparator.inRange(attributeValue) indicats
                                // 1. the final end attribute value < the original value in the isHighEnd = true case
                                // 2. the final end attribute value > the original value in the isHighEnd = false case
                                // need to re-fetch
                                if (keysInIndex == null) {
                                    keysInIndex = getKeysFromIndex(force);
                                }

                                // if the keysInIndex is built successfully, go through re-fetch logic.
                                if (keysInIndex != null) {
                                    attributeValue = reFetchEndValueFromIndex(keysInIndex, reFetchOffset, isHighEnd);
                                    reFetchOffset++;
                                    // set the keySet to null to re-try.
                                    keySet = null;
                                    // after set the keySet to null, this will cause the retry on the while loop.
                                    // this type of retry should be counted in the re-fetch retry.
                                    // decrease the retryCount to compensate it.
                                    retryCount--;
                                } else {
                                    // could not build keysInIndex, re-try from the beginning.
                                    attributeValue = getEndKeyAttributeValue(isHighEnd);
                                    reFetchOffset = 0;
                                    keySet = null;
                                }
                                retryReFetch++;
                            }
                        } else {
                            // keySet.size()==0 && attributeValue!=null
                            // this implies the current end value is no longer valid and the current tran has nothing.
                            // However, because the current attributeValue!=null, means the index may have next end
                            // value.
                            // need to re-fetch from index for the next target attributeValue.
                            if (keysInIndex == null) {
                                keysInIndex = getKeysFromIndex(force);
                            }

                            if (keysInIndex != null) {
                                attributeValue = reFetchEndValueFromIndex(keysInIndex, reFetchOffset, isHighEnd);
                                reFetchOffset++;
                                // set the keySet to null to re-try.
                                keySet = null;
                                // after set the keySet to null, this will cause the retry on the while loop.
                                // this type of retry should be counted in the concurrency check retry.
                                // decrease the retryCount to compensate it.
                                retryCount--;
                            } else {
                                // could not build keysInIndex, re-try from the beginning.
                                attributeValue = getEndKeyAttributeValue(isHighEnd);
                                reFetchOffset = 0;
                                keySet = null;
                            }
                            retryReFetch++;
                        }

                    }

                }

                if (keySet != null) {
                    resultSet.addAll(keySet);
                } else {
                    if (indexPmi != null) {
                        indexPmi.incrementFindCollisionCount(indexName, 1);
                    }
                    HashIndexStatsModuleImpl statsModule = getStatsModule();
                    if (statsModule != null) {
                        statsModule.incrementFindCollisionCount(indexName, 1);
                    }
                    // Whoops, another transaction modified Set obtained from index
                    // while the above was iterating over the Set to perform the
                    // addAll operation. Therefore, we need to retry by starting
                    // over beginning with getting Set from index to pickup changes
                    // from the transaction that just modified the Set.
                    ++retryCount;
                    if (retryCount >= RETRY_LIMIT || retryReFetch >= retryReFetchLimit) {
                        if (!force){
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc,
                                        "executeQueryForEndValue reached retry limit and is locking to finish the lookup: "
                                        + indexName);
                            }
                            force = true;
                        }else{
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc,
                                        "executeQueryForEndValue reached retry limit and is throwing FinderException: "
                                        + indexName);
                            }
                            if (indexPmi != null) {
                                indexPmi.incrementFindFailureCount(indexName, 1);
                            }
                            if (statsModule != null) {
                                statsModule.incrementFindFailureCount(indexName, 1);
                            }
                            throw new FinderException(QUERY_RETRY_LIMIT_EXCEEDED);
                        }
                    }

                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc,
                                "executeQueryForEndValue retrying query since another tran modified index during query: "
                                        + indexName);
                    }
                    retry = true;
                }
            } while (retry);

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "executeQueryForEndValue returning: " + resultSet);
            }

            return resultSet;
        }

        /**
         * support both findMax and findMin methods
         *
         * @param keysInIndex
         *            The keys arrary of the index.
         *
         * @param reFetchOffset
         *            Used to calculate the position of the next end value.
         *
         * @param isHighEnd
         *            indicates whether high or low end re-fetch.
         *
         * @return Object of the next re-fetch end value.
         *
         */
        private Object reFetchEndValueFromIndex(Object[] keysInIndex, int reFetchOffset, boolean isHighEnd) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "reFetchEndValueFromIndex, index = " + indexName + ", keysInIndex = " + keysInIndex
                        + ", numOfReFetchFromIndex = " + reFetchOffset + ", isHighEnd = " + isHighEnd);
            }

            Object attributeValue = null;
            int indexOfFirst = 0;
            int indexOfLast = keysInIndex.length - 1;
            int nextIndex = 0;
            if (isHighEnd) {
                nextIndex = indexOfLast - reFetchOffset;
            } else {
                nextIndex = indexOfFirst + reFetchOffset;
            }

            if (nextIndex <= indexOfLast && nextIndex >= indexOfFirst) {
                attributeValue = keysInIndex[nextIndex];
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "reFetchEndValueFromIndex returning: " + attributeValue);
            }

            return attributeValue;
        }

        /**
         * get the keys array from the index
         *
         * @return keys array from the index
         */
        private Object[] getKeysFromIndex(boolean force) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "getKeysFromIndex");
            }
            Object[] keysInIndex = null;
            int retryKeysInIndexCount = 0;
            boolean retryKeysInIndex = true;
            while (retryKeysInIndex && retryKeysInIndexCount < RETRY_LIMIT) {
                try {
                    if(force) {
                        synchronized (ivIndexSortedMap) {
                            keysInIndex = ivIndexSortedMap.keySet().toArray();
                        }
                    } else {
                        keysInIndex = ivIndexSortedMap.keySet().toArray();
                    }

                    retryKeysInIndex = false;
                } catch (ConcurrentModificationException e) {
                    // This exception occurs when the toArray method attempts to
                    // interate over the Set of keys being added and another
                    // transaction commits and modifies the Set object during
                    // commit processing. Therefore, we need to return a null
                    // reference to inform caller to retry the query since a
                    // concurrent modification occured on the Set.
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "getKeysFromIndex 2372: ConcurrentModificationException occured for index: "
                                + indexName);
                    }
                    retryKeysInIndexCount++;
                }
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "getKeysFromIndex returning: " + keysInIndex);
            }

            return keysInIndex;
        }

        /**
         * Get the current high or low end value
         *
         * @param isHighEnd
         *            indicates whether high or low end attribute value to return.
         */
        private Object getEndKeyAttributeValue(boolean isHighEnd) {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "getEndKeyAttributeValue , isHighEnd = " + isHighEnd);
            }

            ivIndexSortedMap = getIndexSortedMap(null);
            if(ivIndexSortedMap == null) {
                // this indicates no data has ever inserted
                return null;
            }

            Object attributeValue = null;
            try {
                if (isHighEnd) {
                    attributeValue = ivIndexSortedMap.lastKey();
                } else {
                    attributeValue = ivIndexSortedMap.firstKey();
                }
            } catch (java.util.NoSuchElementException e) {
                // the index is empty, though this is abnormal case.
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "getEndKeyAttributeValue returning: " + attributeValue);
            }

            return attributeValue;
        }

    }

    final private class SizeRangeProxy extends RangeProxy {
        Set[] ivCollectionSizeIndex;

        /**
         * Construct a SizeRangeProxy proxy.
         */
        SizeRangeProxy(MapIndexInfo map) {
            super(map, true);
            if(ivDiskBacked)
            {
                throw new ObjectGridRuntimeException("SizeRange queries cannot be supported on index "+indexName+" cannot be supported for a grid running in diskoffload mode");
            }
            ivCollectionSizeIndex = collectionSizeIndex;
        }

        public Iterator findAll(Object attributeValue) throws FinderException {

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "SizeRangeProxy.findAll for size = " + attributeValue + ", indexName = " + indexName);
            }

            if (attributeValue == null) {
                throw new IllegalArgumentException(
                        "SizeRangeProxy: the attributeValue argument of findAll method must be a non null reference");
            }

            Integer sizeInteger = (Integer) attributeValue;
            int size = sizeInteger.intValue();

            boolean outOfRange = false;
            if (size >= ivCollectionSizeIndex.length) {
                // out of range, no furhter process
                outOfRange = true;
            }

            boolean sizeZeroInRange = false;
            if (size == 0) {
                sizeZeroInRange = true;
            }

            ProxyEQComparator ivEQ = new ProxyEQComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
            ivEQ.isCollectionSizeComparator = true;
            ivEQ.size = size;
            Set resultSet = Collections.EMPTY_SET;
            int retryCount = 0;
            boolean retry;
            boolean force = false;
            do {
                // Variables that need to be re-initialize each time thru loop.
                retry = false;
                Set s = Collections.EMPTY_SET;

                if (!outOfRange) {
                    s = ivCollectionSizeIndex[size];
                    if (s == null) {
                        s = Collections.EMPTY_SET;
                    }
                }

                Set keySet = null;
                if (sizeZeroInRange && ivAssociationTupleIndex) {
                    // special case
                    // in entity case, there won't be size==0, so the processSet(s, comparator) can be skipped.
                    ProxyEQNullComparator ivEQNull = new ProxyEQNullComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                    keySet = processSet(keysWithNullAttribute, ivEQNull,force);
                } else {
                    // normal case
                    keySet = processSet(s, ivEQ,force);
                    /*
                     * if(keySet != null){ // continute; otherwise, will go into re-try mode if(sizeZeroInRange &&
                     * ivAssociationTupleIndex) { // in entity toMany case, need to include null set // s =
                     * keysWithNullAttribute; Set keySet2 = processSet(keysWithNullAttribute, ivEQNull);
                     * if(keySet2==null){ // need re-try, set keySet to null keySet = null; } else {
                     * keySet.addAll(keySet2); } } }
                     */
                }

                if (keySet != null) {
                    resultSet = keySet;
                    continue;
                } else {
                    // Whoops, another transaction modified Set obtained from index
                    // while the above was iterating over the Set to perform the
                    // addAll operation. Therefore, we need to retry by starting
                    // over beginning with getting Set from index to pickup changes
                    // from the transaction that just modified the Set.
                    ++retryCount;
                    if (retryCount >= RETRY_LIMIT) {
                        if (!force){
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc, "SizeRangeProxy.findAll reached retry limit and is locking to finish the lookup: "
                                        + indexName);
                            }
                            force = true;
                        }else{
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc, "SizeRangeProxy.findAll reached retry limit and is throwing FinderException: "
                                        + indexName);
                            }
                            throw new FinderException("SizeRangeProxy: query retry limit exceeded");
                        }
                    }

                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc,
                                "SizeRangeProxy.findAll retrying query since another tran modified index during query: "
                                        + indexName);
                    }
                    retry = true;
                }
            } while (retry);

            // Return iterator for result list created by above loop.
            Iterator result = new CollectionIterator(resultSet);

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "SizeRangeProxy.findAll returning: " + result);
            }

            return result;
        }

        public Iterator findGreaterEqual(Object attributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "SizeRangeProxy.findGreaterEqual for size = " + attributeValue + ", indexName = "
                        + indexName);
            }

            if (attributeValue == null) {
                throw new IllegalArgumentException(
                        "SizeRangeProxy: the attributeValue argument of findGreaterEqual method must be a non null reference");
            }

            Integer sizeInteger = (Integer) attributeValue;
            int size = sizeInteger.intValue();

            boolean sizeZeroInRange = false;
            if (size == 0) {
                sizeZeroInRange = true;
            }

            // Use the greater than comparator for range check.
            ProxyGEComparator ivGE = new ProxyGEComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
            ivGE.isCollectionSizeComparator = true;
            ivGE.size = size;
            ProxyComparator comparator = ivGE;

            Set resultSet = Collections.EMPTY_SET;
            int retryCount = 0;
            boolean retry;
            boolean force = false;
            do {
                // Variables that need to be re-initialize each time thru loop.
                retry = false;
                Set s = getCompleteSet(size, (ivCollectionSizeIndex.length - 1), true, true);
                Set keySet = processSet(s, comparator,force);

                if (keySet != null) {
                    // continute; otherwise, will go into re-try mode
                    if (sizeZeroInRange && ivAssociationTupleIndex) {
                        // in entity toMany case, need to include null set
                        // s = keysWithNullAttribute;
                        ProxyEQNullComparator ivEQNull = new ProxyEQNullComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                        Set keySet2 = processSet(keysWithNullAttribute, ivEQNull, force);
                        if (keySet2 == null) {
                            // need re-try, set keySet to null
                            keySet = null;
                        } else {
                            keySet.addAll(keySet2);
                        }
                    }
                }

                if (keySet != null) {
                    resultSet = keySet;
                    continue;
                } else {
                    // Whoops, another transaction modified Set obtained from index
                    // while the above was iterating over the Set to perform the
                    // addAll operation. Therefore, we need to retry by starting
                    // over beginning with getting Set from index to pickup changes
                    // from the transaction that just modified the Set.
                    ++retryCount;
                    if (retryCount >= RETRY_LIMIT) {
                        if (!force){
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc,
                                        "SizeRangeProxy.findGreaterEqual reached retry limit and locking to finish the lookup: "
                                        + indexName);
                            }
                            force = true;
                        }else{
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc,
                                        "SizeRangeProxy.findGreaterEqual reached retry limit and is throwing FinderException: "
                                        + indexName);
                            }
                            throw new FinderException("SizeRangeProxy: query retry limit exceeded");
                        }
                    }

                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc,
                                "SizeRangeProxy.findGreaterEqual retrying query since another tran modified index during query: "
                                        + indexName);
                    }
                    retry = true;
                }
            } while (retry);

            // Return iterator for result list created by above loop.
            Iterator result = new CollectionIterator(resultSet);

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "SizeRangeProxy.findGreaterEqual returning: " + result);
            }

            return result;
        }

        public Iterator findAll() throws FinderException {
            throw new UnsupportedOperationException("SizeRangeProxy: findAll is not supported");
        }

        public Iterator findGreater(Object attributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "SizeRangeProxy.findGreater for size = " + attributeValue + ", indexName = " + indexName);
            }

            if (attributeValue == null) {
                throw new IllegalArgumentException(
                        "SizeRangeProxy.findGreater: the attributeValue argument of findGreater method must be a non null reference");
            }

            Integer sizeInteger = (Integer) attributeValue;
            int size = sizeInteger.intValue();

            boolean sizeZeroInRange = false;
            if (size < 0) {
                sizeZeroInRange = true;
            }

            // Use the greater comparator for range check.
            ProxyGTComparator ivGT = new ProxyGTComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
            ivGT.isCollectionSizeComparator = true;
            ivGT.size = size;
            ProxyComparator comparator = ivGT;

            Set resultSet = Collections.EMPTY_SET;
            int retryCount = 0;
            boolean retry;
            boolean force = false;
            do {
                // Variables that need to be re-initialize each time thru loop.
                retry = false;
                Set s = getCompleteSet(size, (ivCollectionSizeIndex.length - 1), false, true);
                Set keySet = processSet(s, comparator, force);

                if (keySet != null) {
                    // continute; otherwise, will go into re-try mode
                    if (sizeZeroInRange && ivAssociationTupleIndex) {
                        // in entity toMany case, need to include null set
                        // s = keysWithNullAttribute;
                        ProxyEQNullComparator ivEQNull = new ProxyEQNullComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                        Set keySet2 = processSet(keysWithNullAttribute, ivEQNull, force);
                        if (keySet2 == null) {
                            // need re-try, set keySet to null
                            keySet = null;
                        } else {
                            keySet.addAll(keySet2);
                        }
                    }
                }

                if (keySet != null) {
                    resultSet = keySet;
                    continue;
                } else {
                    // Whoops, another transaction modified Set obtained from index
                    // while the above was iterating over the Set to perform the
                    // addAll operation. Therefore, we need to retry by starting
                    // over beginning with getting Set from index to pickup changes
                    // from the transaction that just modified the Set.
                    ++retryCount;
                    if (retryCount >= RETRY_LIMIT) {
                        if (!force){
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc,
                                        "SizeRangeProxy.findGreater reached retry limit and is locking to finish the lookup: "
                                        + indexName);
                            }
                            force = true;
                        }else{
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc,
                                        "SizeRangeProxy.findGreater reached retry limit and is throwing FinderException: "
                                        + indexName);
                            }
                            throw new FinderException("SizeRangeProxy: query retry limit exceeded");
                        }
                    }

                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc,
                                "SizeRangeProxy.findGreater retrying query since another tran modified index during query: "
                                        + indexName);
                    }
                    retry = true;
                }
            } while (retry);

            // Return iterator for result list created by above loop.
            Iterator result = new CollectionIterator(resultSet);

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "SizeRangeProxy.findGreater returning: " + result);
            }

            return result;
        }

        public Iterator findLess(Object attributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "SizeRangeProxy.findLess for size = " + attributeValue + ", indexName = " + indexName);
            }

            if (attributeValue == null) {
                throw new IllegalArgumentException(
                        "SizeRangeProxy: the attributeValue argument of findLess method must be a non null reference");
            }

            Integer sizeInteger = (Integer) attributeValue;
            int size = sizeInteger.intValue();

            boolean sizeZeroInRange = false;
            if (size > 0) {
                sizeZeroInRange = true;
            }
            ProxyLTComparator ivLT = new ProxyLTComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
            ivLT.isCollectionSizeComparator = true;
            ivLT.size = size;
            ProxyComparator comparator = ivLT;

            Set resultSet = Collections.EMPTY_SET;
            int retryCount = 0;
            boolean retry;
            boolean force = false;
            do {
                // Variables that need to be re-initialize each time thru loop.
                retry = false;
                Set s = getCompleteSet(0, size, true, false);
                Set keySet = processSet(s, comparator, force);

                if (keySet != null) {
                    // continute; otherwise, will go into re-try mode
                    if (sizeZeroInRange && ivAssociationTupleIndex) {
                        // in entity toMany case, need to include null set
                        // s = keysWithNullAttribute;
                        ProxyEQNullComparator ivEQNull = new ProxyEQNullComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                        Set keySet2 = processSet(keysWithNullAttribute, ivEQNull,force);
                        if (keySet2 == null) {
                            // need re-try, set keySet to null
                            keySet = null;
                        } else {
                            keySet.addAll(keySet2);
                        }
                    }
                }

                if (keySet != null) {
                    resultSet = keySet;
                    continue;
                } else {
                    // Whoops, another transaction modified Set obtained from index
                    // while the above was iterating over the Set to perform the
                    // addAll operation. Therefore, we need to retry by starting
                    // over beginning with getting Set from index to pickup changes
                    // from the transaction that just modified the Set.
                    ++retryCount;
                    if (retryCount >= RETRY_LIMIT) {
                        if (!force){
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc,
                                        "SizeRangeProxy.findLess reached retry limit and is locking to finish the lookup: "
                                        + indexName);
                            }
                            force = true;
                        }else{
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc,
                                        "SizeRangeProxy.findLess reached retry limit and is throwing FinderException: "
                                        + indexName);
                            }
                            throw new FinderException("SizeRangeProxy: query retry limit exceeded");
                        }
                    }

                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc,
                                "SizeRangeProxy.findLess retrying query since another tran modified index during query: "
                                        + indexName);
                    }
                    retry = true;
                }
            } while (retry);

            // Return iterator for result list created by above loop.
            Iterator result = new CollectionIterator(resultSet);

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "SizeRangeProxy.findLess returning: " + result);
            }

            return result;
        }

        public Iterator findLessEqual(Object attributeValue) throws FinderException {
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr
                        .entry(tc, "SizeRangeProxy.findLessEqual for size = " + attributeValue + ", indexName = "
                                + indexName);
            }

            if (attributeValue == null) {
                throw new IllegalArgumentException(
                        "SizeRangeProx: the attributeValue argument of findLessEqual method must be a non null reference");
            }

            Integer sizeInteger = (Integer) attributeValue;
            int size = sizeInteger.intValue();

            boolean sizeZeroInRange = false;
            if (size >= 0) {
                sizeZeroInRange = true;
            }

            ProxyLEComparator ivLE = new ProxyLEComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);            ivLE.isCollectionSizeComparator = true;
            ivLE.size = size;
            ProxyComparator comparator = ivLE;

            Set resultSet = Collections.EMPTY_SET;
            int retryCount = 0;
            boolean retry;
            boolean force = false;
            do {
                // Variables that need to be re-initialize each time thru loop.
                retry = false;
                Set s = getCompleteSet(0, size, true, true);
                Set keySet = processSet(s, comparator,force);

                if (keySet != null) {
                    // continute; otherwise, will go into re-try mode
                    if (sizeZeroInRange && ivAssociationTupleIndex) {
                        // in entity toMany case, need to include null set
                        // s = keysWithNullAttribute;
                        ProxyEQNullComparator ivEQNull = new ProxyEQNullComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                        Set keySet2 = processSet(keysWithNullAttribute, ivEQNull,force);
                        if (keySet2 == null) {
                            // need re-try, set keySet to null
                            keySet = null;
                        } else {
                            keySet.addAll(keySet2);
                        }
                    }
                }

                if (keySet != null) {
                    resultSet = keySet;
                    continue;
                } else {
                    // Whoops, another transaction modified Set obtained from index
                    // while the above was iterating over the Set to perform the
                    // addAll operation. Therefore, we need to retry by starting
                    // over beginning with getting Set from index to pickup changes
                    // from the transaction that just modified the Set.
                    ++retryCount;
                    if (retryCount >= RETRY_LIMIT) {
                        if (!force){
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc,
                                        "SizeRangeProxy.findLessEqual reached retry limit and is locking to finish the lookup: "
                                        + indexName);
                            }
                            force = true;
                        }else{
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc,
                                        "SizeRangeProxy.findLessEqual reached retry limit and is throwing FinderException: "
                                        + indexName);
                            }
                            throw new FinderException("SizeRangeProxy: query retry limit exceeded");
                        }
                    }

                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc,
                                "SizeRangeProxy.findLessEqual retrying query since another tran modified index during query: "
                                        + indexName);
                    }
                    retry = true;
                }
            } while (retry);

            // Return iterator for result list created by above loop.
            Iterator result = new CollectionIterator(resultSet);

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "SizeRangeProxy.findLessEqual returning: " + result);
            }

            return result;
        }

        public Iterator findRange(Object lowAttributeValue, Object highAttributeValue) throws FinderException {
            return findRange(lowAttributeValue, highAttributeValue, true, false);
        }

        public Iterator findRange(Object lowAttributeValue, Object highAttributeValue,
                boolean includesLowAttributeValue, boolean includesHighAttributeValue) throws FinderException {

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.entry(tc, "SizeRangeProxy.findRange (" + lowAttributeValue + ", " + highAttributeValue
                        + ") , indexName = " + indexName);
            }

            if (lowAttributeValue == null) {
                throw new IllegalArgumentException(
                        "SizeRangeProxy.findRange: the lowAttributeValue argument of findRange method must be a non null reference");
            }

            if (highAttributeValue == null) {
                throw new IllegalArgumentException(
                        "SizeRangeProxy.findRange: the highAttributeValue argument of findRange method must be a non null reference");
            }

            Integer lowSizeInteger = (Integer) lowAttributeValue;
            Integer highSizeInteger = (Integer) highAttributeValue;
            int lowSize = lowSizeInteger.intValue();
            int highSize = highSizeInteger.intValue();

            if (lowSize > highSize) {
                return new CollectionIterator(Collections.EMPTY_SET);
            }

            boolean sizeZeroInRange = false;
            if (highSize > 0 || (highSize == 0 && includesHighAttributeValue)) {
                if (lowSize < 0 || (lowSize == 0 && includesLowAttributeValue)) {
                    sizeZeroInRange = true;
                }
            }

            // determine the comparator
            ProxyRangeComparator comparator = null;
            if (includesLowAttributeValue) {
                if (includesHighAttributeValue) {
                    ProxyRangeIncludesBothEndComparator ivRangeIncludesBothEnd = new ProxyRangeIncludesBothEndComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                    comparator = ivRangeIncludesBothEnd;
                } else {
                    ProxyRangeComparator ivRange = new ProxyRangeComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                    comparator = ivRange;
                }

            } else {
                if (includesHighAttributeValue) {
                    ProxyRangeIncludesHighEndComparator ivRangeIncludesHighEnd = new ProxyRangeIncludesHighEndComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                    comparator = ivRangeIncludesHighEnd;
                } else {
                    ProxyRangeExcludesBothEndComparator ivRangeExcludesBothEnd = new ProxyRangeExcludesBothEndComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                    comparator = ivRangeExcludesBothEnd;
                }
            }

            comparator.isCollectionSizeComparator = true;
            comparator.ivHighSize = highSize;
            comparator.ivLowSize = lowSize;

            Set resultSet = Collections.EMPTY_SET;
            int retryCount = 0;
            boolean retry;
            boolean force = false;
            do {
                // Variables that need to be re-initialize each time thru loop.
                retry = false;
                Set s = getCompleteSet(lowSize, highSize, includesLowAttributeValue, includesHighAttributeValue);
                Set keySet = processSet(s, comparator,force);

                if (keySet != null) {
                    // continute; otherwise, will go into re-try mode
                    if (sizeZeroInRange && ivAssociationTupleIndex) {
                        // in entity toMany case, need to include null set
                        // s = keysWithNullAttribute;
                        ProxyEQNullComparator ivEQNull = new ProxyEQNullComparator(ivAssociationTupleIndex,isCollectionAttribute,isComparable,useEqualsMethodForComparison);
                        Set keySet2 = processSet(keysWithNullAttribute, ivEQNull,force);
                        if (keySet2 == null) {
                            // need re-try, set keySet to null
                            keySet = null;
                        } else {
                            keySet.addAll(keySet2);
                        }
                    }
                }

                if (keySet != null) {
                    resultSet = keySet;
                    continue;
                } else {
                    // Whoops, another transaction modified Set obtained from index
                    // while the above was iterating over the Set to perform the
                    // addAll operation. Therefore, we need to retry by starting
                    // over beginning with getting Set from index to pickup changes
                    // from the transaction that just modified the Set.
                    ++retryCount;
                    if (retryCount >= RETRY_LIMIT) {
                        if (!force){
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc, "findLessEqual reached retry limit and is locking to finish the lookup: "
                                        + indexName);
                            }
                            force = true;
                        }else{
                            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                                Tr.debug(tc, "findLessEqual reached retry limit and is throwing FinderException: "
                                        + indexName);
                            }
                            throw new FinderException(QUERY_RETRY_LIMIT_EXCEEDED);
                        }
                    }

                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc, "findLessEqual retrying query since another tran modified index during query: "
                                + indexName);
                    }
                    retry = true;
                }
            } while (retry);

            // Return iterator for result list created by above loop.
            Iterator result = new CollectionIterator(resultSet);

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.exit(tc, "SizeRangeProxy.findLessEqual returning: " + result);
            }

            return result;
        }

        public Iterator findMax() throws FinderException {
            throw new UnsupportedOperationException("SizeRangeProxy: findMax is not supported");
        }

        /**
         * @see com.ibm.websphere.objectgrid.plugins.index.MapRangeIndex#findMin()
         */
        public Iterator findMin() throws FinderException {
            throw new UnsupportedOperationException("SizeRangeProxy: findMin is not supported");
        }

        /**
         * If return null, that means unexpected error occurred and the caller may have to re-try.
         *
         * @param start
         * @param end
         * @param includeStart
         * @param includeEnd
         * @return
         */
        private Set getCompleteSet(int start, int end, boolean includeStart, boolean includeEnd) {
            // The caller has to ensure the start and end are in the range (0, ivCollectionSizeIndex.length-1)
            Set completeSet = new LinkedHashSet();
            // the collection size index is based on int
            // > start is equivalent to >=(start+1), and so on
            // < end is equivalent to <=(end-1), and so on

            int startPoint = start;
            if (startPoint < 0) {
                startPoint = 0;
            } else {
                if (!includeStart) {
                    startPoint = startPoint + 1;
                }
            }

            int endPoint = end;
            if (endPoint > (ivCollectionSizeIndex.length - 1)) {
                endPoint = ivCollectionSizeIndex.length - 1;
            } else {
                if (!includeEnd) {
                    endPoint = end - 1;
                }
            }

            // the for loop should include both startPoint and endPoint that have been justified by includeStart and
            // includeEnd flag
            for (int i = startPoint; i <= endPoint; i++) {
                try {
                    Set s = ivCollectionSizeIndex[i];
                    if (s != null) {
                        completeSet = copySnapshotFromSet(completeSet, s);
                    }
                } catch (ConcurrentModificationException e) {
                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                        Tr.debug(tc,
                                "SizeRangeProxy.getCompleteSet: ConcurrentModificationException occured for size index: "
                                        + indexName);
                    }

                    // indicates unable to get a complete set.
                    completeSet = null;
                    // exist the loop, otherwise will cause infinite loop.
                    break;
                }
            }

            if (ObjectGridManagerImpl.isTraceEnabled && tc.isDebugEnabled()) {
                Tr.debug(tc, "SizeRangeProxy.getCompleteSet: start=" + start + ", end=" + end + ", includeStart="
                        + includeStart + ", includeEnd=" + includeEnd + ", completeSet = " + completeSet);
            }

            return completeSet;
        }
    }

    /* (non-Javadoc)
     * @see com.ibm.websphere.objectgrid.plugins.BackingMapPlugin#setBackingMap(com.ibm.websphere.objectgrid.BackingMap)
     */
    public void setBackingMap(BackingMap map) {
        this.backingMap = map;       
    }

    /* (non-Javadoc)
     * @see com.ibm.websphere.objectgrid.plugins.BackingMapPlugin#getBackingMap()
     */
    public BackingMap getBackingMap() {
        return backingMap;
    }

    /* (non-Javadoc)
     * @see com.ibm.websphere.objectgrid.lifecycle.BackingMapPlugin#initialize()
     */
    public void initialize() throws ObjectGridRuntimeException {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.debug(tc, "initialize idx=" + this.indexName + ", map=" + backingMap.getName());
        }

        if(state.testState(LifecycleState.State.INITIALIZING, Operation.LT)) {
            state.setState(LifecycleState.State.INITIALIZING);
            // Register ourselves with the map
            keyConf = (KeyConfig)this.backingMap;
            this.backingMap.addMapEventListener(bmapLifecycleListener);
            state.setState(LifecycleState.State.INITIALIZED);

            // d12187 - Allow attribute-less indexes for indexing the entire value.
            //if(!ivPOJOKeyIndex && (attributeName == null || attributeName.trim().length() == 0)) {
            //    throw new IllegalStateException("Unable to initialize index: " + getName() + ".  AttributeName not set.");
            //}
        }

        // in static index case, when this initialize() method is called, backingMap.getState() is INITIALIZING
        // in dynamic index case, the backingMap.getState() is ONLINE, because creating dynamic index only allowed on initialized map.
        if(backingMap.getState() == BackingMapLifecycleListener.State.ONLINE && this.mapSerializer == null) {
            // dynamic map case, need to process serializer related work.
            bmapLifecycleListener.processEventStateInitialized(backingMap);
        }        
    }

    /* (non-Javadoc)
     * @see com.ibm.websphere.objectgrid.plugins.Initializable#isInitialized()
     */
    public boolean isInitialized() {
        return state.testState(LifecycleState.State.INITIALIZED, Operation.GTEQ);
    }

    /* (non-Javadoc)
     * @see com.ibm.websphere.objectgrid.lifecycle.BackingMapPlugin#destroy()
     */
    public void destroy() {
        if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
            Tr.debug(tc, "destroy idx=" + this.indexName + ", map=" + backingMap.getName());
        }
        
        if(this.globalIndex != null) {
        	globalIndex.destroy();
        }
        
        if(state.testState(LifecycleState.State.DESTROYING, Operation.LT)) {
            state.setState(LifecycleState.State.DESTROYING);

            state.setState(LifecycleState.State.DESTROYED);
        }
    }

    /* (non-Javadoc)
     * @see com.ibm.websphere.objectgrid.plugins.Destroyable#isDestroyed()
     */
    public boolean isDestroyed() {
        return state.testState(LifecycleState.State.DESTROYED, Operation.GTEQ);
    }


    /**
     * Using a private inner class to not expose the fact that the HashIndex is also a listener.
     */
    private class BMapLifecycleListener implements BackingMapLifecycleListenerInternal {
        /* (non-Javadoc)
         * @see com.ibm.websphere.objectgrid.plugins.MapLifecycleListener#backingMapStateChanged(com.ibm.websphere.objectgrid.plugins.MapLifecycleListener.State, com.ibm.websphere.objectgrid.BackingMap)
         */
        public void backingMapStateChanged(BackingMapLifecycleListener.LifecycleEvent event) {
            BackingMap map = event.getBackingMap();
            if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                Tr.debug(tc, "backingMapStateChanged idx=" + HashIndex.this.indexName + ", event=" + event.getState() + ", map=" + map.getName());
            }

            // Some plug-ins are only available after various stages of map start...

            try {
                // TODO DS:  Should we instead have a start event for this plugin?  So we can fail it?  Look at spring/javabean etc.
                switch(event.getState()) {
                case INITIALIZED:
                	// this event may occur during ObjectGrid deactivation too.
                	processEventStateInitialized(map);
                    break;
                case STARTING:
                	// in OSGI env, map serializer plugin may be swap and the map will go through life cycle event again,
                	// however, there will be not INITIALIZED event in such case.
                	// The sequence is like: OFFLINE -> STARTING -> ONLINE
                	// need to check if the map serializer has been changed, if so, process map serializer again.
                	if((map.getSerializerAccessor().getMapSerializerPlugin() != HashIndex.this.mapSerializer)) {
                		processEventStateInitialized(map);
                	}
                    break;
                case DESTROYED:
                    // In general its cheaper to recreate large collections
                    // than clear existing ones. We don't expect to be used
                    // again, but we'll keep the state of this object valid
                    // and just re-initialize the collections storing keys.
                    initializeCollections();
                    break;
                }
                
                // design change: alway enable multi-types mode for serilaizer case if it is not composite index 
                if(useSerializer && !isComposite) {
                	setIsMultiTypes(true);
                }

                // TEST support only:
                if(forceMultiTypesMode && !isComposite) {
                    setIsMultiTypes(true);
                    System.out.println("indexName="+indexName + ", forceMultiTypesMode="+forceMultiTypesMode + ", isMultiTypes="+isMultiTypes + ", attributeName="+attributeName);
                    //if(ivPOJOKeyIndex) {
                         //in POJO case, there is no keySerializerAttributeName, just same as value index case, attributeName
                    //     Tr.warning(tc, NLSConstants.HASHINDEX_ATTRIBUTE_NOT_FOUND_IN_SERIALIZER_METADATA_CWOBJ0065, new Object[] {indexName, map.getName(), com.ibm.ws.xs.util.Messages.getMsg(NLSConstants.KEY),attributeName, "KeyDataDescriptor"});
                    //} else {
                    //    Tr.warning(tc, NLSConstants.HASHINDEX_ATTRIBUTE_NOT_FOUND_IN_SERIALIZER_METADATA_CWOBJ0065, new Object[] {indexName, map.getName(), com.ibm.ws.xs.util.Messages.getMsg(NLSConstants.VALUE),attributeName, "ValueDataDescriptor"});
                    //}
                }
            } catch (Exception e) {
                // Any failure should be fatal for the map.
                FFDCFilter.processException(e, CLASS_NAME + "backingMapStateChanged", "5904");
                throw new LifecycleFailedException(e.getMessage(), e);
            }
        }

        private void processEventStateInitialized (BackingMap map) {
            HashIndex.this.mapSerializer = map.getSerializerAccessor().getMapSerializerPlugin();
            
            isAttributeNameSet  = attributeName != null && attributeName.trim().length() != 0;
            
            // Process any metadata provided by a serializer, if present.
            if (mapSerializer != null) {
                // d12187 - Allow attribute-less indexes for indexing the entire value.
            	//if(attributeName == null || attributeName.trim().length() == 0) {
                //    throw new IllegalStateException("Unable to initialize index: " + getName() + ".  AttributeName not set.");
                //}

                MapDataDescriptor mdd = mapSerializer.getMapDataDescriptor();
                boolean useKeySerializer = mapSerializer.getKeySerializerPlugin() != null;
                boolean useValueSerializer = mapSerializer.getValueSerializerPlugin() != null;

                isKeySerializer = false; // Will flip on this value if attributeName is key.xxx

                keyConf = (KeyConfig)map;

                // If no serializers, no additional configuration is needed at this stage.
                if (!(useKeySerializer || useValueSerializer)) {
                    return;
                }

                // Even if there are serializers, then the user still may be indexing a 
                // POJO for the key or value.

                // If a KeySerializer is defined, then determine what the key path looks like.
                String keyAddrName = null;
                String pathSep = null;
                KeyDataDescriptor kdd = null;
                if (useKeySerializer) {
                    // KeySerializer
                    kdd = HashIndex.this.mapSerializer.getKeySerializerPlugin().getKeyDataDescriptor();
                    keyAddrName = mdd.getAddressableKeyName();
                    if(kdd != null) {
                    	pathSep = kdd.getPathSeparator();
                    }
                }
                else {
                    // If there is no key serializer, but they have identified 
                    // a POJO key index, then assume they are indexing a POJO key and NOT using the
                    // value serializer.
                    if (ivPOJOKeyIndex) {
                        if (tc.isDebugEnabled()) {
                            Tr.debug(tc, "ivPOJOKeyIndex when no key serializer for a key attribute in a mixed serializer configuration: " + attributeName + " for map: " + map.getName() + " of ObjectGrid: " +  map.getObjectGrid().getName());
                        }
                        return;
                    }
                }
                
                // If there is an attributeName defined, we need to determine 
                // if it's a path to the KeySerializer or ValueSerializer.
                
                // If there is no attributeName defined, or if just the key path is specified, then the
                // entire key/value is being indexed.
                String keySerializerAttributeName = null;
                
                
                if(!isAttributeNameSet) {
                    // If no attribute name, then we assume the entire key or value data object is indexed.
                    if(ivPOJOKeyIndex) {
                        isKeySerializer = true;
                    } else {
                        isKeySerializer = false;
                    }
                } else {
                    if(keyAddrName != null && pathSep != null) {
                        String keyIdentifier = keyAddrName + kdd.getPathSeparator();
                        if(isComposite) {
                            if(ivPOJOKeyAttributeIndexes == null) {
                                ivPOJOKeyAttributeIndexes = new boolean[attributeNamesArray.length];
                            }
    
                            // composite index: we don't support multi-types on composite index
                            // and do not allow collection type attribute exists inside composite index
    
                            // remove key identifier from key attributes
                            for(int i=0; i<attributeNamesArray.length; i++) {
                                if(attributeNamesArray[i].startsWith(keyIdentifier) || ivPOJOKeyIndex) {
                                    // note: must check if serializer metadata contains this key attribute before removing key identifier.
                                    if(!((BaseMap)map).isCopyToBytesXDF() && !((SerializerAccessorImpl)map.getSerializerAccessor()).getCanonicalAttributes().containsKey(attributeNamesArray[i])) {
                                        // The key attribute is not found in key metadata
                                        Tr.warning(tc, NLSConstants.HASHINDEX_ATTRIBUTE_NOT_FOUND_IN_SERIALIZER_METADATA_CWOBJ0065, new Object[] {indexName, map.getName(), com.ibm.ws.xs.util.Messages.getMsg(NLSConstants.KEY),attributeNamesArray[i], "KeyDataDescriptor"});
                                    }
                                    // key attribute, remove key identifier, so later can retrieve attribute value from key serilaizer.
                                    if(!ivPOJOKeyIndex) {
                                        attributeNamesArray[i] = attributeNamesArray[i].substring(keyIdentifier.length());
                                    }
                                    ivPOJOKeyAttributeIndexes[i] = true;
                                } else {
                                    // note: The state INITIALIZED may occur again during tear down,
                                    // The key attributes have been processed in the 1st call, so the ivPOJOKeyAttributeIndexes is set
                                    // need to check ivPOJOKeyAttributeIndexes first.
                                    if(ivPOJOKeyAttributeIndexes[i]) {
                                        if(!((BaseMap)map).isCopyToBytesXDF() && !((SerializerAccessorImpl)map.getSerializerAccessor()).getCanonicalAttributes().containsKey(keyIdentifier + attributeNamesArray[i])) {
                                            // The key attribute is not found in key metadata
                                            Tr.warning(tc, NLSConstants.HASHINDEX_ATTRIBUTE_NOT_FOUND_IN_SERIALIZER_METADATA_CWOBJ0065, new Object[] {indexName, map.getName(), com.ibm.ws.xs.util.Messages.getMsg(NLSConstants.KEY),attributeNamesArray[i], "KeyDataDescriptor"});
                                        }
                                    } else {
                                        // value attribute
                                        if(!((BaseMap)map).isCopyToBytesXDF() && !ivPOJOKeyAttributeIndexes[i] && !((SerializerAccessorImpl)map.getSerializerAccessor()).getCanonicalAttributes().containsKey(attributeNamesArray[i])) {
                                            // The value attribute is not found in value metadata
                                            Tr.warning(tc, NLSConstants.HASHINDEX_ATTRIBUTE_NOT_FOUND_IN_SERIALIZER_METADATA_CWOBJ0065, new Object[] {indexName, map.getName(), com.ibm.ws.xs.util.Messages.getMsg(NLSConstants.VALUE),attributeNamesArray[i], "ValueDataDescriptor"});
                                        }
                                    }
                                }
                            }
    
                        } else {
                            // single attribute index
                            // Check and see if our attribute name addresses part of the key, or if teh ivPojoKeyIndex is defined
                            // key.id
                            // keyName = key
                            // pathsep = .
                            // attribute name = key.id
                            if(attributeName.startsWith(keyIdentifier) || ivPOJOKeyIndex) {
                                isKeySerializer = true;
                                // If the POJOKeyIndex flag is set, then take the attribute at face value.
                                // Otherwise, assume that the key prefix is part of the attribute name.
                                keySerializerAttributeName = attributeName;
                                if(!ivPOJOKeyIndex) {
                                    keySerializerAttributeName = attributeName.substring(keyIdentifier.length());
                                }
                                if(tc.isDebugEnabled()) {
                                    Tr.debug(tc, "Using a KeyDataSerializer for attribute: " + keySerializerAttributeName + " for map: " + map.getName() + " of ObjectGrid: " +  map.getObjectGrid().getName());
                                }
                                if(!((BaseMap)map).isCopyToBytesXDF() && !((SerializerAccessorImpl)map.getSerializerAccessor()).getCanonicalAttributes().containsKey(attributeName)) {
                                    // The key attribute is not found in key metadata, support multi-types
                                    // setIsMultiTypes(true); 
                                    Tr.warning(tc, NLSConstants.HASHINDEX_ATTRIBUTE_NOT_FOUND_IN_SERIALIZER_METADATA_CWOBJ0065, new Object[] {indexName, map.getName(), com.ibm.ws.xs.util.Messages.getMsg(NLSConstants.KEY),keySerializerAttributeName, "KeyDataDescriptor"});
                                }
                            } else {
                                // attribute is intended to be on the value.
                                isKeySerializer = false;
                                if (!useValueSerializer) {
                                    if (tc.isDebugEnabled()) {
                                        Tr.debug(tc, "No value serializer for a value attribute in a mixed serializer configuration: " + attributeName + " for map: " + map.getName() + " of ObjectGrid: " +  map.getObjectGrid().getName());
                                    }
                                    return;
                                }
                                if(!((BaseMap)map).isCopyToBytesXDF() && !((SerializerAccessorImpl)map.getSerializerAccessor()).getCanonicalAttributes().containsKey(attributeName)) {
                                    // The value attribute is not found in value metadata, support multi-types
                                    // setIsMultiTypes(true);
                                    Tr.warning(tc, NLSConstants.HASHINDEX_ATTRIBUTE_NOT_FOUND_IN_SERIALIZER_METADATA_CWOBJ0065, new Object[] {indexName, map.getName(), com.ibm.ws.xs.util.Messages.getMsg(NLSConstants.VALUE),attributeName, "ValueDataDescriptor"});
                                }
    
                                if (tc.isDebugEnabled()) {
                                    Tr.debug(tc, "Using a ValueDataSerializer for attribute: " + attributeName + " for map: " + map.getName() + " of ObjectGrid: " +  map.getObjectGrid().getName());
                                }
                            }
                        }
                    } else {
                        // no key identifier, assume all from value.
                        isKeySerializer = false;
                        if(isComposite) {
                            for(int i=0; i<attributeNamesArray.length; i++) {
                                if(!((BaseMap)map).isCopyToBytesXDF() && !((SerializerAccessorImpl)map.getSerializerAccessor()).getCanonicalAttributes().containsKey(attributeNamesArray[i])) {
                                    // The value attribute is not found in value metadata
                                    Tr.warning(tc, NLSConstants.HASHINDEX_ATTRIBUTE_NOT_FOUND_IN_SERIALIZER_METADATA_CWOBJ0065, new Object[] {indexName, map.getName(), com.ibm.ws.xs.util.Messages.getMsg(NLSConstants.VALUE),attributeNamesArray[i], "ValueDataDescriptor"});
                                    // composite index can not support multi-types
                                }
                            }
                        } else {
                            if(!((BaseMap)map).isCopyToBytesXDF() && !((SerializerAccessorImpl)map.getSerializerAccessor()).getCanonicalAttributes().containsKey(attributeName)) {
                                // The value attribute is not found in value metadata, support multi-types
                                // setIsMultiTypes(true);
                                Tr.warning(tc, NLSConstants.HASHINDEX_ATTRIBUTE_NOT_FOUND_IN_SERIALIZER_METADATA_CWOBJ0065, new Object[] {indexName, map.getName(), com.ibm.ws.xs.util.Messages.getMsg(NLSConstants.VALUE),attributeName, "ValueDataDescriptor"});
                            }
                        }
                    }
                }

                useSerializer = (useKeySerializer && isKeySerializer) || (useValueSerializer && !isKeySerializer);

                if(!isComposite) {
                    DataDescriptor dd = null;
                    if (isKeySerializer) {
                        dd = HashIndex.this.mapSerializer.getKeySerializerPlugin().getKeyDataDescriptor();
                    } else {
                        dd = HashIndex.this.mapSerializer.getValueSerializerPlugin().getValueDataDescriptor();
                    }
                    HashIndex.this.isCollectionAttribute = false;
                    if(dd != null && isAttributeNameSet) {
                        Map attributeMap = dd.getAttributes();

                        attributeDD = (Attribute)attributeMap.get(attributeName);
                        if(attributeDD != null) {
                            HashIndex.this.isCollectionAttribute = attributeDD.isCollection();
                        }
                    }

                    // Cache the serializer AttributeContext
                    if (isKeySerializer) {
                        // Need to use the NON prefixed attribute name here. The serializer doesn't use the prefix,
                        // only the map serializer and the canonical and full list of attributes.
                        serAttributeContextKey = HashIndex.this.mapSerializer.getKeySerializerPlugin().getAttributeContexts(keySerializerAttributeName);
                    } else {
                        serAttributeContextValue = HashIndex.this.mapSerializer.getValueSerializerPlugin().getAttributeContexts(attributeName);
                    }

                } else {
                    // composite index
                    checkCollectionAttributeIncluded(HashIndex.this.mapSerializer);

                    // Build the attribute context for the key and value composite.
                    ArrayList<String> valueAttributes = new ArrayList<String>();
                    ArrayList<String> keyAttributes = new ArrayList<String>();
                    for (int i = 0; i < attributeNamesArray.length; i++) {
                        if (ivPOJOKeyAttributeIndexes[i]) {
                            keyAttributes.add(attributeNamesArray[i]);
                        } else {
                            valueAttributes.add(attributeNamesArray[i]);
                        }
                    }
                    if (keyAttributes.size() > 0)
                        serAttributeContextKey = HashIndex.this.mapSerializer.getValueSerializerPlugin()
                                .getAttributeContexts((String[]) keyAttributes.toArray(new String[keyAttributes.size()]));
                    if (valueAttributes.size() > 0)
                        serAttributeContextValue = HashIndex.this.mapSerializer.getValueSerializerPlugin()
                                .getAttributeContexts((String[]) valueAttributes.toArray(new String[valueAttributes.size()]));

                    if (ObjectGridManagerImpl.isTraceEnabled && tc.isEntryEnabled()) {
                        Tr.exit(tc, "BMapLifecycleListener.processEventStateInitialized: index name = " + indexName
                                + " , attributeNames = " + ObjectGridUtil.getObjectArrayString(attributeNamesArray));
                    }
                }
            }
        }

        private void checkCollectionAttributeIncluded(MapSerializerPlugin mapSer) {
            //MapSerializerPlugin mapSer = map.getSerializerAccessor().getMapSerializerPlugin();
            MapDataDescriptor mapDD = mapSer.getMapDataDescriptor();
            if(mapDD != null) {
                KeySerializerPlugin kSer = mapSer.getKeySerializerPlugin();
                ValueSerializerPlugin vSer = mapSer.getValueSerializerPlugin();
                KeyDataDescriptor keyDD = null;
                ValueDataDescriptor valDD = null;
                Map<String, Attribute> keyAttributes = null;
                Map<String, Attribute> valueAttributes = null;
                if(kSer != null) {
                    keyDD = kSer.getKeyDataDescriptor();
                    if(keyDD != null) {
                        keyAttributes = keyDD.getAttributes();
                    }
                }
                if(vSer != null) {
                    valDD = vSer.getValueDataDescriptor();
                    if(valDD != null) {
                        valueAttributes = valDD.getAttributes();
                    }
                }

                for(int i=0; i<attributeNamesArray.length; i++) {
                    Attribute att = null;
                    if(ivPOJOKeyAttributeIndexes[i]) {
                        if(keyAttributes != null) {
                            att = keyAttributes.get(attributeNamesArray[i]);
                        }
                    } else {
                        // value attribute
                        if(valueAttributes != null) {
                            att = valueAttributes.get(attributeNamesArray[i]);
                        }
                    }

                       if(att != null && att.isCollection()) {
                           // if throw this error, this occurs in og.getSession() call that trigger OG initialization.
                           throw new ObjectGridRuntimeException("Multi-valued association: " + attributeNamesArray[i] + " is not supported in composite index: " + indexName);
                    }
                }
            }
        }
    }

    /**
     * This method must be called during initialization
     *
     * @param isMultiTypes
     */
    private void setIsMultiTypes (boolean isMultiTypes) {
        this.isMultiTypes = isMultiTypes;
        // if map serializer swapped, the attribute type may change
        // besides, the state change from OFFLINE to STARTING and to ONLINE.
        // During STARTING, if map serilaizer change, it processes map serializer again, and call this method again.
        // so need to check if typeIndexesMap==null.
        // if typeIndexesMap==null, means this is first time to initialize, create a new index structure.
        // otherwise, continue to use the existing index structure.
        if(typeIndexesMap == null) {
        	this.typeIndexesMap = new ConcurrentHashMap();
        }
        index = null; // single type index can be null.
    }

    /**
     * when calling this method and the multi-types is enabled, the passed in attributeObejct parameter can not be null.
     *
     * @param attributeObejct can NOT be null in multi-types case
     * @return
     */
    Map getCurrentIndex(Object attributeObejct) {
        if(!isMultiTypes) {
            return index;
        } else {
            Map typeIndex = null;
            if (inverseRangeIndexEnabled) {
                typeIndex = (Map)typeIndexesMap.get(attributeObejct);
            } else {
                // in multi types mode, the passed attributeObejct can not be null.
                typeIndex = (Map)typeIndexesMap.get(attributeObejct.getClass().getName());
            }
            if(typeIndex == null) {
                // InverseRangeIndex: createTypeIndex for attributeObject, which will be identifier or non-range attributes
                if (inverseRangeIndexEnabled) {
                    typeIndex = createTypeIndex(attributeObejct, ivRangeIndex);
                } else {
                    // create a new type index, considering if this is for range index
                    typeIndex = createTypeIndex(attributeObejct.getClass().getName(), ivRangeIndex);
                }
            }
            return typeIndex;

        }
    }

    /**
     * This method is to return SortedMap to support range index query.
     *
     * @param attributeObejct if null return the first found index to support index query without parameter, such as findMax.
     * @return
     */
    private SortedMap getCurrentRangeIndex(Object attributeObejct) {
        if(!isMultiTypes) {
            return (SortedMap)index;
        } else {
            SortedMap typeIndex = null;
            if(attributeObejct != null) {
                typeIndex = (SortedMap)typeIndexesMap.get(attributeObejct.getClass().getName());
                if(typeIndex == null) {
                    typeIndex = (SortedMap)createTypeIndex(attributeObejct.getClass().getName(), true);
                }
            } else {
                // to support index query w/o parameter, only return the first found type index.
                Iterator iter = typeIndexesMap.values().iterator();
                if(iter.hasNext()) {
                    typeIndex = (SortedMap)iter.next();
                }
            }
            return typeIndex;

        }
    }

    /**
     * This should be the only place to create a type index
     *
     * @param typeIndexName
     * @return
     */
    private synchronized Map createTypeIndex(Object typeIndexName, boolean isRange) {
        Map typeIndex = (Map)typeIndexesMap.get(typeIndexName);
        if(typeIndex == null) {
            if(isRange) {
                typeIndex = new ConcurrentSkipListMap();
            } else if (inverseRangeIndexEnabled) {
                //InverseRangeIndex: Type index will be a ConcurrentSkipListMap using InverseRangeComparator
                typeIndex = new ConcurrentSkipListMap(new InverseRangeComparator());
            } else {
                typeIndex = new ConcurrentHashMap();
            }
            // double check again
            if(typeIndexesMap.get(typeIndexName) == null) {
                typeIndexesMap.put(typeIndexName, typeIndex);
            }
            typeIndex = (Map)typeIndexesMap.get(typeIndexName);
            // at this point, if typeIndex is null, report error.
            if(typeIndex == null) {
                throw new ObjectGridRuntimeException("The typeIndex is null for the type [" + typeIndexName + "]");
            }
        }
        return typeIndex;
    }
    
    void clearGlobalIndex() {
    	// child class GridIndex needs to handle this call.
    	globalIndex.clear();
    }
    
    private synchronized void createGlobalIndex() throws ObjectGridRuntimeException {
    	if(globalIndex==null) {
    		globalIndex = new GlobalIndexImpl(this.backingMap, indexName, globalIndexMapName);
    		globalIndex.setUseAsyncQueue(globalIndexAsyncUpdate);
    		globalIndex.setDebug(debug);    		
    		globalIndex.initialize();
    	}
    }

    /**
     * This flag is only used by POJO testcases that wishes to run in multi-types mode.
     * After formal Serializer HashIndex multi-types testcases are complete, relatd flag and method can be deleted.
     */
    private static boolean forceMultiTypesMode = false;
    /**
     * This method only for testing multi types support using HashIndex POJO testcases.
     * HashIndex POJO testcases can use utility method to force to enable multi types mode and test it.
     *
     * @param enableMultiTypesMode
     */
    private static void forceMultiTypesMode(boolean enableMultiTypesMode) {
        forceMultiTypesMode = enableMultiTypesMode;
        System.out.println("HashIndex global forceMultiTypesMode is set to " + forceMultiTypesMode);
    }
    
    /**
     * Answer true if the index is a single-attribute key index.
     */
    private boolean isNonCompositeKeyIndex() {
        return !isComposite && (ivKeyTupleAttributeIndex || ivPOJOKeyIndex ||  isKeySerializer);
    }
   
}